#!/bin/sh

### Tunable Parameters
#
# Specifies the number of beacons being calibrated.
#

NUMBEACONS=4

#
# STDDEVTHRESH is a threshold value expressed in (cm)
# This value is used to define
#
# 1) The threshold for determine a stable distance measurement during
#    the calibration phase.  A distance measurement is "stable" when 
#    the stddev value of the distances collected over 
#    the last WINDOW_FILT ms falls below STDDEVTHRESH.
#
# 2) The threshold for filtering outlier distance samples during the
#    tracking phase.  Currently, sample distances that are > 3*STDDEVTHRESH
#    are filtered.  
#
# The recommended value for Cricket v1 is 15 and for Cricket v2 10 cm.
#

STDDEVTHRESH=10

#
# Size of history window in milliseconds. For Cricket v1, the recommended
# value is 15000.  For Cricket v2, the recommended value is 4000.  In all 
# cases, the mode value of the distances collected in the history window 
# is used to estimate the true distance between a beacon and the listener.
#

WINDOW_FILT=4000

#
# Size of the GUI window in pixels (x,y-dimension ~ WIN_SIZE x WIN_SIZE)
#

WIN_SIZE=500

# The following specification is deprecated.  Now MAXCOORD
# just specifies the initial coordinate system size as described
# below.  If any of the beacon coordinates lie outside of this
# coordinate system, BeaconConfig now re-scales the coordinate system
# so that all of the beacons will show within the window.
#
# MAXCOORD specifies the size of the coordinate system in cm.
# Express coordinate boundary as (x1,y1,x2,y2), where (x1,y1) 
# is the lower left hand corner coordinate and (x2,y2) is the 
# upper right hand corner. Then setting the MAXCOORD value below 
# defines the screen coordinates with boundaries
# (-MAXCOORD, -MAXCOORD, +MAXCOORD, +MAXCOORD).  The (0,0) origin is 
# in the center of the screen.  MAXCOORD is in units of cm.  
# So if MAXCOORD=400, you get a 8m x 8m area.  The scale of 
# true-to-life distance to screen pixels will thus be
#
#   (2*MAXCOORD)/WIN_SIZE  ( units: cm/screen_pixel )
#

MAXCOORD=15

#
# GUI grid size in cm
#

GRID_SIZE=50

#
# CRICKETD_IP_ADDR specifies the ip location of cricketd (aka gpsd).
# Use 127.0.0.1 for localhost.
#

CRICKETD_IP_ADDR=10.0.0.100
#CRICKETD_IP_ADDR=127.0.0.1

#
# This defines the maximum separation distance during calibration.
# That is, the listener must be held within SEPDIST cm directly below
# a beacon during calibrahtion.  This also implies that beacons should
# be placed at least SEPDIST cm apart.  Otherwise, BeaconConfig cannot
# determine which beacon it is trying to calibrate.
#

#SEPDIST=100
SEPDIST=5000

#
# Uncomment the following to enable connection with BeaconConfigRemote.
# If the following is uncommented AND no BeaconConfigRemote connects
# to the local BeaconConfig program, the GUI may "hang" due to network
# timeouts.
#

REMOTE_ENABLE="-r"

#
############################################# End of Tunable Params

if [ "$OS" = "Windows_NT" ]
then SEP=";"
else SEP=":"
fi

CLASSPATH=".${SEP}beaconfinder.jar${SEP}beaconconfig.jar${SEP}cricketdaemon.jar${SEP}Jama-1.0.1.jar${SEP}viewfinder.jar${SEP}gnu.getopt.jar"

#Usage: BeaconFinderApp cricketdaemon_ip cricketdaemon_port stddev
java -cp $CLASSPATH beaconfinder.BeaconFinderApp localhost 5001 $STDDEVTHRESH &

# units are in cm, except for winsize, which is in pixel
#Usage: BeaconConfig numBeacons closeness_thresh stddev_thresh winsize maxcoord gridstep

# debugging flags
# use jdb attach jdbconn

#DEBUG="-Xdebug -Xrunjdwp:transport=dt_shmem,address=8000,server=y,suspend=n"
#DEBUG=

java -cp $CLASSPATH beaconconfig.BeaconConfig -n $NUMBEACONS -c $SEPDIST -s $STDDEVTHRESH -W $WIN_SIZE -w $MAXCOORD -g $GRID_SIZE $REMOTE_ENABLE &
java -cp $CLASSPATH cricketdaemon.CricketDaemon -g$CRICKETD_IP_ADDR -S$WINDOW_FILT
#jdb -classpath $CLASSPATH cricketdaemon.CricketDaemon

