/*
 * BeaconData class holds data for a beacon; namely its ID, distance_to (in
 * us) to origin, distance_to to x-hat, distance_to to y-hat, and calculated xyz
 * coordinates.  ID is immutable, but distance_to 'distances' can be changed
 * using given methods.
 */

import java.lang.StrictMath;

public class BeaconData{
    
    private static final int NUM_SAMPLES = 5;

    private String id;
    private int x, y, z;
    private int p_origin, p_xhat, p_yhat;
    private int[] distance_to_origin = new int[NUM_SAMPLES];
    private int[] distance_to_xhat = new int[NUM_SAMPLES];
    private int[] distance_to_yhat = new int[NUM_SAMPLES];
    private boolean valid;
    private boolean cvalid;

    public BeaconData(){
// 	this.id = new String();
// 	this.x = 0;
// 	this.y = 0;
	this.valid = false;
	this.cvalid = false;
	this.p_origin = 0;
	this.p_xhat = 0;
	this.p_yhat = 0;

    }

    public boolean valid(){
	return valid;
    }

    public boolean cvalid(){
	return cvalid;
    }

    public void cvalid(boolean cvalid){
	this.cvalid = cvalid;
    }

    public void id(String id){
	this.id = id;
	this.distance_to_origin[p_origin] = 0;
	this.distance_to_xhat[p_xhat] = 0;
	this.distance_to_yhat[p_yhat] = 0;
	this.valid = true;
    }

    public String id(){
	return id;
    }

    private void recalc(){
	if ((distance_to_origin[p_origin] == 0) || (distance_to_xhat[p_xhat] == 0) ||
	    (distance_to_yhat[p_yhat] == 0)) {
	    System.out.println("recalc aborted; one or more distances = 0");
	    this.cvalid = false;
	}
	else {
	    System.out.println("distance to origin = " + distance_to_origin[p_origin] + " cm");
	    System.out.println("distance to xhat = " + distance_to_xhat[p_xhat] + " cm");
	    System.out.println("distance to yhat = " + distance_to_yhat[p_yhat] + " cm");
	    double a = distance_to_origin[p_origin];
	    double b = distance_to_yhat[p_yhat];
	    double c = distance_to_xhat[p_xhat];
	    //System.out.println(a + " " + b + " " + c);
	    double tempx =  ((-1.0/100.0)*(c*c))+25.0+((1.0/100.0)*(a*a));
	    double tempy = ((-1.0/100.0)*(b*b))+25.0+((1.0/100.0)*(a*a));
 	    double tempz = (1.0/100.0)*(StrictMath.sqrt(-12500000.0-(c*c*c*c)+(5000.0*c*c)+(2.0*c*c*a*a)+(2.0*b*b*a*a)-(2.0*a*a*a*a)-(b*b*b*b)+(5000.0*b*b)));
	    //System.out.println("Beacon " + id + "'s coordinates are (" + tempx + "," + tempy + "," + tempz + ")");
	    x = (int)tempx;
	    y = (int)tempy;
	    z = (int)tempz;
	    System.out.println("Beacon " + id + "'s coordinates are (" + x + "," + y + "," + z + ")");
	    this.cvalid = true;

	}
	
    }

    public int[] origin(){
	return distance_to_origin;
    }

    public void origin(int dist){
	if (this.distance_to_origin[p_origin] != dist){
	    System.out.println("Updating beacon data, recalculating coordinates for beacon " + id);
	    this.distance_to_origin[p_origin] = dist;
	    this.recalc();
	}
    }

    public int[] xhat(){
	return distance_to_xhat;
    }

    public void xhat(int dist){
	if (this.distance_to_xhat[p_xhat] != dist){
	    System.out.println("Updating beacon data, recalculating coordinates for beacon " + id);
	    this.distance_to_xhat[p_xhat] = dist;
	    this.recalc();
	}
    }

    public int[] yhat(){
	return distance_to_yhat;
    }

    public void yhat(int dist){
	if (this.distance_to_yhat[p_yhat] != dist){
	    System.out.println("Updating beacon data, recalculating coordinates for beacon " + id);
	    this.distance_to_yhat[p_yhat] = dist;
	    this.recalc();
	}
    }
    
    public void coords(int x, int y, int z){
	this.x = x;
	this.y = y;
	this.z = z;
	this.cvalid = true;
    }

    public int xcoord(){
	return x;
    }

    public int ycoord(){
	return y;
    }

    public int zcoord(){
	return z;
    }

//     public void putdata(String id, int distance_to_origin, int distance_to_xhat, int distance_to_yhat){
// 	this.id = id;
// 	this.distance_to_origin = distance_to_origin;
// 	this.distance_to_xhat = distance_to_xhat;
// 	this.distance_to_yhat = distance_to_yhat;
// 	this.valid = true;
//     }

//     public void putdata_c(String id, int x, int y, int z){
// 	this.id = id;
// 	this.x = x;
// 	this.y = y;
// 	this.z = z;
// 	this.valid = true;
//     }

//     public void setdistance_to(int distance_to_origin, int distance_to_xhat, int distance_to_yhat){
// 	this.distance_to_origin = distance_to_origin;
//         this.distance_to_xhat = distance_to_xhat;
//         this.distance_to_yhat = distance_to_yhat;
//     }



//     public void setcoords(int x, int y, int z){
// 	this.x = x;
// 	this.y = y;
// 	this.z = z;
//     }

}
