// This class taken from Kevin (Jiawen) Chen's Cricket3D application
// Modified by Roshan Baliga <roshan@mit.edu>, March 2004

import java.util.*;
import java.io.*;
import java.net.*;

// $Cricket2,space=NTT1,id=20,dist=102
public class CricketClient extends Thread
{
    // Network I/O
    private String ip;
    private int port;
    private Socket socket;
    private BufferedReader socketReader;
    private OutputStream outputStream;

    // Status
    private String id;
    private char type;
    private int distance;
    private boolean online;
    private boolean frozen;

    // Callback list
    private LinkedList callbacks;

    private WallsDisplay display;

    public CricketClient(String ip, int port, WallsDisplay display)
    {
	this.ip = ip;
	this.port = port;
	// this.id = id;
	this.display = display;
	online = false; // only there so that connect() can be reused
	frozen = false;
	callbacks = new LinkedList();
    }

    public void freeze() {
	
	if (frozen) {
	    frozen = false;
		System.out.println("unfreezing!");
		}
	else {
	    frozen = true;
	    System.out.println("freezing!");	
	}
    }
    
    public synchronized void connect()
		throws IOException
    {
	System.out.println("Connecting to " + ip + ":" + port);
	if (online)
	    disconnect();

 	socket = new Socket(ip, port);
 	socketReader = new BufferedReader
 	    (new InputStreamReader(socket.getInputStream()));
 	outputStream = socket.getOutputStream();
	online = true;
	if (online) {
	    System.out.println("online is true");
	}
    }

    public synchronized void connect(String ip, int port)
	throws IOException
    {
	this.ip = ip;
	this.port = port;
	connect();
    }

    public synchronized void disconnect()
	throws IOException
    {
	if (online)
	    {
		// outputStream.write("r\n".getBytes());
		socket.close();
		socketReader = null;
		outputStream = null;
		online = false;
	    }
    }

//     public synchronized void register(Callback cb)
//     {
// 	callbacks.addLast(cb);
//     }

//     public synchronized void deregister(Callback cb)
//     {
// 	callbacks.remove(cb);
//     }

    public void run()
    {
	System.out.println("Running...");
// 	// send "r\n" to gpsd, to register
// 		try
// 		    { outputStream.write("r\n".getBytes()); }
// 	catch (IOException ie)
// 	    {
// 		System.out.println("Unable to write to socket, stopping...");
// 		try
// 		    { disconnect(); }
// 		catch (IOException iee)
// 		    { System.out.println("Could not close socket, sorry!"); }
// 	    }

	String line;
	synchronized(this)
	    {
		while (online)
		    {
			try
			    {
				line = socketReader.readLine();
// 				update(convertRawToCricketData(line));

				// next 'println' commented out, so we
				// only print lines when not frozen.
//				System.out.println(line);
				if (!frozen) {
				    parseline(line, display);
				    // System.out.println(line.substring(16));

				}
 			    }
			catch (IOException ie)
			    {
				System.out.println
				    ("Communications error, stopping...");
				try
				    { disconnect(); }
				catch (IOException iee)
				    {
					System.out.println
					    ("Could not close socket, sorry!");
				    }
			    }
		    }
	    }
    }

    public static void tokenit(String restofline, char type, WallsDisplay display) {
	StringTokenizer st = new StringTokenizer(restofline, ";");
	while (st.hasMoreTokens()) {
	    //System.out.println(st.nextToken());
	    String mytoken = st.nextToken();
	    if (mytoken.length() > 10) {
		// id:
		String id = mytoken.substring(0, mytoken.indexOf("d")).trim();
		if (id.equals("0")){
		}
		else if (id.equals("1")){
		}
		else if (id.equals("36")){
		}
		else { //this section is only run if id is NOT 0, 1, or 36

		    // distance in microseconds: (yes, i know time is
		    // not a measure of distance. but have you even
		    // thought about what you answer when a friend
		    // asks, "how far is it from here to building
		    // W20?"  you answer, "oh, about five minutes." so
		    // there!)

		    int distance = (int)Float.parseFloat(mytoken.substring(mytoken.indexOf("=") + 1).trim());
		    // this distance value is in microseconds tof for US
		    // (assume 30 microseconds per centimeter)
		    // System.out.println(id);		   
		    // System.out.println(distance);
		    System.out.println("Distance from beacon " + id + " to " + type + " is " + distance/30 + " cm.");
		    display.updateBeacon(id, type, distance/30);		    
		}
	    }
	}
    }
    
    public static void parseline(String rawline, WallsDisplay display) {
	System.out.println(rawline);
	if (rawline.length() > 30) {
	    if (rawline.substring(2, 6).equals("from") ) {		
		if (rawline.substring(6, 9).trim().equals("0")){
		    char type = 'x';
		    // System.out.println("Type is " + type + ", distances:");
		    // System.out.println(rawline.substring(17));
		    tokenit(rawline.substring(17), type, display);
		}
		else if (rawline.substring(6, 9).trim().equals("1")) {
		    char type = 'y';
		    // System.out.println("Type is " + type);
		    // System.out.println(rawline.substring(17));
		    tokenit(rawline.substring(17), type, display);		
		}
		else if (rawline.substring(6, 9).trim().equals("36")){
		    char type = 'o';
		    // System.out.println("Type is " + type);
		    // System.out.println(rawline.substring(17));
		    tokenit(rawline.substring(17), type, display);		
		}
		else {
		    System.out.println("(Skipping the line printed immediately above this.)");
		}
	    }
	}
    }
}
	
/*
 * Everything below this line is stuff from Kevin Chen's
 * implementation that I decided I didn't really need.
 */

// This part actually works with certain real data formats
// 	StringTokenizer st = new StringTokenizer(rawline);
// 	String id = st.nextToken();
// 	st.nextToken();
// 	Float xfloat = new Float(Float.parseFloat(st.nextToken()));
// 	int x = xfloat.intValue();
// 	st.nextToken();
// 	Float yfloat = new Float(Float.parseFloat(st.nextToken()));
// 	int y = yfloat.intValue();
// 	System.out.println("id is " + id + ", x is " + x + ", y is " + y);
// 	display.updateBeacon(id, x, y);


	// String id = st.nextToken();
// 	int x = Integer.parseInt(st.nextToken());
// 	int y = Integer.parseInt(st.nextToken());
// 	System.out.println("id is " + id + ", x is " + x + ", y is " + y);
	//	display.updateBeacon(id, x, y);
	// we need to updated the display; maybe i need to implement
	// a callback or something in here?
	//	this.this.this.updateBeacon(id, x, y);

    

//     private synchronized void update(CricketData cd)
//     {
// 	if (cd != null)
// 	    for(Iterator itr=callbacks.iterator(); itr.hasNext();)
// 		((Callback)(itr.next())).callback(cd, "Cricket", id);
//     }

    /* Line format for a cricket string as of this rev is:
       $Cricket2,space=SPACENAME,id=IDINTEGER,dist=DISTINTEGER
    */
//     public static CricketData convertRawToCricketData(String rawData)
//     {
// 	StringTokenizer st = new StringTokenizer(rawData, ",=");
// 	String useless = st.nextToken(); // gives "$Cricket2"
// 	if (useless.equals("GPSD")) // first line is GPSD, ignore it
// 	    return null;
// 	else
// 	    {
// 		st.nextToken(); // gives "space="
// 		String name = st.nextToken();
// 		st.nextToken(); // gives "id="
// 		// give it the actual id
// 		int id = Integer.parseInt(st.nextToken(), 16) - 32;
// 		st.nextToken(); // gives "dist="
// 		String dist = st.nextToken();
// 		int parsedDist = Integer.parseInt(dist, 16);
// 		return new CricketData(name, id,
// 				       ((double)(parsedDist - 36))
// 				       * 2.2 / 2.54,  // convert to inches
// 				       System.currentTimeMillis());
// 	    }
//     }

//     public static void main(String[] args)
// 	throws IOException
//     {
// 	CricketClient cc = new CricketClient("18.24.6.167", 2947, "Listener");
// 	cc.start();
// 	while(true);
//     }
