/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class BeaconConfigRemote 
{
    NetworkClient nc;

    public BeaconConfigRemote(String ipaddr)
    {
	CustomFrame f = new CustomFrame("Beacon Configuration", 310, 150);
	Container container = f.getContentPane();
	BorderLayout layout = new BorderLayout();
	container.setLayout(layout);
	f.setBackground(Color.white);

	nc = new NetworkClient(ipaddr);

	RemoteSketchPanel sp = new RemoteSketchPanel(null, nc, true);
	RemoteControls cntl = new RemoteControls(null, null, nc, true);
	RemoteSolverPanel solver = new RemoteSolverPanel(null, null, 0, 0, 0, nc);

	container.add(cntl, "North");
	container.add(sp, "Center");
	container.add(solver, "South");

	solver.setBorder(BeaconConfig.createBorder());

	Controls.getResetButton().addActionListener(sp);
	nc.addRemoteSyncListener(sp);
	nc.addRemoteSyncListener(cntl);
	nc.addRemoteSyncListener(solver);

	nc.start();
	f.validate();
	f.pack();
	f.setVisible(true);
    }

    public static void main(String[] args) {
	if(args.length < 1) {
	    Util.assert(false, "Usage: java beaconconfig.BeaconConfigRemote ipaddr");
	}
	try {
	    String look = UIManager.getCrossPlatformLookAndFeelClassName();
	    System.out.println(look);
	    UIManager.setLookAndFeel(look);
	} 
	catch (Exception e) { }
	BeaconConfigRemote bcr = new BeaconConfigRemote(args[0]);
    }
}

