/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// no rotation
class Circle extends Vertex implements Sketchable
{
    public static final double TOLERANCE = 15;

    public double radius;

    public Circle(double x, double y, double r)
    {
	super(x, y);
	update(x, y, r);
	super.sketchedObjects.add(this);
    }

    public void update(double x, double y, double r)
    {
	radius = r;
	this.x = x;
	this.y = y;
    }

    public boolean inBound(double x, double y)
    {
	double dist = rad2D(this.x, this.y, x, y);
	return Math.abs(dist - radius) <= TOLERANCE;
    }

    public double rad2D(double px, double py, double qx, double qy)
    {
	return Math.sqrt(sqr(px - qx) + sqr(py - qy));
    }

    public double sqr(double r)
    {
	return r*r;
    }

    public void draw(Graphics g, DrawingArea drawArea)
    {
	Vertex pp = drawArea.transform(this.x-radius, this.y+radius);
	Vertex qq = drawArea.transform(this.x+radius, this.y-radius);

	//changeColor(Color.red.darker().darker());
	g.setColor(Color.red.darker().darker());

	// java-ism
	g.drawOval(Math.round((float)pp.x), Math.round((float)pp.y), 
		   Math.round((float)Math.abs(qq.x-pp.x)), Math.round((float)Math.abs(qq.y-pp.y)));

    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier) 
    { 
	double newRad = rAnchor;
	double newX   = cAnchor.x;
	double newY   = cAnchor.y;

	newRad = Math.sqrt(distSqr(cAnchor.x, cAnchor.y, cAnchor.z, x, y, z));
	update(newX, newY, newRad);
    }

    Vertex cAnchor = new Vertex(0, 0);
    double rAnchor;
    public void mousePressed(double x, double y, int modifier) 
    { 
	cAnchor.x = this.x;
	cAnchor.y = this.y;
	cAnchor.z = this.z;
	rAnchor   = radius;
    }

    public Vertex addPoint(double x, double y)
    {
	return null;
    }

    public Vertex updateCoord(double x, double y)
    {
	radius = rad2D(this.x, this.y, x, y);
	return null;
    }
}

