/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;


import cricketdaemon.clientlib.ServerBroker;
import cricketdaemon.clientlib.data.BeaconRecord;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class DrawingArea 
    extends JPanel
    implements MouseListener, MouseMotionListener, ComponentListener
	       //, KeyListener
{
    public static final double AREAMARGIN = 50; // draw area margin

    private Node prevMousePos, selectPos, anchorPoint;
    private Image img;

    public static final int nodeSize = 8; // assume this to be even
    public static final int nodeSizeDiv2 = nodeSize/2;

    /* transform stuff */
    public static Vertex minCoord, maxCoord, translate;
    public double scaleX, scaleY;

    Vertex currentSelectObject;
    Vertex currentMouseOverObject;

    private boolean gridOn = false;
    private boolean mouseEnabled = true;
    private boolean manualMode = false;
    private boolean annulusMode = false;

    //private static final int COORD_WIDTH = 150;
    //public static final int GRID_STEP = 24;

    //private int coordWidth, coordHeight;
    public  int gridStep;

    private Font font;

    private Dimension oldDim, dDim;

    public DrawingArea(int width, int coordWidth, int gridStep) 
    {
	super();

	font = new Font("Arial", Font.PLAIN, 18);

	if(mouseEnabled) {
	    addMouseListener(this);
	    addMouseMotionListener(this);
	}

	this.gridStep  = gridStep;
	this.translate = new Vertex(0, 0);
	this.minCoord  = new Vertex(0, 0);
	this.maxCoord  = new Vertex(0, 0);
	this.oldDim    = new Dimension(width, width);
	this.dDim      = new Dimension(0, 0);
	setDimensions(width, width, coordWidth, coordWidth);
    }

    private void setDimensions(int width, int height, 
			       int coordWidth, int coordHeight)
    {
	//System.out.println(oldDim+"\n"+width+" "+height+" "+coordWidth+" "+coordHeight);
	//preferredSize = new Dimension(width, width);
	setSize(width, height);
	setPreferredSize(getSize());
	
	// moves (0,0) to center of the canvas

	translate.x = width/2.0;
	translate.y = height/2.0;

	/* coordinates are in centimeter unit */
	minCoord.x = -coordWidth;
	minCoord.y = -coordHeight;

	maxCoord.x = coordWidth;
	maxCoord.y = coordHeight;
	
	scaleX = width/(maxCoord.x-minCoord.x);
	scaleY = -height/(maxCoord.y-minCoord.y);  

	// keep the scale squared
	if(Math.abs(scaleX) < Math.abs(scaleY))
	    scaleY = -scaleX;
	if(Math.abs(scaleY) < Math.abs(scaleX))
	    scaleX = -scaleY;

	img = createImage(getPreferredSize().width, 
			  getPreferredSize().height);
    }

    // Initialize data structure
    public void initialize() 
    {
	anchorPoint  = new Node(-10, -10);
	prevMousePos = new Node(-10, -10);
	selectPos    = new Node(-10, -10);
	currentSelectObject = null;
	currentMouseOverObject = null;
	cricketListener = null;
	/*
	img = createImage(getPreferredSize().width, 
			  getPreferredSize().height);
	*/
	Vertex.clearList();
	repaint();
    }

    public void mousePressed(MouseEvent e) {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	Util.dbgPrintln("mpressed: "+modifier+" "+MouseEvent.BUTTON3_MASK+
			"at (" +x+","+y+").");

	anchorPoint.setLocation(x, y);
	prevMousePos.setLocation(x, y); 
	selectPos.setLocation(x, y);

	if(manualMode) {
	    Vertex v = invTransformStatic(x, y);
	    currentSelectObject = findObject(v.x, v.y);
	    
	    // checking MouseEvent.BUTTON1_MASK doesn't work for netscape,
	    // so check for button3 and reverse the if-condition claues instead
	    if(currentSelectObject != null) {
		currentSelectObject.mousePressed(v.x, v.y, modifier);
		/* disable deleting beacons
		if ((modifier & MouseEvent.BUTTON3_MASK) > 0) {
		   currentSelectObject.remove();
		   currentSelectObject = null;
		}
		*/
	    }
	}

	repaint();
    }

    private boolean isVerticalDisplacement(int x, int y)
    {
	Util.assert(selectPos != null, 
		    "DrawingArea::isVerticalDisplacement");
	double slope = (double) (x - selectPos.x)/(y - selectPos.y);
	return Math.abs(slope) >= 1.0;
    }

    public void mouseDragged(MouseEvent e) 
    {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	/* to restrict displacement to either HORIZONTAL or VERTICAL */
	if ((modifier & MouseEvent.ALT_MASK) > 0) {
	    if(isVerticalDisplacement(x, y))
		y = selectPos.y;
	    else
		x = selectPos.x;
	}

	int xx = Math.max(Math.min(x, getPreferredSize().width-nodeSizeDiv2), nodeSizeDiv2);
	int yy = Math.max(Math.min(y, getPreferredSize().height-nodeSizeDiv2), nodeSizeDiv2);

	Util.assert(prevMousePos != null, "DrawingArea::mouseDragged()");
	Util.dbgPrintln("Dragging: (" + xx + ", " + yy + ")");
	prevMousePos.setLocation(xx, yy);

	//if(currentObjectUnderMouse != null && !insertMode) {
	Vertex v = invTransformStatic(xx, yy);
	Vertex anchor = invTransform(anchorPoint.x, anchorPoint.y);
	if(currentSelectObject != null) 
	    currentSelectObject.mouseDragged(v.x, v.y, anchor, modifier);
	    
	repaint();
    }
    
    public void mouseMoved(MouseEvent e) { 
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	Vertex v = invTransformStatic(x, y);
	Vertex f = findObject(v.x, v.y);

	if(currentMouseOverObject != null) {
	    currentMouseOverObject.changeColor(currentMouseOverObject.DEFAULT_COLOR);
	}
	if(f != null && f instanceof Beacon) {
	    currentMouseOverObject = f;
	    f.changeColor(Color.darkGray);
	    repaint();
	}
	else if(currentMouseOverObject != null) {
	    currentMouseOverObject = null;
	    repaint();
	}
    }

    public void setGrid(boolean selected) {
	gridOn = selected;
	repaint();
    }

    public void setManualOverride(boolean selected) {
	manualMode = selected;
    }

    public void setAnnulus(boolean selected) {
	annulusMode = selected;
	repaint();
    }

    public boolean isAnnulusMode() {
	return annulusMode;
    }

    public void addBeacon(BeaconRecord br, ServerBroker s)
    {
	new Beacon(br, s);
	double x = maxCoord.x;
	double y = maxCoord.y;

	boolean change = false;

	if(Math.abs(br.pos.x) > (x-AREAMARGIN)) {
	    change = true;
	    x = Math.abs(br.pos.x) + AREAMARGIN;
	}

	if(Math.abs(br.pos.y) > (y-AREAMARGIN)) {
	    change = true;
	    y = Math.abs(br.pos.y) + AREAMARGIN;
	}
	
	// keep a square formation
	if(x > y)
	    y = x;
	if(y > x)
	    x = y;

	if(change) {
	    //System.out.println("addBeacon called before setDimension mx="+maxCoord.x+" my="+
	    //maxCoord.y+" bx="+br.pos.x+" by="+br.pos.y);
	    setDimensions(getSize().width, getSize().height,
			  (int)Math.round(x),
			  (int)Math.round(y));
	}
	
	repaint();
    }

    // very ugly abstraction.  should decouple beacon management from
    // drawing routines!  oh well, quick hack.
    public BeaconRecord findBeacon(BeaconRecord br)
    {
	return Beacon.findBeacon(br);
    }

    private SquareVertex cricketListener = null;

    private long lastStreamerAdd = 0;
    private long streamerSamplingPeriod = 200;
    private long streamerDelay = 3000;
    private boolean streamerEnabled = false;

    public void setStreamerSamplingPeriod(long t) { streamerSamplingPeriod = t; }
    public void setStreamerDelay(long t) { streamerDelay = t; }
    public void setStreamer(boolean b) { streamerEnabled = b; }

    public synchronized void moveCricketListener(double x, double y, double z)
    {
	StreamerVertex.refresh();
	if(cricketListener == null) {
	    cricketListener = new SquareVertex(x, y, z);
	}
	else {
	    cricketListener.x = x;
	    cricketListener.y = y;
	    cricketListener.z = z;
	}

	long curTime = System.currentTimeMillis();
	if((streamerSamplingPeriod > 0) &&
	   (streamerEnabled) &&
	   ((curTime - lastStreamerAdd) > streamerSamplingPeriod)) {
	    new StreamerVertex(x, y, z, streamerDelay);
	    lastStreamerAdd = curTime;
	}

	repaint();
    }

    public double getCricketListenerHeight()
    {
	return cricketListener.z;
    }

    public void mouseReleased(MouseEvent e) 
    {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	Util.dbgPrintln("mouseReleased "+x+" "+y);

	anchorPoint.setLocation(-10, -10);
	prevMousePos.setLocation(-10, -10);
	selectPos.setLocation(-10, -10);

	if(currentSelectObject != null)
	    currentSelectObject.mouseReleased(x, y);

	currentSelectObject = null;
	repaint();
    }

    public void mouseClicked(MouseEvent e) { return; }
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }

    public void update(Graphics g)
    {
	//System.out.println("update");
	//double buffering to avoid flickering
	if(img != null) {
	    Graphics gc = img.getGraphics();
	    paintComponent(gc);
	    g.drawImage(img, 0, 0, this);
	}
    }

    private Dimension diffDim(int w, int h)
    {
	dDim.setSize(w - oldDim.width, h - oldDim.height);
	if(Math.abs(dDim.width) > 0 || Math.abs(dDim.height) > 0)
	    return dDim;
	else
	    return null;
    }

    public void paintComponent(Graphics g) 
    {
	//System.out.println(getSize());
	Dimension diff = diffDim(getSize().width, getSize().height);
	if(diff != null) {
	    double sx = (double) (oldDim.width + diff.width)/oldDim.width;
	    double sy = (double) (oldDim.height + diff.height)/oldDim.height;

	    setDimensions(getSize().width, getSize().height,
			  (int) Math.round(sx * maxCoord.x),
			  (int) Math.round(sy * maxCoord.y));
	    oldDim.setSize(getSize());
	}

	super.paintComponent(g);

	g.setFont(font);
	g.clearRect(0, 0, 
		    getSize().width, 
		    getSize().height);

	if(anchorPoint != null) {
	    // dragging
	    g.setColor(Color.gray);
	    Vertex start = anchorPoint.draw(g, this);
	    Vertex end  = prevMousePos.draw(g, this);

	    //labelling the distance of the two points
	    double dist = Math.sqrt(start.distSqr(end));
	    g.drawString(Util.textOf(dist, 2), 
			 Math.round((anchorPoint.x+prevMousePos.x)/2),
			 Math.round((anchorPoint.y+prevMousePos.y)/2));
	    g.drawLine(anchorPoint.x, anchorPoint.y,
		       prevMousePos.x, prevMousePos.y);
	}

	synchronized(this) {
	    Vertex.drawAll(g, this);
	    if(cricketListener != null)
		cricketListener.draw(g, this);
	}
	
	//System.out.println("wassup "+gridStep);

	drawGrid(g, 1, gridStep);
	//drawGrid(g, 1, gridStep, coordWidth, getSize().width/2);

	//System.out.println("grid returns");
    }


    //public void drawGrid(Graphics g, int size, float space, 
    //float realWidth, int screenWidth)
    public void drawGrid(Graphics g, int size, float space)
    {
	/*
	float scale = screenWidth/realWidth;
	int ulTrans = screenWidth;

	float i = 0;
	g.setColor(Color.black);


	  // assumes squared screens 
	do {
	    int x = Math.round(i*scale)+ulTrans;
	    int nx = Math.round(-i*scale)+ulTrans;
	    
	    if(gridOn) {
		g.drawLine(x, 0, x, 2*ulTrans);
		g.drawLine(0, x, 2*ulTrans, x);
		g.drawLine(nx, 0, nx, 2*ulTrans);
		g.drawLine(0, nx, 2*ulTrans, nx);
	    }
	    else {
		g.drawOval(x-size, ulTrans-size, 2*size, 2*size);
		g.drawOval(nx-size, ulTrans-size, 2*size, 2*size);
		g.drawOval(ulTrans-size, x-size, 2*size, 2*size);
		g.drawOval(ulTrans-size, nx-size, 2*size, 2*size);
	    }

	    i += space;
	} while(i <= realWidth);	
	*/

	// assumes maxCoord.x == minCoord.x, same for y
	int screenWidth  = getSize().width/2;
	int screenHeight = getSize().height/2;

	for(float i = 0; i <= maxCoord.x; i += space) {
	    int x  = (int)Math.round(i*scaleX)+screenWidth;
	    int nx = (int)Math.round(-i*scaleX)+screenWidth;

	    if(gridOn) {
		g.drawLine(x, 0, x, 2*screenHeight);
		g.drawLine(nx, 0, nx, 2*screenHeight);
	    }
	    else {
		g.drawOval(x-size, screenHeight-size, 2*size, 2*size);
		g.drawOval(nx-size, screenHeight-size, 2*size, 2*size);
	    }
	}

	for(float i = 0; i <= maxCoord.y; i += space) {
	    int x  = (int)Math.round(i*scaleY)+screenHeight;
	    int nx = (int)Math.round(-i*scaleY)+screenHeight;

	    if(gridOn) {
		g.drawLine(0, x, 2*screenWidth, x);
		g.drawLine(0, nx, 2*screenWidth, nx);
	    }
	    else {
		g.drawOval(screenWidth-size, x-size, 2*size, 2*size);
		g.drawOval(screenWidth-size, nx-size, 2*size, 2*size);
	    }
	}
    }
    

    public Vertex transform(double x, double y)
    {
	return new Vertex(x*scaleX+translate.x, y*scaleY+translate.y);
    }

    public Vertex invTransform(double x, double y)
    {
	return new Vertex((x-translate.x)/scaleX,(y-translate.y)/scaleY);
    }

    private Vertex transformStatic = new Vertex(0,0);
    public Vertex invTransformStatic(double x, double y)
    {
	 transformStatic.x = (x-translate.x)/scaleX;
	 transformStatic.y = (y-translate.y)/scaleY;
	 return transformStatic;
    }

    public Vertex findObject(double x, double y)
    {
	return Vertex.findObject(x, y);
    }

    /* sketch routines */
    Sketchable sketchObject = null;
    public static final int RECTANGLE = 0;
    public static final int PLINE = 1;
    public static final int CIRCLE = 2;

    public void newSketchObject(double x, double y, int type)
    {
	System.out.println("DrawingArea::newSketchObject "+type);

	if(sketchObject instanceof PLine && type == PLINE) {
	    //add point
	    sketchObject.addPoint(x, y);
	    return;
	}

	if(sketchObject != null) 
	    sketchObject.remove();
	
	switch(type) {
	case RECTANGLE:
	    sketchObject = new Rectangle(x, y);
	    break;
	case PLINE:
	    sketchObject = new PLine(x, y);
	    break;
	case CIRCLE:
	    sketchObject = new Circle(x, y, 0);
	    break;
	default:
	    Util.assert(false, "DrawingArea::newSketchObject invalid type: "+type);
	}
    }

    public void penUp(double x, double y)
    {
	System.out.println("DrawingArea::PenUp");
	if(sketchObject != null) {
	    sketchObject.updateCoord(x, y);
	    sketchObject = null;
	}
    }

    public void componentHidden(ComponentEvent e) { }

    public void componentMoved(ComponentEvent e) { }

    public void componentResized(ComponentEvent e) 
    { 
	repaint();
    }

    public void componentShown(ComponentEvent e) { }

    /*
    public void keyPressed(KeyEvent e) 
    {
	int k = KeyEvent.VK_UNDEFINED;
	if(e.isActionKey())
	    k = e.getKeyCode();
	if(k == 0)
	    k = (int) Character.toLowerCase(e.getKeyChar());
	Util.dbgPrintln("keypressed: "+k);
	switch(k) {
	case KeyEvent.VK_UP:
	    break;
	case KeyEvent.VK_DOWN:
	    break;
	}
    }
    
    public void keyReleased(KeyEvent e) { return; }

    public void keyTyped(KeyEvent e) { return; }
    */

}



