/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

class NetworkClient extends Thread
    implements ActionListener
{
    /**
     * Sets the period of sending registration messages to the 
     * cricketdaemon.Server
     */
    //public static final int TIMEOUT = 1000;
    public static final int BUFSIZE = 1500;

    public static final String REGISTER_TOKEN = NetworkServer.REGISTER_TOKEN;

    private Socket socket;
    private ObjectOutputStream netOut;
    private ObjectInputStream netIn;
    private InputStream auxNetIn;
    private String address;
    private int port;

    private ArrayList remoteSyncListeners;

    /**
     * Sends a registration message to the cricketdaemon.server.Server.
     */
    private boolean connect()
    {
	try {
	    System.out.println("NetworkClient::connecting...");
	    if(socket != null)
		socket.close();
	    socket = new Socket(InetAddress.getByName(address), port);
	    
	    //socket.setSoTimeout(TIMEOUT);
	    socket.setTcpNoDelay(true);

	    netOut = new ObjectOutputStream(socket.getOutputStream());
	    auxNetIn = socket.getInputStream(); //new BufferedInputStream(socket.getInputStream()));
	    netIn = new ObjectInputStream(auxNetIn);
	    
	    //netOut.writeObject(REGISTER_TOKEN);
	    //netOut.flush();
	}
	catch(IOException e) {
	    Util.dbgPrintln("NetworkClient::connect() timeout");
	    return false;
	}
	return true;
    }

    public NetworkClient() 
    {
	this("127.0.0.1");
    }

    public NetworkClient(String addr) 
    {
	address = addr;
	port    = NetworkServer.PORT;
	remoteSyncListeners = new ArrayList();
    }

    private boolean reconnect = true;

    /**
     * Reconnects whenever there is an error.
     */
    public void run()
    {
	long pollPeriod = 50;
	while(true) {
	    try {
		synchronized(this) {
		    while(true) {
			if(reconnect) 
			    reconnect = !connect();
			else {
			    Util.dbgPrintln("NetworkClient::poll "+auxNetIn.available());

			    if(auxNetIn.available() > 0) 
				dispatch(netIn.readObject());
			}

			try {
			    if(reconnect)
				wait(500);
			    else
				wait(pollPeriod);
			}
			catch (InterruptedException e) {
			}
		    }
		}
	    }
	    catch (ClassNotFoundException cnf) {
		Util.assert(false, "NetworkClient::run()");
	    }
	    catch (IOException io) {
		reconnect = true;
		Util.dbgPrintln("NetworkClient::run() timeout");
	    }
	}
    }

    private void dispatch(Object o) {
	Util.dbgPrintln("NetworkClient::dispatch\n"+o);
	
	if(o instanceof RemoteSyncEvent) {
	    RemoteSyncEvent e = (RemoteSyncEvent) o;
	    Iterator it = remoteSyncListeners.iterator();
	    while(it.hasNext()) {
		((RemoteSyncListener)it.next()).remoteSyncEvent(e);
	    }
	}
    }

    public void addRemoteSyncListener(RemoteSyncListener l)
    {
	remoteSyncListeners.add(l);
    }

    public void actionPerformed(ActionEvent e)
    {
	Util.dbgPrintln("NetworkClient::actionPerformed");
	transmitEvent(e);
    }

    private synchronized void transmitEvent(EventObject e)
    {
	try {
	    if(netOut == null)
		throw new IOException();
	    netOut.writeObject(e);
	}
	catch (IOException ie) {
	    reconnect = true;
	    notify();
	}
    }

    static public void main(String[] args) 
    {
	//Util.init("NetworkClient.log");
	NetworkClient nc = new NetworkClient();
	

    }
}
