/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class PLine extends Vertex implements Sketchable
{
    ArrayList vertices = new ArrayList();

    public PLine(double x, double y) 
    {
	super(x, y);
	super.sketchedObjects.add(this);
	DEFAULT_COLOR = Color.red.darker().darker();
	changeColor(DEFAULT_COLOR);
	init();
    }

    private void init()
    {
	vertices.clear();
	vertices.add(this);
	addPoint(x, y);
    }

    public boolean inBound(double x, double y)
    {
	return false;
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	super.draw(g, drawArea);

	Vertex vprev = null;
	for(int c = 0; c < vertices.size(); c++) {
	    Vertex v = (Vertex) vertices.get(c);
	    //v.draw(g, drawArea);

	    if(vprev != null) {
		Vertex start = drawArea.transform(vprev.x, vprev.y);
		Vertex end   = drawArea.transform(v.x, v.y);
		g.drawLine((int) start.x, (int) start.y, (int) end.x, (int) end.y);
	    }
	    vprev = v;
	}
    }

    public String toString()
    {
	String result = "[";
	result += super.toString();
	for(int c = 0; c < vertices.size(); c++) {
	    Vertex v = (Vertex) vertices.get(c);
	    result += " "+v;
	}
	result += "]";
	return result;
    }

    public Vertex addPoint(double x, double y) 
    {
	System.out.println("PLine:: addPoint");
	Vertex v = new Vertex(x, y);
	v.DEFAULT_COLOR = Color.red.darker().darker();
	v.changeColor(v.DEFAULT_COLOR);
	vertices.add(v);
	super.sketchedObjects.add(v);
	return v;
    }

    public void remove()
    {
	Iterator it = vertices.iterator();
	while(it.hasNext()) {
	    Object o = it.next();
	    super.sketchedObjects.remove(o);
	}
	vertices.clear();
    }
    
    public Object clone()
    {
	Util.assert(false, "Shape::clone not implemented");
	return null;
    }

    public Vertex updateCoord(double x, double y)
    {
	Util.assert(vertices.size() > 1, "PLine::updateCoord");
	Vertex v = (Vertex) vertices.get(vertices.size() - 1);
	v.mouseDragged(x, y, null, 0);
	return v;
    }
}
