/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class Rectangle extends PLine
{
    /* first and third corners are opposite */
    Vertex firstCorner;
    Vertex secondCorner;
    Vertex thirdCorner;
    Vertex fourthCorner;

    public Rectangle(double x, double y) 
    {
	super(x, y);
	firstCorner = this;
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	if(fourthCorner != null) {
	    if(firstCorner.modified) {
		secondCorner.x = firstCorner.x;
		fourthCorner.y = firstCorner.y;
	    }
	    else if(secondCorner.modified) {
		firstCorner.x = secondCorner.x;
		thirdCorner.y = secondCorner.y;
	    }
	    else if(thirdCorner.modified) {
		secondCorner.y = thirdCorner.y;
		fourthCorner.x = thirdCorner.x;
	    }
	    else if(fourthCorner.modified) {
		thirdCorner.x  = fourthCorner.x;
		firstCorner.y  = fourthCorner.y;
	    }

	
	    super.draw(g, drawArea);
	    Vertex start = drawArea.transform(fourthCorner.x, fourthCorner.y);
	    Vertex end   = drawArea.transform(firstCorner.x, firstCorner.y);
	    g.drawLine((int) start.x, (int) start.y, (int) end.x, (int) end.y);
	}
    }

    public Vertex addPoint(double x, double y)
    {
	Util.dbgPrintln("Rectangle addPoint");
	if(thirdCorner == null) {
	    // this.x is firstCorner.x, etc.
	    // x is thirdCorner.x, etc.
	    secondCorner = super.addPoint(this.x, y);
	    thirdCorner  = super.addPoint(x, y);
	    fourthCorner = super.addPoint(x, this.y);
	}
	return firstCorner;
    }

    public Vertex updateCoord(double x, double y)
    {
	thirdCorner.mouseDragged(x, y, null, 0);
	return thirdCorner;
    }
}
