/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.event.*;
import java.util.*;

class RemoteSketchPanel 
    extends SketchPanel implements RemoteSyncListener
{
    /* states of the root BeaconConfigApp */
    private RemoteSyncEvent currentStates;
    private EventListener eventHandler;
    private boolean override;

    public RemoteSketchPanel(SolverPanel sp, EventListener e, boolean override)
    {
	super(sp);
	currentStates = new RemoteSyncEvent();
	currentStates.sketchPanelStates = super.getStates();
	eventHandler = e;
	this.override = override;
    }

    private void updateRemote()
    {
	if(!override) {
	    currentStates = new RemoteSyncEvent();
	    currentStates.sketchPanelStates = super.getStates();
	}
	if(eventHandler instanceof RemoteSyncListener) {
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
	}
    }

    public void enableButtons(boolean v)
    {
	super.enableButtons(v);
	updateRemote();
    }

    public void actionPerformed(ActionEvent e) 
    {
	Util.dbgPrintln("RemoteSketchPanel::actionPerformed "+
			e.getActionCommand());
	if(!override) { // root app
	    super.actionPerformed(e);
	    currentStates = new RemoteSyncEvent();
	    currentStates.sketchPanelStates = super.getStates();
	}
	else {
	    // remote app: restore old states, then wait for ack from
	    // root application
	    setCurrentStates(currentStates);
	}

	if(eventHandler instanceof ActionListener)
	    ((ActionListener) eventHandler).actionPerformed(e);
	else if(eventHandler instanceof RemoteSyncListener)
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
    }

    private void setCurrentStates(RemoteSyncEvent cs)
    {
	Util.dbgPrintln("RemoteSketchPanel::setCurrentStates");
	currentStates = cs;
	if(cs.sketchPanelStates != null) {
	    Boolean penUpSelected = (Boolean) cs.sketchPanelStates.get(0);
	    Boolean penUpEnabled  = (Boolean) cs.sketchPanelStates.get(1);
	    Boolean rectSelected  = (Boolean) cs.sketchPanelStates.get(2);
	    Boolean rectEnabled   = (Boolean) cs.sketchPanelStates.get(3);
	    Boolean lineSelected  = (Boolean) cs.sketchPanelStates.get(4);
	    Boolean lineEnabled   = (Boolean) cs.sketchPanelStates.get(5);
	    Boolean circSelected  = (Boolean) cs.sketchPanelStates.get(6);
	    Boolean circEnabled   = (Boolean) cs.sketchPanelStates.get(7);
	    penUpControl.setSelected(penUpSelected.booleanValue());
	    rectControl.setSelected(rectSelected.booleanValue());
	    lineControl.setSelected(lineSelected.booleanValue());
	    circControl.setSelected(circSelected.booleanValue());

	    penUpControl.setEnabled(penUpEnabled.booleanValue());
	    rectControl.setEnabled(rectEnabled.booleanValue());
	    lineControl.setEnabled(lineEnabled.booleanValue());
	    circControl.setEnabled(circEnabled.booleanValue());
	}
	else {
	    penUpControl.setSelected(true);
	    rectControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(false);

	    enableButtons(false);
	}
	updateHighlighting();
    }

    // called by the Network Client in remote app
    public void remoteSyncEvent(RemoteSyncEvent e) {
	Util.dbgPrintln("RemoteSketchPanel::remoteSyncEvent");
	setCurrentStates(e);
    }

}
