/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.event.*;
import java.util.*;

class RemoteSolverPanel 
    extends SolverPanel implements RemoteSyncListener
{
    /* states of the root BeaconConfigApp */
    private RemoteSyncEvent currentStates;
    private EventListener eventHandler;

    public RemoteSolverPanel(BeaconConfig bc, DrawingArea da, int closeness, 
			     int stddev, 
			     int enoughsamples,
			     EventListener e)
    {
	super(bc, da, closeness, stddev, enoughsamples);
	currentStates = new RemoteSyncEvent();
	currentStates.status = super.status.getText();
	currentStates.errmsg = super.errmsg.getText();
	eventHandler = e;
    }

    protected void changeStatus(String s)
    {
	super.changeStatus(s);
	//System.out.println("changeStatus: "+s);
	currentStates = new RemoteSyncEvent();
	currentStates.status = super.status.getText();

	if(eventHandler != null) // could be null when super is called within constructor
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
    }

    protected boolean errMsg(boolean beep, String s) {
	if(!super.errMsg(beep, s))
	    return false;

	//System.out.println("errMsg "+s);

	currentStates = new RemoteSyncEvent();
	currentStates.errmsg = super.errmsg.getText();

	if(eventHandler != null) // could be null when super is called within constructor
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);

	return true;
    }
    

    private void setCurrentStates(RemoteSyncEvent cs)
    {
	Util.dbgPrintln("RemoteSolverPanel::setCurrentStates");
	currentStates = cs;

	//System.out.println("Client: s="+cs.status+" e="+cs.errmsg);
	
	if(cs.status != null)
	    super.status.setText(cs.status);
	if(cs.errmsg != null)
	    super.errmsg.setText(cs.errmsg);
    }

    // called by the Network Client in remote app
    public void remoteSyncEvent(RemoteSyncEvent e) {
	Util.dbgPrintln("RemoteControls::remoteSyncEvent");
	setCurrentStates(e);
    }

}
