/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.util.*;
import java.io.*;

class RemoteSyncEvent 
    extends EventObject implements Serializable
{
    ArrayList sketchPanelStates;
    ArrayList controlPanelStates;

    String status;
    String errmsg;

    public RemoteSyncEvent()
    {
	super(new Object()); //hack
	sketchPanelStates = null;
	controlPanelStates = null;
	status = null;
	errmsg = null;
    }

    public String toString()
    {
	String result = "";
	Iterator it;
	if(sketchPanelStates != null) {
	    result += "SketchPanelStates: ";
	    it = sketchPanelStates.iterator();
	    while(it.hasNext()) 
		result += it.next()+" ";
	    result += "\n";
	}
	if(controlPanelStates != null) {
	    result += "ControlPanelStates: ";
	    it = controlPanelStates.iterator();
	    while(it.hasNext()) 
		result += it.next()+" ";
	    result += "\n";
	}
	if(status != null)
	    result += "Status: "+status + "\n";

	if(errmsg != null)
	    result += "ErrMsg: "+errmsg + "\n";

	return result;
    }

    public static void main(String[] args) 
	throws IOException, ClassNotFoundException
    {
	
	RemoteSyncEvent foo = new RemoteSyncEvent();
	foo.sketchPanelStates.add(new String("foobar"));
	foo.sketchPanelStates.add(new String("foobar 2"));
	foo.controlPanelStates.add(new String("bar"));
	
	FileOutputStream ostream = new FileOutputStream("t.tmp");
	ObjectOutputStream p = new ObjectOutputStream(ostream);

	p.writeObject(foo);
	p.flush();
	ostream.close();

	System.out.println("deserialization");
	
	FileInputStream istream = new FileInputStream("t.tmp");
	ObjectInputStream ip = new ObjectInputStream(istream);

	RemoteSyncEvent ifoo = (RemoteSyncEvent) ip.readObject();

	System.out.println(ifoo);
	istream.close();
 
    }

    public void update(RemoteSyncEvent e)
    {
	if (e.sketchPanelStates != null)
	    sketchPanelStates = e.sketchPanelStates;
	if (e.controlPanelStates != null)
	    controlPanelStates = e.controlPanelStates;
	if (e.status != null)
	    status = e.status;
	if (e.errmsg != null)
	    errmsg = e.errmsg;
	//System.out.println("Update: "+this);
    }

    public Object clone()
    {
	RemoteSyncEvent result = new RemoteSyncEvent();
	result.sketchPanelStates = sketchPanelStates;
	result.controlPanelStates = controlPanelStates;
	result.status = status;
	result.errmsg = errmsg;
	return result;
    }
}
