/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class SketchPanel extends JPanel 
    implements ActionListener
{
    private SketchControl sketcher;

    protected JRadioButton lineControl;
    protected JRadioButton rectControl;
    protected JRadioButton circControl;
    protected JRadioButton penUpControl;
    private JLabel penDn;

    private static String lineControlLabel = "Line";
    private static String rectControlLabel = "Rect";
    private static String circControlLabel = "Circ";
    private static String penUpControlLabel = "PenUp";

    public SketchPanel(SketchControl s) 
    {
	//((FlowLayout)getLayout()).setHgap(0);
	setLayout(new GridLayout(1, 5));
	sketcher = s;
	penUpControl = addRadioButton(penUpControlLabel);
	penDn = new JLabel("PenDn:", JLabel.CENTER);
	//penDn.setMaximumSize(new Dimension(20,17));
	//System.out.println(penDn.getMaximumSize());
	add(penDn);
	lineControl = addRadioButton(lineControlLabel);
	rectControl = addRadioButton(rectControlLabel);
	circControl = addRadioButton(circControlLabel);
	reset();
    }

    private boolean matchRadioAction(ActionEvent e, JRadioButton t) 
    {
	return e.getActionCommand().equals(t.getText());
    }

    private JRadioButton addRadioButton(String name)
    {
	JRadioButton result = new JRadioButton(name);
	result.setHorizontalAlignment(JRadioButton.CENTER);
	result.setSelected(false);
	result.setEnabled(false);
	result.setActionCommand(result.getText());
	result.addActionListener(this);
	add(result);	
	return result;
    }

    public void actionPerformed(ActionEvent e)
    {
	if(e.getActionCommand().equals(Controls.getResetButton().getText())) {
	    reset();
	}
	else if(matchRadioAction(e, lineControl)) {
	    if(sketcher != null)
		sketcher.setLinePenDn(true);
	    penUpControl.setSelected(false);
	    lineControl.setSelected(true);
	    circControl.setSelected(false);
	    rectControl.setSelected(false);
	}
	else if(matchRadioAction(e, circControl)) {
	    sketcher.setCircPenDn(true);
	    penUpControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(true);
	    rectControl.setSelected(false);
	}
	else if(matchRadioAction(e, rectControl)) {
	    sketcher.setRectPenDn(true);
	    penUpControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(false);
	    rectControl.setSelected(true);
	}
	else if(matchRadioAction(e, penUpControl)) {
	    reset();
	}
	updateHighlighting();
    }

    protected void updateHighlighting()
    {
	penUpControl.setForeground(penUpControl.isSelected() ?
				   Color.black : Color.gray);
	penDn.setForeground(penUpControl.isSelected() ?
			    Color.gray : Color.black);
    }

    public void reset()
    {
	Util.dbgPrintln("SketchPanel::reset()");
	if(sketcher != null) {
	    sketcher.setCircPenDn(false);
	    sketcher.setRectPenDn(false);
	    sketcher.setLinePenDn(false);
	}
	penUpControl.setSelected(true);
	rectControl.setSelected(false);
	lineControl.setSelected(false);
	circControl.setSelected(false);	

	updateHighlighting();
    }

    public void enableButtons(boolean v)
    {
	if(v != penUpControl.isEnabled()) {
	    penUpControl.setEnabled(v);
	    rectControl.setEnabled(v);
	    lineControl.setEnabled(v);
	    circControl.setEnabled(v);	
	}
    }

    public ArrayList getStates()
    {
	ArrayList result = new ArrayList();
	result.add(new Boolean(penUpControl.isSelected()));
	result.add(new Boolean(penUpControl.isEnabled()));
	result.add(new Boolean(rectControl.isSelected()));
	result.add(new Boolean(rectControl.isEnabled()));
	result.add(new Boolean(lineControl.isSelected()));
	result.add(new Boolean(lineControl.isEnabled()));
	result.add(new Boolean(circControl.isSelected()));
	result.add(new Boolean(circControl.isEnabled()));
	return result;
    }
}




