/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;
import java.awt.*;
import java.util.*;

class StreamerVertex extends SquareVertex
{
    private long expiryTime;

    public StreamerVertex(double x, double y, double z, long timeout)
    {
	super(x, y, z);
	super.streamer.add(this);
	super.drawCoord = false;
	expiryTime = System.currentTimeMillis() + timeout;
    }

    public long getExpiryTime() { return expiryTime; }

    // not thread safe!
    public static void refresh()
    {
	Iterator it  = streamer.iterator();
	long curTime = System.currentTimeMillis();
	while(it.hasNext()) {
	    StreamerVertex v = (StreamerVertex) it.next();
	    if(v.getExpiryTime() < curTime)
		it.remove();
	    else
		break; // optimization, assumes streamer has increasing time order
	}
    }
}
