/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconconfig;
import java.awt.*;
import java.util.*;
import javax.swing.*;

public class Vertex
{
    public static ArrayList beacons          = new ArrayList();
    public static ArrayList sketchedObjects  = new ArrayList();
    public static ArrayList streamer         = new ArrayList();

    public static final double TOLERANCE = 5;

    public double x;
    public double y;
    public double z;

    protected Vertex transformed_xyz;
    protected boolean modified = false;
    
    public Vertex(double x, double y) 
    {
	this.x = x; this.y = y;
	DEFAULT_COLOR = Color.blue;
	changeColor(DEFAULT_COLOR);
    }

    public static void clearList()
    {
	beacons.clear();
	sketchedObjects.clear();
	streamer.clear();
    }

    public boolean inBound(double x, double y)
    {
	//System.out.println(this.x+" "+this.y);
	return
	    Math.abs(this.x - x) < TOLERANCE &&
	    Math.abs(this.y - y) < TOLERANCE;
    }

    public static void drawAll(Graphics g, DrawingArea drawArea)
    {
	for(int i = 0; i < sketchedObjects.size(); i++) {
	    ((Vertex)sketchedObjects.get(i)).draw(g, drawArea);
	}	
	for(int i = 0; i < beacons.size(); i++) {
	    ((Vertex)beacons.get(i)).draw(g, drawArea);
	}
	for(int i = 0; i < streamer.size(); i++) {
	    ((Vertex)streamer.get(i)).draw(g, drawArea);
	}
    }

    public  Color DEFAULT_COLOR;
    private Color color;

    public void changeColor(Color c) {
	color = c;
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	modified = false;
	g.setColor(color);

	Vertex v = drawArea.transform(x, y);

	//System.out.println(v.x+" : "+v.y);

	g.drawString("("+Util.textOf(x, 2)+", "+Util.textOf(y, 2)+")",
		     Math.round((float)(v.x))+2,
		     Math.round((float)(v.y))+ 12);


	g.fillOval(Math.round((float)(v.x - DrawingArea.nodeSizeDiv2)), 
		   Math.round((float)(v.y - DrawingArea.nodeSizeDiv2)), 
		   DrawingArea.nodeSize, 
		   DrawingArea.nodeSize);

	transformed_xyz = v;
    }

    public String toString()
    {
	return "("+x+","+y+")";
    }

    public double distSqr(Vertex v)
    {
	return distSqr(x, y, z, v.x, v.y, v.z);
    }

    public double distSqr(double xx, double yy, double zz)
    {
	return distSqr(x, y, z, xx, yy, zz);
    }

    public double distSqr(double X, double Y, double Z, double XX, double YY, double ZZ)
    {
	//System.out.println(X+" "+Y+" "+Z+" "+XX+" "+YY+" "+ZZ);
	return (X - XX)*(X - XX) + (Y - YY)*(Y - YY) + (Z-ZZ)*(Z-ZZ);
    }

    public boolean equals(Object o)
    {
	return this == o;
    }

    public static Vertex findObject(double x, double y)
    {
	for(int i = 0; i < sketchedObjects.size(); i++) {
	    Vertex obj = (Vertex) sketchedObjects.get(i);
	    if(obj.inBound(x, y))
		return obj;
	}

	for(int i = 0; i < beacons.size(); i++) {
	    Vertex obj = (Vertex) beacons.get(i);
	    if(obj.inBound(x, y))
		return obj;
	}
	return null;
    }

    public void mousePressed(double x, double y, int modifier)
    {
	return;
    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier)
    {
	this.x = x; this.y = y;
	modified = true;
    }

    public void mouseReleased(double x, double y)
    {
	return;
    }

    public void remove()
    {
	return;
    }

    public Object clone()
    {
	return new Vertex(x, y);
    }
}
