/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconfinder;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import samplemon.Mapping;

class BarGraph extends JPanel
{
    private int col;     // TableModel column selected for graph
    private double xlim; // x range value
    private int width;   // gui window width

    private static final int leftMargin = 20;
    private static final int barMargin = 150;
    private static final int barShim  = 10;
    private static final int rightMargin = 50;
    private static final int yMargin = 20;

    private Image img;

    private static final int barThickness = 20;
    private static final int ySpace = 10;

    private double stddevThresh;

    // Color scheme from ptolemy plot.
    // Default _colors, by data set.
    // There are 11 colors so that combined with the
    // 10 marks of the Plot class, we can distinguish 110
    // distinct data sets.
    public static Color[] colors = {
        new Color(0xff0000),   // red
        new Color(0x0000ff),   // blue
        new Color(0x00aaaa),   // cyan-ish
        new Color(0x000000),   // black
        new Color(0xffa500),   // orange
        new Color(0x53868b),   // cadetblue4
        new Color(0xff7f50),   // coral
        new Color(0x45ab1f),   // dark green-ish
        new Color(0x90422d),   // sienna-ish
        new Color(0xa0a0a0),   // grey-ish
        new Color(0x14ff14),   // green-ish
    };

    private Dimension preferredSize;
    public Dimension getPreferredSize() 
    {
	return preferredSize;
    }


    public BarGraph(int c, double x, int w, double stddevThresh) 
    {
	col = c; xlim = x; width = w;
	this.stddevThresh = stddevThresh;
	preferredSize = new Dimension(w, 220);
	img = createImage(getPreferredSize().width, 
			  getPreferredSize().height);
    }

    ArrayList names = new ArrayList();
    ArrayList values = new ArrayList();

    private String graphName;

    public synchronized void updateGraph(TableModel t)
    {
	names.clear();
	values.clear();
	graphName = t.getColumnName(col);
	for(int i = 0; i < t.getRowCount(); i++) {
	    String n = (String) t.getValueAt(i, 0);
	    String v = (String) t.getValueAt(i, col);
	    names.add(n);
	    values.add(Double.valueOf(v));
	}
	repaint();
    }

    public void update(Graphics g)
    {
	//double buffering to avoid flickering
	if(img != null) {
	    Graphics gc = img.getGraphics();
	    paintComponent(gc);
	    g.drawImage(img, 0, 0, this);
	}
    }

    public synchronized void paintComponent(Graphics g) 
    {
	super.paintComponent(g);

	g.setFont(GUI.font);
	g.clearRect(0, 0, 
		    getPreferredSize().width, 
		    getPreferredSize().height);

	double minWidth = 0.1;

	double xscale = (double) (width-(barMargin+rightMargin))/xlim;
	int y = yMargin;
	
	// draw Title
	if(graphName != null) {
	    g.setColor(Color.black);
	    g.drawString(graphName, 
			 (getPreferredSize().width -
			 g.getFontMetrics().stringWidth(graphName))/2,
			 y+g.getFontMetrics().getHeight()/2);
	}

	y = yMargin + g.getFontMetrics().getHeight() + ySpace;

	int textOffset = barThickness - Math.abs(barThickness - g.getFontMetrics().getHeight())/2 - 1;

	for(int i = 0; i < names.size(); i++) {
	    String s = (String) names.get(i);
	    double v = ((Double) values.get(i)).doubleValue();
	    int length = (int) Math.round((v + minWidth) * xscale);
	    
	    g.setColor(Color.black);
	    g.drawString(s, leftMargin, y+textOffset);
	    
	    //g.setColor(Color.blue);
	    g.setColor(colors[Mapping.setMap(names, s) % colors.length]);

	    g.fillRect(barMargin-barShim, y, length+barShim, barThickness);

	    y += ySpace+barThickness;
	}

	drawVertLine(g, "0", (int) Math.round(minWidth*xscale), Color.black);

	drawVertLine(g, Double.toString(xlim), 
		     (int) Math.round((xlim+minWidth)*xscale), Color.black);

	/* The static value gets loaded at loadtime and is never updated by
	   the runtime of SolverPanel.  Another words, this doesn't work */
	/*
	drawVertLine(g, Double.toString(SolverPanel.STDDEV_THRESH), 
		     (int) Math.round((SolverPanel.STDDEV_THRESH+minWidth)*xscale), Color.red);
	*/
	drawVertLine(g, Double.toString(stddevThresh), 
		     (int) Math.round((stddevThresh+minWidth)*xscale), Color.red);

    }

    public void drawVertLine(Graphics g, String s, int x, Color c)
    {
	g.setColor(c);
	g.fillRect(x+barMargin, yMargin+g.getFontMetrics().getHeight(), 2,
		   getPreferredSize().height - 3*yMargin - g.getFontMetrics().getHeight());
	g.drawString(s, x+barMargin, 
		     getPreferredSize().height - yMargin);// - g.getFontMetrics.getHeight());
    }

}
