/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package beaconfinder;

import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;
import cricketdaemon.UnitConv;

import javax.swing.table.*;
import java.util.*;

class TableModel extends AbstractTableModel 
{
    private Object[] beaconData, calcBeaconData, orderedCalcBeaconData;
    public int curBeaconInd;
    private static final long TIMEOUT = 20000;

    private long clockOffset;

    public String[] columnNames;

    private String[]
	columnNamesCentimeters = 
    { 
	"name", "age (ms)", "mode (cm)", "mean (cm)", 
	"med (cm)", "max (cm)", "min (cm)", "stdv (cm)"//, "coordinates"
    };

    private String[]
	columnNamesInches = 
    { 
	"name", "age (ms)", "mode (in)", "mean (in)", 
	"med (in)", "max (in)", "min (in)", "stdv (in)"//, "coordinates"
    };
    
    public TableModel()
    {
	super();

	if(BeaconFinderApp.useMetric)
	    columnNames = columnNamesCentimeters;
	else 
	    columnNames = columnNamesInches;
    }

    public void setClockOffset(long off)
    {
	clockOffset = off;
    }

    private long currentTime()
    {
	return System.currentTimeMillis() + clockOffset;
    }

    public void setCalcBeaconData(Object[] a)
    {
	calcBeaconData = a;
    }

    public void setBeaconsHeard(ArrayList beaconsHeard)
    {
	beaconData = makeSet(beaconData, beaconsHeard);
	/*
	  for(int i = 0; i < beaconData.length; i++)
	  System.out.println(beaconData[i]);
	*/
    }

    public String getColumnName(int col) 
    {
	return columnNames[col];
    }

    public int getRowCount() { 
	if(beaconData == null)
	    return 0;
	else {
	    return beaconData.length; 
	}
    }
    public int getColumnCount() { return columnNames.length; }

    public Object getValueAt(int row, int col) {
	if(beaconData == null)
	    return null;

	if(row >= beaconData.length)
	    return null;

	BeaconRecord c = (BeaconRecord) beaconData[row];

	switch(col) {
	case 8:
	    if(c.pos == null)
		return null;
	    return 
		Util.valueToString(c.pos.x, 0.1)+" "+
		Util.valueToString(c.pos.y, 0.1)+" "+
		Util.valueToString(c.pos.z, 0.1);
	case 0:
	    return c.uniqueName;
	case 1:
	    if(c.lastUpdate == null)
		return null;
	    long diff = currentTime() - c.lastUpdate.longValue();
	    return String.valueOf(diff);
	}

	if(c.distStat == null)
	    return null;

	// fixme: very ugly code
	if(BeaconFinderApp.useMetric)
	    return colConversionCentimeters(col, row, c);
	else 
	    return colConversionInches(col, row, c);
    }

    private Object colConversionInches(int col, int row, BeaconRecord c) {
	switch(col) {
	case 2:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.mode), 0.001);
	case 3:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.mean), 0.001);
	case 4:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.median), 0.001);
	case 5:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.max), 0.001);
	case 6:
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.min), 0.001);
	case 7:
	    // need to re-center offset
	    return Util.valueToString(UnitConv.cricketDistToIN(c.distStat.stddev), 0.001);
	case 9:	    
	    BeaconRecord calcDist = null;
	    if(orderedCalcBeaconData != null)
		calcDist = (BeaconRecord) orderedCalcBeaconData[row];
	    
	    if(calcDist == null)
		return null;
	    else
		return Util.valueToString(UnitConv.cricketDistToIN(calcDist.distStat.dist), 0.001);
	}
	return null;
    }

    private Object colConversionCentimeters(int col, int row, BeaconRecord c) {
	switch(col) {
	case 2:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.mode), 0.001);
	case 3:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.mean), 0.001);
	case 4:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.median), 0.001);
	case 5:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.max), 0.001);
	case 6:
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.min), 0.001);
	case 7:
	    // need to re-center offset
	    return Util.valueToString(UnitConv.cricketDistToCM(c.distStat.stddev), 0.001);
	case 9:	    
	    BeaconRecord calcDist = null;
	    if(orderedCalcBeaconData != null)
		calcDist = (BeaconRecord) orderedCalcBeaconData[row];
	    
	    if(calcDist == null)
		return null;
	    else
		return Util.valueToString(UnitConv.cricketDistToCM(calcDist.distStat.dist), 0.001);
	}
	return null;
    }

    public Class getColumnClass(int col) { return String.class; }

    public void updateBeaconsHeard() 
    {
	if(beaconData == null) {
	    curBeaconInd = 0;
	    return;
	}

	ArrayList result = new ArrayList();
	long curTime = currentTime();
	for(int i = 0; i < beaconData.length; i++) {
	    BeaconRecord br = (BeaconRecord) beaconData[i];
	    if(!timeOut(br.lastUpdate, curTime))
		result.add(beaconData[i]);
	}
	//	System.out.println("beaconData foo "+beaconData.length);
	beaconData = result.toArray();

	//	System.out.println("beaconData bar "+beaconData.length);

	Arrays.sort(beaconData);
	curBeaconInd = findClosest(beaconData);
	if(calcBeaconData != null)
	    orderedCalcBeaconData = orderData(beaconData, calcBeaconData);
	else
	    orderedCalcBeaconData = null;

	fireTableDataChanged();
    }

    private boolean timeOut(Long lastUpdate, long curTime)
    {
	if(lastUpdate == null)
	    return false;
	return (lastUpdate.longValue() + TIMEOUT) < curTime;
    }

    private Object[] orderData(Object[] target, Object[] source)
    {
	Object[] result = new Object[target.length];

	for(int i = 0; i < result.length; i++) {
	    Associable t = (Associable) target[i];
	    for(int j = 0; j < source.length; j++) {
		Associable s = (Associable) source[j];
		if(s.isAssociated(t)) {
		    result[i] = source[j];
		    break;
		}
	    }
	}

	return result;
    }


    private Object[] makeSet(Object[] orig, ArrayList more) 
    {
	if(orig != null) {
	    for(int i = 0; i < orig.length; i++) {
		if(!more.contains((BeaconRecord)orig[i]))
		    more.add(orig[i]);
	    }
	}
	return more.toArray();
    }

    private int findClosest(Object[] o)
    {
	double curMin = Double.MAX_VALUE;
	int curMinIndex = -1;
	
	for(int i = 0; i < o.length; i++) {
	    BeaconRecord r = (BeaconRecord) o[i];
	    double comp = r.distStat.mode;
	    if(comp < curMin) {
		curMin = comp;
		curMinIndex = i;
	    }
	}
	
	return curMinIndex;
    }
    
}
