/* acad2ug.c : routines to convert DXF entities */
#include "acad2ug.h"

#define TRUE 1
#define FALSE 0


/* Global variables */

/* DXF groups consist of a code and a value */
int DXFgroupcode ;
char DXFgroupvalue[256];
/* id of last vertex and face in current block */
int Lastv =0;
int Lastw =0;
int Lastf =0;

int Lineno=0, /* line number of current Entity in dxf file  */
CurLineno=0;  /* Current line number in dxf file  */

/*Table of layers containing only ones that are turned on */
LAYER  *Layertable = NULL;
/*Total number of layers */
int  Numlayers =1;
short Nolayerfile = TRUE; /* there is an user wants-specified layer file ? */

short Nocolorfile = TRUE; /* there is an user-specified color file ? */
float Colortable[256][4];
char Prefix[50];
short Inablock = FALSE;   /* Processing BLOCKS section */
short InEntities = FALSE; /* Processing ENTITIES section */
short WantFix1 = FALSE;   
short WantFix2 = FALSE;   
short WantFix3 = FALSE;   
short Want2D = FALSE;   
short WantPlanv = FALSE;   
short WantPrefix = FALSE;   
short WantNoDefs = FALSE;   
short Debugging =0; 
short WantArcs = TRUE;
short WantMIT = FALSE;

/* direction of z axis for Entity Coordinate system (ECS)*/
VERTEX Ecs;
/* World Coordinate system (WCS) axes */   
static VERTEX Wcs_x = {1.0,0.0,0.0};
static VERTEX Wcs_y = {0.0,1.0,0.0};
static VERTEX Wcs_z = {0.0,0.0,1.0};

/* Matrix to transform from ECS to WCS */
double Ecs_matrix[4][4];

/* Each DXF BLOCK will become a BLK structure */
BLK *First_block = NULL;
BLK *Last_block = NULL;
BLK *Cur_block = NULL;
INSTANCE *Instance_list= NULL;


/* MakeECSmatrix():
 * Makes a  matrix 'm' to convert ECS to WCS, using the algorithm 
 * in AUTOCAD manual (release 10) page 410 . 
 */
MakeECSmatrix()
{
   VERTEX ax, ay;
   double length;

   if (fabs(Ecs.x) < 1/64. && fabs(Ecs.y) < 1/64. )
      Cross(&ax,Wcs_y,Ecs);
   else
      Cross(&ax,Wcs_z,Ecs);
   
   /* normalize ax */
   length = sqrt(ax.x * ax.x + ax.y * ax.y + ax.z * ax.z );
   ax.x /= length; ax.y /= length; ax.z /= length;
   
   Cross(&ay, Ecs, ax);
   /* normalize ay */
   length = sqrt(ay.x * ay.x + ay.y * ay.y + ay.z * ay.z );
   if (length != 0.){
      ay.x /= length; ay.y /= length; ay.z /= length;
   }
   
   Ecs_matrix[0][0]  =  ax.x;
   Ecs_matrix[0][1]  =  ax.y;
   Ecs_matrix[0][2]  =  ax.z;
   Ecs_matrix[0][3]  =  0.;
   
   Ecs_matrix[1][0]  =  ay.x;
   Ecs_matrix[1][1]  =  ay.y;
   Ecs_matrix[1][2]  =  ay.z;
   Ecs_matrix[1][3]  =  0.;
   
   Ecs_matrix[2][0]  =  Ecs.x;
   Ecs_matrix[2][1]  =  Ecs.y;
   Ecs_matrix[2][2]  =  Ecs.z;
   Ecs_matrix[2][3]  =  0.;
   
   Ecs_matrix[3][0]  =  0.;
   Ecs_matrix[3][1]  =  0.;
   Ecs_matrix[3][2]  =  0.;
   Ecs_matrix[3][3]  =  1.;
}

/* EcsToWcs(vrtx):
 * Converts a vertex in Entity Coordinate System to World Coordinate
 * system by multiplying the vertex with the Ecs Matrix
 */
EcsToWcs(vrtx)
VERTEX *vrtx;
{
   VERTEX tempvrtx;
   tempvrtx.x = Ecs_matrix[0][0] * vrtx->x +
            Ecs_matrix[1][0] * vrtx->y +
            Ecs_matrix[2][0] * vrtx->z ;
   tempvrtx.y = Ecs_matrix[0][1] * vrtx->x +
            Ecs_matrix[1][1] * vrtx->y +
            Ecs_matrix[2][1] * vrtx->z ;
   tempvrtx.z = Ecs_matrix[0][2] * vrtx->x +
            Ecs_matrix[1][2] * vrtx->y +
            Ecs_matrix[2][2] * vrtx->z ;
   vrtx->x = tempvrtx.x;
   vrtx->y = tempvrtx.y;
   vrtx->z = tempvrtx.z;
}


/* dispatch_on_group(code,type):
 * Looks at the type of an entity and calls functions to 
 * process its groups
 */
dispatch_on_group(code,type)
int code;
char type[];
{
   char layer[50];
   int layerindex;

   if (code != 0) { 
   /* skip the junk groups that appear before a new entity */
     Skip("");
     return;
   }
   Lineno = CurLineno;   /* begining of a new entity */
   if(!strcmp(type,"SECTION")){
       ProcessSection();
   }else if(!strcmp(type,"ENDBLK")){
       Inablock = FALSE;
       Skip(type);
   }else if(!strcmp(type,"SEQEND")){
      Skip(type);
   }else if(!strcmp(type,"ENDSEC")) {
      Skip(type);
   }else if (!strcmp(type,"POINT")){
      Skip(type);
   }else if (!strcmp(type,"TEXT")){
      Skip(type); 
   }else if (!strcmp(type,"SHAPE")){
      Skip(type);
   }else if (!strcmp(type,"ATTDEF")){
      Skip(type);
   }else if (!strcmp(type,"ATTRIB")){
      Skip(type);
   }else if (!strcmp(type,"VERTEX")){
      Skip(type);
   }else if (!strcmp(type,"DIMENSION")){
      Skip(type);
   }else if (!strcmp(type,"BLOCK")){
      Inablock = TRUE;
      ProcessBlock();
   }else if(!strcmp(type,"LINE")) {
     ProcessLine();  
   }
   else if (!strcmp(type,"INSERT"))
      ProcessInsert();
   else if (!strcmp(type,"CIRCLE")) {
     ProcessCircle();
     /*Skip(type);*/
   }
   else if (!strcmp(type,"ARC")) {
     if (WantArcs)
       ProcessArc();
     else Skip(type);
   }
   else if (!strcmp(type,"SOLID"))
      ProcessSolid();
   else if (!strcmp(type,"POLYLINE")) {
     ProcessPolyline(); 
   }
   else if (!strcmp(type,"3DLINE"))
      ProcessLine();
   else if (!strcmp(type,"3DFACE"))
      Process3dface();
   else{
      fprintf(Errorfile,"Unkown entity : %s in line %d\n",type,Lineno);
      Skip ("");
   }
}  


ProcessSection()
{
     GetTwoLines();
     if(!strcmp(DXFgroupvalue,"ENTITIES")){
        InEntities  = TRUE;
        MakeBlock("Entity");
        Skip("");
     } else if(!strcmp(DXFgroupvalue,"BLOCKS")){
        Inablock = TRUE;
        Skip("");
     }else
        Skip("SECTION");
}

ProcessBlock()
{
   BLK *newblk; 
   char layer[100];
   
   while(1){ 
      GetTwoLines();
      switch (DXFgroupcode){
         case 2:		/*name of Block */
              newblk = MakeBlock(DXFgroupvalue); 
              break;
         case 8:                 /*layer of Block */
            strcpy(layer,DXFgroupvalue);break;
         case 10 :            
            newblk->base.x = atof(DXFgroupvalue);break;     
         case 20 :            
            newblk->base.y = atof(DXFgroupvalue);break;     
         case 30 :            
            newblk->base.z = atof(DXFgroupvalue);break;     
	case 62: /*color */
          break;
         case 70 : 
            /* flagbits  */
            break;        
         case  0 :	    /*an entity in Block reached */
              return;  
     }
   }
}
  
error()
{
   fprintf(Errorfile,"error on  line %d of %s.dxf\n",Lineno,Mainfilename);
   exit(2);
}


ProcessLine()
{
   VERTEX v[4], start, finish;
   double thickness=0.0;
   short useEcs = FALSE;
   int i;
   int layerindex;

   start.z = finish.z = 0.0;
   Ecs.x = Ecs.y = Ecs.z =0.0;
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 5:	/* name -- ? seth */
	    break;
         case 8:
	   /*printf(" in layer %s",DXFgroupvalue);*/
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("LINE");
              return;
          }
          break;
	case 10:
	   start.x = atof(DXFgroupvalue);break;
	case 20:
	   start.y = atof(DXFgroupvalue);break;
	case 30:
	   start.z = atof(DXFgroupvalue);break;
	case 11:
	   finish.x = atof(DXFgroupvalue);break;
	case 21:
	   finish.y = atof(DXFgroupvalue);break;
        case 31:
	   finish.z = atof(DXFgroupvalue);break;
        case 38:
	   start.z = atof(DXFgroupvalue);
	   finish.z = atof(DXFgroupvalue);break;
	case 39:
	  thickness = atof(DXFgroupvalue);break;
	case 62: /*color */
          break;
	case 210:
          useEcs = TRUE;
	  Ecs.x = atof(DXFgroupvalue);break;
	case 220:
	  Ecs.y = atof(DXFgroupvalue);break;
	case 230:
	  Ecs.z = atof(DXFgroupvalue);break;
	case 0:
           v[0] = start; v[1] = finish;
           start.z += thickness; finish.z += thickness;
	   v[2] = finish; v[3] = start; 
           if(useEcs){
              MakeECSmatrix();
              for(i=0;i<4;i++)
                 EcsToWcs(&v[i]);
           } 
           if (fabs(thickness) > EPSILON)
	        MakeFace(v,4,layerindex);
	   else if (Want2D ) {
	     MakeWire(v,2,layerindex);
	   }
           return;
       default: 
          fprintf(Errorfile,"Line : unknown group %d\n",DXFgroupcode);
          break;
      }
   }
}
                                 

Make_arc(start_angle,end_angle,center,radius,thickness,useEcs,layerindex) 
double start_angle,end_angle;
VERTEX  center;
double radius,thickness;
short useEcs;
int layerindex;
{
   int    i,j,num;		
   VERTEX verts[ARC_PTS];/* corners of generated polygon */
   double inc;		/* increment to increase angle */

   if (end_angle < start_angle) end_angle = end_angle + 360;

   /* convert start angle to radians and decide number of polygons to draw */
   num = (int) (ARC_PTS * fabs(end_angle - start_angle) / 360.0);
   if (num < 12) num = 12;
   inc = ((end_angle - start_angle) / num) * (PI / 180.0);
   start_angle = start_angle * PI / 180.0;
   if (fabs(thickness) > EPSILON) {
   /* arc has thickness, generate polygons to approximate cylinder sides */
      for (i = 0; i < num; i++) {
	 verts[0].x = center.x+cos(start_angle+i*inc)*radius;
	 verts[0].y = center.y+sin(start_angle+i*inc)*radius; 
	 verts[0].z = center.z;
	 verts[1].x = center.x+cos(start_angle+(i+1)*inc)*radius;
	 verts[1].y = center.y+sin(start_angle+(i+1)*inc)*radius;
	 verts[1].z = center.z;
	 verts[2].x = center.x+cos(start_angle+(i+1)*inc)*radius;
	 verts[2].y = center.y+sin(start_angle+(i+1)*inc)*radius;
	 verts[2].z = center.z+thickness;
	 verts[3].x = center.x+cos(start_angle+i*inc)*radius;
	 verts[3].y = center.y+sin(start_angle+i*inc)*radius;
	 verts[3].z = center.z+thickness;
         if(useEcs){
           MakeECSmatrix();
           for(j=0;j<4;j++)
             EcsToWcs(&verts[j]);
         }
         MakeFace(verts,4,layerindex); 
      }
   } else if (Want2D) {
      for (i = 0; i <= num; i++) {
	 verts[i].x = center.x+cos(start_angle+i*inc)*radius;
	 verts[i].y = center.y+sin(start_angle+i*inc)*radius; 
	 verts[i].z = center.z;
      }
      if(useEcs){
        MakeECSmatrix();
        for(j=0;j<num;j++)
          EcsToWcs(&verts[j]);
      }
      MakeWire(verts,num+1,layerindex); /* -1 by A&W*/ 
   }
}

                                    
ProcessCircle()
{
   VERTEX center;
   double thickness=0.0,radius=0.0;
   short useEcs = FALSE;
   int layerindex;

   center.z =0.0;
   Ecs.x = Ecs.y = Ecs.z =0.0;
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 5:	/* name -- ? seth */
	    break;
         case 8:              
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("CIRCLE");
              return;
          }
          break;
	case 10:
	   center.x = atof(DXFgroupvalue);break;
	case 20:
	   center.y = atof(DXFgroupvalue);break;
	case 30:
        case 38:
	   center.z = atof(DXFgroupvalue);break;
	case 39:
	  thickness = atof(DXFgroupvalue);break;
	case 40:
	   radius = atof(DXFgroupvalue);break;
	case 62:
          break;
        case 210:
	   /* X extrusion direction of z axis for Entity Cordinate System */ 
           useEcs = TRUE;
	   Ecs.x = atof(DXFgroupvalue);break;
        case 220:
	   /* Y extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.y = atof(DXFgroupvalue);break;
        case 230:
	   /* Z extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.z = atof(DXFgroupvalue);break;
        case 0:
           Make_arc(0.0,359.9999,center,radius,thickness,useEcs,layerindex);
           return;
        default:
           fprintf(Errorfile,"Circle: unknown group %d\n",DXFgroupcode);
           break;
      }
   }
}

                                    
ProcessArc()
{
   VERTEX center;
   double thickness=0.0,radius=0.0,
          start_ang=0.0,end_ang=0.0;
   short useEcs = FALSE;
   int layerindex;
 
   center.z =0.0;
   Ecs.x = Ecs.y =Ecs.z =0.0;
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 5:	/* name -- ? seth */
	    break;
         case 8:              
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("ARC");
              return;
          }
	  /*move arcs into door layer*/
	  if (WantMIT)
	    layerindex = Wantlayer(MIT_DOOR_LAYER); /* place arcs in door layer */
          break;
	case 10:
	   center.x = atof(DXFgroupvalue);break;
	case 20:
	   center.y = atof(DXFgroupvalue);break;
	case 30:
        case 38:
	   center.z = atof(DXFgroupvalue);break;
	case 39:
	  thickness = atof(DXFgroupvalue);break;
	case 40:
	   radius = atof(DXFgroupvalue);break;
	case 50:
	   start_ang = atof(DXFgroupvalue);break;
	case 51:
	   end_ang = atof(DXFgroupvalue);break;
	case 62: /*color */
          break;
        case 210:
	   /* X extrusion direction of z axis for Entity Cordinate System */ 
           useEcs = TRUE;
	   Ecs.x = atof(DXFgroupvalue);break;
        case 220:
	   /* Y extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.y = atof(DXFgroupvalue);break;
        case 230:
	   /* Z extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.z = atof(DXFgroupvalue);break;
        case 0:
           Make_arc(start_ang,end_ang,center,radius,thickness,useEcs,layerindex);
           return;
        default:
           fprintf(Errorfile,"Arc: unknown group %d\n",DXFgroupcode);
           break;
      }
   }
}

ProcessInsert()
{
   double x=1.,y=1.,z=1.,angle=0. ;  /*scale factors and  rotation angle*/
   char name[50],*trans,tempstmt[50];
   INST_STMT *stmt; 
   short useECS = FALSE;
   int i,j;
   BLK *ablk;
   VERTEX start;
   int layerindex;

   start.x = start.y = start.z =0.0;
   Ecs.x =Ecs.y = Ecs.z =0.0; 
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 2:
	  strcpy(name,DXFgroupvalue);break;
	case 8:
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("INSERT");
              return;
          }
          break;
	case 10:
	   start.x = atof(DXFgroupvalue);break;
	case 20:
	   start.y = atof(DXFgroupvalue);break;
	case 30:
	case 38:
	   start.z = atof(DXFgroupvalue);break;
	case 41:
	   x = atof(DXFgroupvalue);break;
	case 42:
	   y = atof(DXFgroupvalue);break;
	case 43:
	   z = atof(DXFgroupvalue);break;
	case 44:
        case 45:
	   /*column and row spacing? */ break;
        case 50:
	   /* rotation angle */ 
	   angle = atof(DXFgroupvalue);break;
	case 62: /*color */
          break;
        case 66:
	   break;
        case 210:
	   /* X extrusion direction of z axis for Entity Cordinate System */ 
           useECS = TRUE;
	   Ecs.x = atof(DXFgroupvalue);break;
        case 220:
	   /* Y extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.y = atof(DXFgroupvalue);break;
        case 230:
	   /* Z extrusion direction of z axis for Entity Cordinate System */ 
	   Ecs.z = atof(DXFgroupvalue);break;
	case 0: /* end of insert */
            MakeInstance(name);
            stmt = MakeInstanceStmt(Cur_block);
            stmt->layerindex = layerindex;
            stmt->name = Instance_list->name;
            stmt->blk = Findblk(name);
            trans = stmt->transforms;
            trans[0] = '\0';
            if ((x*y*z) < 0){ 
            /* if odd number of mirroring,
               record mirrored instance so that order of vertices will be
               considered in reverse*/
               Instance_list->mirror = TRUE;
            } 
          
            /* it seems that the base is always origin, but just incase... 
	    if(ablk->base.x != 0.){
 		sprintf(tempstmt," -tx %g", -ablk->base.x);
 		strcat(trans,tempstmt);
	    }
	    if(ablk->base.y != 0.){
 		sprintf(tempstmt," -ty %g", -ablk->base.y);
 		strcat(trans,tempstmt);
	    }
	    if(ablk->base.z != 0.){
 		sprintf(tempstmt," -tz %g", -ablk->base.z);
 		strcat(trans,tempstmt);
	    }
            */
            
	    if(x == -1. ){
 		strcat(trans," -mx");
	    }else if(x < 0. ){
 		sprintf(tempstmt," -sx %g -mx",-x);
 		strcat(trans,tempstmt);
	    }else if (x != 1.){
 		sprintf(tempstmt," -sx %g",x);
 		strcat(trans,tempstmt);
            }
	    if(y == -1. ){
 		strcat(trans," -my");
	    }else if(y< 0. ){
 		sprintf(tempstmt," -sy %g -my",-y);
 		strcat(trans,tempstmt);
	    }else if (y !=1.){
 		sprintf(tempstmt," -sy %g",y);
 		strcat(trans,tempstmt);
            }
	    
	    if(z == -1. ){
 		strcat(trans," -mz");
	    }else if(z< 0. ){
 		sprintf(tempstmt," -sz %g -mz",-z);
 		strcat(trans,tempstmt);
	    }else if (z != 1.){
 		sprintf(tempstmt," -sz  %g",z);
 		strcat(trans,tempstmt);
            }
	    
	    if(angle != 0.){
 		sprintf(tempstmt," -rz %g",angle);
 		strcat(trans,tempstmt);
	    }
            /*
	    if(start.x != 0. || ablk->base.x != 0.){
 		sprintf(tempstmt," -tx %g",start.x + ablk->base.x );
 		strcat(trans,tempstmt);
	    }
	    if(start.y != 0 || ablk->base.y != 0.){
 		sprintf(tempstmt," -ty %g",start.y + ablk->base.y );
 		strcat(trans,tempstmt);
	    }
	    if(start.z != 0 || ablk->base.z != 0.){
 		sprintf(tempstmt," -tz %g",start.z + ablk->base.z);
 		strcat(trans,tempstmt);
            }
            */
	    if(start.x != 0. ){
 		sprintf(tempstmt," -tx %g",start.x);
 		strcat(trans,tempstmt);
	    }
	    if(start.y != 0 ){
 		sprintf(tempstmt," -ty %g",start.y );
 		strcat(trans,tempstmt);
	    }
	    if(start.z != 0 ){
 		sprintf(tempstmt," -tz %g",start.z );
 		strcat(trans,tempstmt);
            }
	    if(useECS){
                MakeECSmatrix();
 		sprintf(tempstmt," -M4 ");
 		strcat(trans,tempstmt);
                for (i=0;i<4;i++)
                   for (j=0;j<4;j++){
 		      sprintf(tempstmt,"%g ",Ecs_matrix[i][j]);
 		      strcat(trans,tempstmt);
                   }
            }
            return;
       default: 
           fprintf(Errorfile,"Insert: unknown group %d\n",DXFgroupcode);
           break;
     }
   }
}                            

              
GetVertices(vertex,j,start_width,end_width)
VERTEX vertex[];
int *j;
double *start_width,*end_width;
{
   int flags;
   static float defaultz = 0.0;

   /* incase there is no x,y for this vertex */
   vertex[*j].x = vertex[*j].y = 0.0;
   /* incase there is no z for this vertex */
   vertex[*j].z = defaultz;
   start_width[*j] = 0.;
   end_width[*j] = 0.; 
   
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 5:	    /* vertex name? -- seth */
	case 8:
           break;
	case 10:
	   vertex[*j].x = atof(DXFgroupvalue);break;
	case 20:
	   vertex[*j].y = atof(DXFgroupvalue);break;
	case 30:
	case 38:
	case 39:
	   defaultz = vertex[*j].z = atof(DXFgroupvalue); break; 
	case 62: /*color */
          break;
	case 70:
	   flags = atoi(DXFgroupvalue);
	   if(flags & CONTROLVERTEX) {
	      /*ignore control vertices */
              /* if (*j > 0) */
		/* 	(*j)--; */
	      }
           break;
	case 40: 
             start_width[*j] = atof(DXFgroupvalue);
             break;
	case 41:
             end_width[*j] = atof(DXFgroupvalue);
             break;
	case 42:
             fprintf(Errorfile,"Bulge factor ignored;lineno:%d\n",CurLineno);
             break;
	case 50:
           break;
	case 0:
	    if (*j < MAXPOLYVERT-1)	{
		(*j)++;
		vertex[*j].z  = 0.0; /* incase there is no z for this vertex */
		}
	    else    {
		fprintf ( stderr, "overflowed MAXPOLYVERTS!  *j == %d in line %d\n", *j, CurLineno );
		}

           if (!strcmp(DXFgroupvalue,"SEQEND"))
	      return;
           else 
              break;
        default: 
           fprintf(Errorfile,"Vertex: unknown group %d\n",DXFgroupcode);
           break;
      }
   }
}


Addthickness(w,thickness)
VERTEX w[];
double thickness;
{
         w[2].x = w[1].x;  
	 w[2].y = w[1].y;  
	 w[2].z = w[1].z + thickness;  
	 w[3].x = w[0].x;  
	 w[3].y = w[0].y;  
	 w[3].z = w[0].z + thickness;  
}

/*
  For each pair of vertices find two parallel lines which are
  apart by width and intersect them with two  parallel lines
  for the next pair. Find all such intersections and make
  faces, with four sides. If thick add thickness.
*/ 

AddWidth(verts,num,start_width,end_width,thickness,closed,useEcs,layerindex)
VERTEX verts[];
int num;
double start_width[], end_width[], thickness;
short closed,useEcs;
int  layerindex;
{
  VERTEX vtop[MAXPOLYVERT],vbottom[MAXPOLYVERT];
  VERTEX vtopthick[MAXPOLYVERT],vbottomthick[MAXPOLYVERT];
  VERTEX v[4];
  int i,j,k,last;
  double length,sine,cosine,
         trans_x,trans_y,
	 new_x, new_y;

   if (num < 2) return;
   j = num;
   if(closed){
      verts[j++] = verts[0];     
      verts[j++] = verts[1];     
   }
/*
 fprintf(Errorfile,"Wide POLY: lineno = %d layer %s\n",Lineno,Layertable[layerindex].name);
   fprintf(Errorfile,"originals\n");
   for(i=0;i < j ;i++)
       fprintf(Errorfile,"x=%.1f y=%.1f	s=%.1f e=%.1f\n",verts[i].x,verts[i].y,
					start_width[i],end_width[i]);
*/
 
  for(i=0,k=0;i<j-1;i++,k +=2){
     if ( verts[i].z != verts[(i+1) % j].z ){
       fprintf(Errorfile,"Wide polyline not in xy plane;lineno:%d \n",Lineno);
       return;
     }
     /* translate the line so that first vertex is origin */ 
     trans_x=  verts[i+1].x - verts[i].x;
     trans_y = verts[i+1].y - verts[i].y;
     length = sqrt( trans_x * trans_x + trans_y * trans_y);
     sine = trans_y / length;   /* sin of angle between line and x axis */
     cosine = trans_x / length; /* cos of angle between line and x axis */
  
     /* find coordinates of a point along line that has a length
     of start_width/2  from origin */
     new_x = (start_width[i] * 0.5) * cosine;  
     new_y = (start_width[i] * 0.5)* sine;  

     /* rotate this point around origin 90 and -90 degrees and translate
        back to get new coordinates  */
     vtop[k].x = -new_y + verts[i].x;
     vtop[k].y = new_x + verts[i].y;
     vbottom[k].x = new_y + verts[i].x;
     vbottom[k].y = -new_x + verts[i].y;
     
     /* translate the line so that second  vertex is origin */ 
     trans_x=  verts[i].x - verts[i+1].x;
     trans_y = verts[i].y - verts[i+1].y;
     length = sqrt( trans_x * trans_x + trans_y * trans_y);
   
     sine = trans_y / length;   /* sin of angle between line and x axis */
     cosine = trans_x / length; /* cos of angle between line and x axis */
     
     /* find coordinates of a point along line that has a length
        of end_width/2  from origin */
     new_x = (end_width[i]/2) * cosine;  
     new_y = (end_width[i]/2) * sine;  
   
     /* rotate this point around origin 90 and -90 degrees and translate
        back to get new coordinates  */
     vbottom[k+1].x = -new_y + verts[i+1].x;
     vbottom[k+1].y = new_x + verts[i+1].y;
     vtop[k+1].x = new_y + verts[i+1].x;
     vtop[k+1].y = -new_x + verts[i+1].y;
   }

   last = k-1;
 /* 
   fprintf(Errorfile,"----\n"); 
   for(i=0;i<=last;i++){
       fprintf(Errorfile,"bottom:x=%.1f y=%.1f   top:x=%.1f y=%.1f\n",vbottom[i].x,
       vbottom[i].y, vtop[i].x,vtop[i].y);
   }
*/
   for(i=1,k=0;i<j-1; i++, k+=2){
     if(!Intersect(&v[0],vbottom[k],vbottom[k+1],vbottom[k+2],vbottom[k+3])){
        printf("Parrallel lines have no intersection; %d;\n",Lineno);
        return; 
     }
     vbottom[i] = v[0];
   }
   
   for(i=1,k=0;i<j-1; i++, k+=2){
     if(!Intersect(&v[0],vtop[k],vtop[k+1], vtop[k+2],vtop[k+3])){
        printf("Parrallel lines have no intersection; %d;\n",Lineno);
        return;
     }
     vtop[i] = v[0];
   }
   if(closed){
      vbottom[0] = vbottom[j-2];
      vtop[0] = vtop[j-2];
      j--;
   }else{
      vbottom[j-1] = vbottom[last];
      vtop[j-1] = vtop[last];
   }
   for(i=0;i<j;i++){
      vtop[i].z = verts[0].z;
      vbottom[i].z = verts[0].z;
   }
   /* if thick  make extruded set of vertices */ 
   if(fabs(thickness) >  EPSILON)
     for(i=0;i<j;i++){
        vtopthick[i] = vtop[i];
        vbottomthick[i] = vbottom[i];
        vtopthick[i].z += thickness; 
        vbottomthick[i].z += thickness; 
     } 
  if(useEcs)
    for(k=0;k<j;k++){
      EcsToWcs(&vtop[k]);
      EcsToWcs(&vbottom[k]);
    }
   /* take two vertices from top and two from bottom and make a face*/  
   for(i=0;i<j-1;i++){
     v[0]= vtop[i];
     v[1]= vtop[i+1];
     v[2]= vbottom[i+1];
     v[3]= vbottom[i];
     MakeFace(v,4,layerindex);
   } 
   if(fabs(thickness) >  EPSILON){
     if(useEcs)
       for(k=0;k<j;k++){
         EcsToWcs(&vtopthick[k]);
         EcsToWcs(&vbottomthick[k]);
       }
     /* make faces for top */
     for(i=0;i<j-1;i++){
        v[0]= vbottomthick[i];
        v[1]= vbottomthick[i+1];
        v[2]= vtopthick[i+1];
        v[3]= vtopthick[i];
        MakeFace(v,4,layerindex);
     }
     /* make faces for sides */
     for(i=0;i<j-1;i++){
        v[0]= vtopthick[i];
        v[1]= vtopthick[i+1];
        v[2]= vtop[i+1];
        v[3]= vtop[i];
        MakeFace(v,4,layerindex);
        v[0]= vbottom[i];
        v[1]= vbottom[i+1];
        v[2]= vbottomthick[i+1];
        v[3]= vbottomthick[i];
        MakeFace(v,4,layerindex);
     }
     /* if the shape is  not closed make two end  faces */
     if (!closed){
        v[0]= vbottom[0];
        v[1]= vbottomthick[0];
        v[2]= vtopthick[0];
        v[3]= vtop[0];
        MakeFace(v,4,layerindex);
        v[0]= vbottom[j-1];
        v[1]= vtop[j-1];
        v[2]= vtopthick[j-1];
        v[3]= vbottomthick[j-1];
        MakeFace(v,4,layerindex);
     }
    }
  /* 
   printf("===\n");
   for(i=0;i <j ;i++)
       printf("bottom:x=%.1f y=%.1f z=%.3f   top:x=%.1f y=%.1f z=%.3f\n",vbottom[i].x,
       vbottom[i].y,vbottom[i].z, vtop[i].x,vtop[i].y,vtop[i].z);
   
*/
}

Process3Dmesh(v,flag,m,n,mdensity,ndensity,closed,layerindex)
VERTEX v[];
int flag;
int m,n,
mdensity,ndensity; /* not implemented */
short closed;
int layerindex;
{
   int             row, row_next;
   int             column, column_next;
   VERTEX          w[4],trngle1[3],trngle2[3];


   for (row=0; row < m - 1; row++){
      row_next = row+1;
      for (column=0; column < n - 1; column++){
	 column_next = column+1;
	 w[0].x = v[row*n + column].x;
	 w[0].y = v[row*n + column].y;
	 w[0].z = v[row*n + column].z;

	 w[1].x = v[row_next*n + column].x;
	 w[1].y = v[row_next*n + column].y;
	 w[1].z = v[row_next*n + column].z;

	 w[2].x = v[row_next*n + column_next].x;
	 w[2].y = v[row_next*n + column_next].y;
	 w[2].z = v[row_next*n + column_next].z;

	 w[3].x = v[row*n + column_next].x;
	 w[3].y = v[row*n + column_next].y;
	 w[3].z = v[row*n + column_next].z;

         if(WantPlanv){
	    MakeFace(w,4,layerindex);
         }else{
	    /* tesselate the polygon into two triangles */
	    if ((((w[3].x-w[1].x)*(w[3].x-w[1].x))+
	       ((w[3].y-w[1].y)*(w[3].y-w[1].y))+
	       ((w[3].z-w[1].z)*(w[3].z-w[1].z))) <
	       (((w[2].x-w[0].x)*(w[2].x-w[0].x))+
	       ((w[2].y-w[0].y)*(w[2].y-w[0].y))+ 
	       ((w[2].z-w[0].z)*(w[2].z-w[0].z)))){
	          trngle1[0] = w[0];
	          trngle1[1] = trngle2[0] = w[1];
	          trngle1[2] = trngle2[2] = w[3];
	          trngle2[1] = w[2];
	    }else{
	       trngle1[0] = trngle2[0] = w[0];
	       trngle1[1] = w[1];
	       trngle1[2] = trngle2[1] = w[2];
	       trngle2[2] = w[3];
	    }
	    MakeFace(trngle1,3,layerindex);
	    MakeFace(trngle2,3,layerindex);
         }

      }
   }

/* if the surface is closed , close the polygon */
   if (flag & POLY3DCLOSEDN ){
      for (column=0; column < n - 1; column++){
	 column_next = column+1;
	 w[0].x = v[(m -1)*n + column].x;
	 w[0].y = v[(m -1)*n + column].y;
	 w[0].z = v[(m -1)*n + column].z;

	 w[1].x = v[column].x;
	 w[1].y = v[column].y;
	 w[1].z = v[column].z;

	 w[2].x = v[column_next].x;
	 w[2].y = v[column_next].y;
	 w[2].z = v[column_next].z;

	 w[3].x = v[(m -1)*n + column_next].x;
	 w[3].y = v[(m -1)*n + column_next].y;
	 w[3].z = v[(m -1)*n + column_next].z;

         if(WantPlanv){
	    MakeFace(w,4,layerindex);
         }else{
	    /* tesselate the polygon into two triangles */
	    if ((((w[3].x-w[1].x)*(w[3].x-w[1].x))+
	       ((w[3].y-w[1].y)*(w[3].y-w[1].y))+
	       ((w[3].z-w[1].z)*(w[3].z-w[1].z))) <
	       (((w[2].x-w[0].x)*(w[2].x-w[0].x))+
	       ((w[2].y-w[0].y)*(w[2].y-w[0].y))+ 
	       ((w[2].z-w[0].z)*(w[2].z-w[0].z)))){
	          trngle1[0] = w[0];
	          trngle1[1] = trngle2[0] = w[1];
	          trngle1[2] = trngle2[2] = w[3];
	          trngle2[1] = w[2];
	       }
	    else {
	       trngle1[0] = trngle2[0] = w[0];
	       trngle1[1] = w[1];
	       trngle1[2] = trngle2[1] = w[2];
	       trngle2[2] = w[3];
	    }
	    MakeFace(trngle1,3,layerindex);
	    MakeFace(trngle2,3,layerindex);
         }

      }
   }

/* if the surface is closed in m direction  , close the polygon */
   if (closed){
      for (row=0; row < m - 1; row++){
	 row_next = row+1;
	 w[0].x = v[(n -1)*m + row].x;
	 w[0].y = v[(n -1)*m + row].y;
	 w[0].z = v[(n -1)*m + row].z;

	 w[1].x = v[row].x;
	 w[1].y = v[row].y;
	 w[1].z = v[row].z;

	 w[2].x = v[row_next].x;
	 w[2].y = v[row_next].y;
	 w[2].z = v[row_next].z;

	 w[3].x = v[(n -1)*m + row_next].x;
	 w[3].y = v[(n -1)*m + row_next].y;
	 w[3].z = v[(n -1)*m + row_next].z;

         if(WantPlanv){
	    MakeFace(w,4,layerindex);
         }else{
	    /* tesselate the polygon into two triangles */
	    if ((((w[3].x-w[1].x)*(w[3].x-w[1].x))+
	       ((w[3].y-w[1].y)*(w[3].y-w[1].y))+
	       ((w[3].z-w[1].z)*(w[3].z-w[1].z))) <
	       (((w[2].x-w[0].x)*(w[2].x-w[0].x))+
	       ((w[2].y-w[0].y)*(w[2].y-w[0].y))+ 
	       ((w[2].z-w[0].z)*(w[2].z-w[0].z)))){
	          trngle1[0] = w[0];
	          trngle1[1] = trngle2[0] = w[1];
	          trngle1[2] = trngle2[2] = w[3];
	          trngle2[1] = w[2];
	       }
	    else {
	       trngle1[0] = trngle2[0] = w[0];
	       trngle1[1] = w[1];
	       trngle1[2] = trngle2[1] = w[2];
	       trngle2[2] = w[3];
	    }
	    MakeFace(trngle1,3,layerindex);
	    MakeFace(trngle2,3,layerindex);
         }

      }
   }

}
             
          
ProcessPolyline()
{
   int i=0,j=0,k=0;
   short isthick   = FALSE,
         closed    = FALSE,
         useSwidth  = FALSE,
         useEwidth  = FALSE,
         useEcs    = FALSE,
         ismesh    = FALSE;
   int flags;
   int m,n,mdensity=0,ndensity=0;
   VERTEX v[MAXPOLYVERT];
   VERTEX w[4];
   VERTEX start;
   double thickness =0.0;
   double start_width[MAXPOLYVERT],end_width[MAXPOLYVERT];
   int layerindex;
    
   for(i=0;i<MAXPOLYVERT;i++)
       v[i].z =0.;
   Ecs.x = Ecs.y = Ecs.z =0.0;
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
	case 5:	/* name -- ? seth */
	    break;
	case 8 :
	  /*printf(" in layer %s",DXFgroupvalue);*/
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("POLYLINE");
              return;
          }
          break;
	case 10:
	   start.x = atof(DXFgroupvalue);break;
	case 20:
	   start.y = atof(DXFgroupvalue);break;
	case 30:
	case 38:
	   start.z = atof(DXFgroupvalue);break;
	case 39:
	   thickness = atof(DXFgroupvalue);
           if(fabs(thickness) > EPSILON)
	     isthick = TRUE;
           break;
	case 40:
	   start_width[0] = atof(DXFgroupvalue);
           if (start_width[0] > EPSILON){
              useSwidth = TRUE;
              for(i=0;i<MAXPOLYVERT;i++)
	         start_width[i]  = start_width[0]; 
           }
           break;
	case 41:
	   end_width[0] = atof(DXFgroupvalue);
           useEwidth = TRUE;
	   if(end_width[0] > EPSILON){
              useEwidth = TRUE;
              for(i=0;i<MAXPOLYVERT;i++)
	         end_width[i]  = end_width[0]; 
           }
           break;
	case 62: /*color */
          break;
	case 66:
           break;
	case 70:
	   flags = atoi(DXFgroupvalue);
           if (flags & CLOSED ){
               closed = TRUE;
           }
           if (flags & CURVEFITTED){
               fprintf(Errorfile,"Curve-fitted polyline ;lineno:%d\n",Lineno);
           }
           if (flags & SPLINEFITTED){
               fprintf(Errorfile,"Spline polyline ;lineno:%d\n",Lineno);
           }
           if (ismesh = (flags & POLY3DMESH)){
               fprintf(Errorfile,"3D polyline mesh;lineno:%d\n",Lineno);
           }
           if (flags & POLY3D){
               fprintf(Errorfile,"3D polyline;lineno:%d\n",Lineno);
           }
           break; 
	case 71:
           m = atoi(DXFgroupvalue);break;
	case 72:
           n = atoi(DXFgroupvalue);break;
	case 73:
           mdensity = atoi(DXFgroupvalue);break;
	case 74:
           ndensity = atoi(DXFgroupvalue);break;
	case 210:
           useEcs = TRUE;
	   Ecs.x  = atof(DXFgroupvalue);break;
	case 220:
	   Ecs.y  = atof(DXFgroupvalue);break;
	case 230:
	   Ecs.z  = atof(DXFgroupvalue);break;
	case 0:
           if (ismesh){
              GetVertices(v,&j,start_width,end_width);
              Process3Dmesh(v,flags,m,n,mdensity,ndensity,closed,layerindex);
              return;
           }
           GetVertices(v,&j,start_width,end_width);
           /* if last vertex is same as first one delete it and
             set the closed flag on, so everything stays consistent */
           if(v[0].x == v[j-1].x &&
              v[0].y == v[j-1].y &&
              v[0].z == v[j-1].z ){
              j--;
              closed = TRUE;
           }
           CheckVertices(v,&j);
	   if (useEcs)
              MakeECSmatrix();
           if ( !closed && !isthick && !(useSwidth || useEwidth) ){
           /*polyline is 2D i.e. not thick or closed or has no width */
              if(Want2D){
	         if (useEcs){
	            for(k=0;k<j;k++)
	              EcsToWcs(&v[k]);
                 }
	         MakeWire(v,j,layerindex);
              }
	      return;
           }
	   if (closed && !(isthick || useSwidth  || useEwidth )){
           /* this is just a closed simple polyline */
	      if (useEcs){
	        for(k=0;k<j;k++)
	          EcsToWcs(&v[k]);
              }
	      MakeFace(v,j,layerindex);
              return;
           } 
	   if (closed){
             if (j < 3){
               fprintf(Errorfile,
               "Can't close a polyline with two vertices;lineno=%d;layer=%s \n",
                Lineno,Layertable[layerindex].name);
                closed = FALSE;
             }
           }
	   if ( useSwidth  || useEwidth ){
            /* wide polyline or wide and thick polyline */
	      AddWidth(v,j,start_width,end_width,thickness,closed,
                       useEcs,layerindex);
               return;
           } 
	   if (closed){
            /* closed and thick polyline */
             if (v[j-1].x != v[0].x ||
                   v[j-1].y != v[0].y ){    
                /* End vertex not the same as first;so add first to end */
		 v[j++] = v[0];
              }
           }
	   if (isthick){
              for(i=0;i<j -1;i++){
              /* take pairs of vertices and make a  thick polygon */
                 w[0] = v[i];
                 w[1] = v[i+1];
	         Addthickness(w,thickness);
	         if (useEcs)
	           for(k=0;k<4;k++)
	             EcsToWcs(&w[k]);
	         MakeFace(w,4,layerindex);
               }
	  }
          return;
       default:
           fprintf(Errorfile,"Polyline: unknown group %d\n",DXFgroupcode);
           break;
     }
  }
}
       
                               
ProcessSolid()
{
   int i,j=0;
   short useEcs  = FALSE;
   VERTEX v[4], 
          w[4], u[4],temp; 
   double thickness =0.0,elevation=0.0;
   int layerindex;

   for(i=0;i<4;i++){
     v[i].z =0.;
     w[i].z =0.;
     u[i].z =0.;
   } 
   Ecs.x = Ecs.y = Ecs.z =0.0;
   j = -1;
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
      case 5 : break;
        case 8 :
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("SOLID");
              return;
          }
          break;
        case 10:
        case 11:
        case 12:
        case 13:
           v[++j].x =  atof(DXFgroupvalue);break;
	case 20:
	case 21:
	case 22:
	case 23:
	   v[j].y = atof(DXFgroupvalue);break;
	case 30:
	case 31:
	case 32:
	case 33:
	   v[j].z = atof(DXFgroupvalue);break;
	case 38:
	   elevation = atof(DXFgroupvalue);break;
	case 39:
	   thickness = atof(DXFgroupvalue);break;
	case 62: /*color */
          break;
	case 210:
           useEcs = TRUE;
	   Ecs.x = atof(DXFgroupvalue);break;
	case 220:
	   Ecs.y = atof(DXFgroupvalue);break;
	case 230:
	   Ecs.z = atof(DXFgroupvalue);break;
        case 0:
            if(elevation != 0.0)
               for(i=0;i<4;i++)
                  v[i].z =elevation;
            j++;
            if(j==4) {
             /* if four sided polygon, switch third and fourth 
                vertex to eliminate AUTOCAD vertex order oddity */
               temp = v[2];
               v[2] = v[3];
               v[3] = temp;
            }
	    if ( fabs(thickness) > EPSILON){
	       for(i=0;i<j;i++){
		    w[i].x = v[i].x ;
		    w[i].y = v[i].y ;
		    w[i].z = v[i].z  + thickness;
               }
            }
            if(useEcs){
              MakeECSmatrix();
              for(i=0;i<j;i++){
                 EcsToWcs(&v[i]);
                 EcsToWcs(&w[i]);
              }
            }
            MakeFace(v,j,layerindex);
	    if ( fabs(thickness) > EPSILON){
               /* make the top face */
               MakeFace(w,j,layerindex);
               /* make side faces */
	       for(i=0;i<j;i++){
                  u[0] = v[i];
                  u[1] = v[(i+1) % j];
                  u[2] = w[(i+1) % j];
                  u[3] = w[i];
                  MakeFace(u,4,layerindex);
	       }
	    }
            return;
        default : 
           fprintf(Errorfile,"Solid: unknown group %d\n",DXFgroupcode);
           break;
      }
   }
}

                                      
Process3dface()
{
   int j=0;
   VERTEX  v[4];
   int layerindex;

   for(j=0;j< 4;j++)
     v[j].z = 0.0; 
   while(1){
      GetTwoLines();
      switch (DXFgroupcode){
        case 8 :
          if (!(layerindex = Wantlayer(DXFgroupvalue))){
              Skip("3DFACE");
              return;
          }
          break;
        case 10:
	   v[0].x = atof(DXFgroupvalue);j=1;break;
        case 11:
	   v[1].x = atof(DXFgroupvalue);j=2;break;
        case 12:
	   v[2].x = atof(DXFgroupvalue);j=3;break;
        case 13:
	   v[3].x = atof(DXFgroupvalue);j=4;break;
	case 20:
	   v[0].y = atof(DXFgroupvalue);break;
	case 21:
	   v[1].y = atof(DXFgroupvalue);break;
	case 22:
	   v[2].y = atof(DXFgroupvalue);break;
	case 23:
	   v[3].y = atof(DXFgroupvalue);break;
	case 30:
	   v[0].z = atof(DXFgroupvalue);break;
	case 31:
	   v[1].z = atof(DXFgroupvalue);break;
	case 32:
	   v[2].z = atof(DXFgroupvalue);break;
	case 33:
	   v[3].z = atof(DXFgroupvalue);break;
	case 62: /*color */
          break;
	case 70: /*flag bits ignored */ break;
        case 0:
           MakeFace(v,j,layerindex);
           return;
        default: 
           fprintf(Errorfile,"3dface: unknown group %d\n",DXFgroupcode);
           break;
       }
   }
}

main(argc,argv)
int argc;
char *argv[];
{  
   char type[50];

   start_timer();
   Process_args(argc,argv);
   Setup_files();
  
   /* read the DXF file two lines at a time and process groups */
   GetTwoLines();
   while(1){
      if(!strcmp(DXFgroupvalue,"EOF")) break;
      dispatch_on_group(DXFgroupcode,DXFgroupvalue);
   }
      
   fclose(Dxffile);
   MakeInstance("Entity");  /* Make the top instance */
#ifdef PLANV
   if (WantPlanv) {
      Write_temp_ugfile();
      Make_flatfile();
      Read_flatfile();
      WritePlanv();
   }
#endif PLANV
#ifdef FIXDXF
   if (WantFix2 || WantFix3) Fix_orients2();
   if (WantFix1) {
      Write_temp_ugfile();
      Make_flatfile();
      Read_flatfile();
   }
   if (WantFix1 ) Fix_orients1();
#endif FIXDXF
   printextents();     
   fclose(Errorfile);
   Write_ugfile();
   end_timer();
   get_time_diff();
   print_time_diff("convert");
}
