#include <gl.h>
#include <device.h>
#include "acad2ug.h"
#include "fix1.h"

static char *Flatfilename = "Flat";
FILE *Flatfile;
VERTEX View[MAXVIEWPOINTS];
FBUF_INFO *Fbuf_id;
int Numviews,Total_faces=0;
Matrix idmat = {1.0,0.0,0.0,0.0,
                0.0,1.0,0.0,0.0,
                0.0,0.0,1.0,0.0,
                0.0,0.0,0.0,1.0};
Make_flatfile(){
   FILE *Flatfile;
   char command[80];
   
   sprintf(command,"ugflatten -l < tempugfile  > %s ", Flatfilename);
   system(command);
}


INSTANCE *
Findinst(name)
char name[];
{
   INSTANCE *inst = Instance_list;
   while ( inst != NULL) { 
    if(!strcmp(name,inst->name)) 
      return(inst);
    inst = inst->next;
   }
   return(NULL);
}


VERTEX *
ParseVertex(name,index)
char name[];
int *index;
{
    char inst_name[50];
    char block_name[50];
    char id[50];
    int underscore[10];
    int uscores =0;
    int start;
    int i,j,k;
    BLK *blk;
    INSTANCE *inst;

   /* name is of form  instname_..._instname_vINT   */
   /* the last instname is of form   #blockname#INT   */
  
   /* find position of all underscores */
   for(i=0;name[i] != '\0';i++)
       if (name[i] == '_'){
         underscore[uscores++] = i;
       } 
   start = (uscores < 2 ? 0 : underscore[uscores -2]); 
   if (name[start] == '_')
       start++;
   if (name[start] == '#')
       start++;
   /* extract the block name; start form one to last underscore */
   for(i=start,k=0;name[i] != '#';i++,k++)
     block_name[k] = name[i];
   block_name[k] = '\0'; 
  
    /* extract the instance name */
   for(i=start-1,k=0;name[i] != '_';i++,k++)
     inst_name[k] = name[i];
   inst_name[k] = '\0'; 
  
    /* extract the integer id at the end of name */
   start = underscore[uscores -1] + 3;
   for(i=start,k=0; name[i] != '\0'; i++,k++)
     id[k] = name[i];
   id[k] = '\0'; 
   *index = atoi(id);
  
   /* find block and instance  corresponding to this instance */
   blk = Findblk(block_name);  
   inst = Findinst(inst_name);
   return(inst->vlist);
}


Read_flatfile()
{
   FILE *Flatfile;
   char  Flatfilename[50];
   char string[256];
   char code, v_name[50], f_name[50];
   int i,index;
   VERTEX *v_listp;
   
   sprintf(Flatfilename,"Flat");
   
   /* Open the file */
   if((Flatfile = fopen(Flatfilename,"r")) == NULL){
      printf("can't open file %s\n",Flatfilename);
      exit(1);
   }

   while (fscanf(Flatfile,"%c",&code) != EOF ){
       if (code == VERTEXCODE){
          fscanf(Flatfile,"%s",string) ;
          v_listp = ParseVertex(string,&index);
          fscanf(Flatfile,"%s",string);           
          v_listp[index].x = atof(string);
          fscanf(Flatfile,"%s",string);           
          v_listp[index].y = atof(string);
          fscanf(Flatfile,"%s",string);           
          v_listp[index].z = atof(string);
          fgets(string,256,Flatfile); 
       }else if (code == FACECODE || code == WIRECODE){
          fclose(Flatfile);
          return;
      }else{ /* skip */
            fgets(string,256,Flatfile); 
      }
   }
}

ReadViewPoints()
{
   int i=0;
   char useless[50],c,line[256];
   int numread;
   FILE *viewfile;

   if((viewfile = fopen(Viewfilename,"r")) == NULL){
      printf("can't open file %s\n",Viewfilename);
      return(0);
   }
   i=0;
   while(fgets(line,256,viewfile)){ /*
      if(numread = sscanf(line,"%c %c %s %s %lg %s %lg %s %lg",
          &c,&c, useless,useless,&(View[i].x),useless,
          &(View[i].y), useless,&(View[i].z)) == 9){
       */
      if(numread = sscanf(line,"%g  %g %g",
          &(View[i].x), &(View[i].y),&(View[i].z)) == 3){
           i++;
      }else{
        printf("error in reading viewfile,lineno =%d\n",i);
        return(0);
      }
   }
   Numviews =i;
   fclose (viewfile);
   return(1);
}


Render()
{
   int i,j,k=0,index;
   FACE *face;
   BLK  *blk; 
   INSTANCE *inst;
   float vector[3];
      
   for (i=0;i<Total_faces;i++){
      face = Fbuf_id[i].face ;
      inst = Fbuf_id[i].inst ;
      blk = inst->blockp;
      bgnpolygon();
      cpack(i);
      for(j=0;j<face->numverts;j++){
         index = face->v_id_list[j];
         vector[0] = inst->vlist[index].x;
         vector[1] = inst->vlist[index].y;
         vector[2] = inst->vlist[index].z;
         v3f(vector);
      }
      endpolygon();
  }

}

FindNormals()
{
  FACE *face;
  int index1,index2,index3,i ;
  VERTEX *verts,a,b,c;
 
  for(i=0;i<Total_faces;i++){ 
     verts = Fbuf_id[i].inst->vlist;
     face = Fbuf_id[i].face;
     index1 = face->v_id_list[0];
     index2 = face->v_id_list[1];
     index3 = face->v_id_list[2];
     Find_vec(verts[index1],verts[index2],&a); 
     Find_vec(verts[index2],verts[index3],&b);
     Cross(&c,a,b);
     Fbuf_id[i].normal.x = c.x;
     Fbuf_id[i].normal.y = c.y;
     Fbuf_id[i].normal.z = c.z;
     Fbuf_id[i].normal.w = -Dot(c,verts[index1]);
  }
} 
  

  
void Orient(viewpnt)
VERTEX viewpnt;
{
   unsigned long pixel[RIGHTW-LEFTW+1][TOPW-BOTTOMW+1]; /*rendered faces */
   int i,j,k,num;
   double result;
   FACE *face;
   INSTANCE *inst;
   VERTEX4 a;

   a.x = viewpnt.x;
   a.y = viewpnt.y;
   a.z = viewpnt.z;
   a.w = 1;
   num = lrectread(0,0,RIGHTW-LEFTW,TOPW-BOTTOMW, &pixel[0][0]);/*read pixels */
   for(i=0;i<Total_faces;i++)
     Fbuf_id[i].done = 0;
   for(i=0;i< RIGHTW - LEFTW;i++)
     for(j=0;j<TOPW - BOTTOMW;j++){
       if(pixel[i][j] > Total_faces) /* pixel does not have a face color */
         continue;
       /* consider face if it covers atleast 20 pixels */
       if(Fbuf_id[pixel[i][j]].done == 20){
          Fbuf_id[pixel[i][j]].done++ ;
          face = Fbuf_id[pixel[i][j]].face;
          inst = Fbuf_id[pixel[i][j]].inst;
          if(inst->mirror == TRUE) continue;
          result = Dot4(Fbuf_id[pixel[i][j]].normal,a); 
          if(result<0){
             face->flips++; /* this face should be fliped */
          }else if(result > 0){
             face->oks++; /*this face seems ok */
          }
       }else if(Fbuf_id[pixel[i][j]].done < 20)
          Fbuf_id[pixel[i][j]].done++ ;
        else
         continue;
     }
}


FixErrors()
{
   
   FACE *face;
   BLK  *blk;
   int i;
   int numdoubles=0;

   for(blk=First_block;blk != NULL; blk = blk->next){
      for (i=0; i<blk->numfaces;i++){
         face = &blk->flist[i];
         if( (face->oks > 0 ) ||  (face->flips > 0) ){
            if(face->flips > face->oks )
               face->check = FLIP;
            else
               face->check = OK;
         }
         if( (face->oks > 0 ) && (face->flips > 0) ){
            face->error = TRUE;
            /*if both sides seen more than once, make this face double
              but let the result of simple fix override  
            if((face->oks > 1 && face->flips > 1) &&
               (face->oks < 11 && face->flips < 11)){
               face->check = DOUBLE;
               if(blk== Last_block)
                  numdoubles++;
            } */
         }
      }
   }
   printf("Fix resulted in %d double sided faces \n",numdoubles);
}

         
Fix_orients1()
{
   int i,j,k=0;
   long window;
   INSTANCE *inst;
   BLK *blk;
   VERTEX points[4];
   VERTEX look_vec;
 
   if (!ReadViewPoints())
     return;
   /* find total number of faces ,and set Fbuf ids*/
   for (inst = Instance_list; inst != NULL; inst= inst->next){
      blk = inst->blockp;
      Total_faces += blk->numfaces;
   }
   Fbuf_id = CALLOC(Total_faces,FBUF_INFO);
   for (inst = Instance_list; inst != NULL; inst= inst->next){
      blk = inst->blockp;
      for(i=0;i<blk->numfaces;i++){
          Fbuf_id[k].face = &blk->flist[i];
          Fbuf_id[k++].inst = inst;
      }
   }
   FindNormals();


   keepaspect(1,1);
   prefposition(LEFTW,RIGHTW,BOTTOMW,TOPW);
   foreground();
   window = winopen("fix faces ");
   winset(window);
   RGBmode();
   gconfig();
   zbuffer(1);
   /* points to look at : corners of the building */ 
    points[0].x = Minx;
    points[0].y = Miny;
    points[1].x = Minx;
    points[1].y = Maxy;
    points[2].x = Maxx;
    points[2].y = Miny;
    points[3].x = Maxx;
    points[3].y = Maxy;
   /* look from viewpoints */   
   perspective(900,1,3.0,1000.0);
   for (i=0;i<Numviews;i++){
   /* for each viewpoint,pick 4 points to look at  and  render the scene */
      points[0].z =View[i].z;
      points[1].z =View[i].z;
      points[2].z =View[i].z;
      points[3].z =View[i].z;
      for (j=0;j<4;j++){
        if(!Find_vec(View[i],points[j],&look_vec)){
          printf("bad viewpoint ignored :x=%.3f,y=%.3f,z=%.3f\n",
                              View[i].x,View[i].y, View[i].z);
        }else{
           View_scene(View[i],points[j]);
           Orient(View[i]); 
        }
     }
  }
  winclose(winget());
  FixErrors();
}


View_scene(from,to)
VERTEX from,to;
{
   int window;
   VERTEX look_vec;
   char str[80];
        cpack(0xffffff);
        clear();
        zclear();
        pushmatrix();
        lookat(from.x,from.y,from.z,to.x,to.y,to.z,0); 
        Render();
        popmatrix();
}
