#include "acad2ug.h"
#include "fix2.h"

#define TRUE 1
#define FALSE 0

/* A routine for fixing face orientations, using architectural
   properties of buildings.
   Divide all faces into 4 sections; Parrallel to the XY,YZ,ZX 
   and Skew sections. Then sort each section.
   Ripple through all sections (except the skew ones) and decide
   about the orientation by finding sides of faces. 
*/

int Maxnumxys, Maxnumyzs, Maxnumzxs;
PLANE_PTR  *XYplane,*YZplane,*ZXplane;
PLANE *Skewplane;
int Num_xy=0,Num_yz =0, Num_zx=0,Skews=0;

double Wall_widths[NUMWALLWIDTHS] = {WALLWIDTHS};

Find_face_planes(blk)
BLK  *blk;
{
   FACE *face;
   int i;

   for (i=0; i<blk->numfaces;i++){
      face = &blk->flist[i];
      if (fabs(face->normal[0]) == 1. )
         face->orient = YZ;
      else if (fabs(face->normal[1]) == 1. )
         face->orient = ZX;
      else if (fabs(face->normal[2]) == 1. )
         face->orient = XY;
      else
         face->orient = SKEW;
   }
} 

Find_face_extents(face,aplane,verts)
FACE *face;
PLANE *aplane;
VERTEX *verts;
{
   int j,index;

   aplane->minx =aplane->miny = aplane->minz = BIGFLOAT;
   aplane->maxx =aplane->maxy = aplane->maxz = SMALLFLOAT;
   for(j=0;j<face->numverts;j++){
      index = face->v_id_list[j];
      if (verts[index].x < aplane->minx)
            aplane->minx = verts[index].x;
      if (verts[index].x > aplane->maxx)
            aplane->maxx = verts[index].x;
      if (verts[index].y < aplane->miny)
            aplane->miny = verts[index].y;
      if (verts[index].y > aplane->maxy)
            aplane->maxy = verts[index].y;
      if (verts[index].z < aplane->minz)
            aplane->minz = verts[index].z;
      if (verts[index].z > aplane->maxz)
            aplane->maxz = verts[index].z;
   }
}

Exchange (p,q)
PLANE *p,*q;
{ 
   PLANE temp;
   temp.value = p->value;
   temp.face = p->face;
   temp.minx = p->minx ; temp.maxx = p->maxx ;
   temp.miny = p->miny ; temp.maxy = p->maxy ;
   temp.minz = p->minz ; temp.maxz = p->maxz ;
   p->value = q->value;
   p->face = q->face;
   p->minx = q->minx ; p->maxx = q->maxx ;
   p->miny = q->miny ; p->maxy = q->maxy ;
   p->minz = q->minz ; p->maxz = q->maxz ;
   q->value = temp.value;
   q->face = temp.face;
   q->minx = temp.minx ; q->maxx = temp.maxx ;
   q->miny = temp.miny ; q->maxy = temp.maxy ;
   q->minz = temp.minz ; q->maxz = temp.maxz ;
}

Init_planes()
{
   Maxnumxys= Maxnumyzs= Maxnumzxs = 100;
   XYplane = CALLOC(100,PLANE_PTR);
   YZplane = CALLOC(100,PLANE_PTR);
   ZXplane = CALLOC(100,PLANE_PTR);
}

Expand(plane_type)
short plane_type;
{
  switch(plane_type) {
    case XY : 
       Maxnumxys += 100;
       XYplane = REALLOC(XYplane,Maxnumxys,PLANE_PTR);
       break;
    case ZX : 
       Maxnumzxs += 100;
       ZXplane = REALLOC(ZXplane,Maxnumzxs,PLANE_PTR);
       break;
    case YZ : 
       Maxnumyzs += 100;
       YZplane = REALLOC(YZplane,Maxnumyzs,PLANE_PTR);
       break;
    default : break;
 }
}

/* Divide_into_planes:
   Divide the faces in the entity block into four sections.
   Parrallel to the XY,YZ,ZX and Skew sections.
*/
Divide_into_planes(blk)
BLK *blk;
{
   FACE *face;
   VERTEX *verts;
   PLANE *aplane,*p;
   int  i,k;
 
   verts = blk->vlist; 
   
   for(i=0;i<blk->numfaces;i++){
      aplane = MALLOC(PLANE);
      aplane->face = &blk->flist[i];
      aplane->next = NULL;
      aplane->pside = NONE;
      face = &blk->flist[i];
      Find_face_extents(face,aplane,verts);
      if (aplane->face->orient == YZ){
         /* face prallel to YZ plane */
         aplane->value = aplane->minx;
         aplane->length = aplane->maxy-aplane->miny;
         /* find a list in the array for it */
         for(k=0;k<Num_yz;k++){
           if (fabs(YZplane[k]->value - aplane->value) < 0.001)
             break;
         }
         /* insert it in the right order according to its lengthin list*/
         if(k == Num_yz){
            Num_yz++;
            YZplane[k] = aplane;
         }else{
            for(p=YZplane[k];p->next!=NULL && p->length>aplane->length;p=p->next);
            if(p->next ==NULL && p->length > aplane->length){
               p->next = aplane;
            }else{
               aplane->next = p->next; 
               p->next = aplane;
               Exchange(p,aplane);
            }
         } 
         if (Num_yz == Maxnumyzs){
            fprintf(stderr,"not enough yz\n");
            Expand(YZ);
         }
      }else if (aplane->face->orient == ZX){
         /* face parallel to ZX plane */
         aplane->value = aplane->miny;
         aplane->length =aplane->maxx-aplane->minx;
         for(k=0;k<Num_zx;k++)
           if (fabs(ZXplane[k]->value - aplane->miny) < 0.001 )
             break;
         /* insert it in the right order according to length in list*/
         if(k == Num_zx){
            Num_zx++;
            ZXplane[k] = aplane;
         }else{
            for(p=ZXplane[k];p->next!=NULL && p->length > aplane->length;p=p->next);
            if(p->next ==NULL && p->length > aplane->length){
                    p->next = aplane;
            }else{
               aplane->next = p->next; 
               p->next = aplane;
               Exchange(p,aplane);
            }
         } 
         if (Num_zx == Maxnumzxs){
            fprintf(stderr,"notenoughzx");
            Expand(ZX);
          }
      }else if (aplane->face->orient == XY){
         /* face parallel to XY plane */
         aplane->value = aplane->minz;
         aplane->length =aplane->maxx-aplane->minx;
         for(k=0;k<Num_xy;k++)
           if (fabs(XYplane[k]->value - aplane->minz) < 0.001)
             break;
         /* insert it in the right order according to length in list*/
         if(k == Num_xy){
            Num_xy++;
            XYplane[k] = aplane;
         }else{
            for(p=XYplane[k];p->next!=NULL && p->length > aplane->length;p=p->next);
            if(p->next ==NULL && p->length > aplane->length){
                    p->next = aplane;
            }else{
               aplane->next = p->next; 
               p->next = aplane;
               Exchange(p,aplane);
            }
         } 
         if (Num_xy == Maxnumxys){
            fprintf(stderr,"notenoughxy");
            Expand(XY);
          }
      }else {
         aplane->next = Skewplane;
         Skewplane = aplane;
         Skews++;
      }    
   }
   fprintf(stderr,"divided into %d XY, %d YZ, %d ZX, %d Skews\n",
          Num_xy,Num_yz,Num_zx,Skews);
}


Sort_planes(pl)
PLANE_TYPE pl;
{
   
   PLANE *temp ,**plane;
   int i, PASS ; 
   int EXCHANGE = TRUE;
   switch (pl) {
      case XY:   
               PASS = Num_xy -1;
               plane = XYplane;
               break;
      case YZ:   
               PASS = Num_yz -1;
               plane = YZplane;
               break;
      case ZX:   
               PASS = Num_zx -1;
               plane = ZXplane;
               break;
   }
   do{
      EXCHANGE = FALSE;
      for(i=0;i<PASS;i++){
        if(plane[i]->value > plane[i+1]->value){
           EXCHANGE = TRUE;
           temp = plane[i];
           plane[i]= plane[i+1];
           plane[i+1] = temp;
        }
      }
   }while(EXCHANGE);
           
}

/* Look in the plane array and try to find a match for "plane"
   which is in the ith plane. Find a match that is as close as
   possible to this plane. Preference is for a match of the 
   same size, then a difference equal to the distance between
   match and plane (one wall corner), then a difference equal to 
   twice that distance (two wall corners), then a  match of larger 
   size and finally any overlapping match 
*/

Overlap(min1,max1,min2,max2)
double min1, max1, min2,max2;
{
   if(min2 >= min1 && min2 < max1)
     return 1;
   if(min1 >= min2 && min1 < max2)
     return 1;
   return 0;
}

PLANE
*Find_match(reverse,plane,i,plane_type,match_type)
short reverse;
PLANE *plane;
int i;
short plane_type;
MATCH_TYPE *match_type;
{
   int j,begin,end,step;
   PLANE *match,*candidate=NULL;
   double dist;
  
   *match_type = NOTYPE;
   switch(plane_type){
     case XY:
            if (reverse){
              begin = i+1; end = Num_xy-1; step = 1;
            }else{
              begin = i-1; end = -1; step = -1;
            }
            for(j=begin ;j != end; j += step){
              dist = fabs (XYplane[i]->value - XYplane[j]->value );
              for(match=XYplane[j];match != NULL; match=match->next){
                 if(!Overlap(plane->minx,plane->maxx,match->minx,match->maxx) ||
                    !Overlap(plane->miny,plane->maxy,match->miny,match->maxy))
                   continue;
                 if(fabs(match->miny - plane->miny) < 0.001 &&
                    fabs(match->maxy - plane->maxy) < 0.001 && 
                    fabs(match->minx - plane->minx) < 0.001 && 
                    fabs(match->maxx - plane->maxx) < 0.001) {
                    *match_type = TYPE1; /* exactly same */
                    return (match);
                 }else if(fabs(match->miny - plane->miny) < 0.001 ||
                    fabs(match->maxy - plane->maxy) < 0.001 || 
                    fabs(match->minx - plane->minx) < 0.001 || 
                    fabs(match->maxx - plane->maxx) < 0.001){
                    *match_type = TYPE2; /* one side same */
                    candidate = match;
                 }else if((match->miny >= plane->miny &&
                           match->maxy <= plane->maxy && 
                           match->minx >= plane->minx && 
                           match->maxx <= plane->maxx) ||
                          (match->miny <= plane->miny &&
                           match->maxy >= plane->maxy && 
                           match->minx <= plane->minx && 
                           match->maxx >= plane->maxx) ){
                    if (*match_type != TYPE2) {
                       *match_type = TYPE3; /* totally inside */
                       candidate = match;
                    }
                 }else if(!candidate || match->length > candidate->length){
                       *match_type = TYPE4;
                       candidate = match;
                 }
	      }
              if(candidate != NULL)
                  break;
            }
            break;
     case YZ:
            if (reverse){
              begin = i+1; end = Num_yz-1; step = 1;
            }else{
              begin = i-1; end = -1; step = -1;
            }
            for(j=begin ;j != end; j += step){
              dist = fabs (YZplane[i]->value - YZplane[j]->value );
              for(match=YZplane[j];match != NULL; match=match->next){
                 if(!Overlap(plane->minz,plane->maxz,match->minz,match->maxz) ||
                    !Overlap(plane->miny,plane->maxy,match->miny,match->maxy))
                    continue;
                 if((fabs(match->miny - plane->miny) < 0.001) &&
                    (fabs(match->maxy - plane->maxy) >= dist - 0.001) &&
                    (fabs(match->maxy - plane->maxy) <= dist + 0.001) ){
                    *match_type = TYPE1;
                    return (match);
                 }else if((fabs(match->maxy - plane->maxy) < 0.001) &&
                    (fabs(match->miny - plane->miny) >= dist - 0.001) &&
                    (fabs(match->miny - plane->miny) <= dist + 0.001) ){
                    *match_type = TYPE1;
                    return (match);
                 }else if ((fabs(match->miny - plane->miny) >= dist - 0.001) &&
                           (fabs(match->miny - plane->miny) <= dist + 0.001) &&
                           (fabs(match->maxy - plane->maxy) >= dist - 0.001) &&
                           (fabs(match->maxy - plane->maxy) <= dist + 0.001)){
                    *match_type = TYPE1;
                    return (match);
                 }else if (fabs(match->miny - plane->miny) < 0.001 &&
                           fabs(match->maxy - plane->maxy) < 0.001) {
                    *match_type = TYPE1;
                    return(match);
                 }else if (fabs(match->miny - plane->miny) < 0.001 ||
                           fabs(match->maxy - plane->maxy) < 0.001) {
                    *match_type = TYPE2;
                    candidate = match;
                 }else if( (match->miny >= plane->miny &&
                            match->maxy <= plane->maxy) || 
                           (plane->miny >= match->miny &&
                            plane->maxy <= match->maxy)) {
                    if (*match_type != TYPE2) {
                       *match_type = TYPE3; /* totally inside */
                       candidate = match;
                    }
                 }else if(!candidate){
                     candidate = match;
                     *match_type = TYPE4; 
                 }
	      }
              if(candidate != NULL)
                  break;
            }
            break;
     case ZX:
            if (reverse){
              begin = i+1; end = Num_zx-1; step = 1;
            }else{
              begin = i-1; end = -1; step = -1;
            }
            for(j=begin ;j != end; j += step){
              dist = fabs(ZXplane[i]->value - ZXplane[j]->value );
              for(match=ZXplane[j];match != NULL; match=match->next){
                 if(!Overlap(plane->minz,plane->maxz,match->minz,match->maxz) ||
                    !Overlap(plane->minx,plane->maxx,match->minx,match->maxx))
                    continue;
                 if(fabs(match->minx - plane->minx) <= 0.001 &&
                    fabs(match->maxx - plane->maxx) >= dist - 0.001 &&
                    fabs(match->maxx - plane->maxx) <= dist + 0.001){
                    *match_type = TYPE1;
                    return (match);
                 }else if(fabs(match->maxx - plane->maxx) < 0.001 &&
                    fabs(match->minx - plane->minx) >= dist - 0.001 &&
                    fabs(match->minx - plane->minx) <= dist + 0.001){
                    *match_type = TYPE1;
                    return (match);
                 }else if (fabs(match->minx - plane->minx) >= dist - 0.001 &&
                           fabs(match->minx - plane->minx) <= dist + 0.001 &&
                           fabs(match->maxx - plane->maxx) >= dist - 0.001 &&
                           fabs(match->maxx - plane->maxx) <= dist + 0.001){
                    *match_type = TYPE1;
                    return (match);
                 }else if (fabs(match->minx - plane->minx) < 0.001 &&
                           fabs(match->maxx - plane->maxx) < 0.001) {
                    *match_type = TYPE1;
                    return(match);
                 }else if (fabs(match->minx - plane->minx) < 0.001 ||
                           fabs(match->maxx - plane->maxx) < 0.001) {
                    *match_type = TYPE2;
                    candidate = match;
                 }else if( (match->minx >= plane->minx &&
                            match->maxx <= plane->maxx) || 
                           (plane->minx >= match->minx &&
                            plane->maxx <= match->maxx)) {
                    if (*match_type != TYPE2) {
                       *match_type = TYPE3; /* totally inside */
                       candidate = match;
                    }
                 }else if(!candidate){
                     candidate = match;
                     *match_type = TYPE4; 
                 }
	         
	      }
              if(candidate != NULL)
                  break;
            }
            break;
     default: break;

  }
  return(candidate);
}


Is_a_wall(w1,w2)
PLANE *w1,*w2;
{
   int i;
   double width;
   width = fabs(w1->value - w2->value);
/*   for(i=0;i<NUMWALLWIDTHS;i++)
     if(width == Wall_widths[i])
        return(TRUE); 
*/
   if (width >= MINWALLWIDTH || width <= MAXWALLWIDTH)
     return(TRUE);
   return(FALSE);
}


Findstring(str1,str2)
char *str1, *str2;
{
   int length1, length2,i,j,num;
   length1 = strlen(str1);
   length2 = strlen(str2);

   for(i=0;i<= (length2 -length1);i++){
      num =0;
      for(j=0;j<length1;j++){
        if(str2[i+j] == str1[j])
          num++; 
      }
      if(num == length1)
        return(TRUE);
   }
   return(FALSE);
}

 
Init_orient_XYs()
{
   char string[50];
   int i;
   PLANE *plane;
   int num;
   for(i=0;i < Num_xy;i++){
      num=0;
      for(plane = XYplane[i];plane != NULL;plane = plane->next){
        num++;
        if(Findstring(CEILINGSTR,Layertable[plane->face->layerindex].name) ){
             plane->pside = DOWN;
        }else if(Findstring(FLOORSTR,Layertable[plane->face->layerindex].name)){
             plane->pside = UP;
        }
      }
   }
}

orient_cl_fl(maxwidth,minwidth,type,samelayer,large)
double maxwidth,minwidth;
MATCH_TYPE type;
short samelayer;
short large;
{
   int i,j;
   PLANE *plane,*match1, *match2, *match; 
   short side;
   MATCH_TYPE match_type;
  

   /* take each XYpolygon , find the first match for it in the previous 
      layer and decide its orientation */
   for(i=1;i < Num_xy-1 ;i++)
      for(plane = XYplane[i];plane != NULL;plane = plane->next){
        match1 = Find_match(0,plane,i,XY,&match_type);
        match2 = Find_match(0,plane,i,XY,&match_type);
        if(match1 == NULL && match2 == NULL)  continue;
        if(match1 == NULL)
            match = match2;
        else if (match2 == NULL)
            match = match1;
        else if (fabs(match2->value - plane->value) < 
                 fabs(match1->value - plane->value))
            match = match2;
        else 
            match = match1;
        if (type != match_type) continue;
        if(samelayer && (plane->face->layerindex != match->face->layerindex))
           continue;
        if(fabs(plane->value - match->value) > maxwidth ||
              fabs(plane->value - match->value) <= minwidth)
           continue;
        if(large && (plane->maxy - plane->miny < MINROOMWIDTH  ||
                     match->maxy - match->miny < MINROOMWIDTH) )
           continue;
        if(match->pside != UP){ 
           match->pside = DOWN;
           plane->pside = UP;
           continue;
        }
       
      }
}


init_boundarywalls()
{
   int i,j;
   PLANE *plane;
   /* Initialize the boundary planes */ 
   for(plane = YZplane[0];plane != NULL;plane = plane->next)
             plane->pside = WEST;
   for(plane = YZplane[Num_yz -1];plane != NULL;plane = plane->next)
            plane->pside = EAST;
   for(plane = ZXplane[0];plane != NULL;plane = plane->next)
              plane->pside = SOUTH;
   for(plane = ZXplane[Num_zx -1];plane != NULL;plane = plane->next)
             plane->pside = NORTH;
}

orient_wall(maxwidth,minwidth,type,samelayer,large)
double maxwidth,minwidth;
MATCH_TYPE type;
short samelayer;
short large;
{
   int i,j;
   PLANE *plane,*match,*match1,*match2; 
   short side;
   MATCH_TYPE match_type;
  

   /* take each YZpolygon , find the first match for it in the previous 
      layer and decide its orientation */
   for(i=1;i < Num_yz-1 ;i++)
      for(plane = YZplane[i];plane != NULL;plane = plane->next){
        match1 = Find_match(0,plane,i,YZ,&match_type);
        match2 = Find_match(0,plane,i,YZ,&match_type);
        if(match1 == NULL && match2 == NULL)  continue;
        if(match1 == NULL)
            match = match2;
        else if (match2 == NULL)
            match = match1;
        else if (fabs(match2->value - plane->value) < 
                 fabs(match1->value - plane->value))
            match = match2;
        else 
            match = match1;
        if (type != match_type) continue;
        if(samelayer && (plane->face->layerindex != match->face->layerindex))
           continue;
        if(fabs(plane->value - match->value) > maxwidth ||
              fabs(plane->value - match->value) <= minwidth)
           continue;
        if(large && (plane->maxy - plane->miny < MINROOMWIDTH  ||
                     match->maxy - match->miny < MINROOMWIDTH) )
           continue;
           /* If the distance between this polygon and its match is 
     	     close to minimum wall width then this polygon is
             a side of wall and should  point EAST 
	   */
        if(match->pside != EAST){ 
           match->pside = WEST;
           plane->pside = EAST;
           continue;
        }
       
      }


   /* take each ZXpolygon , find the first match for it in the previous 
      layer and decide its orientation */
   for(i=1;i < Num_zx-1;i++)
      for(plane = ZXplane[i];plane != NULL;plane = plane->next){
        match1 = Find_match(0,plane,i,ZX,&match_type);
        match2 = Find_match(0,plane,i,ZX,&match_type);
        if(match1 == NULL && match2 == NULL)  continue;
        if(match1 == NULL)
            match = match2;
        else if (match2 == NULL)
            match = match1;
        else if (fabs(match2->value - plane->value) < 
                 fabs(match1->value - plane->value))
            match = match2;
        else 
            match = match1;
        if (type != match_type) continue;
        if(samelayer && (plane->face->layerindex != match->face->layerindex))
           continue;
        if(fabs(plane->value - match->value) > maxwidth ||
              fabs(plane->value - match->value) <= minwidth)
           continue;
        if(large &&  (plane->maxx - plane->minx < MINROOMWIDTH  ||
                      match->maxx - match->minx < MINROOMWIDTH) )
           continue;
           /* If the distance between this polygon and its match is 
             close to minimum wall width then this polygon is
             a side of wall and should  point right */
        if(match->pside  != NORTH){
           match->pside = SOUTH;
           plane->pside = NORTH;
           continue;
        }
     }
}


contradict(m1,m2,s)
PLANE *m1,*m2;
short *s;
{
     PLANE *m;
     if(m1 == NULL || m2 == NULL) 
        return(TRUE);
     if(m1->pside == NONE || m2->pside== NONE) 
        return(TRUE);
     if(m1->pside != m2->pside)
        return(TRUE);
     switch(m1->pside){
        case WEST: *s = EAST; break;
        case EAST: *s = WEST; break;
        case NORTH: *s = SOUTH; break;
        case SOUTH: *s = NORTH; break;
        case UP: *s = DOWN; break;
        case DOWN: *s = UP; break;
      }
      return(FALSE);
}

Check_bothsides()
{
   int i,j;
   PLANE *plane,*match1,*match2; 
   short side;
   MATCH_TYPE match_type;
   /* go through all planes and if there is a polygon without
      orientation, find matches on two sides  of it; if these 
      matches have the same orientation  
      then set the orientation  of the face to be the reverse */
   for(i=1;i < Num_yz-1 ;i++)
      for(plane = YZplane[i];plane != NULL;plane = plane->next){
        if(plane->pside != NONE) continue;
        match1 = Find_match(0,plane,i,YZ,&match_type);
        match2 = Find_match(1,plane,i,YZ,&match_type);
        if(!contradict(match1,match2,&side)){
             plane->pside = side;
        }
      }

   for(i=1;i < Num_zx - 1;i++)
      for(plane = ZXplane[i];plane != NULL;plane = plane->next){
        if(plane->pside != NONE)
             continue;
        match1 = Find_match(0,plane,i,ZX,&match_type);
        match2 = Find_match(1,plane,i,ZX,&match_type);
        if(!contradict(match1,match2,&side)){
             plane->pside = side;
        }
      }
   for(i=1;i < Num_xy - 1;i++)
      for(plane = XYplane[i];plane != NULL;plane = plane->next){
        if(plane->pside != NONE)
             continue;
        match1 = Find_match(0,plane,i,XY,&match_type);
        match2 = Find_match(1,plane,i,XY,&match_type);
        if(!contradict(match1,match2,&side)){
             plane->pside = side;
        }
      }
}

Fix_face(face,side)
FACE *face;
short side;
{
    switch (side) {
    case DOWN:
       if(face->normal[2] > 0.){
          face->check = FLIP;
       }else if (face->normal[2] < 0.){
          face->check = OK; 
       }
       break;
    case UP :
       if(face->normal[2] < 0.){
          face->check = FLIP;
       }else if (face->normal[2] > 0.){
          face->check = OK; 
       }
       break;
    case WEST:
       if(face->normal[0] > 0.){
          face->check = FLIP;
       }else if (face->normal[0] < 0.){
          face->check = OK;
       }
       break;
    case EAST :
       if(face->normal[0] < 0.){
          face->check = FLIP;
       }else if (face->normal[0] > 0.){
          face->check = OK; 
       }
       break;
    case SOUTH:
       if(face->normal[1] > 0.){
          face->check = FLIP;
       }else if (face->normal[1] < 0.){
          face->check = OK; 
       }
       break;
    case NORTH :
       if(face->normal[1] < 0.){
          face->check = FLIP; 
       }else if (face->normal[1] > 0.){
          face->check = OK;
       }
       break;
       default :  break;
   }
} 


Fix_face_orients()
{
   int i,j;
   PLANE *plane; 
   FACE *face;

   for(i=0;i<Num_xy;i++)
      for(plane = XYplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          Fix_face(face,plane->pside);
      } 
   for(i=0;i<Num_zx;i++)
      for(plane= ZXplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          Fix_face(face,plane->pside);
      } 
   for(i=0;i<Num_yz;i++)
      for(plane = YZplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          Fix_face(face,plane->pside); 
      }
}

Apply_to_planes()
{
   int i,j;
   PLANE *plane; 
   FACE *face;

   for(i=0;i<Num_xy;i++)
      for(plane = XYplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          if(plane->pside == NONE && face->check != NOTCHECKED){
               switch (face->check) {
                  case OK : 
                        if (face->normal[2] > 0.)
                         plane->pside = UP;
                        else if (face->normal[2] < 0.) 
                         plane->pside = DOWN;
                  case FLIP : 
                        if (face->normal[2] > 0.)
                         plane->pside = DOWN;
                        else if (face->normal[2] < 0.) 
                         plane->pside = UP;
               }
          } 
      } 
   for(i=0;i<Num_zx;i++)
      for(plane= ZXplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          if(plane->pside == NONE && face->check != NOTCHECKED){
               switch (face->check) {
                  case OK : 
                        if (face->normal[1] > 0.)
                         plane->pside = NORTH;
                        else if (face->normal[1] < 0.) 
                         plane->pside = SOUTH;
                  case FLIP : 
                        if (face->normal[1] > 0.)
                         plane->pside = SOUTH ;
                        else if (face->normal[1] < 0.) 
                         plane->pside = NORTH;
               }
          } 
      } 
   for(i=0;i<Num_yz;i++)
      for(plane = YZplane[i];plane != NULL;plane = plane->next){
          face = plane->face;
          if(plane->pside == NONE && face->check != NOTCHECKED){
               switch (face->check) {
                  case OK : 
                        if (face->normal[0] > 0.)
                         plane->pside = EAST;
                        else if (face->normal[0] < 0.) 
                         plane->pside = WEST ;
                  case FLIP : 
                        if (face->normal[0] > 0.)
                         plane->pside = WEST;
                        else if (face->normal[0] < 0.) 
                         plane->pside = EAST;
               }
          } 
      }
}


Fix_orients2()
{
  BLK *blk;
  int num,i;
  PLANE *plane; 
  double wallwidth;
  double floorwidth;

  blk = Last_block;
  Init_planes();
  Find_face_planes(blk);
  Divide_into_planes(blk);
  Sort_planes(XY);
  Sort_planes(YZ);
  Sort_planes(ZX);

  fprintf(stderr,"PROCESSING XY planes\n");
  if (WantFix3)
     Init_orient_XYs();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE1,0,1);
     fprintf(stderr,"         ,large,type1,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE2,0,1);
     fprintf(stderr,"         ,large,type2,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE3,0,1);
     fprintf(stderr,"         ,large,type3,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE4,0,1);
     fprintf(stderr,"         ,large,type4,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE1,0,0);
     fprintf(stderr,"         ,     ,type1,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE2,0,0);
     fprintf(stderr,"         ,     ,type2,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE3,0,0);
     fprintf(stderr,"         ,     ,type3,%.3f: ",MINROOMHEIGHT);
     report();
  orient_cl_fl(MAXROOMHEIGHT,MINROOMHEIGHT,TYPE4,0,0);
     fprintf(stderr,"         ,     ,type4,%.3f: ",MINROOMHEIGHT);
     report();
 
  fprintf(stderr,"PROCESSING YZ and ZX planes\n");
  init_boundarywalls(); 
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth -4.,TYPE1,1,1);
     orient_wall(wallwidth,wallwidth -4.,TYPE1,1,1);
  }
  fprintf(stderr,"samelayer,large,type1,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth -4.,TYPE1,0,1);
     orient_wall(wallwidth,wallwidth -4.,TYPE1,0,1);
  }
  fprintf(stderr,"         ,large,type1,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE2,1,1);
     orient_wall(wallwidth,wallwidth -4.,TYPE2,1,1);
  }
  fprintf(stderr,"samelayer,large,type2,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE2,0,1);
     orient_wall(wallwidth,wallwidth -4.,TYPE2,0,1);
  }
  fprintf(stderr,"         ,large,type2,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE1,1,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE1,1,0);
  }
  fprintf(stderr,"samelayer,     ,type1,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE1,0,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE1,0,0);
  }
  fprintf(stderr,"         ,     ,type1,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE2,1,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE2,1,0);
  }
  fprintf(stderr,"samelayer,     ,type2,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE2,0,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE2,0,0);
  }
  fprintf(stderr,"         ,     ,type2,%.3f: ",wallwidth);
  report();
  
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE3,1,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE3,1,0);
  }
  fprintf(stderr,"samelayer,     ,type3,%.3f: ",wallwidth);
  report();
  for(wallwidth = MINWALLWIDTH,floorwidth = MINFLOORWIDTH; 
      wallwidth <= MAXWALLWIDTH ; wallwidth += 4,floorwidth += 4){ 
     if (floorwidth < MAXFLOORWIDTH) 
        orient_cl_fl(floorwidth,floorwidth - 4.,TYPE3,0,0);
     orient_wall(wallwidth,wallwidth -4.,TYPE3,0,0);
  }
  fprintf(stderr,"         ,     ,type3,%.3f: ",wallwidth);
  report();
  
  Check_bothsides();
  fprintf(stderr,"nocontradiction                 : ");
  report();
  Fix_face_orients();
  Propagate_orientation(blk);  
  Apply_to_planes();
  fprintf(stderr,"propagation                     : ");
  report();
  
  Check_bothsides();
  fprintf(stderr,"nocontradiction                 : ");
  report();
  Mark_consistentplanes();
  fprintf(stderr,"consistent planes               : ");
  report();
  Fix_face_orients();
 /* mark();*/
  
}

report()
{
   int i,num,total;
   PLANE *plane; 
  
   num =0; total =0;
   for(i=0;i<Num_xy ;i++)
      for(plane=XYplane[i];plane != NULL; plane = plane->next){
         total++;
         if(plane->pside != NONE )
            num++;
   }
   fprintf(stderr,"XY= %.3f%% ",(double)(num * 100.)/(double)total);
   num =0; total =0;
   for(i=0;i<Num_yz ;i++)
      for(plane=YZplane[i];plane != NULL; plane = plane->next){
         total++;
         if(plane->pside != NONE )
            num++;
   }
   fprintf(stderr,"YZ= %.3f%% ",(double)(num * 100.)/(double)total);
   num =0; total =0;
   for(i=0;i<Num_zx ;i++)
      for(plane=ZXplane[i];plane != NULL; plane = plane->next){
         total++;
         if(plane->pside != NONE )
            num++;
   }
   fprintf(stderr,"ZX= %.3f%%\n",(double)(num * 100.)/(double)total);
}


mark()
{
   int i,num,total;
   PLANE *plane; 
  
   for(i=0;i<Num_xy ;i++)
      for(plane=XYplane[i];plane != NULL; plane = plane->next){
         if(plane->pside != NONE )
            plane->face->layerindex = 0;
   }
   for(i=0;i<Num_yz ;i++)
      for(plane=YZplane[i];plane != NULL; plane = plane->next){
         if(plane->pside != NONE )
            plane->face->layerindex = 0;
   }
   for(i=0;i<Num_zx ;i++)
      for(plane=ZXplane[i];plane != NULL; plane = plane->next){
         total++;
         if(plane->pside != NONE )
            plane->face->layerindex = 0;
   }
}

/* As a last step, go through each of the plane layers and look 
   at the orientation of all polygons in the plane. If all point 
   in the same direction then mark the unmarked ones to this direction.*/
Mark_consistentplanes()
{  
   int i,j,x=0,y=0,z=0;
   PLANE *plane; 
   char c;
   int ups,downs,nones,easts,wests,souths,norths;

   for(i=0;i<Num_xy ;i++){
      nones = ups = downs =0;
      for(plane=XYplane[i];plane != NULL; plane = plane->next){
         if(plane->pside == DOWN){
             downs++;
         } else if(plane->pside == UP){
             ups++;
         }else 
             nones++;
      }
      if (ups > 0 && downs > 0)
        break;
      else if (ups > 0){
         for(plane=XYplane[i];plane != NULL; plane = plane->next)
            plane->pside = UP;
      }else if (downs > 0){
         for(plane=XYplane[i];plane != NULL; plane = plane->next)
           plane->pside = DOWN;
      }
   } 
   for(i=0;i<Num_yz ;i++){
      nones = easts = wests =0;
      for(plane=YZplane[i];plane != NULL; plane = plane->next){
         if(plane->pside == EAST){
             easts++;
         } else if(plane->pside == WEST){
             wests++;
         }else 
             nones++;
      }
      if (easts > 0 && wests > 0){
        break;
      }else if (easts > 0){
        for(plane=YZplane[i];plane != NULL; plane = plane->next)
            plane->pside = EAST;
      }else if (wests > 0){
         for(plane=YZplane[i];plane != NULL; plane = plane->next)
            plane->pside = WEST;
      }
   }
   for(i=0;i<Num_zx ;i++){
      nones = norths = souths =0;
      for(plane=ZXplane[i];plane != NULL; plane = plane->next){
         if(plane->pside == NORTH){
             norths++;
         } else if(plane->pside == SOUTH){
             souths++;
         }else 
             nones++;
      }
      if (norths > 0 && souths > 0){
        break;
      }else if (norths > 0){
         for(plane=ZXplane[i];plane != NULL; plane = plane->next)
            plane->pside = NORTH;
      }else if (souths > 0){
         for(plane=ZXplane[i];plane != NULL; plane = plane->next)
            plane->pside = SOUTH;
      }
   }
}
