#include "acad2ug.h"

#define B_EPSILON    0.03125       /* 1/16 of an inch */

EDGE  *Is_in_edgel(edgel,numedges,index1,index2,vlist)
EDGE *edgel;
int numedges;
int index1,index2;
VERTEX *vlist;
{
   EDGE *curedge;
   int i;

   for(i =0; i< numedges; i++) {
         curedge = &edgel[i];
         if((fabs(curedge->v1[0] - vlist[index1].x) < B_EPSILON) &&
            (fabs(curedge->v1[1] - vlist[index1].y) < B_EPSILON) &&
            (fabs(curedge->v1[2] - vlist[index1].z) < B_EPSILON) &&
            (fabs(curedge->v2[0] - vlist[index2].x) < B_EPSILON) &&
            (fabs(curedge->v2[1] - vlist[index2].y) < B_EPSILON) &&
            (fabs(curedge->v2[2] - vlist[index2].z) < B_EPSILON) ){
               return(curedge);
         }
   }
   return(NULL);
}


*Make_edgelist(blk)
BLK *blk;
{

   EDGE *edgel;
   int numedges =0 ;
   EDGE *edge;
   FACE *face;
   VERTEX *vlist;
   int i,j,index1,index2,numverts;
 
   edgel = CALLOC((blk->numfaces * 4), EDGE);
   vlist = blk->vlist; 
   for(i=0;i< blk->numfaces; i++){
      face = &blk->flist[i];
      numverts = face->numverts;
      for(j=0;j<numverts;j++){
         index1= face->v_id_list[j];
         index2= face->v_id_list[(j+1) % numverts];
         if( edge = Is_in_edgel(edgel,numedges,index1,index2,vlist)){
             if(edge->facecount > 2) {
                 edge->facecount = 3;
                 break;
             }
             edge->faceinfo[1].face = face;;
             edge->faceinfo[1].edge_dir = OK;
             edge->facecount++;
         }else if( edge = Is_in_edgel(edgel,numedges,index2,index1,vlist)){
             if(edge->facecount > 2) {
                 edge->facecount = 3;
                 break;
             }
             edge->faceinfo[1].face = face;;
             edge->faceinfo[1].edge_dir = FLIP;
             edge->facecount++;
         }else{
             edge = &edgel[numedges];
             numedges++;
             edge->faceinfo[0].face = face;
             edge->faceinfo[0].edge_dir = OK;
             edge->facecount = 1;
             edge->v1[0] = vlist[index1].x ;
             edge->v1[1] = vlist[index1].y ;
             edge->v1[2] = vlist[index1].z ;
             edge->v2[0] = vlist[index2].x ;
             edge->v2[1] = vlist[index2].y ;
             edge->v2[2] = vlist[index2].z ;
        }
      }
   }
   blk->elist =edgel; blk->numedges = numedges; 
}

/* returns true if absolute values of normals same */
normals_equal(n1,n2)
float n1[3],n2[3];
{
    if((fabs(fabs(n1[0]) - fabs(n2[0])) < 0.001) &&
       (fabs(fabs(n1[1]) - fabs(n2[1])) < 0.001) &&
       (fabs(fabs(n1[2]) - fabs(n2[2])) < 0.001)){
           return(1);
    }
    return(0);
}

/*
Propagate_plane(face,blk)
FACE *face;
BLK *blk;
{
   FACEINFO *faceinfo1, *faceinfo2;
   FACE *face1, *face2;
   int i;

   if(blk->numfaces <= 0)
     return;
  
   for(i=0;i<blk->numfaces;i++){
      if(face->orient != blk->flist[i].orient)
         continue;
      if(!normals_equal(face->normal,blk->flist[i].normal))
         continue;
      if( fabs(face->normal[3]) != fabs(blk->flist[i].normal[3]) )
         continue;
      if(face->normal[3] == blk->flist[i].normal[3]){
        blk->flist[i].check = face->check ;
      }else{
        if(face->check == FLIP)
           blk->flist[i].check = OK;
        else if(face->check == OK)
           blk->flist[i].check = FLIP;
      }
   }
}
*/

Propagate_orientation(blk)
BLK *blk;
{
   EDGE *edgel;
   FACEINFO faceinfo1, faceinfo2;
   FACE *face1, *face2;
   EDGE *edge;
   int pass=0, numdone=0;
   short progress;
   int numedges;
   int i;

   if (blk->elist == NULL) 
      Make_edgelist(blk);

   edgel = blk->elist;
   progress =1;
   while(progress){
      progress =0;   
      for (i=0;i<blk->numedges;i++){
         edge = &edgel[i];
         if (edge->facecount == 2){
             faceinfo1 = edge->faceinfo[0];
             faceinfo2 = edge->faceinfo[1];
             face1 = faceinfo1.face;
             face2 = faceinfo2.face;
             if(face1->check != NOTCHECKED && face2->check != NOTCHECKED)
                continue;
             if(face1->check == NOTCHECKED && face2->check == NOTCHECKED)
                continue;
             if(face2->check == NOTCHECKED && face1->check != NOTCHECKED) {
                if(faceinfo1.edge_dir == faceinfo2.edge_dir){
                progress =1;
                numdone++;
                   if(face1->check == OK)
                      face2->check = FLIP;
                   else if(face1->check == FLIP)
                      face2->check = OK;
                }else{
                progress =1;
                numdone++;
                      face2->check = face1->check;
                }
            }else if(face1->check == NOTCHECKED && face2->check != NOTCHECKED) {
                if(faceinfo2.edge_dir == faceinfo1.edge_dir){
                progress =1;
                numdone++;
                   if(face2->check == OK)
                      face1->check = FLIP;
                   else if(face2->check == FLIP)
                      face1->check = OK;
                }else{
                progress =1;
                numdone++;
                   face1->check = face2->check;
                }
             }
                    
         }
      }/*for*/
   } /*while*/
/*   Free_edgel(edgel);*/
   fprintf(stderr,"progress in propagation = %d faces\n",numdone);
     
}
      
/* 
Free_edgel(el)
EDGE *el;
{
   EDGE *e, *nexte;
   FACEP *fp, *nextfp;

   e=el;
   while (e != NULL){
      nexte= e->next;
      fp = e->faceinfo;
      while(fp != NULL){
         nextfp = fp->next;
         free(fp);
         fp = nextfp;
      }
      free(e);
      e = nexte;
   }
   el = NULL;
}
*/
