#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <malloc.h>
#include <ctype.h>
#include <math.h>

#define BIGFLOAT   99999999.0
#define SMALLFLOAT   -99999999.0
#define EPSILON  0.000001   
#define DEFAULTTHICK  1.0   
#define PI 3.14159265358979323846  
#define MINLAYERS 50  
#define MAXPOLYVERT  400  
#define MAXFACEVERT  30  
#define MAXWIREVERT  100  
#define MINFACEINBLK     50 
#define MINVERTINBLK     100 
#define MINFACEINENT     300
#define MINVERTINENT     1000 
#define MINWIRES         100 
#define MAXVIEWPOINTS    50 
#define ARC_PTS     28 
#define  CLOSED  0X1
#define  CURVEFITTED  0X2
#define  SPLINEFITTED  0X4
#define   POLY3D  0X8
#define   POLY3DMESH  0X10
#define   POLY3DCLOSEDN  0X20
#define   CONTROLVERTEX 0X10
#define CALLOC(n,t) (t *)calloc((unsigned)(n),sizeof(t))
#define REALLOC(p,n,t)  (t *)realloc((char *)p,(unsigned)(n * sizeof(t)))
#define MALLOC(t) (t *)malloc(sizeof(t))

#define   NOTHING 0
#define   WANTED 1
#define   UNWANTED -1

#define   ALL 0x0
#define   XY  0x1
#define   YZ  0x2
#define   ZX  0x4
#define   SKEW  0x8

typedef struct {
   char name[50];
   char ugname[50];
   int  colorindex; 
}LAYER;

typedef struct {
  double x,y,z;
}VERTEX;

typedef struct {
  double x,y,z,w;
}VERTEX4;

typedef enum {NOTCHECKED,OK,FLIP,DOUBLE} CHECK;

typedef struct aface {
  int layerindex;
  double normal[4];
  int lineno ; 
  int   v_id_list[MAXFACEVERT];
  int numverts;
  int flips ;   /* the next five fields are used for fixing face orientations*/ 
  int oks ; 
  CHECK check;
  short orient;
  short error;
}FACE;

typedef struct awire {
  int layerindex;
  int lineno ; 
  int   v_id_list[MAXWIREVERT];
  int numverts;
}WIRE;

typedef struct facep {
   FACE *face;
   CHECK edge_dir;
}FACEINFO;

typedef struct stmt {
  char*name;
  struct ablock *blk;
  int layerindex;
  char transforms[256];
  struct stmt  *next;
}INST_STMT;

typedef struct edge { /* edge list used for orientation propagation */
   int facecount;
   short type;
   double v1[3], v2[3];
   FACEINFO faceinfo[2];
}EDGE;

typedef struct ablock {
  char name[50];
  VERTEX base;
  int numverts;
  int numfaces;
  int numwires;
  int numedges;
  int numinstances; /* number of instances of this block */
  VERTEX *vlist;    /* pointer to vertex array */
  WIRE  *wlist;     /* pointer to wire   array */
  FACE  *flist;     /* pointer to face   array */
  EDGE  *elist;     /* pointer to edge  array */
  INST_STMT  *inst_stmt;     /* pointer to instance statements */
  struct ablock *next;
}BLK;

typedef struct instance{ /* used for scene flattening */
  char name[50];
  BLK *blockp;
  VERTEX *vlist;   /* pointer to vertex array */
  short mirror;   /* indicates if this instance is an mirror image; if so the 
                    order of vertices should be reversed */
  struct instance  *next;
}INSTANCE;

extern FILE *Errorfile, *Dxffile;
extern char Ugfilename[], Mainfilename[], Planfilename[], Viewfilename[];
extern char Prefix[];
extern short WantFix1, WantFix2, WantFix3, Want2D, WantPlanv,WantPrefix;
extern int DXFgroupcode ;
extern short Nolayerfile, Nocolorfile;
extern char DXFgroupvalue[256];
extern int Lastv , Lastw, Lastf ;
extern int Lineno, CurLineno;
extern LAYER *Layertable;
extern float Colortable[256][4];
extern int  Numlayers ;
extern short Inablock , InEntities ;
extern double Minx,Maxx,Miny,Maxy,Minz,Maxz;
extern VERTEX Ecs;
extern double Ecs_matrix[4][4];
extern BLK *First_block, *Last_block, *Cur_block;
extern INSTANCE *Instance_list;
extern BLK *Findblk();
extern INST_STMT *MakeInstanceStmt();
extern BLK  *MakeBlock();
extern short Debugging;
extern double Dot();
extern double Dot4();
