/* input.c : routines to get user arguments and read DXF file */
#include "acad2ug.h"

#define TRUE 1
#define FALSE 0

/* files and filenames used for output and  input */
FILE *Dxffile,*Ugfile,*Layerfile,*Colorfile, *Errorfile;
char Dxffilename[50], Ugfilename[50],Layerfilename[50],Colorfilename[50],
     Viewfilename[50],Errorfilename[50],Planfilename[50], Mainfilename[50];

Print_help()
{
  fprintf(stderr,"-2        : process two-dimensional entities;\n");
  fprintf(stderr,"-c <file> : use color table in <file>;\n");
  fprintf(stderr,"            <File> format is <colorindex r g b>;\n");
  fprintf(stderr,"            Default:  random colors will be assigned to layers.\n");
  fprintf(stderr,"-d        : include DXF line numbers as comments in ug file.\n");
  fprintf(stderr,"-f1 <file>: Fix orientation using zbuffer method;\n");
  fprintf(stderr,"            <File> is the file containing viewpoints.\n");
  fprintf(stderr,"-f2       : Fix orientation using wall thicknesses;\n");
  fprintf(stderr,"-f3       : Fix orientation using wall thicknesses and make layers\n");
  fprintf(stderr,"            with string \"CL\" in them point down and \"FL\" point up.\n");
  fprintf(stderr,"-l <file> : Use only layers in <file> that have flag Y;\n");
  fprintf(stderr,"            <File> format is <layername Y/N colorindex>;\n");
  fprintf(stderr,"            Default:  all layers will be converted.\n");
  fprintf(stderr,"-p        : Make plan view; output is a binary file with extension .planv\n");
  fprintf(stderr,"            containing color values for a 100x100 pixel image of a plan view.\n");
  fprintf(stderr,"-r <prefix>: Optional prefix to be used with color names\n");
  fprintf(stderr,"example   : acad2ug -l layerfile  floor1\n");
  fprintf(stderr,"            will convert floor1.dxf to floor1.ug; only layers\n");
  fprintf(stderr,"            indicated in layerfile with flag Y will be converted;\n");
  fprintf(stderr,"            file floor1.errors contains detected errors.\n ");
}

Process_args(argc,argv)
int argc;
char *argv[];
{
  int i;
   if (argc < 2){ 
     fprintf(stderr,"usage : acad2ug <flags> <Dxffile>\n");
     fprintf(stderr,"use acad2ug -h for information \n");
      exit(1);
   }
   
   for(i=1;i<argc;i++){
         if(argv[i][0] == '-' ){
            if(argv[i][1] == 'h'){
               Print_help();
               exit(1);
            }
            else if(argv[i][1] == '2')
                Want2D = TRUE;
            else if(argv[i][1] == 'd')
                Debugging = TRUE;
            else if(argv[i][1] == 'f' && argv[i][2] == '1'){
                WantFix1 = TRUE;
                strcpy(Viewfilename,argv[++i]);
            }else if(argv[i][1] == 'f' && argv[i][2] == '2')
                WantFix2 = TRUE;
            else if(argv[i][1] == 'f' && argv[i][2] == '3')
                WantFix3 = TRUE;
            else if(argv[i][1] == 'p'){
                WantPlanv = TRUE;
                Want2D = TRUE;
            }else if(argv[i][1] == 'l'){
               sprintf(Layerfilename,"%s",argv[++i]);
               Nolayerfile = FALSE;
            }else if(argv[i][1] == 'c'){
               sprintf(Colorfilename,"%s",argv[++i]);
                Nocolorfile = FALSE;
            }else if(argv[i][1] == 'r'){
               sprintf(Prefix,"%s",argv[++i]);
               WantPrefix = TRUE;
            }else
               fprintf(stderr,"Unknown argument ignored : %s\n",argv[i]);
         }
   }
   /* expect file name to be the last argument */ 
   if(argv[argc-1][0] == '-'){
    fprintf(stderr,"last argument should be the file name\n");
     exit(1);
   }
   sprintf(Mainfilename,"%s",argv[argc-1]);
   if (WantPlanv) /* do not need orientation fix for plan view */
       WantFix1=WantFix2= FALSE;
}

Setup_files(){
   char answer;
   int i,j,index;
   char line[256];

   sprintf(Dxffilename,"%s.dxf",Mainfilename);
   if((Dxffile = fopen(Dxffilename,"r")) == NULL){
     fprintf(stderr,"can't open file %s\n",Dxffilename);
      exit(1);
   }
   if(WantPlanv){
     sprintf(Planfilename,"%s.planv",Mainfilename);
     sprintf(Ugfilename,"%s.planv.ug",Mainfilename);
   }else 
     sprintf(Ugfilename,"%s.ug",Mainfilename);
   
   sprintf(Errorfilename,"%s.errors",Mainfilename);
   if((Errorfile = fopen(Errorfilename,"w")) == NULL){
     fprintf(stderr,"can't open file %s\n",Errorfilename);
      exit(1);
   }
   /* open layer file */ 
   Layertable = CALLOC(MINLAYERS,LAYER);
   if(!Nolayerfile){
     fprintf(stderr,"Using file %s for layers\n",Layerfilename);
      if((Layerfile = fopen(Layerfilename,"r")) == NULL){
         fprintf(stderr,"can't open file %s\n",Layerfilename);
           exit(1);
      }
      while(fscanf(Layerfile,"%s  %c %d \n",Layertable[Numlayers].name,&answer,
             &Layertable[Numlayers].colorindex)!= EOF){
          if (answer == 'Y' || answer == 'y') {
             Numlayers++;
             if (Numlayers % MINLAYERS == 0){
                Layertable = REALLOC(Layertable,(Numlayers + MINLAYERS),LAYER);
                if (!Layertable){
                  fprintf(stderr,"Unable to allocate space for layer table\n");
                   exit(0);
                }
             }
          }
      }
      fclose(Layerfile);   
   }
   for(i=0;i<256;i++)
      for(j=0;j<4;j++)
         Colortable[i][j] = 0.;
      
   /* if there is a color file read it;otherwise generate a random color table*/
   if (Nocolorfile) {
      for(i=0;i<256;i++){
            Colortable[i][0] = fabs ((( 2 * random()) % 255) / 255.);
            Colortable[i][1] = fabs ((( 2 * random() ) % 255) / 255.);
            Colortable[i][2] = fabs ((( 2 * random() ) % 255) / 255.);
            Colortable[i][3] = 0.; 
      }
   }else {
      if((Colorfile = fopen(Colorfilename,"r")) == NULL){
         fprintf(stderr,"can't open file %s\n",Colorfilename);
          exit(1);
       }
       while(fgets(line,256,Colorfile) != NULL){
          sscanf(line,"%d",&index);
          if (index < 256)
            sscanf(line,"%d %g %g %g",&index,&Colortable[index][0],
                    &Colortable[index][1],&Colortable[index][2]);
       }
    }
}

/*  GetTwoLines():
 *  Reads two lines (group code and a string) from the dxf file
 */
int GetTwoLines()
{ 
   if (fgets(DXFgroupvalue,256,Dxffile) == NULL)
       error();
   CurLineno++;
   DXFgroupvalue[strlen(DXFgroupvalue) - 1] = '\0';
   DXFgroupcode = atoi(DXFgroupvalue);
   if(fgets(DXFgroupvalue,256,Dxffile) == NULL)
      error();
   CurLineno++;
   DXFgroupvalue[strlen(DXFgroupvalue) - 1] = '\0';
}

Skip(type)
char type[];
{
   if(!strcmp(type,"BLOCK"))
   /* skip to end of block*/
      while(1){
        GetTwoLines();
        if (!strcmp(DXFgroupvalue,"ENDBLK"))
           return;
      }
   else if (!strcmp(type,"VERTEX") || !strcmp(type,"POLYLINE") )
      while(1){
        GetTwoLines();
        if (!strcmp(DXFgroupvalue,"SEQEND"))
           return;
      }
   else if (!strcmp(type,"SECTION"))
      while(1){
        GetTwoLines();
        if (!strcmp(DXFgroupvalue,"ENDSEC"))
           return;
      }
   else  /* skip until next  0 group code */
      while(1){
        GetTwoLines();
        if (DXFgroupcode == 0)
           return;
      }
}


/* Wantlayer(layername):
 * Given a layername, searches the layer list to see if this 
 * layer is wanted. If no layer list is given, the layer is 
 * assumed wanted.
 */
Wantlayer(layername)
char layername[];
{
   int i, result;
   
   for(i=1;i<Numlayers;i++){
      result = strcmp(layername,Layertable[i].name);
      if (result == 0){
         return(i);
      }
   }
   /* if no layer file is provided add this new layer to list */
   if(Nolayerfile) {
     strcpy(Layertable[Numlayers].name , layername);
     Layertable[Numlayers].colorindex = Numlayers;
     fprintf(stderr,"added layer %s \n",Layertable[Numlayers].name );
     Numlayers++;
     if (Numlayers % MINLAYERS == 0){
         Layertable = REALLOC(Layertable,(Numlayers + MINLAYERS),LAYER);
         if(!Layertable) {
            fprintf(stderr,"Unable to allocate space  for layer table\n");
            exit(0);
         }
     }
     return(TRUE);
   }else
     return(FALSE);
}

