/* output.c : routines to write ugfile, plan view file and error file*/
#include "acad2ug.h"
#include "plan.h"
#ifdef PLANV
#include <gl.h>
#include <device.h>
#endif PLANV


printextents()
{
   fprintf(Errorfile,"Extents of the Entity section :\n");
   fprintf(Errorfile,"Minx= %.3f Maxx = %.3f\n", Minx,Maxx);
   fprintf(Errorfile,"Miny= %.3f Maxy = %.3f\n", Miny,Maxy);
   fprintf(Errorfile,"Minz= %.3f Maxz = %.3f\n", Minz,Maxz);
}

Fix_layer_names()
{
   int i,j,k,l;

   /* Dashes not allowed in color names;replaced with underscore*/
   if (WantPrefix)
     for(i=0;i<strlen(Prefix);i++)
       if (Prefix[i] == '-')
          Prefix[i] == '_';
   for(i=1;i<Numlayers;i++){
      k = l =0;
      if (WantPrefix){
        for(k=0,l=0; k < strlen(Prefix);k++,l++)
          Layertable[i].ugname[k] = Prefix[l];
      }else if(isdigit(Layertable[i].name[0])){
          Layertable[i].ugname[0] = 'N';
          k=1; 
      }
      for(j=0;Layertable[i].name[j] != NULL;j++)
         if(Layertable[i].name[j] == '-')
            Layertable[i].ugname[j+k] = '_';
         else
            Layertable[i].ugname[j+k] = Layertable[i].name[j] ;
       Layertable[i].ugname[j+k] = '\0';
   }
}

Write_temp_ugfile(){
   int i,j;
   BLK *blk;
   INST_STMT *inst;
   FILE *ugfile;
 
   if((ugfile = fopen("tempugfile","w")) == NULL){
      fprintf(stderr,"can't open file tempugfile\n");
      exit(1);
   }
   for(blk=First_block; blk != NULL; blk=blk->next){
      /*if(blk->numfaces == 0) continue;*/
      fprintf(ugfile,"def #%s;\n",blk->name);
      /* print vertices */
      for(i=0;i<blk->numverts;i++)
         fprintf(ugfile,"v vv%d %.3f %.3f %.3f;\n",i,
                blk->vlist[i].x, blk->vlist[i].y, blk->vlist[i].z);
       /* print wires */
      for(i=0;i<blk->numwires;i++){
         fprintf(ugfile,"w w%d ( ",i);
         for(j=0;j<blk->wlist[i].numverts;j++)
             fprintf(ugfile,"vv%d ",blk->wlist[i].v_id_list[j]);
         fprintf(ugfile,") ;\n");
      }
       /* print faces */
      for(i=0;i<blk->numfaces;i++){
         fprintf(ugfile,"f f%d ( ",i);
         for(j=0;j<blk->flist[i].numverts;j++)
             fprintf(ugfile,"vv%d ",blk->flist[i].v_id_list[j]);
         fprintf(ugfile,") ;\n");
      }
      /*print instances */
      for(inst=blk->inst_stmt;inst != NULL;inst=inst->next){
         fprintf(ugfile,"i %s (#%s %s ) ;\n",inst->name, inst->blk->name,inst->transforms);
      }
      fprintf(ugfile,"end;\n"); 
   }
   /* print the top instance */
   fprintf(ugfile,"i #Entity#0 (#Entity);\n");
   fclose(ugfile);
}

Write_ablk(blk,ugfile)
BLK *blk;
FILE *ugfile;
{
   int i,j;
   INST_STMT *inst;
   
   fprintf(ugfile,"def #%s;\n",blk->name);
   /* print vertices */
   for(i=0;i<blk->numverts;i++)
      fprintf(ugfile,"v vv%d %.3f %.3f %.3f;\n",i,
             blk->vlist[i].x, blk->vlist[i].y, blk->vlist[i].z);
    /* print wires */
   for(i=0;i<blk->numwires;i++){
      fprintf(ugfile,"w w%d ( ",i);
      for(j=0;j<blk->wlist[i].numverts;j++)
          fprintf(ugfile,"vv%d ",blk->wlist[i].v_id_list[j]);
      fprintf(ugfile,") %s ;", Layertable[blk->wlist[i].layerindex].ugname);
      if(Debugging)
        fprintf(ugfile," {%d} ",blk->wlist[i].lineno);
      fprintf(ugfile,"\n");
   }
    /* print faces */
   for(i=0;i<blk->numfaces;i++){
       fprintf(ugfile,"f f%d ( ",i);
          if(blk->flist[i].check == FLIP){
             for(j=blk->flist[i].numverts -1 ;j > -1;j--)
                fprintf(ugfile,"vv%d ",blk->flist[i].v_id_list[j]);
          }else{
             for(j=0;j<blk->flist[i].numverts;j++)
                fprintf(ugfile,"vv%d ",blk->flist[i].v_id_list[j]);
         }
      fprintf(ugfile,") %s ;", Layertable[blk->flist[i].layerindex].ugname);
      if(Debugging)
        fprintf(ugfile," {%d} ",blk->flist[i].lineno);
      fprintf(ugfile,"\n");
      if(blk->flist[i].check == DOUBLE){
         fprintf(ugfile,"f fd%d ( ",i);
            for(j=blk->flist[i].numverts -1 ;j > -1;j--)
                fprintf(ugfile,"vv%d ",blk->flist[i].v_id_list[j]);
         fprintf(ugfile,") %s;\n", Layertable[blk->flist[i].layerindex].ugname);
      }
   }
   /*print instances */
   for(inst=blk->inst_stmt;inst != NULL;inst=inst->next)
      fprintf(ugfile,"i %s (#%s %s %s ) ;\n",inst->name,inst->blk->name,
              Layertable[inst->layerindex].ugname,inst->transforms);
   fprintf(ugfile,"end;\n"); 
}

Write_ugfile()
{
   int i;
   BLK *blk;
   float r,g,b;
   FILE *ugfile;

   fprintf(stderr,"Writing output\n");
   if((ugfile = fopen(Ugfilename,"w")) == NULL){
      fprintf(stderr,"can't open file %s\n",Ugfilename);
      exit(1);
   }
   Fix_layer_names();
   for(i=1;i<Numlayers ; i++){
      r = Colortable[Layertable[i].colorindex][0] ;
      g = Colortable[Layertable[i].colorindex][1] ;
      b = Colortable[Layertable[i].colorindex][2] ;
      fprintf(ugfile,"c_rgb   %s %.3f %.3f %.3f;\n",Layertable[i].ugname,r,g,b);  
   }
   for(blk=First_block; blk != NULL; blk=blk->next){
      /*if(blk->numfaces == 0 && blk->numwires == 0 && blk->inst_stmt == NULL)
          continue;*/
       Write_ablk(blk,ugfile);
   }
   /* print the top instance */
   if(Last_block->numfaces != 0 || Last_block->numwires !=0 || Last_block->inst_stmt != NULL)
       fprintf(ugfile,"i #Entity#0 (#Entity);\n");
   fclose(ugfile);
}

#ifdef PLANV
static draww_for_planv(wire,vlist)
WIRE *wire;
VERTEX *vlist;
{ 
   int i;
   float vec[2];
   bgnline();
   for(i=0;i<wire->numverts ;i++){
      vec [0] = vlist[wire->v_id_list[i]].x ; 
      vec [1] = vlist[wire->v_id_list[i]].y ;
      v2f(vec); 
   }
   endline();
}

static drawf_for_planv(face,vlist)
FACE *face;
VERTEX vlist[];
{ 
   int i;
   float vec[2];
   bgnline();
   for(i=0;i <face->numverts ;i++){
      vec [0] = vlist[face->v_id_list[i]].x ; 
      vec [1] = vlist[face->v_id_list[i]].y ;
      v2f(vec); 
   }
   vec [0] = vlist[face->v_id_list[0]].x ; 
   vec [1] = vlist[face->v_id_list[0]].y ;
   v2f(vec); 
   endline();
}

WritePlanv()
{
   int i,j,index;
   long num;
   FACE *face;
   WIRE *wire;
   BLK  *blk;
   INSTANCE *inst;
   float vec1[3],vec2[3];
   size_t size;
   unsigned long pixel[PLANRIGHTW-PLANLEFTW+1][PLANTOPW-PLANBOTTOMW+1];
   FILE *planfile; 
   
   fprintf(stderr,"Writing Plan file\n");
   if((planfile = fopen(Planfilename,"w")) == NULL){
      fprintf(stderr,"can't open file %s\n",Planfilename);
      return;
   }
   
   prefposition(PLANLEFTW,PLANRIGHTW,PLANBOTTOMW,PLANTOPW);
   foreground();
   winopen("Planview;Press D key to exit");
   RGBmode();
   gconfig();
   ortho2((Coord)Minx-MARGIN,(Coord)Maxx+MARGIN, 
           (Coord)Miny-MARGIN,(Coord)Maxy+MARGIN);
   cpack(0xffffff);
   clear();

   linewidth(3);
   cpack(0xfffffe);
   for (inst = Instance_list; inst != NULL; inst= inst->next){
      blk = inst->blockp;
      for(i=0;i<blk->numfaces;i++){
          face =  &blk->flist[i];
          if (Layertable[face->layerindex].colorindex == FORBIDDENCOLOR)
             drawf_for_planv(face,inst->vlist); 
      }
      for(i=0;i<blk->numwires;i++){
          wire =  &blk->wlist[i];
          if (Layertable[wire->layerindex].colorindex == FORBIDDENCOLOR)
             draww_for_planv(wire,inst->vlist); 
      }
   }

   linewidth(1);
   for (inst = Instance_list; inst != NULL; inst= inst->next){
      blk = inst->blockp;
      for(i=0;i<blk->numfaces;i++){
          face =  &blk->flist[i];
          if (Layertable[face->layerindex].colorindex == FORBIDDENCOLOR)
             cpack(0x0);
          else
             c4f(Colortable[Layertable[face->layerindex].colorindex]); 
          drawf_for_planv(face,inst->vlist); 
      }
      for(i=0;i<blk->numwires;i++){
          wire =  &blk->wlist[i];
          if (Layertable[wire->layerindex].colorindex == FORBIDDENCOLOR)
             cpack(0x0);
          else
             c4f(Colortable[Layertable[wire->layerindex].colorindex]); 
          draww_for_planv(wire,inst->vlist); 
      }
    }

   num = lrectread(0,0,PLANRIGHTW-PLANLEFTW,PLANTOPW-PLANBOTTOMW,pixel);
   fprintf(planfile,"%.3f %.3f %.3f %.3f %.3f %.3f\n",Minx, Maxx,
                                Miny,Maxy, Minz,Maxz); 
   size = (PLANRIGHTW - PLANLEFTW + 1) * (PLANTOPW - PLANBOTTOMW + 1);
   fwrite((const void)pixel,sizeof(unsigned long),size,planfile);
   fclose(planfile);  
}
#endif PLANV
