/*****************************************************************************
**
** cdt.C: implementation of the C interface defined in cdt.h
**
** Copyright (C) 1995 by Dani Lischinski 
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
******************************************************************************/

#include "quadedge.h"

/* ADDENDUM */
extern "C"
void write_edge(FILE *fp, Edge *edge)
{
    edge->QEdge()->Write(fp);
}

extern "C"
void *create_point( double x, double y )
{
    return new Point2d(x, y);
}

extern "C"
void set_org_endpoint( Edge *e, Point2d *pt ) 
{
    double *p = (double*)pt;
    /*
    printf("%d seqnum %d num %d %f %f\n", 
	   e->isMarked(), e->QEdge()->seqNum, e->num, p[0], p[1]);
    write_edge( stdout, e );
    */
    assert( e->Org() == 0 );
    e->EndPoints( pt, e->Dest() );
}

extern "C"
void set_dest_endpoint( Edge *e, Point2d *pt ) 
{
    assert( e->Dest() == 0 );
    e->EndPoints( e->Org(), pt );
}

extern "C"
void get_edge_endpoints( Edge *e, double *pts ) 
{
    Edge *edge = (Edge *)e;
    Point2d *s = edge->Org();
    Point2d *d = edge->Dest();

    assert(s != 0);
    assert(d != 0);

    double *sd = (double*)s;
    double *dd = (double*)d;

    pts[0] = sd[0];
    pts[1] = sd[1];
    pts[2] = dd[0];
    pts[3] = dd[1]; 
}

extern "C"
void* get_starting_edge( Mesh *mesh ) 
{
    return mesh->getStartingEdge();
}

extern "C"
void* rot( Edge *edge ) 
{
    return edge->Rot();
}

extern "C"
void* inv_rot( Edge *edge ) 
{
    return edge->invRot();
}

extern "C"
void* sym( Edge *edge ) 
{
    return edge->Sym();
}

extern "C"
void* onext( Edge *edge ) 
{
    return edge->Onext();
}

extern "C"
void* oprev( Edge *edge ) 
{
    return edge->Oprev();
}

extern "C"
void* dnext( Edge *edge ) 
{
    return edge->Dnext();
}

extern "C"
void* dprev( Edge *edge ) 
{
    return edge->Dprev();
}

extern "C"
void* lnext( Edge *edge ) 
{
    return edge->Lnext();
}

extern "C"
void* lprev( Edge *edge ) 
{
    return edge->Lprev();
}

extern "C"
void* rnext( Edge *edge ) 
{
    return edge->Rnext();
}

extern "C"
void* rprev( Edge *edge ) 
{
    return edge->Rprev();
}

extern "C"
void org( Edge *edge, double *pt )
{
    double *p = (double *) edge->Org();
    pt[0] = p[0];
    pt[1] = p[1];
}

extern "C"
void dest( Edge *edge, double *pt )
{
    double *p = (double *) edge->Dest();
    pt[0] = p[0];
    pt[1] = p[1];
}

extern "C"
int is_constrained( Edge *edge ) 
{
    return edge->isConstrained();
}

extern "C"
int is_marked_org( Edge *edge ) 
{
    return edge->isMarked();
}

extern "C"
int get_mark_org( Edge *edge )
{
    return edge->GetMark();
}

extern "C"
void mark_edge_org( Edge *e, char color ) 
{
    register Edge *t = e;
    do {
	t->MarkColor(color);
	t = t->Onext();
    } while (t != e);
}

extern "C"
void unmark_mesh( Mesh *mesh ) 
{
    mesh->UnMarkAllQuadEdges();
}

extern "C"
int get_seqnum( Edge *edge )
{
    return edge->QEdge()->seqNum;
}

extern "C"
void* locate( Mesh *mesh, double x, double y )
{
    return mesh->Locate( Point2d(x, y) );
}

extern "C"
void insert_edge_type( Mesh *mesh, int type, double x1, double y1, double x2, double y2 )
{
    double pts[4];
    Point2d p( (x1+x2)/2.0, (y1+y2)/2.0 );
    mesh->InsertEdge(Point2d(x1, y1), Point2d(x2, y2));
    /* locate edge and mark type */
    Edge *e = mesh->Locate( p );

    if(e)
	e->QEdge()->edgeType = type;
    else
	printf("insert_edge_type not found!\n");

    get_edge_endpoints( e, pts );

    /*
      printf("insert_edge_type: %d %f %f %f %f\n",
      type, pts[0], pts[1], pts[2], pts[3] );
    */

    /* the following asserts rectilinearity */
    /*
    if(x1 == x2) {
	//swapping y is allowed
	Assert(x1 == pts[0]);
	Assert((y1 == pts[1] &&	y2 == pts[3]) ||
	       (y1 == pts[3] &&	y2 == pts[1]));
    }
    else {
	Assert(y1 == y2);
	Assert(y1 == pts[1]);
	Assert((x1 == pts[0] && x2 == pts[2]) ||
	       (x1 == pts[2] && x2 == pts[0]));
	       }
    */
}

extern "C"
int get_edge_type( Edge *e )
{
    return e->QEdge()->edgeType;
}

extern "C"
void set_edge_type( Edge *e, int type )
{
    e->QEdge()->edgeType = type;
}

/* ADDENDUM */

// assumes that given points form a bounding convex polygon
extern "C"
void *create_mesh( int num_bdry_vertices, double *bdry_vertices)
{
	return new Mesh( num_bdry_vertices, bdry_vertices );
}

// insert a Delaunay site
extern "C"
void insert_vertex( Mesh *mesh, double x, double y )
{
    mesh->InsertSite(Point2d(x, y));
}

// insert a constraint edge
extern "C"
void insert_edge( Mesh *mesh, double x1, double y1, double x2, double y2 )
{
    mesh->InsertEdge(Point2d(x1, y1), Point2d(x2, y2));
}

extern "C"
void apply_vertices(
	Mesh *mesh, void *apply_arg,
	void (*func)( void *apply_arg, void *vertex ))
{
    mesh->ApplyVertices(func, apply_arg);
}

extern "C"
void apply_edges(
	Mesh *mesh, void *apply_arg,
	void (*func)( void *apply_arg, void *edge, int constrained ))
{
    mesh->ApplyEdges(func, apply_arg);
}

extern "C"
void destroy_mesh (Mesh *mesh)
{
    delete mesh;
}

