/*****************************************************************************
**
** cdt.h: a C interface to the CDT routines
**
** Copyright (C) 1995 by Dani Lischinski 
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
******************************************************************************/

#ifndef CDT_H
#define CDT_H

#ifdef __cplusplus
extern "C" {
#endif

/* 
 * create_mesh initializes a mesh given an array bdry_vertices,
 * assumed to contain 2*num_bdry_vertices coordinates of the
 * boundary vertices of a convex polygon.
 * It return a pointer to the mesh data structure, which should
 * be used as the first argument for the other routines.
 */
void *create_mesh( int num_bdry_vertices, double *bdry_vertices );

void insert_vertex( void *mesh, double x, double y );

void insert_edge( void *mesh, double x1, double y1, double x2, double y2 );

void apply_vertices(
	void *mesh, void *apply_arg,
	void (*func)( void *apply_arg, void *vertex ));
					 
void apply_edges(
		 void *mesh, void *apply_arg,
		 void (*func)( void *apply_arg, void *edge, int constrained ));
    
void draw_mesh( void *mesh, int output_postscript );
    
void destroy_mesh( void *mesh );

/* ADDENDUM */

void get_edge_endpoints( void *e, double *pts );
    void* get_starting_edge( void *mesh );
    void* rot( void *edge );
    void* inv_rot( void *edge );
    void* sym( void *edge );
    void* onext( void *edge );
    void* oprev( void *edge );
    void* dnext( void *edge );
    void* dprev( void *edge );
    void* lnext( void *edge );	
    void* lprev( void *edge );
    void* rnext( void *edge );	
    void* rprev( void *edge );
    void org( void *edge, double *pt );
    void dest( void *edge, double *pt );
    int is_constrained( void *edge );


#define CLEAR 0
#define GREY  1
#define BLACK 2
    int is_marked_org( void *edge );
    int get_mark_org( void *edge );
    void mark_edge_org( void *edge, char color );
    void unmark_mesh( void *mesh );
    int get_seqnum( void *edge );

    void* locate( void *mesh, double x, double y );
    void insert_edge_type( void *mesh, int type, double x1, double y1, double x2, double y2 );
    int get_edge_type( void *e );
    void set_edge_type( void *e, int type );
    void write_edge(FILE *fp, void *edge);
    void* create_point( double x, double y );
    void set_org_endpoint( void *e, void *pt );
    void set_dest_endpoint( void *e, void *pt );

#ifdef __cplusplus
}
#endif

#endif /* CDT_H */
