/**********************************/
/** Author:       Will Leiserson **/
/** Last Updated: 8/9/2001       **/
/** Email:        leisew@rpi.edu **/
/**********************************/

/******************/
/** HEADER FILES **/
/******************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "sug.h"

/**********************/
/** CONSTANTS N SUCH **/
/**********************/

/**************************/
/** FUNCTION DEFINITIONS **/
/**************************/

/** Load:
 *  Read in a Unigraphics File and store the contents in the
 *  passed ugfile structure.
 */
bool
Load( const char *fileName, ugfile *uf )
{
  char line[1024], name[32], lname[32], type, *location;
  FILE *fp;
  vnode *vn, *cvn;
  wnode *wn;
  vert *v1;
  float r, g, b, x, y, z;
  lnode *layer, *current;
  ugstr *cStr, *nStr;
  int i, j, k, cNum;

  /** Initialize the contents of *uf to NULL **/
  uf->name = NULL;
  uf->init = NULL;
  uf->end = NULL;
  uf->wireList = NULL;
  uf->firstWire = NULL;
  uf->vertList = NULL;
  uf->firstVert = NULL;
  uf->hWire = 0;
  uf->hVert = 0;
  uf->maxWire = 0;
  uf->maxVert = 0;
  uf->layerList = NULL;

  /** Open the file **/
  fp = fopen( fileName, "r" );
  if( !fp ) return false;

  uf->name = (char*) calloc( strlen( fileName )+1, sizeof( char ) );
  strcpy( uf->name, fileName );

  /** Set up the first layer definition **/
  uf->layerList = (lnode*) malloc( sizeof( lnode ) );
  current = uf->layerList;
  current->prev = current;
  current->next = current;
  current->name = NULL;

  /** In case there is extra data at the end of the lines containing
      the layer information, store it for the end of the file. **/
  uf->end = (ugstr*) malloc( sizeof( ugstr ) );
  cStr = uf->end;
  cStr->line = NULL;
  cStr->next = NULL;

  /** Start with the colors **/
  while( !feof( fp ) ){
    fgets( line, 256, fp );
    i = sscanf( line, "c_rgb %s %f %f %f ; { color %d }",
		lname, &r, &g, &b, &cNum );
    if( i > 3 ){

      /** Set up a layer (name, rgb values, etc.) **/
      if( current->name == NULL ){
	current->name = (char*) calloc( strlen( lname )+1, sizeof( char ) );
	strcpy( current->name, lname );
      }else{
	layer = (lnode*) malloc( sizeof( lnode ) );
	layer->name = (char*) calloc( strlen( lname )+1, sizeof( char ) );
	strcpy( layer->name, lname );
	layer->prev = current;
	current->next = layer;
	layer->next = uf->layerList;
	uf->layerList->prev = layer;
	current = layer;
      }

      current->r = r;
      current->g = g;
      current->b = b;

      /** If the color numbers appeared after the layer names, put them
	  in at the end. **/
      if( i > 4 ){
	if( cStr->line == NULL ){
	  cStr->line = (char*) calloc( 24, sizeof( char ) );
	  sprintf( cStr->line, "{ color %d }\n", cNum );
	}else{
	  nStr = (ugstr*) malloc( sizeof( ugstr ) );
	  nStr->line = (char*) calloc( 24, sizeof( char ) );
	  sprintf( nStr->line, "{ color %d }\n", cNum );
	  nStr->next = NULL;
	  cStr->next = nStr;
	  cStr = nStr;
	}
      }
    }else{ break; }
  }

  /** Now, all of the layers have been read in.  There may or may
      not be extra information for the next few lines. **/
  uf->init = (ugstr*) malloc( sizeof( ugstr ) );
  cStr = uf->init;
  cStr->next = NULL;
  cStr->line = NULL;

  while( !feof( fp ) ){
    if( sscanf( line, "v vv%d", &cNum ) ){
      /** If it actually matches what a vertex will look like, move
	  on.  Otherwise, read it into the *init list of lines. **/
      break;
    }else{
      if( cStr->line == NULL ){
	cStr->line = (char*) calloc( strlen( line )+1, sizeof( char ) );
	strcpy( cStr->line, line );
      }else{
	nStr = (ugstr*) malloc( sizeof( ugstr ) );
	nStr->line = (char*) calloc( strlen( line )+1, sizeof( char ) );
	strcpy( nStr->line, line );
	cStr->next = nStr;
	nStr->next = NULL;
	cStr = nStr;
      }
    }
    fgets( line, 256, fp );
  }

  /** Ready to read in all of the vertices **/
  uf->maxVert = 700;
  uf->vertList = (vert**) calloc( uf->maxVert, sizeof( vert* ) );
  for( i = 0; i < uf->maxVert; ++i ) uf->vertList[i] = NULL;

  while( !feof( fp ) ){
    i = sscanf( line, "v %s %f %f %f", name, &x, &y, &z );
    if( i > 3 ){

      /** Resize vertList if need be **/
      if( uf->hVert + 1 >= uf->maxVert ){
	uf->maxVert *= 2;
	uf->vertList = (vert**) realloc( uf->vertList, uf->maxVert * sizeof(vert*) );
	for( i = uf->hVert; i < uf->maxVert; ++i )
	  uf->vertList[i] = NULL;
      }

      /** Copy in all of the values that have been read in. **/
      uf->vertList[uf->hVert] = (vert*) malloc( sizeof( vert ) );
      uf->vertList[uf->hVert]->name =
	(char*) calloc( strlen( name )+1, sizeof( char ) );
      strcpy( uf->vertList[uf->hVert]->name, name );
      uf->vertList[uf->hVert]->x  = x;
      uf->vertList[uf->hVert]->y  = y;
      uf->vertList[uf->hVert]->z  = z;
      uf->vertList[uf->hVert]->wl = NULL;
      uf->vertList[uf->hVert]->isUsed    = false;
      uf->vertList[uf->hVert]->sortValue = x + y + z;

      /** Set up pointers **/
      if( uf->hVert != 0 ){
	uf->vertList[uf->hVert]->prev = uf->vertList[uf->hVert-1];
	uf->vertList[uf->hVert-1]->next = uf->vertList[uf->hVert];
      }

      uf->hVert++;
    }else{ break; }

    fgets( line, 256, fp );
  }

  uf->firstVert = uf->vertList[0];

  /** Connect the ends of the list **/
  uf->vertList[0]->prev = uf->vertList[uf->hVert-1];
  uf->vertList[uf->hVert-1]->next = uf->vertList[0];

  /** All done with vertices.  Now do the same for wires.  This is
      a little more involved.  wireList has to be read in, but there
      also have to be associations made between the wires=>vertices
      and vertices=>wires **/

  uf->maxWire = 200;
  uf->wireList = (wire**) calloc( uf->maxWire, sizeof( wire* ) );
  for( i = 0; i < uf->maxWire; ++i ) uf->wireList[i] = NULL;

  while( !feof( fp ) ){
    i = sscanf( line, "%c %s ( ", &type, name );
    if( i > 1 && ( type == 'w' || type == 'f' ) ){
      /** If it caught the type (wire or face) and the name, add the
	  wire and finish processing the string. **/

      if( uf->hWire+1 >= uf->maxWire ){

	/** Allocate more space if need be **/
	uf->maxWire *= 2;
	uf->wireList =
	  (wire**) realloc( uf->wireList, uf->maxWire * sizeof( wire* ) );
	for( i = uf->hWire; i < uf->maxWire; ++i )
	  uf->wireList[i] = NULL;
      }

      /** Set the basic information about the wire. **/
      uf->wireList[uf->hWire] = (wire*) malloc( sizeof( wire ) );
      uf->wireList[uf->hWire]->name =
	(char*) calloc( strlen( name )+1, sizeof( char ) );
      strcpy( uf->wireList[uf->hWire]->name, name );
      if( type == 'w' ){
	uf->wireList[uf->hWire]->isFace = false;
      }else{
	uf->wireList[uf->hWire]->isFace = true;
      }

      /** Move some pointers around. **/
      if( uf->hWire != 0 ){
	uf->wireList[uf->hWire]->prev = uf->wireList[uf->hWire-1];
	uf->wireList[uf->hWire-1]->next = uf->wireList[uf->hWire];
      }

      /** Find all of the vertices that are used in this wire. **/
      uf->wireList[uf->hWire]->vl = (vnode*) malloc( sizeof( vnode ) );
      cvn = uf->wireList[uf->hWire]->vl;
      cvn->v = NULL;
      cvn->next = NULL;
      location = line;
      while( *location != '(' ) ++location;
      ++location;
      while( *location != ')' ){
	while( *location == ' ' ) ++location;
	if( *location != ')' ){
	  i = sscanf( location, "%s", name );

	  if( i > 0 ){
	    i = sscanf( name, "vv%d", &cNum );
	    if( i > 0 ){

	      /** Found a vertex.  Add it to the list in this wire. **/
	      if( cNum >= uf->hVert ) j = uf->hVert - 1;
	      else j = cNum;

	      v1 = uf->vertList[j];
	      sscanf( v1->name, "vv%d", &k );
	      while( k != cNum && j > 0 ){
		--j;
		v1 = uf->vertList[j];
		sscanf( v1->name, "vv%d", &k );
	      }

	      if( cvn->v == NULL ){
		cvn->v = v1;
	      }else{
		vn = (vnode*) malloc( sizeof( vnode ) );
		vn->v = v1;
		vn->prev = cvn;
		cvn->next = vn;
		cvn = vn;
	      }

	      /** Add this wire to the list in v1 **/
	      wn = (wnode*) malloc( sizeof( wnode ) );
	      wn->w = uf->wireList[uf->hWire];

	      if( v1->wl == NULL ){
		v1->isUsed = true;
		v1->wl = wn;
		wn->next = wn;
		wn->prev = wn;
	      }else{
		wn->next = v1->wl;
		wn->prev = v1->wl->prev;
		v1->wl->prev->next = wn;
		v1->wl->prev = wn;
		v1->wl = wn;
	      }
	    }
	  }

	  /** Move *location up. **/
	  while( *location != ' ' && *location != ')' ) ++location;
	}
      }

      /** Tie up the ends of the list of vnodes **/
      uf->wireList[uf->hWire]->vl->prev = cvn;
      cvn->next = uf->wireList[uf->hWire]->vl;

      /** Finally, figure out what layer this wire belongs to. **/
      while( *location == ')' || *location == ' ' ) ++location;
      i = sscanf( location, "%s", lname );

      if( i > 0 ){
	if( lname[strlen( lname )-1] == ';' )
	  lname[strlen( lname )-1] = '\0';

	/** Find the correct layer and put the wire in it. **/
	current = uf->layerList;
	do{
	  if( !strcmp( lname, current->name ) ) break;
	  current = current->next;
	} while( current != uf->layerList );

	uf->wireList[uf->hWire]->layer = current;

      }else{
	uf->wireList[uf->hWire]->layer = NULL;
      }

      uf->hWire++;
    }else{ break; }

    fgets( line, 256, fp );
  }

  /** Connect up the list **/
  uf->firstWire = uf->wireList[0];
  uf->wireList[0]->prev = uf->wireList[uf->hWire-1];
  uf->wireList[uf->hWire-1]->next = uf->wireList[0];

  /** There may be a couple of trailing lines.  Read them in, so as not
      to lose data. **/
  cStr = uf->end;
  while( !feof( fp ) ){
    if( cStr->line == NULL ){
      cStr->line = (char*) calloc( strlen( line )+1, sizeof( char ) );
      strcpy( cStr->line, line );
    }else{
      nStr = (ugstr*) malloc( sizeof( ugstr ) );
      nStr->line = (char*) calloc( strlen( line )+1, sizeof( char ) );
      strcpy( nStr->line, line );
      nStr->next = NULL;
      cStr->next = nStr;
      cStr = nStr;
    }
    fgets( line, 256, fp );
  }

  /** All done. **/
  fclose( fp );
  return true;
}

/** Write:
 *  Prints out the ugfile into a file specified by *fileName, or stdout
 *  if *fileName == "stdout"
 */
bool
Write( const char *fileName, ugfile *uf )
{
  lnode *layer;
  ugstr *currentLine, *faceStr, *wireStr, *endWire, *endFace;
  vnode *vlist;
  vert *v1;
  wire *w1;
  FILE *fp;
  char *buffer;
  int i, j, bufLength, maxLength;

  bufLength = 0;
  maxLength = 64;
  buffer = (char*) calloc( maxLength, sizeof( char ) );

  if( strcmp( fileName, "stdout" ) ){
    fp = fopen( fileName, "w" );
    if( !fp ) return false;
  }else{
    fp = stdout;
  }

  /** Write out the layer names and their rgb values **/
  layer = uf->layerList;
  do{
    fprintf( fp, "c_rgb %s %f %f %f ;\n", layer->name, layer->r, layer->g, layer->b );
    layer = layer->next;
  } while( layer != uf->layerList );

  /** If there is any extra data that initially came right after the
      layers, print it out **/
  for( currentLine = uf->init; currentLine != NULL; currentLine = currentLine->next )
    if( currentLine->line ) fprintf( fp, "%s", currentLine->line );

  /** Print out all of the vertices **/
  v1 = uf->firstVert;
  i = 0;
  do{
    assert( v1 != NULL );

    if( v1->isUsed ){
      sprintf( buffer, "vv%d", i );
      free( v1->name );
      v1->name = (char*) calloc( strlen( buffer )+1, sizeof( char ) );
      strcpy( v1->name, buffer );

      fprintf( fp, "v %s %g %g %g ;\n", v1->name, v1->x, v1->y, v1->z );
      ++i;
    }

    v1 = v1->next;
  } while( v1 != uf->firstVert );

  /** Seperate the wires from the faces in lists. **/
  w1 = uf->firstWire;
  faceStr = NULL; wireStr = NULL;
  i = 0; j = 0;
  do{
    /** Based on whether w1 is a face or a wire, there are different leading
	characters. **/
    if( w1->isFace ) sprintf( buffer, "f f%d (", j );
    else sprintf( buffer, "w w%d (", i );

    bufLength = strlen( buffer ) + 1;

    vlist = w1->vl;

    /** For each vertex in the wire, append the vertex name to the string **/

    do{
      assert( vlist->next != vlist );
      assert( vlist->v != NULL );
      assert( vlist->v->name != NULL );
      assert( vlist != NULL );
      bufLength += strlen( vlist->v->name ) + 1;
      /** Resize the array, if needed. **/
      if( bufLength >= maxLength ){
	maxLength *= 2;
	buffer = (char*) realloc( buffer, maxLength * sizeof( char ) + 1 );
      }

      sprintf( buffer, "%s %s", buffer, vlist->v->name );
      vlist = vlist->next;
    } while( vlist != w1->vl && vlist != NULL );

    bufLength += strlen( " )  ;\n" );
    bufLength += strlen( w1->layer->name );
    if( bufLength >= maxLength ){
      maxLength *= 2;
      buffer = (char*) realloc( buffer, maxLength * sizeof( char ) + 1 );
    }

    sprintf( buffer, "%s ) %s ;\n", buffer, w1->layer->name );
    currentLine = (ugstr*) malloc( sizeof( ugstr ) );
    currentLine->line = (char*) calloc( strlen( buffer )+1, sizeof( char ) );
    strcpy( currentLine->line, buffer );
    currentLine->next = NULL;
    /** The string should look like this, now:
	w w%d ( vv%d vv%d ... vv%d ) layerName ;\n
	     or
	f f%d ( vv%d vv%d ... vv%d ) layerName ;\n
    **/

    if( w1->isFace ){
      if( faceStr == NULL ){
	faceStr = currentLine;
	endFace = faceStr;
      }else{
	endFace->next = currentLine;
	endFace = endFace->next;
      }
      ++j;
    }else{
      if( wireStr == NULL ){
	wireStr = currentLine;
	endWire = wireStr;
      }else{
	endWire->next = currentLine;
	endWire = endWire->next;
      }
      ++i;
    }

    w1 = w1->next;
  } while( w1 != uf->firstWire );

  /** Print out the wires and then the faces **/
  endWire = wireStr;
  while( endWire != NULL ){
    fprintf( fp, "%s", endWire->line );
    wireStr = endWire;
    endWire = endWire->next;
    free( wireStr->line );
    free( wireStr );
  }

  endFace = faceStr;
  while( endFace != NULL ){
    fprintf( fp, "%s", endFace->line );
    faceStr = endFace;
    endFace = endFace->next;
    free( faceStr->line );
    free( faceStr );
  }

  /** Lastly, print out the color data relating to the layers **/
  for( currentLine = uf->end; currentLine != NULL; currentLine = currentLine->next )
    if( currentLine->line != NULL ) fprintf( fp, "%s", currentLine->line );

  if( strcmp( fileName, "stdout" ) ) fclose( fp );

  return true;
}

/** Unload:
 *  Remove all of the data associated with a ugfile, uf.  This erases the whole
 *  data structure.  Make sure you haven't done any of it on your own.  Returns
 *  true if everything was okay.
 */
bool Unload( ugfile *uf )
{
  wire  *w;
  wnode *wn1, *wn2;
  vert  *v;
  vnode *vn1, *vn2;
  lnode *ln1, *ln2;
  ugstr   *line1, *line2;
  int i;

  /** Remove all vnodes from every wire. **/
  for( i = 0; i < uf->hWire; ++i ){

    w = uf->wireList[i];
    if( w != NULL ){
      vn1 = w->vl;
      vn1->prev->next = NULL;

      while( vn1 != NULL ){

	vn2 = vn1->next;
	free( vn1 );
	vn1 = vn2;

      }

      free( w->name );
      free( w );
    }

  }

  /** Remove the list of wires **/
  free( uf->wireList );

  /** Get rid of all of the wnodes from the vertices. **/
  for( i = 0; i < uf->hVert; ++i ){

    v = uf->vertList[i];
    if( v != NULL ){
      if( v->isUsed ){

	wn1 = v->wl;
	wn1->prev->next = NULL;

	while( wn1 != NULL ){

	  wn2 = wn1->next;
	  free( wn1 );
	  wn1 = wn2;

	}

      }

      free( v->name );
      free( v );
    }
  }

  /** Remove the list of vertices **/
  free( uf->vertList );

  /** Remove the layers **/
  ln1 = uf->layerList;
  ln1->prev->next = NULL;

  while( ln1 != NULL ){

    ln2 = ln1->next;
    free( ln1->name );
    free( ln1 );
    ln1 = ln2;

  }

  /** Remove the beginning and ending set of strings **/
  line1 = uf->init;

  while( line1 != NULL ){

    line2 = line1->next;
    free( line1->line );
    free( line1 );
    line1 = line2;

  }

  line1 = uf->end;

  while( line1 != NULL ){

    line2 = line1->next;
    free( line1->line );
    free( line1 );
    line1 = line2;

  }

  /** Free the ugfile that was passed, itself **/
  free( uf->name );
  free( uf );

  return true;
}

/** Standardize:
 *  Modify names so that they fit the WLS (Will Leiserson Standard):
 *  v vv%d etc.
 *  w w%d  etc.
 *  f f%d  etc.
 */
void Standardize( ugfile *uf )
{
  vert *v;
  wire *w;
  char buffer[32];
  int i, j;

  i = 0;
  v = uf->firstVert;

  /** Modify all of the vertices **/
  do{
    free( v->name );
    sprintf( buffer, "v%d", i );
    v->name = (char*) calloc( strlen( buffer ) + 1, sizeof( char ) );
    strcpy( v->name, buffer );
    ++i;
    v = v->next;
  } while( v != uf->firstVert );

  i = j = 0;
  w = uf->firstWire;

  /** Modify the faces and wires **/
  do{
    free( w->name );
    if( w->isFace ){
      sprintf( buffer, "f%d", i );
      ++i;
    }else{
      sprintf( buffer, "w%d", j );
      ++j;
    }
    w->name = (char*) calloc( strlen( buffer ) + 1, sizeof( char ) );
    strcpy( w->name, buffer );
    w = w->next;
  } while( w != uf->firstWire );

  return;
}

/** AddVertex:
 *  Make a new vertex and add it to the list of vertices in uf.  Insert the given values
 *  and return a pointer to it.
 */
vert *AddVertex( ugfile *uf, float x, float y, float z )
{
  vert *newVert;
  int i;

  newVert = (vert*) malloc( sizeof( vert ) );

  newVert->x = x;
  newVert->y = y;
  newVert->z = z;
  newVert->wl = NULL;
  newVert->isUsed = false;
  newVert->name = (char*) calloc( strlen( "newvert" )+1, sizeof( char ) );
  strcpy( newVert->name, "newvert" );

  if( uf->hVert + 2 >= uf->maxVert ){
    uf->maxVert *= 2;
    uf->vertList = (vert**) realloc( uf->vertList, uf->maxVert * sizeof( vert* ) );
    for( i = uf->hVert; i < uf->maxVert; ++i ) uf->vertList[i] = NULL;
  }

  uf->vertList[uf->hVert] = newVert;
  uf->hVert++;

  newVert->next = uf->firstVert;
  newVert->prev = uf->firstVert->prev;
  uf->firstVert->prev->next = newVert;
  uf->firstVert->prev = newVert;

  return newVert;
}

/** AddWire:
 *  Create a new wire and add it to *uf, or merely add an existing wire to *uf.
 *  Either way, return the wire.
 */
wire *AddWire( ugfile *uf, wire *w )
{
  int i;
  wire *newWire;

  /** If no existing wire is passed, make a new one. **/
  if( w != NULL ) newWire = w;
  else{
    newWire = (wire*) malloc( sizeof( wire ) );
    newWire->name = (char*) calloc( strlen( "newwire" )+1, sizeof( char ) );
    strcpy( newWire->name, "newwire" );
    newWire->isFace = false;
    newWire->vl     = NULL;
    newWire->layer  = uf->layerList;
  }

  if( uf->hWire + 1 >= uf->maxWire ){
    /** Resize the wire list if it needs to be resized **/
    uf->maxWire *= 2;
    uf->wireList = (wire**) realloc( uf->wireList, uf->maxWire * sizeof( wire* ) );
    for( i = uf->hWire + 1; i < uf->maxWire; ++i ) uf->wireList[i] = NULL;
  }

  uf->wireList[uf->hWire] = newWire;
  uf->hWire++;

  if( uf->firstWire != NULL ){
    newWire->next = uf->firstWire;
    newWire->prev = uf->firstWire->prev;
    uf->firstWire->prev->next = newWire;
    uf->firstWire->prev = newWire;
  }else{
    uf->firstWire = newWire;
    newWire->next = newWire;
    newWire->prev = newWire;
  }

  return newWire;
}

bool RemoveVertWire( vert *remove, wire *w )
{
  vnode *vn;
  wnode *wn;

  /** Make sure we're not leaving a wire with one vertex **/
  if( w->vl->prev == w->vl->next ) return false;

  /** Get rid of the vertex entry in the wire. **/
  vn = w->vl;
  while( vn->v != remove ) vn = vn->next;
  if( vn == w->vl ) w->vl = w->vl->next;

  assert( vn->v == remove );

  vn->prev->next = vn->next;
  vn->next->prev = vn->prev;
  free( vn );

  /** Get rid of the wire entry in the vertex **/
  wn = remove->wl;
  if( wn == wn->next ){
    /** If this is the only wire in the vertex, mark the vertex as unused. **/

    remove->isUsed = false;
    remove->wl = NULL;
    free( wn );

  }else{

    while( wn->w != w ) wn = wn->next;
    if( wn == remove->wl ) remove->wl = remove->wl->next;
    wn->prev->next = wn->next;
    wn->next->prev = wn->prev;
    free( wn );

  }


  return true;
}

/** RemoveWire:
 *  Gets rid of a wire, completely.
 */
void RemoveWire( ugfile *uf, wire *w )
{
  vnode *vn;
  wnode *wn;
  int i;

  vn = w->vl->next;

  while( vn != w->vl ){

    wn = vn->v->wl->next;
    while( wn->w != w && wn != vn->v->wl ) wn = wn->next;
    assert( wn->w == w );

    /** Get rid of the 'w' entry in vn->v **/
    if( wn == wn->next ){
      free( wn );
      vn->v->wl = NULL;
      vn->v->isUsed = false;
    }else{
      wn->next->prev = wn->prev;
      wn->prev->next = wn->next;
      if( vn->v->wl == wn ) vn->v->wl = wn->next;
      free( wn );
    }

    vn = vn->next;
    free( vn->prev );
  }

  wn = vn->v->wl->next;
  while( wn->w != w && wn != vn->v->wl ) wn = wn->next;
  assert( wn->w == w );

  /** Get rid of the 'w' entry in vn->v **/
  if( wn == wn->next ){
    free( wn );
    vn->v->wl = NULL;
    vn->v->isUsed = false;
  }else{
    wn->next->prev = wn->prev;
    wn->prev->next = wn->next;
    if( vn->v->wl == wn ) vn->v->wl = wn->next;
    free( wn );
  }

  free( vn );

  /** Find w in the uf list and get rid of it. **/
  for( i = 0; i < uf->hWire && uf->wireList[i] != w; ++i );
  assert( uf->wireList[i] == w );

  w->prev->next = w->next;
  w->next->prev = w->prev;
  if( uf->firstWire == w ) uf->firstWire = w->next;
  free( w->name );
  free( w );
  uf->wireList[i] = NULL;

  return;
}

/** AddVertWire:
 *  Add a vertex, insert, into a wire, w, between the vertices, v1, v2.  Add insert
 *  to w's list of vertices, and w into insert's list of wires.
 */
bool AddVertWire( vert *insert, wire *w, vert *v1, vert *v2 )
{
  wnode *wn;
  vnode *vn1, *vn2;

  /** Put w into insert's list of wires **/
  if( insert->wl == NULL ){
    /** Case: insert has no wires in its list, yet **/
    insert->isUsed = true;
    insert->wl = (wnode*) malloc( sizeof( wnode ) );
    insert->wl->w = w;
    insert->wl->next = insert->wl;
    insert->wl->prev = insert->wl;
  }else{
    /** Case: put w at the beginning of insert's list of wires **/
    wn = (wnode*) malloc( sizeof( wnode ) );
    wn->w = w;
    wn->next = insert->wl;
    wn->prev = insert->wl->prev;
    insert->wl->prev->next = wn;
    insert->wl->prev = wn;
    insert->wl = wn;
  }

  /** Put *insert into w's list of vertices **/
  if( w->vl == NULL ){
    /** Case: there are no vertices in w, yet **/
    w->vl = (vnode*) malloc( sizeof( vnode ) );
    w->vl->v = insert;
    w->vl->next = w->vl;
    w->vl->prev = w->vl;
  }else if( v1 == NULL && v2 == NULL ){
    /** Case: put *insert at beginning of vertex list in w **/
    vn1 = (vnode*) malloc( sizeof( vnode ) );
    vn1->v = insert;
    vn1->next = w->vl;
    vn1->prev = w->vl->prev;
    w->vl->prev->next = vn1;
    w->vl->prev = vn1;
    w->vl = vn1;
  }else{
    /** Case: put *insert after v1 in w **/
    vn1 = w->vl;

    while( vn1->v != v1 ) vn1 = vn1->next;

    assert( insert != v1 && insert != v2 );

    vn2 = (vnode*) malloc( sizeof( vnode ) );
    vn2->v = insert;
    vn2->next = vn1->next;
    vn2->prev = vn1;
    vn1->next->prev = vn2;
    vn1->next = vn2;
  }

  return true;
}

/** GetEdges:
 *  Take all of the edges out of a ug file and return an array of them.  Call
 *  it with empty values in n and m.  n will become the highest non-NULL value
 *  in the array, and m is the maximum size of the array.
 */
edge **GetEdges( ugfile *uf, int *n, int *m )
{
  edge **e;
  wire *w;
  vert *v1, *v2;
  vnode *vn;
  int numEdges, i, j;

  /** Initialize the return value: e **/
  numEdges = 512;
  e = (edge**) calloc( numEdges, sizeof( edge* ) );
  for( i = 0; i < numEdges; ++i ) e[i] = NULL;

  i = 0;

  /** Go through each of the wires/faces and get a list of edges **/
  w = uf->firstWire;
  do{
    vn = w->vl;
    v1 = vn->v;
    vn = vn->next;

    do{
      v2 = vn->v;

      /** Check to see if the array needs resizing, before adding an edge **/
      if( i >= numEdges - 1 ){
	numEdges *= 2;
	e = (edge**) realloc( e, numEdges * sizeof( edge* ) );
	if( !e ) return NULL;
	for( j = i; j < numEdges; ++j ) e[j] = NULL;
      }

      /** Set some space **/
      e[i] = (edge*) malloc( sizeof( edge ) );
      if( !e[i] ) return NULL;

      e[i]->v[0] = v1;
      e[i]->v[1] = v2;
      e[i]->hasMatch = false;
      e[i]->w = w;

      /** Set the equation values in e[i] **/
      if( v1->x == v2->x ){
	/** e[i] is vertical **/
	e[i]->direction = VERTICAL;
	e[i]->eq[M] = 0;
	e[i]->eq[B] = v1->x;
	e[i]->sortValue = v1->x;
      }else if( v1->y == v2->y ){
	/** e[i] is horizontal **/
	e[i]->direction = HORIZONTAL;
	e[i]->eq[M] = 0;
	e[i]->eq[B] = v1->y;
	e[i]->sortValue = v1->y;
      }else{
	/** e[i] is sloped **/
	e[i]->direction = SLOPED;
	/** m = (y1-y2)/(x1-x2) **/
	e[i]->eq[M] = ( v1->y - v2->y ) / ( v1->x - v2->x );
	/** b = y-m*x **/
	e[i]->eq[B] = v1->y - ( e[i]->eq[M] * v1->x );
	e[i]->sortValue = e[i]->eq[M];
      }

      /** Cycle to the next two points **/
      ++i;
      v1 = v2;
      vn = vn->next;
    } while( vn != w->vl );

    if( w->isFace ){
      /** Do it all one more time **/
      v2 = vn->v;

      /** Check to see if the array needs resizing, before adding an edge **/
      if( i >= numEdges - 1 ){
	numEdges *= 2;
	e = (edge**) realloc( e, numEdges * sizeof( edge* ) );
	if( !e ) return NULL;
	for( j = i; j < numEdges; ++j ) e[j] = NULL;
      }

      /** Set some space **/
      e[i] = (edge*) malloc( sizeof( edge ) );
      if( !e[i] ) return NULL;

      e[i]->v[0] = v1;
      e[i]->v[1] = v2;
      e[i]->hasMatch = false;
      e[i]->w = w;

      if( v1->x == v2->x ){
	/** e[i] is vertical **/
	e[i]->direction = VERTICAL;
	e[i]->eq[M] = 0;
	e[i]->eq[B] = v1->x;
	e[i]->sortValue = v1->x;
      }else if( v1->y == v2->y ){
	/** e[i] is horizontal **/
	e[i]->direction = HORIZONTAL;
	e[i]->eq[M] = 0;
	e[i]->eq[B] = v1->y;
	e[i]->sortValue = v1->y;
      }else{
	/** e[i] is sloped **/
	e[i]->direction = SLOPED;
	/** m = (y1-y2)/(x1-x2) **/
	e[i]->eq[M] = ( v1->y - v2->y ) / ( v1->x - v2->x );
	/** b = y-m*x **/
	e[i]->eq[B] = v1->y - ( e[i]->eq[M] * v1->x );
	e[i]->sortValue = e[i]->eq[M];
      }
      ++i;
    }

    w = w->next;
  } while( w != uf->firstWire );

  if( n != NULL ) *n = i;
  if( m != NULL ) *m = numEdges;

  return e;
}

/** SplitWire:
 *  Split the wire, *w, on the vertices *v1 and *v2.  Return the segment containing
 *  the passed vertices.  Any new wires generated are added to the ugfile, *uf.
 */
wire *SplitWire( ugfile *uf, wire *w, vert *v1, vert *v2 )
{
  wire  *newWire, *newWire2;
  wnode *wn;
  vnode *vn, *vn1, *vn2;

  /** If this wire has only two vertices to begin with, just return it. **/
  if( w->vl->prev == w->vl->next ) return w;

  /** It's possible that v1 and v2 wrap, if w is a face.  If so, just move
      w->vl back one. **/
  if( w->isFace && w->vl->v == v2 ) w->vl = w->vl->prev;

  if( v2 == NULL ){
    /** Make two wires by splitting on one vertex **/
    /** Not yet implemented **/

    fprintf( stderr, "Error: Unimplemented code for SplitWire in sug.c\n" );

  }else{
    /** Make a wire using v1 and v2, and generate an additional one or two other
        wires in the splitting process. **/

    if( w->vl->v == v1 ){
      /** Only need to generate one additional wire. **/

      newWire = AddWire( uf, NULL );
      newWire->layer = w->layer;

      vn = (vnode*) malloc( sizeof( vnode ) );
      vn->v = v2;

      vn->next          = w->vl->next->next;
      newWire->vl       = vn;
      newWire->vl->prev = w->vl->prev;
      newWire->vl->prev->next = newWire->vl;
      w->vl->prev       = w->vl->next;
      w->vl->next->next = w->vl;

      wn = (wnode*) malloc( sizeof( wnode ) );
      wn->w    = newWire;
      wn->next = v2->wl;
      wn->prev = v2->wl->prev;
      v2->wl->prev->next = wn;
      v2->wl->prev = wn;
      v2->wl = wn;

      if( w->isFace ){
	/** w is a face, so add a node containing v1 to newWire, and add
	    newWire to v1. **/
	wn = (wnode*) malloc( sizeof( wnode ) );
	wn->w = newWire;
	wn->next = v1->wl;
	wn->prev = v1->wl->prev;
	v1->wl->prev->next = wn;
	v1->wl->prev = wn;
	v1->wl = wn;

	vn = (vnode*) malloc( sizeof( vnode ) );
	vn->v = v1;
	vn->next = newWire->vl;
	vn->prev = newWire->vl->prev;
	newWire->vl->prev->next = vn;
	newWire->vl->prev = vn;

	w->isFace = false;
      }

      vn = newWire->vl;
      do{
	wn = vn->v->wl;
	while( wn->w != w && wn->w != newWire ) wn = wn->next;
	wn->w = newWire;
	vn = vn->next;
      } while( vn != newWire->vl );

      strcpy( newWire->name, "wCase1" );

      return w;

    }else if( w->vl->prev->v == v2 ){
      /** v1 - v2 are at the back of the wire.  This wire will stay the same, while
	  the new wire will contain the two new points. **/

      newWire = AddWire( uf, NULL );
      newWire->layer = w->layer;

      vn = (vnode*) malloc( sizeof( vnode ) );
      vn->v = v1;

      newWire->vl = vn;
      vn->next = w->vl->prev;
      vn->prev = w->vl->prev;
      w->vl->prev->prev->next = w->vl;
      w->vl->prev = w->vl->prev->prev;
      vn->next->prev = vn;
      vn->next->next = vn;

      wn = (wnode*) malloc( sizeof( wnode ) );
      wn->w    = newWire;
      wn->next = v1->wl;
      wn->prev = v1->wl->prev;
      v1->wl->prev->next = wn;
      v1->wl->prev = wn;
      v1->wl   = wn;

      if( w->isFace ){
	/** If w is a face, add newWire to the list of wires that use v2.  Also,
	    add a new node to replace the one that was taken. **/
	vn = (vnode*) malloc( sizeof( vnode ) );
	vn->v = v2;
	vn->next = w->vl;
	vn->prev = w->vl->prev;
	w->vl->prev->next = vn;
	w->vl->prev = vn;
	w->vl = vn;

	wn = (wnode*) malloc( sizeof( wnode ) );
	wn->w = newWire;
	wn->next = v2->wl;
	wn->prev = v2->wl->prev;
	v2->wl->prev->next = wn;
	v2->wl->prev = wn;
	v2->wl = wn;

	w->isFace = false;
      }else{
	/** If w is not a face, replace its entry in v2 with newWire. **/
	wn = v2->wl;
	do{
	  wn = wn->next;
	  if( wn->w == w ) wn->w = newWire;
	} while( wn->w != newWire );
      }

      strcpy( newWire->name, "wCase2" );

      return newWire;
    }else{
      /** v1 and v2 are located somewhere in the middle of the wire **/

      vn = w->vl;
      while( vn->v != v2 ) vn = vn->next;

      newWire = AddWire( uf, NULL );
      newWire->layer = w->layer;

      AddVertWire( v2, newWire, NULL, NULL );
      AddVertWire( v1, newWire, NULL, NULL );

      if( w->isFace ){
	/** w is a face.  Therefore, there are only two wires generated by the
	    split. **/

	w->vl = vn;
	w->isFace = false;

      }else{
	/** w is not a face.  Generate one more line, and return the one that has
	    already been generated. **/

	newWire2 = AddWire( uf, NULL );
	newWire2->layer = w->layer;

	vn1 = vn;

	vn2 = w->vl->prev;
	w->vl->prev->next = vn1;
	w->vl->prev = vn1->prev;
	vn1->prev->next = w->vl;
	vn1->prev = vn2;

	vn2 = vn1;
	do{
	  wn = vn2->v->wl;
	  while( wn->w != w ) wn = wn->next;
	  wn->w = newWire2;
	  vn2 = vn2->next;
	} while( vn2 != vn1 );

	newWire2->vl = vn1;
	strcpy( newWire2->name, "wCase3b" );

      }

      strcpy( newWire->name, "wCase3a" );

      return newWire;
    }
  }

  return w;
}

/** MergeWires:
 *  Merge wires, w1 and w2 on the shared vertex.  Return the remaining
 *  wire, or NULL if there was no shared vertex.
 */
wire *MergeWires( wire *w1, wire *w2 )
{
  vnode *vn1, *vn2;
  wire  *complete, *extra;
  wnode *wn;

  vn1 = w1->vl;
  vn2 = w2->vl;

  /** Figure out where the wires meet. **/
  if( vn1->v == vn2->v ){
    ReverseWire( w1 );
    complete = w1;
    extra = w2;
  }else if( vn1->prev->v == vn2->v ){
    complete = w1;
    extra = w2;
  }else if( vn1->v == vn2->prev->v ){
    complete = w2;
    extra = w1;
  }else if( vn1->prev->v == vn2->prev->v ){
    ReverseWire( w2 );
    complete = w1;
    extra = w2;
  }else return NULL;

  vn1 = complete->vl->prev;
  vn2 = extra->vl;

  /** Move pointers around to put the vertices from extra into the vlist for
      complete.  Then free the duplicate vnode, vn2. **/
  complete->vl->prev = vn2->prev;
  vn2->prev->next = complete->vl;
  vn1->next = vn2->next;
  vn1->next->prev = vn1;
  vn1 = vn1->next;
  free( vn2 );

  while( vn1 != complete->vl ){

    wn = vn1->v->wl;
    while( wn->w != extra && wn != vn1->v->wl->prev ) wn = wn->next;

    assert( wn->w == extra );
    wn->w = complete;

    vn1 = vn1->next;
  }

  /** Get rid of the old wire. **/
  extra->next->prev = extra->prev;
  extra->prev->next = extra->next;
  strcpy( extra->name, "extra" );
  extra->next = NULL;
  extra->prev = NULL;

  /** Return the completed wire.  Keep in mind that layer info, amongst other
      useful data may not be accurate. **/
  return complete;
}

/** VertHasWire:
 *  Return the wnode from v containing w if it exists, else return NULL.
 */
wnode *VertHasWire( vert *v, wire *w )
{
  wnode *wn;

  wn = v->wl;
  do{
    if( wn->w == w ) return wn;
    wn = wn->next;
  } while( wn != v->wl );

  return NULL;
}

/** WireHasVert:
 *  Return the vnode from w containing v if it exists, else return NULL.
 */
vnode *WireHasVert( wire *w, vert *v )
{
  vnode *vn;

  vn = w->vl;
  do{
    if( vn->v == v ) return vn;
    vn = vn->next;
  } while( vn != w->vl );

  return NULL;
}

/** CommonWire:
 *  Find and return the wire that is common to v1 and v2.  If it doesn't exist,
 *  return NULL;
 */
wire *CommonWire( vert *v1, vert *v2 )
{
  wnode *wn1, *wn2;

  if( v1->wl == NULL || v2->wl == NULL ) return NULL;

  wn1 = v1->wl;
  do{
    wn2 = v2->wl;
    do{
      if( wn1->w == wn2->w ) return wn1->w;
      wn2 = wn2->next;
    } while( wn2 != v2->wl );
    wn1 = wn1->next;
  } while( wn1 != v1->wl );

  return NULL;
}

/** CommonVert:
 *  Find and return the vert that is common to w1 and w2.  If it doesn't exist,
 *  return NULL;
 */
vert *CommonVert( wire *w1, wire *w2 )
{
  vnode *vn1, *vn2;

  vn1 = w1->vl;
  do{
    vn2 = w2->vl;
    do{
      if( vn2->v == vn1->v ) return vn1->v;
      vn2 = vn2->next;
    } while( vn2 != w2->vl );
    vn1 = vn1->next;
  } while( vn1 != w1->vl );

  return NULL;
}

/** ReverseWire:
 *  Reverse the order of vertices in a wire.  Sets the new w->vl to what was
 *  w->vl->prev.
 */
void ReverseWire( wire *w )
{
  vnode *vn, *swap;

  vn = w->vl;

  do{

    swap = vn->next;
    vn->next = vn->prev;
    vn->prev = swap;

    vn = vn->next;
  } while( vn != w->vl );

  vn = vn->next;
  w->vl = vn;

  return;
}

/** Area:
 *  Find the area of a selected wire (face, if you want the returned value to be
 *  of any significance) by using a line integral from vertex to vertex.  This
 *  value will be negative if the face goes in a counter-clockwise direction.
 */
float Area( wire *w )
{
  float area;
  vnode *vn;
  vert  *v1, *v2;

  area = 0;

  v1 = w->vl->v;
  vn = w->vl->next;

  do{
    v2 = vn->v;

    if( v1->x != v2->x ){
      area += ( (v1->y + v2->y)/2 ) * (v2->x - v1->x);
    }

    v1 = v2;
    vn = vn->next;
  } while( vn != w->vl->next );

  return area;
}

