/*struct.c : routines to make structures for DXF entities */
#include "acad2ug.h"

#define TRUE 1
#define FALSE 0

/* used to keep the extents of the Entity */
double Minx =  BIGFLOAT,     
       Miny =  BIGFLOAT,     
       Minz =  BIGFLOAT,     
       Maxx =  SMALLFLOAT,     
       Maxy =  SMALLFLOAT,     
       Maxz =  SMALLFLOAT;     

extern  double Dot();

/* insert(vrtx):
 * Inserts a vertex in the vertex array and returns its index.
 * Eliminates  duplicate vertices.
 */
int insert(vrtx)
VERTEX vrtx;  
{
   int i;
   
   VERTEX *vlist = Cur_block->vlist; 
   if(Lastv != 0){
     /* search the vlist for the vertex */
     for (i=Lastv-1;i != -1;i--){
       if(fabs(vlist[i].x - vrtx.x ) < EPSILON &&
          fabs(vlist[i].y - vrtx.y ) < EPSILON &&
          fabs(vlist[i].z - vrtx.z ) < EPSILON)
         return(i);
     }

   }

   /* check if vertex list of block needs to be expanded */
   if (InEntities){
     if ( (Lastv % MINVERTINENT) == 0){
        vlist = REALLOC(Cur_block->vlist,(Lastv + MINVERTINENT),VERTEX);
        if (vlist == NULL){
          fprintf(stderr,"out of mem\n");
           exit();
        }
     }
   } else if (Inablock) {
     if ( (Lastv % MINVERTINBLK) == 0){
        vlist = REALLOC(Cur_block->vlist,(Lastv + MINVERTINBLK),VERTEX);
        if (vlist == NULL){
          fprintf(stderr,"out of mem\n");
           exit();
        }
     }
   }

   /* vertex not in list; add it to end */
   vlist[Lastv] = vrtx;
   Lastv++;
   (Cur_block->numverts)++;
   Cur_block->vlist = vlist;
   return(Lastv-1);
}

/* UpdateExtents(num,verts) 
 * Updates the extents of the Entity section as new vertices 
 * are added
 */
void
UpdateExtents(num,verts) 
int num;
VERTEX verts[];
{
   int i;
   if( InEntities ){
     for(i=0;i<num;i++){
        if(Minx > verts[i].x)
            Minx = verts[i].x;
        if(Miny > verts[i].y)
            Miny = verts[i].y;
        if(Minz > verts[i].z)
            Minz = verts[i].z;
        if(Maxx < verts[i].x)
            Maxx = verts[i].x;
        if(Maxy < verts[i].y)
            Maxy = verts[i].y;
        if(Maxz < verts[i].z)
            Maxz = verts[i].z;
      }
   }
}


/* Findblk(name):
 * Given the name of a block, finds the block in the block 
 * list and returns a pointer to the block.
 */
BLK *
Findblk(name)
char name[];
{
   BLK *b;
   b = First_block;
   while ( b != NULL) { 
    if(!strcmp(name,b->name)) 
      return(b);
    b = b->next;
   }
   return(NULL);
}

/* InitBlock(name):
 * Initialized a block structure
 */
BLK 
*InitBlock(name)
char *name;
{
   BLK *newblk;

   newblk = MALLOC(BLK);
   newblk->base.x =0.;
   newblk->base.y =0.;
   newblk->base.z =0.;
   newblk->numverts  =0;
   newblk->numwires  =0;
   newblk->numfaces  =0;
   newblk->numedges  =0;
   newblk->numinstances  =0;
   newblk->inst_stmt = NULL;
   newblk->elist = NULL;
   newblk->next = NULL;
   newblk->wlist = CALLOC(MINWIRES,WIRE);
   if (Inablock){
      newblk->vlist = CALLOC(MINVERTINBLK,VERTEX);
      newblk->flist = CALLOC(MINFACEINBLK,FACE);
   }else if (InEntities) {
      newblk->vlist = CALLOC(MINVERTINENT,VERTEX);
      newblk->flist = CALLOC(MINFACEINENT,FACE);
   }
   strcpy(newblk->name,name);
   return (newblk);
}


/* MakeBlock():
 * Called when an AUTOCAD "BLOCK" entity is reached.  
 * AUTOCAD allows references to blocks prior to their definition.
 * THis is unacceptable in UNIGRAFIX. In this case a structure for
 * the block is already made (when the first reference is made) and
 * is returned . Otherwise a new block structure is created and added
 * to the list.
 */
BLK 
*MakeBlock(name)
char *name;
{
   BLK *newblk;

   newblk = Findblk(name);
   if (newblk == NULL){
      newblk = InitBlock(name);
      if(First_block == NULL)
         First_block = Last_block = newblk; 
      else{
         Last_block->next = newblk;
         Last_block = newblk; 
      }
   }
   Lastv = Lastw = Lastf = 0;
   Cur_block = newblk ; 
   return(newblk);
}


/*MakeInstance(name)
 * Adds a new INSTANCE to Instance_list. For each INSERT entity
 * in the DXF file there will be an INSTANCE structure which points
 * to the block that is being inserted. This structure contains space
 * to hold the transformed vertices of the block after scene flatteining.
 */
void
MakeInstance(name)
char name[];
{
   INSTANCE *new_inst;
   BLK *b;

   b = Findblk(name);
   if (b == NULL){
   /* a reference to  an undefined block ; make an empty block structure
      for it, to be filled later,  and add it to the beginning of list */ 
      b = InitBlock(name);
      b->next = First_block;
      First_block = b;
   }
   new_inst = MALLOC(INSTANCE);
   new_inst->blockp = b; 
   new_inst->mirror = FALSE;
   sprintf(new_inst->name,"#%s#%d",name,b->numinstances);
   (b->numinstances)++;
   new_inst->vlist = CALLOC(b->numverts,VERTEX);
   new_inst->next = Instance_list;
   Instance_list = new_inst;
}


/* MakeInstanceStmt(blk)
 * Adds an Instance statement (equivalent to the UNIGRAPHIX stmt)
 * to the current block.
 */
INST_STMT 
*MakeInstanceStmt(blk)
BLK *blk;
{
   INST_STMT *newstmt;

   newstmt = MALLOC(INST_STMT);
   newstmt->next = blk->inst_stmt;
   blk->inst_stmt = newstmt;
   return(newstmt);
}

/* Expand_facelist()
 * Allocates more memeory to the face list if it needs expansion. Entity
   section is usually larger, so allocate larger chunks.
 */
void
Expand_facelist()
{
   if (InEntities){
     if ( (Lastf % MINFACEINENT) == 0)
        Cur_block->flist=REALLOC(Cur_block->flist,(Lastf+MINFACEINENT),FACE);
   } else if (Inablock) {
     if ( (Lastf % MINFACEINBLK) == 0)
        Cur_block->flist= REALLOC(Cur_block->flist,(Lastf+MINFACEINBLK),FACE);
   }
   if (Cur_block->flist == NULL){
     fprintf(stderr,"out of memory Lineno = %d \n",Lineno);
      exit(0);
   }
}
      
/* Expand_wirelist()
 * Allocates more memeory to the wire list if it needs expansion
 */
void
Expand_wirelist()
{
   if ( (Lastw % MINWIRES) == 0)
        Cur_block->wlist=REALLOC(Cur_block->wlist,(Lastw+MINWIRES),WIRE);
   if (Cur_block->wlist == NULL){
     fprintf(stderr,"out of memory Lineno = %d \n",Lineno);
      exit(0);
   }
}

/* MakeWire(verts,num,layerindex):
   Makes a wire structure 
*/
void
MakeWire(verts,num,layerindex)
VERTEX verts[];
int num;
int layerindex;
{
   int i,id;
   WIRE *wlist;
   /* check for vertices on same line and duplicate adjacent vertices*/
   CheckVertices(verts,&num);
   if (num < 2 ) 
     return;
   Expand_wirelist();
   wlist = Cur_block->wlist; 
   wlist[Lastw].numverts = num;
   wlist[Lastw].layerindex = layerindex;
   wlist[Lastw].lineno = Lineno;
   for(i=0; i<num; i++){
      id = insert(verts[i]);
      wlist[Lastw].v_id_list[i] = id;
   }
   Lastw++;
   (Cur_block->numwires)++;
   UpdateExtents(num,verts);
}

/* MakeFace(verts,num,layerindex):
 * Checks the given set of vertices against common AutoCAD errors,
 * such as multiple vertices, vertices on same line and non-planar
 * surfaces. Makes a face structure from the vertex list and adds
 * it to the face list of the current block.
 */ 
void
MakeFace(verts,num,layerindex)
VERTEX verts[];
int num;
int layerindex;
{
   int i,id;
   FACE *flist;
   double normal[4];
   
   /* check for vertices on same line and duplicate adjacent vertices*/
   CheckVertices(verts,&num);
   if(num < 3){
     fprintf(Errorfile,"Face with two vertices ignored;lineno=%d;Layer=%s;\n",
             Lineno,Layertable[layerindex].name);
     return;
   }
   Find_normal(normal,verts, num);
   if (!PlanarTest(verts,num,normal)){
     fprintf(Errorfile,"Nonplanar face (%d) in block %s:lineno=%d;layer=%s\n",
    Lastf,Cur_block->name, Lineno,Layertable[layerindex].name);
     for(i=0;i<num;i++)
       fprintf(Errorfile,"%.3f %.3f %.3f\n",verts[i].x,verts[i].y,verts[i].z);
     if (!FixPlanar(verts,num,normal,layerindex)){
       Tesselate(verts,num,layerindex);
       /*return;*/
     }
   }
   
   Expand_facelist();
   flist = Cur_block->flist; 
  
   flist[Lastf].numverts = num;
   flist[Lastf].check = NOTCHECKED;
   flist[Lastf].flips =0;
   flist[Lastf].oks = 0;
   flist[Lastf].orient = ALL;
   flist[Lastf].error = FALSE; 
   flist[Lastf].layerindex = layerindex;
   flist[Lastf].lineno = Lineno;
   for(i=0;i<num;i++){
      id = insert(verts[i]);
      flist[Lastf].v_id_list[i] = id;
   }
   flist[Lastf].normal[0] = normal[0];
   flist[Lastf].normal[1] = normal[1];
   flist[Lastf].normal[2] = normal[2];
   flist[Lastf].normal[3] = normal[3];
   Lastf++;
   (Cur_block->numfaces)++;

   UpdateExtents(num,verts);
}
