/*vec.c :  vector operation routines */
#include "acad2ug.h"
#define FALSE 0
#define TRUE 1

int
Normalize(v)
VERTEX *v; 
{
   double length;
   length = sqrt(v->x * v->x + v->y * v->y + v->z * v->z);
   if (length == 0.) return(0);
   v->x /= length;
   v->y /= length;
   v->z /= length;
   return(1);
} 

void
Cross(n,w,u)
VERTEX *n,w,u;
{
   n->x  = w.y * u.z -  w.z * u.y;
   n->y  = -w.x * u.z +  w.z * u.x;
   n->z  = w.x * u.y -  w.y * u.x;
}

double Dot(a,b)
VERTEX a,b;
{
   return(a.x * b.x + a.y * b.y + a.z * b.z);
}

double Dot4(a,b)
VERTEX4 a,b;
{
   return(a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w);
}


/*
 *Find_vec(a,b,c);
 *given two end points a and b finds coordinates of a unit vector in the 
 *direction from a to b 
 */
int
Find_vec(a,b,c)
VERTEX a,b,*c;
{  
   double mag;
   c->x = b.x -a.x;
   c->y = b.y -a.y;
   c->z = b.z -a.z;
   mag = sqrt(c->x *c->x + c->y * c->y + c->z * c->z ); 
   if (mag == 0.){
     c->x = c->y= c->z = 0.;
     return(FALSE);
   }
   c->x /= mag;
   c->y /= mag;
   c->z /= mag;
   return(TRUE);
}

/* Given endpoints  of two lines in xyplane , finds their intersection */
int
Intersect(vec,a,b,c,d)
VERTEX *vec,a,b,c,d;
{
   double slope1,slope2;
   if (a.x == b.x){
      if (c.x == d.x)
        return(0);
      vec->x = a.x;
      slope2 = (c.y - d.y) /(c.x - d.x);
      vec->y = slope2 * (a.x - c.x) + c.y ;
   }else if (c.x == d.x){
      vec->x = c.x;
      slope1 = (a.y - b.y) /(a.x - b.x);
      vec->y = slope1 * (c.x - a.x) + a.y ;
   }else {
      slope1 = (a.y - b.y) /(a.x - b.x);
      slope2 = (c.y - d.y) /(c.x - d.x);
      if (slope1 == slope2)
         return(0);
      vec->x = ((slope2 * c.x)-(slope1 * a.x)+a.y-c.y)/(slope2 -slope1);
      vec->y = ((slope2*a.y)-(slope1*c.y)+(slope1*slope2)*(c.x-a.x))/(slope2-slope1);
   }
   return(1);
}
 


/* Find_normal(normal,verts,num)
 * Finds the normal of a plane defined by a set of vertices using
 * the Newman and Sproull algorithm. This normal defines the 
 * coefficients of a plane that comes close to all vertices. 
 */
void
Find_normal(normal,verts,num)
double *normal;
VERTEX verts[];
int num;
{
  double length;
  int i,j;

  normal[0] = normal[1] = normal[2] = normal[3] = 0.;
  for(i=0; i< num;i++){
    j =(i+1) % num;
    normal[0] += (verts[i].y - verts[j].y) * (verts[i].z + verts[j].z); 
    normal[1] += (verts[i].z - verts[j].z) * (verts[i].x + verts[j].x); 
    normal[2] += (verts[i].x - verts[j].x) * (verts[i].y + verts[j].y); 
  }
  length= sqrt(normal[0]*normal[0] + normal[1]*normal[1] + normal[2]*normal[2]);
  if (length == 0.0){
     normal[0] = normal[1] = normal[2] = normal[3] = 0. ;
  }else{ 
     normal[0] /= length;
     normal[1] /= length;
     normal[2] /= length;
     normal[3] = -(normal[0] * verts[0].x + normal[1] * verts[0].y + 
                   normal[2] * verts[0].z);
  }
}
