package navigator;

import java.awt.*;
import java.awt.geom.*;

class Arrow
{
    public static final int NORTH      = 0;
    public static final int WEST       = 1;
    public static final int SOUTH      = 2;
    public static final int EAST       = 3;

    public static final int STRAIGHT   = 4;
    public static final int TURN_RIGHT = 5;
    public static final int TURN_LEFT  = 6;

    public static Polygon[] straightArrow, turnRightArrow, turnLeftArrow;

    static {
	Rectangle r;
	AffineTransform trans;

	straightArrow = new Polygon[4];
	straightArrow[NORTH] = new Polygon();
	straightArrow[NORTH].addPoint(0, 28);
	straightArrow[NORTH].addPoint(12, 40);
	straightArrow[NORTH].addPoint(24, 28);
	straightArrow[NORTH].addPoint(16, 28);
	straightArrow[NORTH].addPoint(16, 0);
	straightArrow[NORTH].addPoint(8, 0);
	straightArrow[NORTH].addPoint(8, 28);
	r                    = straightArrow[NORTH].getBounds();
	trans                = AffineTransform.getTranslateInstance(-r.width/2, -r.height/2);
	straightArrow[NORTH] = getInstance(trans, NORTH, STRAIGHT);

	turnRightArrow = new Polygon[4];
	turnRightArrow[NORTH] = new Polygon();
	turnRightArrow[NORTH].addPoint(0, 0);
	turnRightArrow[NORTH].addPoint(0, 34);
	turnRightArrow[NORTH].addPoint(14, 34);
	turnRightArrow[NORTH].addPoint(14, 40);
	turnRightArrow[NORTH].addPoint(24, 30);
	turnRightArrow[NORTH].addPoint(14, 20);
	turnRightArrow[NORTH].addPoint(14, 26);
	turnRightArrow[NORTH].addPoint(5, 26);
	turnRightArrow[NORTH].addPoint(5, 0);
	r                     = turnRightArrow[NORTH].getBounds();
	trans                 = AffineTransform.getTranslateInstance(-r.width/2, -r.height/2);
	turnRightArrow[NORTH] = getInstance(trans, NORTH, TURN_RIGHT);

	turnLeftArrow = new Polygon[4];
	turnLeftArrow[NORTH] = new Polygon();
	turnLeftArrow[NORTH].addPoint(24, 0);
	turnLeftArrow[NORTH].addPoint(24, 34);
	turnLeftArrow[NORTH].addPoint(10, 34);
	turnLeftArrow[NORTH].addPoint(10, 40);
	turnLeftArrow[NORTH].addPoint(0, 30);
	turnLeftArrow[NORTH].addPoint(10, 20);
	turnLeftArrow[NORTH].addPoint(10, 26);
	turnLeftArrow[NORTH].addPoint(19, 26);
	turnLeftArrow[NORTH].addPoint(19, 0);
	r                    = turnLeftArrow[NORTH].getBounds();
	trans                = AffineTransform.getTranslateInstance(-r.width/2, -r.height/2);
	turnLeftArrow[NORTH] = getInstance(trans, NORTH, TURN_LEFT);

	for(int i = 1; i < 4; i++) {
	    AffineTransform rot   = AffineTransform.getRotateInstance(i*Math.PI/2);
	    straightArrow[i]  = getInstance(rot, NORTH, STRAIGHT);
	    turnRightArrow[i] = getInstance(rot, NORTH, TURN_RIGHT);
	    turnLeftArrow[i]  = getInstance(rot, NORTH, TURN_LEFT);
	}
    };

    public static Polygon getInstance(AffineTransform t, int orientation, int type)
    {
	Polygon[] arrow = straightArrow;
	
	if(type == TURN_RIGHT)
	    arrow = turnRightArrow;
	else if(type == TURN_LEFT)
	    arrow = turnLeftArrow;

	Polygon result = new Polygon();
	float[] pts    = new float[6];

	PathIterator path = arrow[orientation].getPathIterator(t);
	while(!path.isDone()) {
	    path.currentSegment(pts);
	    result.addPoint(Math.round(pts[0]), Math.round(pts[1]));
	    path.next();
	}

	return result;
    }

    public static Rectangle getBounds(int type, int orientation)
    {
	switch(type) {
	case STRAIGHT:
	    return straightArrow[orientation].getBounds();
	case TURN_RIGHT:
	    return turnRightArrow[orientation].getBounds();
	case TURN_LEFT:
	    return turnLeftArrow[orientation].getBounds();
	default:
	    Util.assert(false, "Arrow::getBounds() wrong type: "+type);
	}
	return null;
    }
}


