package navigator;

import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;

class CricketInterface extends LocationManager
    implements Callback
{
    public static final double BEACONDIST_THRESH = 80; // in inches

    public CricketInterface(GUI g, Mesh m, PathPlanner p, String hostname, int port, String method, String beaconTransFile)
    {
	super(g, m, p, beaconTransFile);
	
	BitSet mask = new BitSet();
    	//mask.set(Broker.BEACONSTAT);

	if(method.equals("space")) {
	    mask.set(Broker.SPACE);
	}
	else
	    mask.set(Broker.DEVICECOORD);

	initTrials = 0;

	ServerBroker s = new ServerBroker(hostname, port);
	s.register(this, mask);
	s.start();
    }

    public void callback(CricketData data, BitSet mask)
    {
	//System.out.println("CricketInterface::callback() "+data);

	if(mask.get(Broker.DEVICECOORD)) {
	    Position p = data.getDevicePosition();
	    coordUpdate(p.x, p.y);
	}
    
	/*
	if(mask.get(Broker.SPACE)) {
	    String s = data.getCurrentSpace(); // w/o id
	    System.out.println(s);
	    Position p = (Position) beaconTable.get(s);
	    if(p != null) {
		query.setLocation(p.x, p.y);
		updateLocation();
		gui.updateIcon(getX(), getY(), s);
	    }
	}
	*/

	/* should really verify this (complex) algorithm
	if(mask.get(Broker.BEACONSTAT)) {
	    ArrayList beacons = data.getBeaconsHeard();
	    // find two closest beacons
	    Object[] closestPair = findClosestPair(curPos, 
						 data.getBeaconsHeard());
	    // BeaconRecordz
	    if(closestPair != null) {
		Util.assert(closestPair.length >= 1, "CricketInterface");
		if(closestPair[0] instanceof Point2D) {
		    coordUpdate(((Point2D) closestPair[0]).getX(), 
				((Point2D) closestPair[0]).getY());
		    //updateLocation((Point2D) closestPair[0], 
		    //(Edge) closestPair[1]);
		}
		else {
		    // ...find x, y or cur edge from this information
		    Object[] midEdge = findMidEdge((Edge) closestPair[0], (Edge) closestPair[1]);
		    
		    if(midEdge == null) {
			// both vertices are in the same cell
			Point2D a = (Point2D) closestPair[2];
			Point2D b = (Point2D) closestPair[3];
			coordUpdate(avg(a.getX(), b.getX()), 
				    avg(a.getY(), b.getY()));
			//query.setLocation(avg(a.getX(), b.getX()), 
			//avg(a.getY(), b.getY()));
			//updateLocation( query, (Edge) closestPair[0] );
		    }
		    else {
			if(mesh.shortestPath( curEdge, (Edge) midEdge[0], 
					      distLim, 0 ) != null) {
			    System.out.println("update2: "+midEdge[1]);
			    updateLocation( (Point2D) midEdge[1], (Edge) midEdge[0] );
			}
		    }
		}
	    }
	}
	*/
    }

    private double avg(double a, double b)
    {
	return (a+b)/2;
    }

    private Object[] findMidEdge(Edge s, Edge d)
    {
	Path path = mesh.shortestPath( s, d );
	if(path.size() <= 2) // same cell or adjacent cell
	    return null;

	Object[] result = new Object[2];
	TreeNode t1 = (TreeNode) path.get(path.size()/2);
	result[0] = t1.e[0];  // find the edge first

	path.smooth();
	path.smooth();

	path.printPath();

	t1 = (TreeNode) path.getNodeAtLength(true, path.pathLength()/2);
	TreeNode t2 = (TreeNode) path.get(path.size()/2 - 1);
	
	result[1] = new Point2D.Double(t1.p.getX(), t1.p.getY());
	System.out.println("findMidEdge: "+result[1]);
	return result;
    }

    private Edge getEdge(double x, double y)
    {
	query.setLocation(x, y);
	pointLoc.setQuery(query);
	mesh.bfs( curEdge, pointLoc );
	//System.out.println("query: "+query);
	return pointLoc.getResult();
    }

    // returns the two closest beacons to loc.  If loc
    // is within a threshold value of either one of the closest beacons, return
    // the closest beacon instead.
    private Object[] findClosestPair(Point2D loc, ArrayList beacons)
    {
	Util.assert(beacons.size() >= 1, "CricketInterface::findClosestPair()");
	BeaconRecord br1 = (BeaconRecord) beacons.get(0);
	Position p = (Position) beaconTable.get(br1.space);	
	Edge edge = getEdge(p.x, p.y);
	Util.assert(edge != null, "CricketInterface::findClosestPair() 3");

	Object[] result = new Object[4];
	if(beacons.size() == 1) {
	    result[0] = new Point2D.Double(p.x, p.y);
	    result[1] = edge;
	    result[2] = result[0];
	    return result;
	}

	Position p1, p2;
	double d1, d2;
	p1 = p;
	d1 = br1.distStat.mean;
	BeaconRecord br2 = (BeaconRecord) beacons.get(1);
	p2 = (Position) beaconTable.get(br2.space);
	d2 = br2.distStat.mean;

	System.out.println(br1.space+" "+d1);
	System.out.println(br2.space+" "+d2);

	for(int i = 2; i < beacons.size(); i++) {
	    BeaconRecord br = (BeaconRecord) beacons.get(i);
	    p = (Position) beaconTable.get(br.space);
	    double dist = br.distStat.mean;
	    System.out.println(br.space+" "+dist);
	    if(dist < d1) {
		p1 = p;
		d1 = dist;
		br1 = br;
	    }
	    else if(dist < d2) {
		p2 = p;
		d2 = dist;
		br2 = br;
	    }
	}
	

	System.out.println(br1.space+" "+br2.space);

	if((d1 < BEACONDIST_THRESH) && (d1 <= d2)) {
	    result[0] = new Point2D.Double(p1.x, p1.y);
	    result[1] = getEdge(p1.x, p1.y);
	    result[2] = result[0];
	    return result;
	}
	else if ((d2 < BEACONDIST_THRESH) && (d2 <= d1)) {
	    result[0] = new Point2D.Double(p2.x, p2.y);
	    result[1] = getEdge(p2.x, p2.y);
	    result[2] = result[0];
	    return result;
	}
	
	result[0] = getEdge(p1.x, p1.y);
	result[1] = getEdge(p2.x, p2.y);
	result[2] = new Point2D.Double(p1.x, p1.y);
	result[3] = new Point2D.Double(p2.x, p2.y);


	System.out.println("findClosestPair: "+result[0]);
	System.out.println("findClosestPair: "+result[1]);

	System.out.println("findClosestPair: "+result[2]);
	System.out.println("findClosestPair: "+result[3]);
	return result;
    }

    double X = 0, Y = 0;
    //double velocity = 50*2.54/1000;
    int velocityInv = 8;
    private void coordUpdate(double x, double y)
    {
	if(initTrials < 5) {
	    initCoord(x, y);
	} else if(initTrials == 5) {
	    query.setLocation(x, y);
	    updateLocation();
	    initTrials++;
	}

	query.setLocation(x, y);
	pointLoc.setQuery(query);
	mesh.bfs( curEdge, pointLoc );
	Edge dest = pointLoc.getResult();
	int distLim = (int) (System.currentTimeMillis()-lastUpdate)/velocityInv;
	System.out.println(distLim);
	if(mesh.shortestPath( curEdge, dest, distLim, 0 ) != null) {
	    System.out.println("update: "+query);
	    updateLocation( query, dest );
	}
    }

    int initTrials;
    private void initCoord(double x, double y) 
    {
	if(dist(x, y, X, Y) > 50) {
	    X = x; Y = y;
	    lastUpdate = System.currentTimeMillis();
	}
	else {
	    initTrials++;
	}
    }

    private double dist(double x, double y, double xx, double yy)
    {
	return Math.sqrt((x - xx)*(x - xx) + (y - yy)*(y - yy));
    }
}

