package navigator;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;

public class GUI 
    implements 
	MeshProcessor,
	GraphProcessor,
	MouseListener,
	MouseMotionListener,
	AdjustmentListener,
	ComponentListener
{
    protected DrawingArea drawArea;
    private AffineTransform trans;
    private AffineTransform scale;
    protected AffineTransform totalTransform;

    Mesh mesh;

    float scaleX = (float) 1.0;
    float scaleY = (float) 1.0;

    int maxX, maxY;

    protected Edge curEdge;
    int curMaxX, curMaxY;
    int curX, curY;
    int curWinWidth, curWinHeight;

    JScrollBar vScrollBar, hScrollBar;
    int vScrollWidth, hScrollHeight;
    public CustomFrame f;

    Point2D arrowOrigin;
    ReachablePaths localPaths;
    int arrowOrient;
    int arrowType;

    Path curPath     = null;
    Path end2EndPath = null;

    /* path arrow orientation and type */
    private Point2D pathArrowOrigin;
    private int pathArrowOrient;
    private int pathArrowType;    

    /* used to disable/enable various rendering */
    protected boolean triangulateEnable, e2ePathEnable, 
	localPathEnable, curPathEnable, correctionEnable, 
	wpGraphEnable, coordEnable;

    //protected Controls cntl;
    public Controls cntl; // demohack

    ArrayList spaceLabels;

    // need an api that updates the mesh and labels dynamically
    public GUI(Mesh m, ArrayList l, boolean interactive)
    {
	this((float)m.boundingBox.getX(), 
	     (float)m.boundingBox.getY(), 
	     (float)(m.boundingBox.getX()+m.boundingBox.getWidth()),
	     (float)(m.boundingBox.getY()+m.boundingBox.getHeight()), 
	     m, l, interactive);
    }

    public GUI(float minX, float minY, float maxX, float maxY, 
	       Mesh m, ArrayList l, boolean interactive)
    {
	mesh = m; 
	spaceLabels = l;

	cntl = new Controls(this, interactive);

	arrowOrigin     = new Point2D.Float(0, 0);
	arrowOrient     = -1;
	arrowType       = -1;

	pathArrowOrigin = new Point2D.Float(0, 0);
	pathArrowOrient = -1;
	pathArrowType   = -1;

	totalTransform = new AffineTransform();
	scale = AffineTransform.getScaleInstance(1.0, 1.0);
	trans = AffineTransform.getTranslateInstance(0, 0);

	curMaxX = this.maxX = Math.round(minX + maxX);
	curMaxY = this.maxY = Math.round(minY + maxY);
	
	//curWinWidth  = 452;
	//curWinHeight = 452;

	curWinWidth  = 320;
	curWinHeight = 240;

	curEdge = null;
	curX = (this.maxX-curWinWidth)/2;
	curY = (this.maxY-curWinHeight)/2;
	
	f = new CustomFrame("CricketNav", curWinWidth, curWinHeight);
	f.getContentPane().setLayout(new BorderLayout());

	
	drawArea = new DrawingArea();
	drawArea.addMouseListener(this);
	drawArea.addMouseMotionListener(this);
	
	hScrollBar = new JScrollBar(JScrollBar.HORIZONTAL, curX,
				    curWinWidth, 0, this.maxX);
	vScrollBar = new JScrollBar(JScrollBar.VERTICAL, curY,
				    curWinHeight, 0, this.maxY);
	hScrollBar.addAdjustmentListener(this);
	vScrollBar.addAdjustmentListener(this);

	f.getContentPane().add(cntl, "North");	
	f.getContentPane().add(drawArea, "Center");	
	f.getContentPane().add(hScrollBar, "South");	
	f.getContentPane().add(vScrollBar, "East");	

	f.validate();
	f.pack();
	f.setVisible(true);
	
	vScrollWidth = (int) Math.round(vScrollBar.getSize().getWidth());
	hScrollHeight = (int) Math.round(hScrollBar.getSize().getHeight());

	reSize(curWinWidth, curWinHeight);
	f.addComponentListener(this);
	zoomOut();
	zoomOut();
	zoomOut();
	zoomOut();
    }


    private void reSize(int x, int y)
    {
	x -= vScrollWidth;
	y -= hScrollHeight;

	//	System.out.println(x+" "+y);

	drawArea.setPreferredSize(new Dimension(x, y));
	Image img = f.createImage(x,y);
	Graphics gc = img.getGraphics();
	drawArea.setImageAndGC(img, gc);       
    }
    
    public boolean procGraphEdge(Edge e)
    {
	Edge edge = e;
	do {
	    if(!edge.isMarked(Edge.getBlack())) {
		drawEdge(edge.rot());
		if(edge.sym().next != null && wpGraphEnable)
		    drawEdge(Color.cyan, edge);
	    }
	    edge = edge.oNext();
	} while (edge != e);
	return false;
    }
    
    public void procMeshEdge(Edge e)
    {
	procMeshEdgeHelper(null, e);
	Edge erot = e.rot();
	if(erot.next != null && erot.sym().next != null && wpGraphEnable)
	    procMeshEdgeHelper(Color.cyan, erot);
    }

    Point2D tempP = new Point2D.Double();
    Point2D tempQ = new Point2D.Double();
    Point2D tempR = new Point2D.Double();

    private void procMeshEdgeHelper(Color color, Edge e)
    {
	Point2D p = e.org();
	Point2D q = e.dest();
	
	totalTransform.transform(p, tempP);
	totalTransform.transform(q, tempQ);	

	//e.setPtrans(p);
	//e.sym().setPtrans(q);

	line.setLine(tempP.getX(), tempP.getY(), tempQ.getX(), tempQ.getY());
	if(line.intersects(0, 0, curWinWidth, curWinHeight )) {
	    if(color == null)
		drawEdge(e);
	    else
		drawEdge(color, e);
	}
    }

    

    private final double factor = 0.25;
    public boolean shouldSearch(Edge src, Edge dst)
    {
	Point2D p = dst.org();
	Point2D q = dst.dest();
	
	/* inefficient, as new Point2D objects are created */
	totalTransform.transform(p, tempP);
	totalTransform.transform(q, tempQ);	

	//dst.setPtrans(p);
	//src.setPtrans(q);

	line.setLine(tempP.getX(), tempP.getY(), tempQ.getX(), tempQ.getY());
	// allow search on a larger visibility window...
	return line.intersects(-factor*curWinWidth, -factor*curWinHeight, 
			       (1+factor)*curWinWidth, (1+factor)*curWinHeight);
    }

    public void updateCurPath(Path p) { curPath = p; }
    public void updateE2EPath(Path p) { end2EndPath = p; }
    public void updateLocalPaths(ReachablePaths p) { localPaths = p; }

    public void updateArrow(double x, double y, int orient, int type)
    {
	arrowOrigin.setLocation(x, y);
	arrowType = type;
	arrowOrient = orient;
    }
    
    public void updatePathArrow(double x, double y, int orient, int type)
    {
	pathArrowOrigin.setLocation(x, y);
	pathArrowOrient = orient;
	pathArrowType   = type;
	snapWindowToWorldCoord(pathArrowOrigin);
    }

    public void snapWindowToWorldCoord(Point2D origin)
    {
	scale.setToScale(scaleX, -scaleY);
	trans.setToTranslation(0, curMaxY);
	
	totalTransform.setToIdentity();
	// applies scale first, then trans
	totalTransform.concatenate(trans);
	totalTransform.concatenate(scale);

	totalTransform.transform( origin, tempP );
	curX = (int) Math.round(tempP.getX() - (curWinWidth-vScrollWidth)/2.0);
	curY = (int) Math.round(tempP.getY() - (curWinHeight-hScrollHeight)/2.0);
    }

    private void recalculateTransforms()
    {
	scale.setToScale(scaleX, -scaleY);

	float rX = (float) (curX + curWinWidth/2)/curMaxX;
	float rY = (float) (curY + curWinHeight/2)/curMaxY;

	curMaxX = Math.round(scaleX * maxX);
	curMaxY = Math.round(scaleY * maxY);

	curX = Math.round(rX*curMaxX) - curWinWidth/2;
	curY = Math.round(rY*curMaxY) - curWinHeight/2;

	recalculating = true; 
	// used to consume the AdjustmentEvent
	// induced by the next two methods (setValues())
	hScrollBar.setValues(curX, curWinWidth, 0, curMaxX);
	vScrollBar.setValues(curY, curWinHeight, 0, curMaxY);
	trans.setToTranslation(-curX, curMaxY-curY);

	totalTransform.setToIdentity();
	// applies scale first, then trans
	totalTransform.concatenate(trans);
	totalTransform.concatenate(scale);
    }

    public AffineTransform getTransform() { return totalTransform; }

    //hack
    public static boolean demo = true;

    Point2D iconOrigin = new Point2D.Double();
    String iconLabel;
    public void updateIcon(double x, double y, String label)
    {
	iconOrigin.setLocation(x, y);
	iconLabel = label;
    }

    private ArrayList beacons = null;
    public void updateBeacons(ArrayList b)
    {
	beacons = b;
    }

    public void updateTimer(long delaySinceLastUpdate)
    {
	cntl.updateTimer(delaySinceLastUpdate);
    }

    public void refresh(Edge e)
    {
	curEdge = e;
	if(drawArea == null)
	    return;

	drawArea.flush();
	recalculateTransforms();
	Graphics gc = drawArea.getContext();
	drawChild(gc, curWinWidth, curWinHeight, totalTransform );

	ServiceTime.printElapsedTime("refresh recalcTransforms");


	if(e != null) {
	    // optimize display
	    Util.assert(e.isFaceOriented(), "GUI::refresh()");
	    mesh.bfs(e, this);
	    ServiceTime.printElapsedTime("refresh bfs");
	}
	else {
	    mesh.applyEdges(this);
	    ServiceTime.printElapsedTime("refresh applyEdges");
	}

	if(demo && beacons != null)
	    drawBeacons(Color.green, beacons );

	//if(demo && iconLabel != null)
	//drawIcon(Color.green, iconOrigin, iconLabel);

	if(spaceLabels != null)
	    drawLabels(Color.white, gc, totalTransform);

	ServiceTime.printElapsedTime("refresh drawLabels");

	if(arrowOrient >= 0 && correctionEnable)
	    drawArrow(Color.red, arrowOrigin, arrowOrient, arrowType, 0.75, false);

	if(pathArrowOrient >= 0)
	    drawArrow(Color.yellow, pathArrowOrigin, pathArrowOrient, pathArrowType, 1.0, true);

	if(localPaths != null && localPathEnable) {
	    //	    System.out.println("drawing paths");
	    localPaths.drawPaths( gc, Color.green, 
				  curWinWidth, curWinHeight,
				  totalTransform );
	}
	if(end2EndPath != null) {
	    end2EndPath.drawDest( gc, Color.red, totalTransform );
	    if(e2ePathEnable)
		end2EndPath.drawPath( gc, Color.pink, 
				      curWinWidth, curWinHeight,
				      totalTransform );
	}
	if(curPath != null && curPathEnable) {
	    curPath.drawPath( gc, Color.red, 
			      curWinWidth, curWinHeight,
			      totalTransform );
	}
	if(curPath != null) {
	    //drawDisc( gc, Color.yellow, curPath, totalTransform );
	}
	ServiceTime.printElapsedTime("refresh before rePaint");
	drawArea.repaint();
	ServiceTime.printElapsedTime("refresh after rePaint");
    }

    protected void drawDisc(Graphics gc, Color color, Path path, AffineTransform trans)
    {
	//Point2D origin = ((TreeNode) path.get(path.size()-1)).p;
	// FIXME: reachable paths should be expressed in euclidean distance
	// instead of hop counts.  also, the following should pick a 
	// fixed distance, rather than some relative hop count distance
	Point2D origin = ((TreeNode) path.get(path.size()-1)).p;
	totalTransform.transform(origin, tempP);
	gc.setColor(color);
	gc.fillOval((int)Math.round(tempP.getX()) - 4, 
		    (int) Math.round(tempP.getY()) - 4, 8, 8);

    }

    // used to allow inherited classes to draw their elements
    protected void drawChild(Graphics gc, int curWinWidth,
			     int curWinHeight, AffineTransform trans)
    {
	return;
    }

    Line2D line = new Line2D.Float();

    public void drawEdge(Edge e)
    {
	//Util.assert(!e.isFaceOriented(), "GUI::drawEdge()");
	if(e.isInvalid())
	    drawEdge(Color.white, e);
	else if(e.isDoorJamb())
	    drawEdge(Color.red, e);
	else {
	    if(!triangulateEnable)
		return;
	    drawEdge(Color.blue, e);
	}
    }

    public void drawEdge(Color color, Edge e)
    {
	Graphics gc = drawArea.getContext();
	gc.setColor(color);

	/*
	  Point2D p = e.getPtrans();
	  Point2D q = e.sym().getPtrans();
	  
	  if(p == null) {
	  p = e.org();
	  p = totalTransform.transform(p, null);
	  e.setPtrans(p);
	  }
	  if(q == null) {
	  q = e.dest();
	  q = totalTransform.transform(q, null);	
	  e.sym().setPtrans(q);
	  }
	*/

	Point2D p = e.org();
	Point2D q = e.dest();
	totalTransform.transform(p, tempP);
	totalTransform.transform(q, tempQ);	

	/* debug -- draw the center coordinate */
	if( coordEnable ) {
	  Point2D r = e.rot().org();
	  totalTransform.transform(r, tempR);	
	  
	  gc.drawString(Math.round(r.getX())+" "+Math.round(r.getY()), 
	  Math.round((float)tempR.getX()), 
	  Math.round((float)tempR.getY()));
	}

	gc.drawLine(Math.round((float)tempP.getX()), 
		    Math.round((float)tempP.getY()), 
		    Math.round((float)tempQ.getX()), 
		    Math.round((float)tempQ.getY()));
    }


    AffineTransform arrowScale = AffineTransform.getScaleInstance(1.0, 1.0);

    private void drawArrow(Color color, Point2D origin, int orient, int type, double scale, boolean addPoint)
    {
	AffineTransform translateArrow = 
	    AffineTransform.getTranslateInstance( origin.getX(), origin.getY() );
	AffineTransform transformArrow = (AffineTransform) totalTransform.clone();
	transformArrow.concatenate(translateArrow);

	arrowScale.setToScale(scale, scale);
	transformArrow.concatenate(arrowScale);

	Polygon arrow = Arrow.getInstance(transformArrow, orient, type);

	if(arrow.intersects( 0, 0, curWinWidth, curWinHeight )) {
	    Graphics gc = drawArea.getContext();
	    gc.setColor(color);
	    gc.drawPolygon(arrow);
	    
	    totalTransform.transform(origin, tempP);
	    if(addPoint)
		gc.fillOval((int)Math.round(tempP.getX()) - 3, 
			    (int) Math.round(tempP.getY()) - 3, 6, 6);
	}
    }

    private void drawLabels(Color color, Graphics g, AffineTransform trans)
    {
	SpaceLabel.setFont(trans);
	for(int i = 0; i < spaceLabels.size(); i++) {
	    SpaceLabel l = (SpaceLabel) spaceLabels.get(i);
	    l.draw(color, g, trans);
	}
    }

    private static String FONTNAME = "Arial";
    private static int FONTSTYLE = Font.BOLD;
    private static int FONTSIZE = 18;

    private FontMetrics fontMetrics;
    private Font plain = new Font(FONTNAME, Font.PLAIN, 12);
    private Font bold  = new Font(FONTNAME, FONTSTYLE, FONTSIZE);
    private void setFont(Graphics g, Font f)
    {
	g.setFont(f);
	fontMetrics = g.getFontMetrics();
    }


    private void drawBeacons(Color color, ArrayList l)
    {
	for(int i = 0; i < l.size(); i++) {
	    BeaconLabel bl = (BeaconLabel) l.get(i);
	    drawIcon(color, bl.origin, bl.label);
	}
    }

    private static final int BEACONSIZE = 4;
    private void drawIcon(Color color, Point2D origin, String label)
    {
	Point2D tempP = totalTransform.transform(origin, null);

	Graphics gc = drawArea.getContext();
	//int width = fontMetrics.stringWidth(label);
	int ix    = (int)Math.round(tempP.getX())-BEACONSIZE/2;
	int iy    = (int)Math.round(tempP.getY())-BEACONSIZE/2;
	gc.setColor(color);
	gc.drawString(label, ix, iy);
	gc.fillRect(ix, iy, BEACONSIZE, BEACONSIZE);

	//System.out.println("x: "+origin.getX()+" "+xx+" y: "+origin.getY()+" "+tempP.getY());
	//setFont(gc, plain);
    }


    private boolean mouseDragged;
    private int mouseX, mouseY;

    public void mouseClicked(MouseEvent e) { return; }
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }
    public void mousePressed(MouseEvent e) 
    { 
	mouseDragged = false;
	mouseX = e.getX();
	mouseY = e.getY();
    }
    public void mouseReleased(MouseEvent e) 
    {
	int modifier = e.getModifiers();
	if((modifier & MouseEvent.SHIFT_MASK) > 0) {
	    if(mouseDragged) {
		curX = curX - (e.getX() - mouseX);
		curY = curY - (e.getY() - mouseY);
		refresh(curEdge);
	    }
	    else {
		if((modifier & MouseEvent.BUTTON3_MASK) > 0)
		    zoomOut();
		else
		    zoomIn();
		refresh(curEdge);
	    }
	}
    }
    public void mouseDragged(MouseEvent e) 
    {
	mouseDragged = true;
    }
    public void mouseMoved(MouseEvent e) { return ; }

    public void zoomIn()
    {
	//	System.out.println("zoomIn()");
	if(scaleX < 3) {
	    scaleX += 0.1;
	    scaleY += 0.1;
	}
    }

    public void zoomOut()	
    {
	//	System.out.println("zoomOut()");
	if(scaleX > 0.2) {
	    scaleX -= 0.1;
	    scaleY -= 0.1;
	}
    }

    private boolean recalculating = false;
    public void adjustmentValueChanged(AdjustmentEvent e) 
    {
	//	System.out.println(e.getValue());
	if(recalculating) {
	    recalculating = false;
	    // consume the event thrown by setValues in 
	    // recalculateTransforms()
	    return;
	}
	    
	Object src = e.getSource();
	if(src == vScrollBar)
	    curY = e.getValue();
	else if (src == hScrollBar)
	    curX = e.getValue();
	refresh(null);
    }

    
    public void componentHidden(ComponentEvent e) { return; }
    public void componentMoved(ComponentEvent e) { return; }
    public void componentResized(ComponentEvent e) 
    {
	curWinWidth = (int) e.getComponent().getSize().getWidth();
	curWinHeight = (int) e.getComponent().getSize().getHeight();
	//	System.out.println(curWinWidth+" "+curWinHeight);
	reSize(curWinWidth, curWinHeight);
	refresh(null);
    }
    public void componentShown(ComponentEvent e) { return; }

    /*
      public void addMouseListener(MouseListener l) 
      {
      drawArea.addMouseListener(l);
      }

      public void addMouseMotionListener(MouseMotionListener l) 
      {
      drawArea.addMouseMotionListener(l);
      }
    */

    public void setTriangulate(boolean enable)
    {
	triangulateEnable = enable;
	refresh(curEdge);
    }
    
    public void setE2EPath(boolean enable)
    {
	e2ePathEnable = enable;
	refresh(curEdge);
    }
    
    public void setLocalPath(boolean enable)
    {
	localPathEnable = enable;
	refresh(curEdge);
    }
    
    public void setCurPath(boolean enable)
    {
	curPathEnable = enable;
	refresh(curEdge);
    }
    
    public void setCorrection(boolean enable)
    {
	correctionEnable = enable;
	refresh(curEdge);
    }
    
    public void setWPGraph(boolean enable)
    {
	wpGraphEnable = enable;
	refresh(curEdge);
    }

    public void setCoordinate(boolean enable)
    {
	coordEnable = enable;
	refresh(curEdge);
    }

    private LocationManager locMan;

    public boolean ENABLE_LOCMANMOUSE_DEFAULT = false;

    public void setLocationManager(LocationManager loc)
    {
	locMan = loc;
	setEnableLocManMouseListener(ENABLE_LOCMANMOUSE_DEFAULT);
    }

    public void setEnableLocManMouseListener(boolean enable)
    {
	Util.assert(locMan != null, "GUI::enableLocManMouseListener()");

	// by removing a listener first, we cannot add redundant
	// listeners via this method (i.e., calling this method twice with
	// enable consecutively.
	drawArea.removeMouseListener(locMan);
	drawArea.removeMouseMotionListener(locMan);		    
	
	if(enable) {
	    drawArea.addMouseListener(locMan);
	    drawArea.addMouseMotionListener(locMan);
	}
    }
}
