package navigator;

import java.awt.geom.*;
import java.awt.event.*;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

public class LocationManager extends Thread
    implements MouseListener,
	       MouseMotionListener
{
    final static long REFRESH_PERIOD = 100;

    protected GUI gui;
    protected Mesh mesh;
    protected PathPlanner planner;

    protected NavPoint2D query;
    protected PointLocator pointLoc;

    protected Edge curEdge;
    protected NavPoint2D curPos;

    protected long lastUpdate;

    public Edge getEdge() { return curEdge; }
    public double getX() { return curPos.getX(); }
    public double getY() { return curPos.getY(); }

    Hashtable beaconTable;

    // currently modified to take a .trans file as input
    private ArrayList initTable(Hashtable table, String configFile) 
    {
	InputStream i = ClassLoader.getSystemResourceAsStream(configFile);
	Util.assert(i != null, "CricketInterface::initTable() "+
		    configFile+" not found");
	Properties p = new Properties();
	
	ArrayList result = new ArrayList();

	try {
	    p.load(i);
	}
	catch (IOException io) {
	    io.printStackTrace();
	    Util.assert(false, "CricketInterface::initTable() "+configFile);
	}
	Enumeration entries = p.propertyNames();
	while(entries.hasMoreElements()) {
	    String key = (String) entries.nextElement();
	    System.out.println("fetching key "+key);
	    StringTokenizer entry = 
		new StringTokenizer(p.getProperty(key));

	    entry.nextToken(); // by pass translated beacon name in trans file

	    double x  = Double.parseDouble(entry.nextToken());
	    double y  = Double.parseDouble(entry.nextToken());
	    double z  = Double.parseDouble(entry.nextToken());

	    int delim = key.indexOf("%");
	    String space = key.substring(0,delim);
	    String id = key.substring(delim+1);

	    System.out.println(x+" "+y+" "+z+" "+space+" "+id);
	    table.put(space, new Position(x, y, z));

	    result.add(new BeaconLabel(new Point2D.Double(x, y), space));
	}

	return result;
    }

    public LocationManager(GUI g, Mesh m, PathPlanner p, String beaconTransFile)
    {
	gui = g; mesh = m; planner = p;
	curPos   = new NavPoint2D(0,0);
	pointLoc = new PointLocator();
	query    = new NavPoint2D(0,0);
	curEdge = mesh.currentEdge.rot();
	curPos.setLocation(curEdge.org().getX(), curEdge.org().getY());
	lastUpdate = System.currentTimeMillis();

	beaconTable = new Hashtable();
	
	if(beaconTransFile != null) {
	    ArrayList a = initTable(beaconTable, beaconTransFile);
	    gui.updateBeacons(a);
	}

	//start();
	gui.setLocationManager(this);
    }

    /**
     * If the query point is in valid space, update location.
     */
    public void updateLocation()
    {
	pointLoc.setQuery(query);
	mesh.bfs( curEdge, pointLoc );
	ServiceTime.printElapsedTime("updateLocation bfs end");

	//System.out.println("query: "+query);
	Edge result = pointLoc.getResult();
	if(result != null) {
	    System.out.println("*** FOUND *** "+query);
	    updateLocation( query, result );
	}
    }

    protected void updateLocation(Point2D loc, Edge e)
    {
	curPos.setLocation( loc );
	curEdge = e;
	if (planner != null) {
	    planner.updatePosition(curEdge, curPos.x, curPos.y);
	    ServiceTime.printElapsedTime("planner end");
	}
	lastUpdate = System.currentTimeMillis();
	gui.refresh(curEdge);
    }

    boolean test = false;

    /* for demo only */
    synchronized public void testReset()
    {
	test = false;
	x = 1363;
	y = 975;
	test();
    }

    /* for demo only */
    synchronized public void testEnable()
    {
	try {
	    wait(REFRESH_PERIOD*10);
	} 
	catch(InterruptedException e) {
	}
	test = true;
    }

    synchronized public void run() 
    {
	while(true) {
	    try {
		wait(REFRESH_PERIOD*10);
		gui.updateTimer(System.currentTimeMillis()-lastUpdate);
		if (test)
		    test();
	    } 
	    catch(InterruptedException e) {
		Util.assert(false, "LocationManager interrupted");
	    }
	}
    }

    private boolean mouseDragged;

    private long serviceTime;

    public void mouseClicked(MouseEvent e) { return; }
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }
    public void mousePressed(MouseEvent e) 
    { 
	ServiceTime.printElapsedTime("start timer"); // start timer
	if((e.getModifiers() ^ MouseEvent.BUTTON1_MASK) == 0) {
	    mouseDragged = false;
	    query.setLocation(e.getX(), e.getY());
	    try {
		gui.getTransform().inverseTransform(query, query);
	    } catch (NoninvertibleTransformException ex) {
		Util.assert(false, "QuadEdgeWalker::mousePressed()");
	    }
	    System.out.println("QuadEdgeWalker: "+query);
	}
    }

    //MouseEvent.BUTTON3_MASK
    public void mouseReleased(MouseEvent e) 
    {
	ServiceTime.printElapsedTime("MouseRelased start");
	if(!mouseDragged && 
	   ((e.getModifiers() ^ MouseEvent.BUTTON1_MASK) == 0)) {
	    //System.out.println("QuadEdge mouse "+query+" "+ curEdge);
    	    updateLocation();
	}
	ServiceTime.printElapsedTime("MouseRelased end");
    }

    public void mouseDragged(MouseEvent e) 
    {
	//mouseDragged = true;
    }
    public void mouseMoved(MouseEvent e) { return ; }


    int x = 1363;
    int y = 975;

    protected void test() {
	System.out.println("\n\n x: "+x+" "+y);

	query.setLocation(x, y);

	/*
	  try {
	  gui.getTransform().inverseTransform(query, query);
	  } catch (NoninvertibleTransformException ex) {
	  Util.assert(false, "QuadEdgeWalker::mousePressed()");
	  }
	*/

	ServiceTime.printElapsedTime("MouseRelased start");
	//System.out.println("QuadEdge mouse "+query+" "+ curEdge);
	updateLocation();
	ServiceTime.printElapsedTime("MouseRelased end");

	// demo hack hack hack
	//if (x < 1800)
	if (x < 1760)
	    x += 40;

	//x %= 300;
    }
}



