package navigator;

import java.awt.*;
import java.awt.geom.*;

import java.util.*;

// map matching from space to quad edge
// determine if currently on path
// path planning to the nearest path
class NavArrow
{
    private GUI gui;

    private final int HISTORY = 2500;
    private Path oldPath = null;

    public NavArrow(GUI g)
    {
	gui  = g;
    }

    // Input is a path that leads up to a leaf or an intersection of
    // the end2end path.
    private int prevX   = -1;
    private int prevY   = -1;
    private int prevType   = Arrow.STRAIGHT;
    private int prevOrient = Arrow.NORTH;
    private TreeNode aTreeNode = new TreeNode();
    public void updateArrow(int x, int y, Path path) {
	int type    = Arrow.STRAIGHT;
	int orient  = Arrow.NORTH;
	int leafIndex = 0;

	TreeNode root = (TreeNode) path.get(path.size()-1);
	TreeNode leaf = (TreeNode) path.get(leafIndex);

	// set leaf to the doorway closest to root, 
	// if it exists and if it isn't the root.
	// if root is door, then do not set leaf to the second last
	// as doorway
	for(int i = leafIndex; i < path.size()-1; i++) {
	    TreeNode pt = (TreeNode) path.get(i);
		if(pt.isDoor) {
		    if(i != path.size() - 2 || !root.isDoor) {
			leaf = pt;
			leafIndex = i;
			//System.out.println("leaf Index= "+leafIndex);
		    }
		}
	}

	/* threshold - truncate path only if it's far from doorway, else
	   if curPos in near doorway, let path go through the doorway */
	int pathLengthSqr = root.p.distanceSqInt(leaf.p);
	if(pathLengthSqr >= 10000) {

	    // prune
	    for(int i = 0; i < leafIndex; i++)
		path.remove(0);
	}
	else {
	    leafIndex = 0;
	    leaf = (TreeNode) path.get(leafIndex);
	}


	oldPath = path;
	path = (Path) path.clone();

	aTreeNode.p = new NavPoint2D(x, y);
	path.add(aTreeNode);
	
	// as user is moving along the direction of the path,
	// augment the last link(s) traversed
	if(oldPath != null && pathLengthSqr >= 10000) {
	    int  index = -1;
	    
	    // find intersection with old path
	    for(int i = oldPath.size()-1; i >= 0; i--) {
		if(root.equals(oldPath.get(i))) {
		    index = i;
		    break;
		}
	    }

	    if(index >= 0) {
		for(int i = index+1; i < oldPath.size(); i++) {
		    TreeNode p = (TreeNode) oldPath.get(i);
		    TreeNode c = (TreeNode) path.get(path.size()-1);
		    //System.out.println("p: "+p+" "+root.p.distanceSq(p.p));
		    if(root.p.distanceSqInt(p.p) > HISTORY)
			break;
		    path.add(p);
		}
	    }
	}

	root = (TreeNode) path.get(path.size()-1);

	path.smooth();
	path.smooth();
	path.smooth();
	path.smooth();

	//System.out.println("updateArrow");
	//path.printPath();

	/*
	TreeNode t = maxTriangle( path );
	if( t != null ) {
	    double r = lengthRatio( root, t, leaf );
	    double a = angle( root , t, leaf );
	    System.out.println(r+" "+a);
	    if(r < 1.33 && r > 0.75) {

		if( a < 2*Math.PI/3 ) {
		    boolean rt = rightTurn( root, t, leaf );
		    if(rt) 
			type = Arrow.TURN_RIGHT;
		    else 
			type = Arrow.TURN_LEFT;
		}
	    }
	}
	if(type == Arrow.STRAIGHT)
	    orient = getOrient(root.p, leaf.p);
	else
	    orient = getOrient(root.p, t.p);
	*/

	int or = getTypeOrient(root.p, leaf.p);
	//	System.out.println(or);
	if(or < 4) {
	    type = Arrow.STRAIGHT;
	    orient = or;
	}
	else if(or >= 4) {
	    TreeNode t = path.maxTriangle();
	    if( t != null ) {
		boolean rt = Util.rightTurn( root, t, leaf );
		if(rt) {
		    if(prevType == Arrow.TURN_LEFT) 
			type = Arrow.STRAIGHT;
		    else
			type = Arrow.TURN_RIGHT;
		    //type = Arrow.TURN_RIGHT;
		    if(or < 8)  //SW or NE
			orient = or - 4;
		    else // SE or NW
			orient = or - 8;
		}
		else {
		    if(prevType == Arrow.TURN_RIGHT) 
			type = Arrow.STRAIGHT;
		    else
			type = Arrow.TURN_LEFT;
		    
		    //type = Arrow.TURN_LEFT;

		    if(or < 8) { //SW or NE
			if(or == Arrow.NORTH+4) 
			    orient = Arrow.EAST;
			else 
			    orient = Arrow.WEST;
		    }
		    else { // SE or NW
			if(or == Arrow.WEST+8) 
			    orient = Arrow.NORTH;
			else 
			    orient = Arrow.SOUTH;
		    }
		}
	    }
	    else
		orient = getOrient(root.p, leaf.p);
	    
	    //System.out.println("prevType: "+prevType+" "+orient);
	}
	
	//	System.out.println("update arrow "+orient);
	prevType   = Arrow.STRAIGHT;
	prevOrient = getOrient(prevX, prevY, x, y);
	double diffX = prevX - x; // to space out the error arrow
	double diffY = prevY - y;
	if(prevOrient != orient)
	    gui.updateArrow(prevX + diffX, prevY + diffY, prevOrient, prevType);
	else
	    gui.updateArrow(-1, -1, -1, -1);
	gui.updatePathArrow(x, y, orient, type);
	gui.updateCurPath( path );

	prevX = x;
	prevY = y;
	prevOrient = orient;
	prevType   = type;
    }
    
    public static final double angleThresh0 = Math.tan(Math.PI/8);
    public static final double angleThresh1 = Math.tan(3*Math.PI/8);
    public static int getTypeOrient(NavPoint2D p, NavPoint2D q) 
    {
	float slope = NavPoint2D.slope(p, q);
	//System.out.println(p+" "+q+" "+slope);//+"\n"+e+" "+e.rot());
	//System.out.println(slope+" "+angleThresh0+" "+angleThresh1);
	float aslope = Math.abs(slope);
	if( aslope < angleThresh0  ) {
	    if(p.getX() < q.getX()) {
		//System.out.println("East");
		return Arrow.EAST;
	    } // else
	    //System.out.println("West");
	    return Arrow.WEST;
	}
	else if(aslope < angleThresh1 && slope >= 0 ) {
	    if(p.getX() > q.getX()) {
		//System.out.println("SouthWest");
		return Arrow.SOUTH+4;
	    } // else
	    //System.out.println("NorthEast");
	    return Arrow.NORTH+4;	    
	}
	else if(aslope < angleThresh1 && slope < 0 ) {
	    if(p.getX() > q.getX()) {
		//System.out.println("NorthWest");
		return Arrow.WEST+8;
	    } // else
	    //System.out.println("SouthEast");
	    return Arrow.EAST+8;	    
	}
	else if(aslope > angleThresh1 ) {
	    if(p.getY() < q.getY()) {
		//System.out.println("North");
		return Arrow.NORTH;
	    } // else
	    
	    //System.out.println("South");
	    return Arrow.SOUTH;
	}
	return -1;
    }

    public static int getOrient(NavPoint2D p, NavPoint2D q) 
    {
	return getOrient(p.x, p.y, q.x, q.y);
    }

    public static int getOrient(int x1, int y1, int x2, int y2) 
    {
	float slope = Math.abs(NavPoint2D.slope(x1, y1, x2, y2));
	//System.out.println(p+" "+q+" "+slope);//+"\n"+e+" "+e.rot());
	if( slope < 1 ) {
	    if(x1 < x2) {
		//System.out.println("East");
		return Arrow.EAST;
	    } // else
	    //System.out.println("West");
	    return Arrow.WEST;
	}
	else {
	    if(y1 < y2) {
		//System.out.println("North");
		return Arrow.NORTH;
	    } // else
	    
	    //System.out.println("South");
	    return Arrow.SOUTH;
	}
    }
}
