package navigator;

import java.io.*;
import java.util.*;
import java.awt.geom.*;

import gnu.getopt.*;
import cricketdaemon.CricketDaemon;

public class NavigatorApp
{
    public NavigatorApp(String fileName, String labels, boolean interactive, String method) 
    {
	this(fileName, labels, false, null, 0, method, null);
    }

    public NavigatorApp(String fileName, String labels, boolean interactive, String hostname, int port, String method, String btf)
    {
	// setup map
	Mesh mesh = parseFile(fileName);
	System.out.println("done parsing Mesh");

	ArrayList spaceLabels = SpaceLabel.parse(labels);

	GUI gui  = new GUI(mesh, spaceLabels, interactive);

	// gui.refresh(mesh.currentEdge.rot());
	// Everything below here deals with initializing the
	// path planner and keyboard controls to walk the 
	// spatial graph.  To only render a map of the input,
	// comment everything below here, and uncomment the
	// line above.

	// setup planner
	PathPlanner planner = new PathPlanner(gui, mesh);

	// setup position update mechanism
	LocationManager lm;
	if(interactive)
	    lm = new QuadEdgeWalker(gui, mesh, planner, btf);
	else
	    lm = new CricketInterface(gui, mesh, planner, hostname, port, method, btf);

	// set destination
	//Edge e = setDestination(mesh, planner);
		
	if(method.equals("space")) {
	    Edge e = mesh.currentEdge.rot();
	    gui.demo = true;
	    gui.refresh(e);
	}
	else {
	    Edge e; //= setDestination(mesh, planner, 1868, 1104);
	    if(labels.equals("philip.label"))
		e = setDestination(mesh, planner, 350, 350);
	    else
		e = setDestination(mesh, planner, 1805, 1005);
	    // set gui
	    gui.refresh(e);
	}
	lm.start();
    }

    private Edge setDestination(Mesh mesh, PathPlanner planner)
    {
	//QuadEdge e = mesh.getQuadEdge(3913);
	QuadEdge e = mesh.getQuadEdge(1584);
	
	Util.assert(e != null, "NavigatorApp() invalid destination");
	System.out.println("done getQuadEdge");
	//planner.setDestination(e.edges[1]);
	    
	if(e.edges[3].oNext() != null) {
	    System.out.println("done setdest 3: "+e.edges[3]);
	    planner.setDestination(e.edges[3]);
	}
	else {
	    Util.assert(e.edges[1].oNext() != null, 
			"NavigatorApp() invalid edge for destination");
	    System.out.println("done setdest 1: "+e.edges[1]);
	    planner.setDestination(e.edges[1]);
	}

	return mesh.currentEdge.rot();
    }

    private Edge setDestination(Mesh mesh, PathPlanner planner, 
				int x, int y)
    {
	PointLocator pl = new PointLocator();
	pl.setQuery(new NavPoint2D(x, y));
	mesh.bfs( mesh.currentEdge.rot(), pl );
	
	Edge result = pl.getResult();
	Util.assert(result != null, 
		    "NavigatorApp::setDestination invalid coord: "+x+" "+y);
	//planner.updatePosition(result, x, y);
	if(result.rot() != null)
	    planner.setDestination(result);
	else
	    planner.setDestination(result.sym());
	return result;
    }

    public static Mesh parseFile(String fileName)
    {
	try {
	    FileReader f = new FileReader(fileName);
	    BufferedReader in = new BufferedReader(f);
	   	    
	    Mesh mesh = new Mesh(in);
	    mesh.packMesh();
	    return mesh;
	}
	catch (IOException e) {
	    Util.assert(false, "NavigatorApp() read error: "+fileName);
	}
	return null;
    }
    
    public static void main(String args[]) 
    {
	boolean interactive = false;
	String hostname = "127.0.0.1";
	String filename = "ne43-6.flp";
	String labels   = "ne43-6.label";
	//String beaconTransFile = "ne43-6.trans";
	String beaconTransFile = null;
	String method   = "coord";
	int port = cricketdaemon.server.ServerModule.PORT;

	Getopt g = new Getopt("NavigatorApp", args, "f::h::il::p::s::z");

	int c;
	while((c = g.getopt()) != -1) {
	    switch(c) {
	    case 'f':
		filename = CricketDaemon.getOpt(g, filename);
		break;
	    case 'h':
		hostname = CricketDaemon.getOpt(g, hostname);
		break;
	    case 'i':
		interactive = true;
		break;
	    case 'l':
		labels = CricketDaemon.getOpt(g, labels);
		break;
	    case 'p':
		port = Integer.parseInt(CricketDaemon.getOpt(g, String.valueOf(port)));
		break;
	    case 's':
		method = CricketDaemon.getOpt(g, method);
		break;
	    case 'z':
		filename = "philip.flp";
		labels = "philip.label";
		beaconTransFile = "philip.trans";
		break;
	    default:
		usage();
	    }
	}

	new NavigatorApp(filename, labels, interactive, hostname, port, method, beaconTransFile);
    }

    private static void usage()
    {
	System.err.println("Usage: java navigator.NavigatorApp [-fFLOORPLAN] [-i] [-lLABELSFILE] [-hHOSTADDR] [-pPORT] [-sMETHOD]\n");
	System.exit(1);
    }
}




