package navigator;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

class Path extends ArrayList
{
    private int e2eIntersectIndex = 0;
    private Line2D line;

    public Path()
    {
	super();
	line = new Line2D.Double();
    }

    public void setE2EIndex(int i) { e2eIntersectIndex = i; } 
    public int getE2EIndex() { return e2eIntersectIndex; } 

    Point2D tempP = new Point2D.Double();
    Point2D tempQ = new Point2D.Double();

    public void drawPath( Graphics gc, Color color, int curWinWidth, 
			  int curWinHeight, AffineTransform trans )
    {
	for(int i = 0; i < size()-1; i++) {
	    TreeNode t  = (TreeNode) get(i);
	    TreeNode tp = (TreeNode) get(i+1);

	    /* optimize this later, we cannot use ptrans because bfs
	       have cleared it */

	    //Point2D p  = t.p;
	    //Point2D q  = tp.p;

	    //System.out.print("[[child]] "+t);
	    //	    System.out.println("[[parent]] "+t.parent);

	    trans.transform(t.p, tempP);
	    trans.transform(tp.p, tempQ);	

	    line.setLine(tempP.getX(), tempP.getY(), tempQ.getX(), tempQ.getY());
	    if(line.intersects(0, 0, curWinWidth, curWinHeight )) {
		gc.setColor(color);
		gc.drawLine(Math.round((float)tempP.getX()), 
			    Math.round((float)tempP.getY()), 
			    Math.round((float)tempQ.getX()), 
			    Math.round((float)tempQ.getY()));
	    }
	}
    }

    public void drawDest( Graphics gc, Color color, AffineTransform trans )
    {
	TreeNode t = (TreeNode) get(0);
	
	//Point2D p  = t.p;
	trans.transform(t.p, tempP);

	gc.setColor(color);
	gc.fillOval(Math.round((float)tempP.getX())-5, 
		    Math.round((float)tempP.getY())-5, 10, 10);
    }

    public void smooth()
    {
	if(size() <= 2)
	    return;

	TreeNode treep = (TreeNode) get(0);
	TreeNode last = (TreeNode) get(size()-1);
	for(int i = 1; i < size(); i++) {
	    TreeNode treeq = (TreeNode) remove(i);
	    int xmid = (treep.p.x + treeq.p.x)/2;
	    int ymid = (treep.p.y + treeq.p.y)/2;
	    TreeNode n = new TreeNode();
	    n.p = new NavPoint2D(xmid, ymid);
	    add(i, n);
	    treep = treeq;
	}
	add(last);
    }

    public void printPath()
    {
	for(int i = 0; i < size(); i++) {
	    TreeNode t = (TreeNode) get(i);
	    System.out.println(t.p+" "+t.curDist);
	}
    }

    // Given a vector of leaf to root path of TreeNodes, return the
    // intermediate node that maximizes the area of the triangle formed
    // by the root, leaf, and the intermediate node.
    // Returns null if path length < 3
    public TreeNode maxTriangle()
    {
	if( size() < 3 )
	    return null;

	TreeNode root   = (TreeNode) get(size()-1);
	TreeNode leaf   = (TreeNode) get(0);
	TreeNode result = (TreeNode) get(1);
	double maxArea  = Util.areaSqr( root, result, leaf );
	
	for(int i = 2; i < size()-1; i++) {
	    TreeNode t = (TreeNode) get(i);
	    double a   = Util.areaSqr( root, t, leaf );
	    if( a > maxArea ) {
		maxArea = a;
		result = t;
	    }
	}

	return result;
    }
    
    public Object clone()
    {
	Path result = new Path();
	result.setE2EIndex(e2eIntersectIndex);
	for(int i = 0; i < size(); i++)
	    result.add(get(i));
	return result;
    }

    double pathLength = -1;
    public double pathLength()
    {
	if(size() <= 1)
	    return 0;
	
	double dist = 0;
	TreeNode prev = (TreeNode) get(0);
	for(int i = 1; i < size(); i++) {
	    TreeNode t = (TreeNode) get(i);
	    dist += t.p.distanceSq(prev.p);
	    prev = t;
	}
	pathLength = dist;
	return dist;
    }

    public TreeNode getNodeAtLength(boolean fromBegin, double length)
    {
	if(pathLength < 0)
	    pathLength();
	if(length > pathLength)
	    return null;

	if(fromBegin) {

	    double dist = 0;
	    TreeNode prev = (TreeNode) get(0);
	    TreeNode t = null;
	    for(int i = 1; i < size(); i++) {
		t = (TreeNode) get(i);
		dist += t.p.distanceSq(prev.p);
		if(dist > length)
		    break;
		prev = t;
	    }

	    double lastDist = t.p.distanceSq(prev.p);
	    if(dist - length < lastDist/2)
		return t;
	    else
		return prev;
	}
	else {
	    Util.assert(false, "not yet implemented");
	}
	return null;
    }
    
}

