package navigator;

import java.util.*;

class PathPlanner 
{
    private Mesh mesh;
    private GUI gui;

    private Edge curEdge;
    private int curX;
    private int curY;

    private ReachablePaths reachables;
    private Path curLocalPath;
    private int curLocalPathNum = 0;

    private Path e2ePath = null;
    private NavArrow navArrow;

    /* acts like a smoothing factor */
    public static final int LOCALPATH_RADIUS = 300;
    /* FIXME: this should be adaptable to triangle granularity */

    public PathPlanner(GUI g, Mesh m)
    {
	gui  = g;
	mesh = m;

	curX = -1;
	curY = -1;
	curEdge = null;

	reachables  = new ReachablePaths(); 
	navArrow   = new NavArrow(g);

	Edge curEdge = mesh.currentEdge.rot();
	updatePosition(curEdge, curEdge.org().x, curEdge.org().y);
    }

    // naive implementation!!!
    private void pathMatch( ReachablePaths r, Path e2e )
    {
	int result = -1;
	double maxDist = 0;

	/*
	  System.out.println("e2e: ");
	  e2e.printPath();
	  System.out.println("local: ");
	  System.out.println(curEdge);
	  for(int i = 0; i < r.numPaths(); i++) {
	  Path path = (Path) r.getPath(i);
	  System.out.println("pathNum="+i);
	  path.printPath();
	  }
	*/

	for(int i = 0; i < r.numPaths(); i++) {
	    Path path = r.getPath(i);
	    for(int j = 0; j < path.size(); j++) {
		TreeNode p = (TreeNode) path.get(j);
		for(int k = e2e.size()-1; k >= 0; k--) {
		    TreeNode q = (TreeNode) e2e.get(k);
		    if(p.equals(q)) {
			r.setPathIntersection(i, j);
			if(maxDist <= q.curDist) {
			    result   = i;
			    maxDist  = q.curDist;
			}
			j = path.size();
			break;
		    }
		}
	    }
	}

	/*
	  System.out.println("pathMatch "+result);
	  if(result >= 0)
	  ((Path)r.getPath(result)).printPath();
	*/

	curLocalPathNum = result;
    }

    public void setDestination(Edge dest)
    {
	if(dest != null) {
	    // ask the server for an e2e path
	    Util.assert(dest.isFaceOriented(), 
			"PathPlanner::setDestination() invalid dest "+dest);
	    e2ePath = mesh.shortestPath(curEdge, dest);
	    Util.assert(e2ePath != null, 
			"PathPlanner::setDestination() search failed");
	    gui.updateE2EPath( e2ePath );
	}
	else {
	    e2ePath = null;
	}
    }

    public void updatePosition(Edge e, int x, int y)
    {
	if(x != curX || y != curY) {
	    curX = x;
	    curY = y;
	    if(curEdge != e) {
		curEdge = e;
		ServiceTime.printElapsedTime("updatePosition");
		
		mesh.shortestPath( curEdge , LOCALPATH_RADIUS, 4 );

		ServiceTime.printElapsedTime("updatePosition 2");
		reachables.extractPaths( curEdge.treeNode );

		ServiceTime.printElapsedTime("updatePosition 3");
		if(e2ePath != null ) {
		    pathMatch( reachables, e2ePath );
		    ServiceTime.printElapsedTime("updatePosition 3.1");
		    if(curLocalPathNum < 0) {
			// user has wondered off the current path
			TreeNode dest = (TreeNode) e2ePath.get(0);
			setDestination( dest.e[0] );
			ServiceTime.printElapsedTime("updatePosition 3.2");
			pathMatch( reachables, e2ePath );
			ServiceTime.printElapsedTime("updatePosition 3.3");
		    }
		}
		else
		    curLocalPathNum = 0;
		Util.assert(curLocalPathNum >= 0,
			    "PathPlanner::setDestination() c");
		getLocalPath(); // updates curLocalPath
		ServiceTime.printElapsedTime("updatePosition 4");
		gui.updateLocalPaths(reachables);
		ServiceTime.printElapsedTime("updatePosition 5");
	    }
	    
	    // FIXME: should not use the same curLocalPath 
	    // see room 640A
	    navArrow.updateArrow(curX, curY, curLocalPath);
	    ServiceTime.printElapsedTime("updatePosition 6");
	}
    }
    
    public void toggleCurLocalPath(boolean next)
    {
	if(reachables.numPaths() <= 0)	    
	    return;
	
	if(next)
	    curLocalPathNum  = (curLocalPathNum + 1) % reachables.numPaths();
	else
	    curLocalPathNum  = (curLocalPathNum > 0) ? 
		curLocalPathNum - 1 : reachables.numPaths() - 1;
	
	navArrow.updateArrow(curX, curY, getLocalPath());
    }
    
    public Path getLocalPath()
    {
	Path p            = reachables.getPath(curLocalPathNum);
	int wayPointIndex = reachables.getPathIntersection(curLocalPathNum);
	curLocalPath = new Path();
	for(int i = wayPointIndex; i < p.size(); i++)
	    curLocalPath.add(p.get(i));
	System.out.println(curLocalPath.size());
	return curLocalPath;
    }
}

