package navigator;

import java.awt.geom.*;

class PointLocator 
    implements GraphProcessor
{
    Edge result;
    NavPoint2D query;
    
    public PointLocator()
    {
	result = null;
	query  = null;
    }

    public void setQuery(NavPoint2D q) 
    { 
	result = null;
	query  = q; 
    }
    public Edge getResult() { return result; }

    public boolean shouldSearch(Edge src, Edge dst)
    {
	return dst.isValid();
    }

    // assumes triangulated subdivision
    public boolean procGraphEdge(Edge e)
    {
	Util.assert(e.isFaceOriented(), "PointLocator::isInFace()");
	Util.assert(query != null, 
		    "PointLocator::procGraphEdge() forgot to setQuery?");
	/*
	  Edge t = e;
	  do {
	  if(Util.rightOf(query, t.rot()))
	  return false;
	  t = t.oNext();
	  } while (t != e);
	*/
	if(e.isInFace(query.x, query.y)) {
	    //System.out.println("inface: "+e+"\nsym: "+e.sym());

	    // perf: this is too costly, commented out!
	    //result = closestEdge(e);
	    result = e;
	    return true;
	}
	else 
	    return false;
    }
    
    private Edge closestEdge(Edge e)
    {
	Edge t = e.rot();
	double a = Line2D.ptSegDistSq(t.org().getX(),
				      t.org().getY(),
				      t.dest().getX(),
				      t.dest().getY(),
				      query.getX(), query.getY());
	double b = Line2D.ptSegDistSq(t.dest().getX(),
				      t.dest().getY(),
				      t.lPrev().org().getX(),
				      t.lPrev().org().getY(),
				      query.getX(), query.getY());
	double c = Line2D.ptSegDistSq(t.lPrev().org().getX(),
				      t.lPrev().org().getY(),
				      t.org().getX(),
				      t.org().getY(),
				      query.getX(), query.getY());
	switch(Util.imin(a, b, c)) {
	case 0:
	    return e;
	case 1:
	    return t.lNext().invRot();
	case 2:
	    return t.lPrev().invRot();
	}
	return null;
    }

    public static void main(String[] args)
    {
	Mesh m = NavigatorApp.parseFile(args[0]);
	PointLocator pl = new PointLocator();
	pl.setQuery(new NavPoint2D(700, 715));
	m.bfs(m.currentEdge.rot(), pl);
	System.out.println(pl.getResult());
    }
}

// procGraphEdge returns result
// result contained in point locator

// simulation to application backend...
