package navigator;

import java.util.*;
import java.awt.geom.*;

class QuadEdge
{
    public static final int VALID    = 0;
    public static final int INVALID  = 1;
    public static final int DOORJAMB = 2;

    String id;
    int type;
    
    Edge[] edges;

    public QuadEdge(String id, int type, String[] quadID, int[] quadEdgeID,
		    float[] x, float[] y)
    {
	this.id = id; 
	this.type = type; 
	
	edges = new Edge[4];
	for(int i = 0; i < 4; i++)
	    edges[i] = new Edge(this, i, quadID[i], quadEdgeID[i], x[i], y[i]);
	
    }

    public static QuadEdge getQuadEdge(Hashtable table, int id)
    {
	return (QuadEdge) table.get(Integer.toString(id));
    }

    public void insertTable(Hashtable edgeTable, Hashtable danglingTable)
    {
	Util.assert(!edgeTable.containsKey(id), "Mesh::parseAnEdge()");
	edgeTable.put(id, this);
	
	Vector v;
	for(int i = 0; i < 4; i++) {
	    if(edges[i].org().isInvalid())
		continue;

	    QuadEdge e = (QuadEdge) edgeTable.get(edges[i].quadID);
	    if(e != null) // found QuadEdge
		edges[i].next = e.edges[edges[i].quadEdgeID];
	    else {
		v = (Vector) danglingTable.get(edges[i].quadID);
		if(v == null) {
		    v = new Vector();
		    danglingTable.put(edges[i].quadID, v);
		}
		else {
		    Util.assert(!v.contains(this), 
				"QuadEdge::insertTable()");
		}
		v.add(this);
	    }
	}
	
	// resolve dangling edges that is missing this as reference
	v = (Vector) danglingTable.get(id);
	if(v != null) {
	    for(int i = 0; i < v.size(); i++) {
		QuadEdge e = (QuadEdge) v.get(i);
		e.linkMissingEdge(this);
	    }
	    danglingTable.remove(id);
	}
    }

    public void linkMissingEdge(QuadEdge e)
    {
	for(int i = 0; i < 4; i++) {
	    if(edges[i].quadID.equals(e.id))
		edges[i].next = e.edges[edges[i].quadEdgeID];
	}
    }

    public boolean equals(Object o)
    {
	return ((QuadEdge)o).id.equals(id);
    }
}
