package navigator;

import java.util.*;
import java.awt.event.*;
import java.awt.geom.*;

public class QuadEdgeWalker extends LocationManager
    implements KeyListener
{
    public static final int STEP = 20;
    private boolean autoRefresh;
    
    public QuadEdgeWalker(GUI g, Mesh m, PathPlanner p, String btf)
    {
	super(g, m, p, btf);

	gui.f.addKeyListener(this);
	autoRefresh = true;
	test();
	gui.cntl.lm = this; // demohack
    }

    public QuadEdgeWalker(GUI g, Mesh m, PathPlanner p, boolean ar, String btf)
    {
	this(g, m, p, btf);
	autoRefresh = ar;
    }

    public void keyReleased(KeyEvent e) { return; }
    public void keyTyped(KeyEvent e) { return; }

    public void keyPressed(KeyEvent e)
    {
	int k = KeyEvent.VK_UNDEFINED;
	if(e.isActionKey())
	    k = e.getKeyCode();
	if(k == 0)
	    k = (int) Character.toLowerCase(e.getKeyChar());
	//System.out.println("key typed "+k);

	switch(k) {
	case KeyEvent.VK_UP:
	    step(Arrow.NORTH, true);
	    if (planner != null) 
		planner.updatePosition(curEdge, curPos.x, curPos.y);
	    break;
	case KeyEvent.VK_DOWN:
	    step(Arrow.SOUTH, true);
	    if (planner != null) 
		planner.updatePosition(curEdge, curPos.x, curPos.y);
	    break;
	case KeyEvent.VK_LEFT:
	    step(Arrow.WEST, true);
	    if (planner != null) 
		planner.updatePosition(curEdge, curPos.x, curPos.y);
	    break;
	case KeyEvent.VK_RIGHT:
	    step(Arrow.EAST, true);
	    if (planner != null) 
		planner.updatePosition(curEdge, curPos.x, curPos.y);
	    break;
	case ',':
	    if (planner != null) 
		planner.toggleCurLocalPath(false);
	    break;
	case '.':
	    if (planner != null) 
		planner.toggleCurLocalPath(true);
	    break;
	default:
	    return;
	}

	lastUpdate = System.currentTimeMillis();
	if(autoRefresh)
	    gui.refresh(curEdge);
    }

    private void step(int dir, boolean forward)
    {
	int targetX = curPos.x;
	int targetY = curPos.y;

	System.out.println("targetX "+targetX);
	System.out.println("targetY "+targetY);
	
	if(dir == Arrow.EAST)
	    targetX += ((forward) ? 1 : -1)*STEP;
	else if(dir == Arrow.WEST)
	    targetX += ((forward) ? -1 : 1)*STEP;
	else if(dir == Arrow.NORTH)
	    targetY += ((forward) ? 1 : -1)*STEP;
	else// if(dir == Arrow.SOUTH)
	    targetY += ((forward) ? -1 : 1)*STEP;

	while(!stepTarget(targetX, targetY)) {
	    // targetX and targetY does not lie in a well defined
	    // triangle, so backtrack by one in the appropriate direction
	    if(dir == Arrow.EAST)
		targetX += (forward) ? -1 : 1;
	    else if(dir == Arrow.WEST)
		targetX += (forward) ? 1 : -1;
	    else if(dir == Arrow.NORTH)
		targetY += (forward) ? -1 : 1;
	    else// if(dir == Arrow.SOUTH)
		targetY += (forward) ? 1 : -1;
	    
	}
    }

    // returns true if targetX and targetY lies within a well defined triangle, where
    // well-defined means that the target point lies completely inside the triangle.
    // curEdge is updated to point to one of the dual edges of the triangle containing
    // the target point and curPos is updated to the target point.

    // returns false otherwise (ie the target point lies on one of the triangle edges)
    // In this case, curEdge and curPos are not updated.
    private boolean stepTarget( int targetX, int targetY )
    {
	Edge fromEdge = null;
	boolean done  = false;

	Edge e = curEdge;	
	// the following is commented out because if curPos.x, curPos.y is exactly on 
	// an edge of a face, then it fails. In this case, Line2D.linesIntersect had
	// better indicate that the curPos point is intersecting that edge.  Otherwise,
	// bad things will happen (like the navarrow will get stuck!)
	// REALLY PROBLEMATIC HERE!!!
	//Util.assert(e.isInFace(curPos.x, curPos.y), "QuadEdgeWalker::stepForward() "+e);
	while(!done) {
	    Edge iterStart = e;
	    done = true;
	    do {
		Util.assert(e != null, "QuadEdgeWalker::stepForward() null");

		Point2D eRotOrg  = e.rot().org();
		Point2D eRotDest = e.rot().dest();
		
		//double ***
		boolean inter = 
		    Line2D.linesIntersect(curPos.getX(), curPos.getY(), 
					  (double)targetX, (double)targetY,
					  eRotOrg.getX(), eRotOrg.getY(),
					  eRotDest.getX(), eRotDest.getY());
		if(inter) {
		    //System.out.println("intersection");
		    if(!e.isValid() || e.oNext() == null) {
			//			System.out.println("invalid detected");
			// there is an intersection with an invalid edge
			// so target is not reached!!!
			return false;
		    }
		    
		    if(e.sym() != fromEdge) {
			//			System.out.println("forward edge");
			fromEdge = e;
			e = e.sym();
			done = false; // found exit edge
			break;
		    }
		}
		e = e.oNext();
	    } while( e != iterStart );
	}

	if(e.isInFace(targetX,targetY)) {
	    //	System.out.println("update "+e);
	    curEdge = e;
	    curPos.setLocation(targetX, targetY);
	    return true;
	}
	
	return false;
    }
}
