package navigator;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;

class ReachablePaths
{
    private ArrayList localPaths; // vector of vector of TreeNodes

    public ReachablePaths()
    {
	localPaths = new ArrayList();
    }

    public void extractPaths(TreeNode t)
    {
	localPaths.clear();
	extractPaths_helper(t);
	// assumes uninitialized values are zero
	Util.assert(localPaths.size() > 0, "ReachablePaths::extractPaths()");
    }

    private void extractPaths_helper(TreeNode t)
    {
	if( t.children.size() == 0 ) {
	    Path v = new Path(); // first element becomes the leaf
	    do {
		v.add(t);
		t = t.parent;
	    } while (t != null);
	    localPaths.add(v);
	}
	else {
	    for(int i = 0; i < t.children.size(); i++)
		extractPaths_helper((TreeNode) t.children.get(i));
	}
    }

    public int numPaths() { return localPaths.size(); }
    public Path getPath(int i) { return (Path) localPaths.get(i); }
    public int getPathIntersection(int i) 
    { 
	int result = ((Path)localPaths.get(i)).getE2EIndex();
	return result;
    }

    public void setPathIntersection(int pathIndex, int wayPointIndex)
    {
	Path p = (Path) localPaths.get(pathIndex);
	p.setE2EIndex(wayPointIndex);
    }


    public void drawPaths(  Graphics gc, Color color, int curWinWidth, 
			     int curWinHeight, AffineTransform trans )
    {
	for(int i = 0; i < localPaths.size(); i++)
	    ((Path) localPaths.get(i)).drawPath(gc, color, 
						curWinWidth, curWinHeight, 
						trans);
    }
}


