package navigator;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;

public class SpaceLabel
{
    public static class Label
    {
	public static final int HORIZONTAL = 0;
	public static final int VERTICAL   = 1;
	
	String label;
	float x, y;
	int or;

	Point2D p = new Point2D.Float();
	Point2D P = new Point2D.Float();
	
	
	/*
	public Label (String s, float x, float y, int or) 
	{
	    //label = s; this.x = x; this.y = y; this.or = or;
	}
	*/

	public void draw(Graphics g, AffineTransform trans)
	{
	    // fix orientation later.
	    trans.transform(p, P);
	    ((Graphics2D) g).drawString(label, (float) P.getX(), (float) P.getY());
	}

	public String toString()
	{
	    return label+" "+x+" "+y+" "+or;
	}
    }

    public static final Font masterFont = new Font("Arial", Font.PLAIN, 12);
    static Font font;

    public static void setFont(AffineTransform trans) {
	font = masterFont.deriveFont(trans);
    }

    public static ArrayList parse(String inputFile) {
	BufferedReader in;
	ArrayList result = new ArrayList();
	try {
	    in = new BufferedReader(new FileReader(inputFile));
	    
	    String s = null;
	    while((s = in.readLine()) != null) { // ref line
		Label space = parseSpace(in);
		Label type  = parseLabel(in);
		Label misc  = parseLabel(in);
		Label size  = parseLabel(in);
		Point2D start  = parsePoint(in);
		Point2D finish = parsePoint(in);
		Line2D line = null;
		if(start != null && finish != null)
		    line = new Line2D.Double(start, finish);
		in.readLine(); // blank line

		result.add(new SpaceLabel(space, type, misc, size, line));
	    }
	}
	catch (Exception io) {
	    Util.assert(false, 
			"LabelParser() cannot open file: "+inputFile);
	}
	return result;
    }	


    private static Label parseSpace(BufferedReader in) 
	throws IOException
    {
	
	Label fl = parseLabel(in);
	Label rm = parseLabel(in);

	fl.label += rm.label;
	return fl;
    }

    private static Label parseLabel(BufferedReader in) 
	throws IOException    
    {
	Label result = new Label();

	String input = in.readLine();
	//System.out.println(input);
	StringTokenizer s = new StringTokenizer(input, ":=");

	s.nextToken();  // type
	s.nextToken();  // l

	String label = s.nextToken();

	if(label.equals("x"))
	    return null;

	label = label.substring(0, label.indexOf("x")-1);
	//System.out.println(label);

	result.label = label;
	parseCoord(result, s);
	
	return result;
    }

    private static Point2D parsePoint(BufferedReader in) 
	throws IOException
    {
	
	String input = in.readLine();
	//System.out.println(input);
	StringTokenizer s = new StringTokenizer(input, ":=");
	s.nextToken();  // type
	s.nextToken();  // :

	String sx = s.nextToken();
	sx = sx.substring(0, sx.indexOf("y")-1);
	float x = Float.parseFloat(sx);

	
	String sy = s.nextToken();
	sy = sy.substring(0, sy.indexOf("z")-1);
	float y = Float.parseFloat(sy);

	float z = Float.parseFloat(s.nextToken());
	
	if(x != 0 || y != 0 || z != 0)
	    return new Point2D.Float(x, y);
	else
	    return null;
    }
	

    private static void parseCoord(Label l, StringTokenizer s) 
	throws IOException
    {
	String sx = s.nextToken();
	sx = sx.substring(0, sx.indexOf("y")-1);
	float x = Float.parseFloat(sx);

	
	String sy = s.nextToken();
	sy = sy.substring(0, sy.indexOf("z")-1);
	float y = Float.parseFloat(sy);

	float z = Float.parseFloat(s.nextToken());
	
	l.p.setLocation(x, y);
    }

    Label space = null;
    Label type = null;
    Label misc = null;
    Label size = null;
    Line2D arrow = null;

    public SpaceLabel(Label space, Label type, Label misc, Label size,
		      Line2D arrow) {
	this.space = space;
	this.type  = type;
	this.misc  = misc;
	this.size  = size;
	this.arrow = arrow;
    }
	

    Point2D lineP = new Point2D.Double();
    Point2D lineQ = new Point2D.Double();
    
    public void draw(Color color, Graphics g, AffineTransform trans) {
	//Font originalFont = g.getFont();
	//g.setFont(font);

	g.setColor(color);
	if (space != null) space.draw(g, trans);
	if (type != null) type.draw(g, trans);
	if (misc != null) misc.draw(g, trans);
	if (size != null) size.draw(g, trans);
	if (arrow != null) {
	    trans.transform(arrow.getP1(), lineP);
	    trans.transform(arrow.getP2(), lineQ);
	    g.drawLine((int)lineP.getX(), (int)lineP.getY(), 
		       (int)lineQ.getX(), (int)lineQ.getY());
	}
	//g.setFont(originalFont);
    }

    public String toString() {
	return "s:"+space+" t:"+type+" m:"+misc+" s:"+size+" a:"+arrow;
    }


    public static void main(String[] args) {
	ArrayList a = SpaceLabel.parse(args[0]);

	for(int i = 0; i < a.size(); i++) {
	    System.out.println(a.get(i));
	}
    }
	
}



