package navigator;

import java.util.*;
import java.awt.geom.*;

public class Util 
{
    public static boolean debugMode = false;

    public static void dbgPrintln(String s) 
    {
	if(debugMode) 
	    System.out.println(s);
    }
    
    public static void assert(boolean pred, String errMsg) 
    {
	try {
	    if(!pred)
		throw new Exception();
	}
	catch (Exception e) {
	    System.err.println("ASSERTION FAILED--> "+errMsg);
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    public static boolean rightTurn(TreeNode a, TreeNode b, TreeNode c)
    {
	NavPoint2D n = a.p;
	NavPoint2D p = b.p;
	NavPoint2D q = c.p;

	return rightTurn(n, p, q);
    }

    public static int det(NavPoint2D n, NavPoint2D p, NavPoint2D q)
    {
	int A =(q.x-p.x)*(n.y-p.y);
	int B =(q.y-p.y)*(n.x-p.x);
	return A - B;
    }

    public static boolean rightTurn(NavPoint2D n, NavPoint2D p, NavPoint2D q)
    {
	/* returns true if n, p, q makes a right turn */
	int det = det(n, p, q);

	//System.out.println(A+" "+B+" "+det);
	//Util.assert(det != 0, "NavArrow::rightTurn() determinant is zero "+
	//n+" "+p+" "+q);
	return det < 0;	
    }

    public static boolean leftTurn(NavPoint2D n, NavPoint2D p, NavPoint2D q)
    {
	/* returns true if n, p, q makes a right turn */
	int det = det(n, p, q);
	return det > 0;	
    }

    // Returns TRUE if the point x is strictly to the right of e
    public static boolean rightOf(NavPoint2D x, Edge e)
    {
	return leftTurn(x, e.dest(), e.org());
    }

    public static boolean leftOf(NavPoint2D x, Edge e)
    {
	return rightTurn(x, e.dest(), e.org());
    }

    // computes the ratio of the length AB to length BC
    public static double lengthRatio(TreeNode a, TreeNode b, TreeNode c)
    {
	System.out.println("lengthRatio");
	double A = a.p.distance(b.p);
	double B = b.p.distance(c.p);
	
	return A/B;
    }

    // computes the angle ABC in range 0--PI
    public static double angle(TreeNode a, TreeNode b, TreeNode c)
    {
	System.out.println("angle");
	double A = a.p.distanceSq(b.p);
	double B = b.p.distanceSq(c.p);
	double C = c.p.distanceSq(a.p);

	return Math.acos((A + B - C)/(2*Math.sqrt(A*B)));
    }

    // returns the area squared of triangle formed by a, b, c
    public static int areaSqr(TreeNode a, TreeNode b, TreeNode c)
    {
	//System.out.println("areaSqr");
	int A = a.p.distanceInt(b.p);
	int B = b.p.distanceInt(c.p);
	int C = c.p.distanceInt(a.p);
	
	int S = (A+B+C)/2;

	return S*(S-A)*(S-B)*(S-C);
    }
  
    public static int imin(double a, double b, double c)
    {
	return (a < b) ? (a < c ? 0 : 2)
	    : (b < c ? 1 : 2);
    }

    public static String textOf(double d, int places) 
    {
	String s = String.valueOf(d);
	int endIndex = s.length();
	int decIndex = s.indexOf(".");
	if(decIndex >= 0)
	    endIndex = min(endIndex, decIndex + places + 1);
	
	return s.substring(0, endIndex);
    }

    public static int min(int a, int b)
    {
	if(b < a)
	    return b;
	return a;
    }

}
