/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package pathfinder;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * GUI for the PathFinderApp.
 *
 * @author Allen Miu
 */
public class DestinationGUI
{
    protected JFrame f;
    protected StatusBar status;

    class StatusBar extends JPanel {
	
	private JLabel status;
	
	public StatusBar () {
	    super();
	    setLayout(new FlowLayout());
	    status = _addInfoLabel(" status:", "IDLE");
	}
	
	private JLabel _addInfoLabel(String title, String value) 
	{
	    JLabel t = new JLabel(title, JLabel.LEFT);
	    add(t);
	    
	    if( value != null ) {
		JLabel v = new JLabel(value, JLabel.RIGHT);
		add(v);
		return v;
	    }
	    return null;
	}

	public void setBusy()
	{
	    status.setText("Querying Server for destinations...");
	}

	public void setIdle()
	{
	    status.setText("IDLE");
	}
    }

    static private JList destList;
    static private PathFinderApp pathfinder;

    static MouseListener selectDestListener = new MouseAdapter() {
	    public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
		    int index = destList.locationToIndex(e.getPoint());
		    Waypoint w = (Waypoint) destList.getModel().getElementAt(index);
		    System.out.println("Destination selected: "+w.id);
		    pathfinder.changeDestination(w);
		}
	    }
	};
    
    public DestinationGUI(PathFinderApp p, GUI g, JList dList) 
    {
	pathfinder = p;
	f = new CustomFrame("Destinations", g, 220, 160);
	Container container = f.getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));

	destList = dList;

	JButton update = new JButton("UPDATE");
	update.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    Util.dbgPrintln("UPDATE clicked");
		    setStatusBusy(true);
		    pathfinder.queryDestinations();
		    setStatusBusy(false);
		}
	    });
	container.add(update);

	JScrollPane scroll = new JScrollPane(destList);
	scroll.setPreferredSize(new Dimension(220, 120));
	container.add(scroll);

	status = new StatusBar();
	status.setBorder(BorderFactory.createLoweredBevelBorder());
	container.add(status);

	f.validate();
	f.pack();
	f.setVisible(true);

	Util.dbgPrintln("view created and exposed");
    }

    /* public API for program */

    public void setStatusBusy(boolean busy) {
	if(busy)
	    status.setBusy();
	else
	    status.setIdle();
    }
}



