/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package pathfinder;

import cricketdaemon.reader.sim.*;
import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;
import java.util.*;
import javax.swing.table.*;

/**
 * An indoor navigation application.
 *
 * @author Allen Miu
 */
class PathFinderApp implements Callback, Runnable
{
    static ArrayList configFile;
    GUI gui;

    public PathFinderApp(cricketdaemon.clientlib.Broker c)
			 //guidedog.clientlib.Broker g)
    {
	cricket = c;
	//guidedog = g;
	//queryDestinations();
	gui = new GUI("PathFinder", this);
	path = null;
	//callback(new Position(30, 95, 0));
    }

    public void run()
    {
	while(true) {
	    try {
		synchronized(this) {
		    wait(15000);
		    update();
		}
	    } catch (InterruptedException ie) {
		
	    }
	}
    }

    /* Cricket Client */
    cricketdaemon.clientlib.Broker cricket;
    BeaconRecord currentBeacon;
    Position currentCoord;
    synchronized public void callback(CricketData data, BitSet mask)
    {
	if(mask.get(Broker.DEVICECOORD)) {
	    currentCoord = data.getDevicePosition();
	    gui.setX(currentCoord.x);
	    gui.setY(currentCoord.y);
	    gui.setZ(currentCoord.z);
	}
	if(mask.get(Broker.BEACONSTAT))
	    currentBeacon = data.getCurrentSpaceObject();
	
	/* cricket can issue a callback to ask for another
	   queryDestinations? */
	notify();
    }

    synchronized public void callback(Position pos)
    {
	currentCoord = pos;
	gui.setX(pos.x);
	gui.setY(pos.y);
	gui.setZ(pos.z);
	notify();
    }

    /* GUI */
    public synchronized void changeDestination(Waypoint dst) 
    {
	// just initialize something for the time being

	//destination = dst;
	destination = dst;
	path = null;
	notify();
    }


    /* Guide Dog */

    //guidedog.clientlib.Broker guidedog;
    // query guidedog for a list of possible destinations
    Vector destinations; 
    Waypoint destination;
    Vector path;

    // interface to guidedog/GUI
    private void queryPath(Position src, Waypoint dst) 
    {
	Util.dbgPrintln("PathFinderApp::queryPath()");
	gui.setQueryPathStatus(true);
	// make up a path!
	path = new Vector();
	if(configFile != null) {
	    for(int i = 1; i < configFile.size()-1; i++)
		path.add(configFile.get(i));
	}
	path.add(dst);
	gui.setQueryPathStatus(false);
    }

    public void queryDestinations()
    {
	destinations = new Vector();
	// make up a list of destinations
	if(configFile != null) 
	    destinations.add( configFile.get(configFile.size()-1) );
	gui.setDestinations(destinations);
    }

    private void update() 
    {
	if(path == null) {
	    // need to handle when current is not avail
	    if(currentCoord != null &&
	       destination != null)
		queryPath(currentCoord, destination);
	    else
		return;
	}
	
	int targetIndex = getCurWaypointIndex();

	/* did device stray off current target? */
	/* may need to change coordinate system here */
	if(strayedOff(currentCoord, targetIndex, path)) {
	    queryPath(currentCoord, destination);
	    targetIndex = getCurWaypointIndex();
	}

	/* did device reach current target? */
	int nextIndex = -1;
	if(targetIndex != path.size() - 1) // desination not reached
	    nextIndex = targetIndex+1;
	gui.update(currentCoord, path, targetIndex, nextIndex);
    }

    private double distSquared(Position a, Position b) 
    {
	double dx = a.x - b.x;
	double dy = a.y - b.y;
	return dx*dx + dy*dy;
    }

    // fixme: how to handle teleportation
    private int getCurWaypointIndex()
    {
	double min = Double.MAX_VALUE;
	int pathIndex = -1;

	/* FIXME: should be at least
	   the closest beacon of the same space!!! */
	for(int i = 0; i < path.size(); i++) {
	    Waypoint t = (Waypoint) path.get(i);
	    /* may need to change coordinate system here */
	    double distSq = distSquared(currentCoord, t.pos);
	    System.out.println(distSq);
	    if( distSq < min ) {
		pathIndex = i;
		min = distSq;
	    }
	}
	Util.dbgPrintln("PathFinderApp::getCurWaypointIndex() "+pathIndex);
	return pathIndex;
    }

    private final double THRESH_SQUARED = 100.0;

    private boolean strayedOff(Position cur, int targetIndex, Vector path)
    {
	Waypoint curTarget = (Waypoint) path.get(targetIndex);
	double curTargetDist = distSquared(cur, curTarget.pos);

	Waypoint nextTarget = null;
	if(targetIndex != path.size() - 1)
	    nextTarget = (Waypoint) path.get(targetIndex+1);
	else if(targetIndex > 0)
	    nextTarget = (Waypoint) path.get(targetIndex-1);
	// else targetIndex == 0 == path.size() -1 

	if(nextTarget == null)
	    return curTargetDist > THRESH_SQUARED;
	double targetNextDist = distSquared(curTarget.pos, nextTarget.pos);
	return curTargetDist > targetNextDist;
    }

    public static void main(String[] args) 
    {
	
	if(args[0].equals("-s")) {
	    configFile = new ArrayList();
	    SimReader.initPath(configFile, "pathfinder.conf");
	}

	cricketdaemon.clientlib.Broker c = 
	    new cricketdaemon.clientlib.ServerBroker();
	/*
	  guidedog.clientlib.Broker g =
	  new guidedog.clientlib.ServerBroker();
	*/

	BitSet mask = new BitSet();
	mask.set(Broker.BEACONSTAT);
	mask.set(Broker.DEVICECOORD);

	PathFinderApp pf = new PathFinderApp(c);//, g);
	c.register(pf, mask);
	c.start();

	Thread t = new Thread(pf);
	t.start();

	try { t.join(); } catch (InterruptedException ie) { }
	c.interrupt();
    }
}



