package positiondemo;

import java.io.*;
import java.awt.*;
import java.util.*;

class Map {
    int offsetX, offsetY;
    float scaleX, scaleY;

    ArrayList beacons;
    ArrayList lines;
    ArrayList ovals;

    Map(String fileName) {
	beacons = new ArrayList();
	lines   = new ArrayList();
	ovals   = new ArrayList();
	try {
	    FileReader f = new FileReader(fileName);
	    BufferedReader in = new BufferedReader(f);
	   	    
	    parseScale(in);
	    parseBeacons(in);
	    parseLines(in);
	    parseOvals(in);
	}
	catch (IOException e) {
	    Util.assert(false, "positiondemo.Map() read error: "+fileName);
	}
	
    }

    private void parseScale(BufferedReader in)
	throws IOException
    {
	String line = in.readLine();
	StringTokenizer s = new StringTokenizer(line);

	String x = s.nextToken();
	
	// FIXME: an ulgy hack:  should define a better input file format!
	if(x.toLowerCase().equals("frame")) {
	    GUI.width  = Integer.parseInt(s.nextToken());
	    GUI.height = Integer.parseInt(s.nextToken());
	    s = new StringTokenizer(in.readLine());
	    x = s.nextToken();
	}

	String y = s.nextToken();
	offsetX  = Integer.parseInt(x);
	offsetY  = Integer.parseInt(y);

	x = s.nextToken();
	y = s.nextToken();
	scaleX = Float.parseFloat(x);
	scaleY = Float.parseFloat(y);	
    }

    private void parseBeacons(BufferedReader in)
	throws IOException
    {
	String line = null; 
	while ((line = in.readLine()) != null) {
	    StringTokenizer s = new StringTokenizer(line);
	    if(s.countTokens() != 3)
		break;
	    int x = parseX(s);
	    int y = parseY(s);
	    String label = s.nextToken();
	    beacons.add(new BeaconLabel(x, y, label));
	}
    }

    private int parseX(StringTokenizer s) 
    {
	return transformX(Float.parseFloat(s.nextToken()));
    }

    private int parseY(StringTokenizer s) 
    {
	return transformY(Float.parseFloat(s.nextToken()));
    }
	
    public int transformX(float x)
    {
	return Math.round(x/scaleX)+offsetX;
    }


    public int transformY(float y)
    {
	return Math.round(y/scaleY)+offsetY;
    }


    private void parseLines(BufferedReader in)
	throws IOException
    {
	String line = null; 
	while ((line = in.readLine()) != null) {
	    StringTokenizer s = new StringTokenizer(line);
	    if(s.countTokens() != 4)
		break;
	    int x = parseX(s);
	    int y = parseY(s);
	    int xx = parseX(s);
	    int yy = parseY(s);
	    lines.add(new Line(x, y, xx, yy));
	}
    }

    private void parseOvals(BufferedReader in)
	throws IOException
    {
	String line = null; 
	while ((line = in.readLine()) != null) {
	    StringTokenizer s = new StringTokenizer(line);
	    if(s.countTokens() != 4)
		break;
	    int x = parseX(s);
	    int y = parseY(s);
	    int width = parseX(s)-offsetX;
	    int height = parseY(s)-offsetY;
	    ovals.add(new Oval(x, y, width, height));
	}
    }

    public void draw(Graphics gc)
    {
	gc.setColor(Color.red);
	for(int i = 0; i < beacons.size(); i++)
	    ((BeaconLabel) beacons.get(i)).draw(gc);
	gc.setColor(Color.green);
	for(int i = 0; i < lines.size(); i++)
	    ((Line) lines.get(i)).draw(gc);
	for(int i = 0; i < ovals.size(); i++)
	    ((Oval) ovals.get(i)).draw(gc);
    }
}
