/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package samplemon;

import gnu.getopt.*;
import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;

import javax.swing.*;
import java.awt.*;
import java.util.*;

import cricketdaemon.UnitConv;

/**
 *
 * @author Allen Miu
 */
public class SampleMonApp extends Thread
    implements Callback, Runnable
{
    // screen size
    static final int WIDTH  = 1000;
    static final int HEIGHT  = 300;

    private long timeLastUpdate = 0;
    private static final long REFRESH_PERIOD = 200;

    private JPanel panel;
    private Subplot plotter;
    Broker cricket;

    public static boolean useMetric = true;

    public SampleMonApp(Broker c, double history, double maxy, boolean sm)
    {
	panel = new JPanel();
	GridBagLayout gridbag = new GridBagLayout();
        panel.setLayout(gridbag);	

	plotter = new SampleMonSubplot(panel, 0, history,
					  "Beacon Samples", 
					  "Distance (cm)", 
					  "Time (s)", maxy, sm);
	cricket = c;
	BitSet mask = new BitSet();
	mask.set(Broker.BEACONSTAT);
	mask.set(Broker.CALCBEACONDIST);
	cricket.register(this, mask);

	start();
    }

    public JPanel getPanel() { return panel; }

    public void setHistory(double history) { plotter.setHistory(history); }

    synchronized public void callback(CricketData data, BitSet mask)
    {
	if(mask.get(Broker.BEACONSTAT)) {
	    Iterator it = data.getBeaconsHeard().iterator();
	    while(it.hasNext()) {
		BeaconRecord br = (BeaconRecord) it.next();
		Iterator its = br.distStat.samples.iterator();
		//System.out.println(br.space+" "+br.distStat.samples.size());
		
		while(its.hasNext()) {
		    Sample s = (Sample) its.next();
		    //System.out.println("\t"+ s);
		    plotter.process(br.space, s.time, 
				    UnitConv.cricketDistToCM(s.dist));
		}
	    }
	}
	flush();
    }

    synchronized public void run()
    {
	while(true) {
	    try {
		wait(REFRESH_PERIOD);
	    } catch (InterruptedException ie) {
		
	    }
	    flush();
	}
    }

    synchronized private void flush()
    {
	long curTime = System.currentTimeMillis();
	if((curTime - timeLastUpdate) >= REFRESH_PERIOD) {
	    plotter.flush();
	    timeLastUpdate = curTime;
	}
    }

    public static void usage()
    {
	System.err.println("Usage: java samplemon.SampleMonApp [-H history_in_ms][-S][-y max_y_axis_cm]\n");
	System.exit(1);
    }

    public static void main(String[] args) 
    {
	Getopt g = new Getopt("SampleMonApp", args, "Sy:H:");
	double history = 4000; // in miliis
	double maxy    = 500; // 5 meters
	boolean scanMode = true;

	int c;
	try {
	    while((c = g.getopt()) != -1) {
		switch(c) {
		case 'S':
		    scanMode = false;
		    break;
		case 'y':
		    maxy    = Double.parseDouble(g.getOptarg());
		    break;
		case 'H':
		    history = Double.parseDouble(g.getOptarg());
		    break;
		default:
		    usage();
		}
	    }
	}
	catch (Exception e) {
	    usage();
	}

	Broker s = new ServerBroker();
	SampleMonApp sma = new SampleMonApp(s, history, maxy, scanMode);
	CustomFrame frame = new CustomFrame("Sample Monitor", WIDTH, HEIGHT);
	frame.getContentPane().add(sma.getPanel());
	frame.finishInit();

	s.start();
	try { sma.join(); } catch (InterruptedException ie) { }
	s.interrupt();
    }
}
