package samplemon;

import java.util.*;
import javax.swing.*;
import ptolemy.plot.*;

import java.awt.*;

class SampleMonSubplot extends Subplot
{
    ArrayList srcstr, lastUpdate;
    private long syslastUpdate = 0;
    private double inittime = -1;

    private boolean scanMode;

    //private final double TICKINTERVAL = 0.250; // in seconds
    private final double TICKINTERVAL = 1.0; // in seconds

    public SampleMonSubplot(Container cont, int y, 
			    double defaultHistory, String title, 
			    String ylab, String xlab, double maxy,
			    boolean sm) 
    {
	super(cont, y, defaultHistory);
	setTitle(title);
	setYLabel(ylab);
	setXLabel(xlab);
	setYRange(0, maxy); 
	setImpulses(true);
	srcstr     = new ArrayList();
	lastUpdate = new ArrayList();
	scanMode   = sm;

	if(scanMode) {
	    setXRange(0, history);
	    adjustXTicks();
	    setXLabel(xlab+" (1 Tick = "+TICKINTERVAL+" seconds)");
	}
    }

    public void process(String beaconid, long time, double value)
    {
	if(inittime < 0)
	    inittime = time;

	time -= inittime;

	int set = getSet(beaconid, time);
	if(set >= 0) {
	    super.addPoint(set, time, value);
	}
	
	syslastUpdate = System.currentTimeMillis();
    }

    // must be in same units as in the data set (i.e. milliseconds)
    // this may change for different beacon timing (sigh).
    protected double xOffset()
    {
	if(syslastUpdate > 0) {
	    long result = System.currentTimeMillis() - syslastUpdate;
	    return result;
	}
	else {
	    return 0;
	}
    }

 
    // fixes x tick mark to label current and lag of 
    // history window seconds
    // The xrange in the data set is in millisecond whereas the TICK marks
    // are in seconds
    protected void adjustXTicks()
    {
	double[] range = getXRange();

	// The following computation ensures that the rightmost tick mark
	// is always 0.  addXTick expects to be called with increasing 
	// currange value.  If currange is decreasing, it won't work.
	int n = (int) Math.floor((range[1]-range[0])/(1000*TICKINTERVAL));
	double xtick    = -n*TICKINTERVAL;
	double currange = range[1] + (xtick * 1000);

	// undocumented ptplot method that I added to plot.jar
	// See PlotBox.java after unjarring plot.jar (jar xvf plot.jar)
	clearXTick(false); 

	while(currange <= range[1]) {
	    //System.out.println(n+" "+xtick +" "+currange);
	    String label = "";
	    if(!scanMode)
		label = Double.toString(xtick);

	    addXTick(label, currange);
	    currange += (TICKINTERVAL * 1000);
	    xtick += TICKINTERVAL;
	}
    }

    private void erasePoints(int dataset, double x)
    {
	Vector pts = (Vector)_points.elementAt(dataset);

	int indexToDelete  = -1;
	double rightThresh = (x+0.05*history) % history;

	if(rightThresh > x) {
	    for(int i = 0; i < pts.size(); i++) {
		PlotPoint old = (PlotPoint)(pts.elementAt(i));
		if ((old.originalx > x) && (old.originalx <= rightThresh))
		    indexToDelete = i;
	    }
	}
	else {
	    for(int i = 0; i < pts.size(); i++) {
		PlotPoint old = (PlotPoint)(pts.elementAt(i));
		if ((old.originalx > x) || (old.originalx <= rightThresh))
		    indexToDelete = i;
	    }
	}

	for (int i = 0; i <= indexToDelete; i++)
	    super.erasePoint(dataset, 0);
    }

    synchronized public void addPoints()
    {
	if(!scanMode) {
	    super.addPoints();
	    return;
	}

	Iterator it = pendingPoints.iterator();
	double lastX = Double.NEGATIVE_INFINITY; 
    
	while(it.hasNext()) {
	    Point p  = (Point) it.next();
	    double x = p.x % history;
	    if(p.x > lastX)
		lastX = p.x;
	    super.addPoint(p.set, x, p.y, JOIN);	    
	}

	if(lastX > Double.NEGATIVE_INFINITY) {
	    for(int n = 0; n < getNumDataSets(); n++)
		erasePoints(n, lastX % history);
	}
	pendingPoints.clear();
	
	repaint();
	
    }

    private int getSet(String src, long time)
    {
	int index;

	index = srcstr.indexOf(src);
	
	if(index < 0) {
	    srcstr.add(src);
	    lastUpdate.add(new Long(time));
	}
	else {
	    long t = ((Long) lastUpdate.get(index)).longValue();

	    if(time <= t)
		return -1;
	    lastUpdate.set(index, new Long(time));
	}

	return Mapping.setMap(srcstr, src);
    }

}
