package samplemon;

import ptolemy.plot.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

class Subplot extends EditablePlot
{
    protected ArrayList pendingPoints;
    
    protected double history;
    protected static boolean JOIN = false; // join points with line

    public class Point
    {
	public int set;
	public double x, y;
	
	public Point(int set, double x, double y) 
	{
	    this.set = set; this.x = x; this.y = y;
	}
    }

    public void setHistory(double h) { 
	history = h; 
	setXPersistence(h);
    }

    public Subplot(Container cont, int y, double defaultHistory) 
    {
        GridBagLayout gridbag = (GridBagLayout) cont.getLayout();
        GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
        c.gridy = y;
	c.gridwidth  = 1;
	c.gridheight = 1;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1.0;
	c.weighty = 1.0;
        gridbag.setConstraints(this, c);
        cont.add(this);

	setHistory(defaultHistory);

	setMarksStyle("dots");
	setXLabel("Time");
	setButtons(false);
	setGrid(false);

	pendingPoints = new ArrayList();
    }

    public void addPoint(int set, double x, double y)
    {
	pendingPoints.add(new Point(set, x, y));
    }

    public void flush()
    {
	//notify();
	addPoints();
    }

    // adjust height
    private void adjustYScale(boolean redraw)
    {
	double[] range = getXRange();
	double ymax    = -Double.MAX_VALUE;
	double ymin    = Double.MAX_VALUE;
	double margin  = 0.075;

	for(int c = 0; c < getNumDataSets(); c++) {
	    double [][] data = getData(c); // assume sorted
	    for(int n = 0; n < data[0].length; n++) {
		if(data[0][n] >= range[0]) {
		    ymax = Math.max(ymax, data[1][n]);
		    ymin = Math.min(ymin, data[1][n]);
		    
		    if(data[0][n] > range[1])
			break;
		    
		    //System.out.println(c+" "+data[0][n]+" "+data[1][n]);
		}
	    }
	}
	
	margin *= ymax-ymin;

	//setYRange(ymin-margin, ymax+margin);
	setYRange(0, ymax+margin);
	//System.out.println(margin+" "+ymin+" "+ymax+" "+getNumDataSets());
	if(redraw)
	    repaint();
    }

    // called by addPoints to adjust XTicks
    protected void adjustXTicks()
    {
	return;
    }

    // called by addPoints to adjust X scale offset
    protected double xOffset()
    {
	return 0;
    }

    /* fixme: should check whether the input source is still alive */
    synchronized public void addPoints()
    {
	Iterator it = pendingPoints.iterator();
	double lastX = Double.NEGATIVE_INFINITY; 
	
	while(it.hasNext()) {
	    Point p = (Point) it.next();
	    if(p.x > lastX)
		lastX = p.x;
	    super.addPoint(p.set, p.x, p.y, JOIN);
	}
	pendingPoints.clear();

	double x = lastX + xOffset();
	if(x >= 0)
	    setXRange(x - history, x);
	
	//adjustYScale(false);
	adjustXTicks();

	repaint();
    }

    synchronized public void clear()
    {
	super.clear(false);
	pendingPoints.clear();
    }

    public void process(String id, long time, double val)
    {
	return;
    }

    public static void main(String[] args) 
    {
	CustomFrame fr = new CustomFrame("Test Subplot", 640, 480);
	GridBagLayout gridbag = new GridBagLayout();
        fr.getContentPane().setLayout(gridbag);	
	
	Subplot sb1 = new Subplot(fr, 0, 4000);
	Subplot sb2 = new Subplot(fr, 1, 4000);
	
	fr.finishInit();
	//sb1.start();
	//sb2.start();

	sb1.addPoint(0, 1, 1);
	sb1.addPoint(0, 2, 2);
	sb1.flush();
	sb2.addPoint(0, 1, 1);
	sb2.addPoint(0, 2, 2);
	sb2.flush();
    }
}
