/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package spacedemo;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.*;
import cricketdaemon.clientlib.*;

/**
 * GUI for the ViewFinderApp.
 *
 * @author Allen Miu
 */
public class GUI extends JPanel
{
    protected JFrame f;

    Image im;
    Graphics imgc;

    private static String FONTNAME = "Arial";
    private static int FONTSTYLE = Font.BOLD;
    private static int FONTSIZE = 28;

    private static final int x = 220;
    private static final int y = 310;
    private static Font font;

    private FontMetrics fontMetrics;

    private void drawString(String s, int x, int y) 
    {
	int width  = fontMetrics.stringWidth( s );
	int height = fontMetrics.getHeight();
	int xx    = x-width/2;
	imgc.drawString(s, xx, y);
    }

    public GUI(String title, ArrayList b) 
    {
	beaconCoords = b;

	preferredSize = new Dimension(x,y);
	setBorder(createBorder());

	
	f = new CustomFrame(title);
	Container container = f.getContentPane();
	container.setLayout(new BorderLayout());
	container.add(this, "Center");

	f.setBackground(Color.white);
	f.validate();
	f.pack();
	f.setVisible(true);

	im = createImage(x, y);
	imgc = (Graphics) im.getGraphics();

	font = new Font(FONTNAME, Font.PLAIN, FONTSIZE);
	imgc.setFont(font);
	fontMetrics = imgc.getFontMetrics();

	Util.dbgPrintln("view created and exposed");
    }

    private ArrayList beaconCoords = null;

    private String highlightSpaceBeacon = null;

    public void highlightSpaceBeacon(String beaconName)
    {
	if (highlightSpaceBeacon == null || 
	    !highlightSpaceBeacon.equals(beaconName)) {
	    highlightSpaceBeacon = beaconName;
	    repaint();
	}
    }

    private void _drawBeacons()
    {
	if( beaconCoords != null && imgc != null ) {
	    imgc.clearRect(0, 0, this.x, this.y);
	    for(int i = 0; i < beaconCoords.size(); i++) {
		Coord p = (Coord) beaconCoords.get(i);
		int ix = (int)p.x;
		int iy = (int)p.y;		
		//imgc.clearRect(boundX, boundY, boundSize, boundSize); 
		//System.out.println(highlightSpaceBeacon);
		//System.out.println(p.name);
		
		// erase underline
		imgc.setColor(Color.white);
		System.out.println(p.name);
		imgc.setFont(font);
		fontMetrics = imgc.getFontMetrics();
		System.out.println(fontMetrics);

		int width = fontMetrics.stringWidth( p.name );
		int xx0   = ix -width/2;
		int xx1   = ix +width/2;
		imgc.drawLine(xx0, iy + 1, xx1, iy + 1);
		//updateBoundingBox(xx0, iy+1, xx1, iy+1);		
		
		if(p.name.equals(highlightSpaceBeacon)) {
		    imgc.setColor(Color.red);
		    // underline 
		    imgc.drawLine(ix - width/2, iy + 1, ix + width/2, iy + 1);
		    System.out.println("underlining "+p.name);
		}
		else
		    imgc.setColor(Color.black);

		drawString(p.name, ix, iy);
	    }
	}	
	imgc.setColor(Color.black);
    }

    public void paintComponent(Graphics g) 
    {
        super.paintComponent(g);
	if( im != null ) {
	    _drawBeacons();
	    g.drawImage(im,0,0,null);
	}
    }

    Dimension preferredSize;
    public Dimension getPreferredSize() 
    {
	return preferredSize;
    }

    private Border createBorder() {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }
}
