/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package spacedemo;

import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;
import cricketdaemon.clientlib.sim.*;
import java.io.*;
import java.util.*;


/**
 * Implements the ViewFinder application presented in the
 * Mobicom 2001 paper.  Not yet completed.
 *
 * @author Allen Miu
 */
class SpaceDemo implements Callback
{
    public static Random r;


    // update occurs only if orientation changes by this angle threshold    
    // static final double delta = Math.PI/16.0; 

    String currentSpace;

    static final long TIMEOUT = 10000; // used to fire an update when no events come in 

    GUI gui;
    Broker cricket;

    public SpaceDemo(String file)
    {
	ArrayList beacons = parseFile(file);
	for(int i = 0; i < beacons.size(); i++)
	    System.out.println(beacons);
	gui = new GUI("SpaceDemo", beacons);
	gui.repaint();
    }

    private ArrayList parseFile(String fileName) 
    {
	try {
	    FileReader f = new FileReader(fileName);
	    BufferedReader in = new BufferedReader(f);
	    
	    String line = null; 
	    ArrayList beacons = new ArrayList();
	    while ((line = in.readLine()) != null) {
		StringTokenizer s = new StringTokenizer(line);
		if(s.countTokens() != 3)
		    break;
		int x = Math.round(Float.parseFloat(s.nextToken()));
		int y = Math.round(Float.parseFloat(s.nextToken()));
		String label = s.nextToken();
		beacons.add(new Coord(x, y, label));
	    }
	    
	    in.close();
	    return beacons;
	}
	catch (IOException ioe) {
	    Util.assert(false, "SpaceDemo::parseFile foobar");
	}
	return null;
    }

    /* should map BEACONID to BEACON COORD, etc. */
    synchronized public void callback(CricketData data, BitSet mask)
    {
	if(mask.get(Broker.SPACE)) {
	    /* SpaceModule */
	    BeaconRecord cs = data.getCurrentSpaceObject();
	    if(cs != null && cs.uniqueName != null) {
		int j = cs.uniqueName.lastIndexOf("-");
		if (j > 0)
		    currentSpace = cs.uniqueName.substring(0,j);
		else 
		    currentSpace = cs.uniqueName;
		System.out.println("Current space is "+currentSpace);
		gui.highlightSpaceBeacon(currentSpace);
	    }
	}
    }

    public static void main(String[] args) 
    {
	Broker s = new ServerBroker();

	BitSet mask = new BitSet();
	mask.set(Broker.SPACE);

	String file = "spacedemo.dat";

	if (args.length > 0)
	    file = args[0];

	SpaceDemo sd = new SpaceDemo(file);
	s.register(sd, mask);
	s.start();
	try { s.join(); } catch (InterruptedException ie) { }
    }
}
