package streamredirector;

import java.util.*;

/**
 * This specialized hash table makes sure that when connection state objects are removed,
 * their interrupt() method is called to ensure the timer threads stop running.
 *
 * @see java.util.Hashtable, streamredirector.ConnectionState
 * @author Jorge Rafael Nogueras
 */

public class ConnectionStateHashtable extends Hashtable
{
  /** this variable sets whether or not debug println's are printed */
  protected static final boolean DEBUG = false;

	/**
	 * This method overloads the remove() method in Hashtable; it removes the item mapped to
	 * "key" and returns it. If said object is of type ConnectionState, it will call its
	 * "interrupt()" method to stop its thread.
	 *
	 * @param key the object that needs to be removed
   * @return the object that was removed
	 */
	
	public Object remove (Object key)
	{
		Object obj = super.remove (key);
		
		try
		{
			// objects in this hash table should always be of type ConnectionState, but in case
			// something goes wrong with the following cast, the error will be caught silently and
			// the object removed will be returned as normal
			ConnectionState state = (ConnectionState) obj;
			
			if (state != null)
			{
        if (DEBUG)
          System.out.println ("ConnectionStateHashtable::remove -> stopping timer in connection state: " + state.cookie);

				state.stop();
			}
		}
		catch (Exception e) { e.printStackTrace(); }
			
		return obj;
	}
}