package streamredirector;

/**
 * This class holds the state that the Controller must keep in order to manage its
 * connections. Whenever the Controller establishes a connection, it keeps a reference
 * to all the pertinent variables in this object and stores this object in a hashtable, so
 * it can be looked up later.
 *
 * @see streamredirector.Controller
 * @author Jorge Rafael Nogueras
 */

public class ControllerConnectionState
{
  /** the cookie that uniquely identifies this connection */
  public Long cookie;
  
  /** the server information object */
  public ServerInfo serverInfo;

  /** the sink information object */
  public SinkInfo sinkInfo;

  /** the reference to the server */
  public ServerConnection server;

  /** the reference to the sink */
  public SinkConnection sink;

  /** the transport over which we can send commands to the sink proxy */
  public Transport transport;
  
  /** the thread that is in charge of renewing the connection's lease */
  public ConnectionLeaseRenewer renewer;
  
  /**
   * Initializes the controller connection state with the appropriate parameters
   *
   * @param the_cookie the cookie that uniquely identifies this connection
   * @param the_serverInfo the server information object
   * @param the_sinkInfo the sink information object
   * @param the_server the reference to the server
   * @param the_sink the reference to the sink
   * @param the_transport the transport over which we can send commands to the sink proxy
   * @param the_renewer the thread that is in charge of renewing the connection's lease
   */

  public ControllerConnectionState (Long the_cookie, ServerInfo the_serverInfo, SinkInfo the_sinkInfo,
              ServerConnection the_server, SinkConnection the_sink, Transport the_transport,
              ConnectionLeaseRenewer the_renewer)
  {
    cookie = the_cookie;
    serverInfo = the_serverInfo;
    sinkInfo = the_sinkInfo;
    server = the_server;
    sink = the_sink;
    transport = the_transport;
    renewer = the_renewer;
  }
  
  /**
   * Prints a useful description of this object.
   *
   * @return a String with a useful description of this object
   */

  public String toString()
  {
    return "[" + this.getClass().getName() + ": cookie = " + cookie + ", server info = " + serverInfo +
            ", sink info = " + sinkInfo + ", server = " + server + ", sink = " + sink +
            ", transport = " + transport + ", lease renewer = " + renewer + "]";
  }
}