package streamredirector;

import java.util.*;

import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.data.*;

/**
 *
 * @see ins.api.Application, ins.namespace.NameSpecifier, streamredirector.Controller
 * @author Jorge Rafael Nogueras
 */

public class CricketPhysicalLocationManager extends LocationUpdateRegistrar implements PhysicalLocationManager, Callback
{
  /** this variable sets whether or not debug println's are printed */
  protected static final boolean DEBUG = Controller.DEBUG;
  
  /** specifies the old (previous) location */
  protected String oldLocation = "";

  public CricketPhysicalLocationManager()
  {
  }
  
  public void configure (LocationUpdateListener listener)
  {
    addLocationUpdateListener (listener);

    // we will now start the code that receives Cricket's notification of location changes
    ServerBroker broker = new ServerBroker();
    
    // we specify that we are inly interested in space (location) information
    BitSet mask = new BitSet();
    mask.set (Broker.SPACE);
    
    // we register ourselves and start the thread that communicates with Cricket
    broker.register (this, mask);
    broker.start();
  }

  /**
   * This method satisfies Cricket's Callback interface: this method is called whenever
   * the controller's location changes.
   */

  public void callback (CricketData data, BitSet mask)
  {
    String newLocation = data.getCurrentSpace();

// <HERE> protect against bad Cricket packets
    if (new String (newLocation.getBytes()).equals (new String (new byte[] {1, 1, 1, 0})))
    {
      if (DEBUG)
        System.out.println ("CricketPhysicalLocationManager::callback -> Bad location: "  + newLocation);

      return;
    }
    
    if (newLocation != null)
    {
// <HERE> temporary while Cricket's stuff is fixed    
	//int spaceIndex = newLocation.indexOf (" ");
	//if (spaceIndex != -1)
	//  newLocation = newLocation.substring (0, spaceIndex);
	newLocation = "[location=mit[building=ne43[floor=1[room=" + newLocation + "]]]]";

//      // <HERE> testing old and new location
//      // the old location is the current one
//      String oldLocation = controllerLocation==null? "": controllerLocation.toString();
      
      if (!oldLocation.equals (newLocation))
      {
        if (DEBUG)
          System.out.println ("CricketPhysicalLocationManager::callback -> New location is " + newLocation);
try { Thread.sleep (3000); } catch (Exception e) {}

        // we create a LocationInformation object with the old and new location
        LocationInformation info = new LocationInformation (oldLocation, newLocation);
        
        // next time this callback gets executed, newLocation will be overwritten with the value of
        // the newly-read location, which we will then compare to oldLocation to see if there has been a change
        oldLocation = newLocation;
        
        fireLocationUpdatedEvent (info);
      }
      else
      {
//        if (DEBUG)
//          System.out.println ("CricketPhysicalLocationManager::callback -> Unchanged location");
      }
    }
  }
}









