package streamredirector;

/**
 * This is an HTTP command used for getting a resource. It is created by passing it the
 * file name of the object being created, and the actual HTTP protocol command is generated
 * incorporating the resource name.
 *
 * @see streamredirector.Command, streamredirector.ServerCommand, streamredirector.HTTPCommand, streamredirector.HTTPResumeCommand
 * @author Jorge Rafael Nogueras
 */

public class HTTPGetCommand extends HTTPCommand
{
  /** the file name of the resource being retrieved */
  protected String fileName;
  
  /**
   * Creates an HTTP command, initializing the command that describes it: it is passed
   * the file name that will be incorporated into the full HTTP "GET" command.
   *
   * @param fileName the file name that will be incorporated into the full HTTP "GET" command
   */

	public HTTPGetCommand (String file_name, String the_mimeType)
	{
		super ("GET " + file_name + " HTTP/1.0\r\n\r\n", the_mimeType);
    fileName = file_name;
	}

  /**
   * Prints a useful description of this object.
   *
   * @return a String with a useful description of this object
   */

  public String toString()
  {
    return "[" + this.getClass().getName() + ": GET " + fileName + "]";
  }
}