package streamredirector;

import java.net.*;

/**
 * This concrete implementation of ServerInfo has the knowledge of how to create
 * an HTTPServerConnection.
 *
 * @see streamredirector.HTTPServerConnection
 * @author Jorge Rafael Nogueras
 */

public class HTTPServerInfo extends ServerInfo
{
  /**
   * Initializes the object with the information necessary to connect to the
   * specified server and download the specified resource.
   *
   * @param the_hostName the hostname of the server
   * @param the_port prefix the port where the server is running
   * @param the_path the full path of the filename on the server
   * @param the_selectedResource the resource that was selected for download
   */

  public HTTPServerInfo (String the_hostName, int the_port, String the_path, ResourceEntry the_selectedResource)
  {
    super (the_hostName, the_port, the_path, the_selectedResource);
  }

  /**
   * Initializes the object with the corresponding server's information
   *
   * @param url describes the server information as read from the database
   * @param resourceEntry describes the resource information as read from the database
   */

  public HTTPServerInfo (URL url, ResourceEntry resourceEntry)
  {
    super (url, resourceEntry);
  }
  
  /**
   * This method will create an HTTPServerConnection object that embodies the connection
   * to the HTTP server.
   *
   * @return the HTTPServerConnection object that embodies the connection to the HTTP server
   * @exception Exception if there is an error creating the server connection object
   */

  public ServerConnection getServerConnection() throws Exception
  {
    return new HTTPServerConnection (new URL ("http://" + hostName + ":" + port + "/"));
  }

  /**
   * This method will configure the HTTP resume command used to re-establish a connection when
   * the connection must migrate to another sink.
   *
   * @return a configured HTTPResumeCommand object that can be used to re-establish a connection
   * @exception Exception if there is an error configuring the resume command
   */

  public ServerCommand configureResumeCommand (ServerCommand resumeCommand) throws Exception
  {
      // <HERE> change this later
      ((HTTPResumeCommand) resumeCommand).setFileName(path);
      return resumeCommand;
  }
}

