package streamredirector;

/**
 * This interface describes a class whose responsibility it is to listen to messages coming
 * from its Transport counterpart. It simply has two methods, one that tells it to start listening for incoming
 * messages and one used to tell it to stop. Exactly how it listens for incoming messages is
 * transport-dependent (it may bind to a TCP socket, a UDP socket, it may register with an RMI registry
 * or it may be an INS-aware application, for instance).
 *
 * @see streamredirector.Transport, streamredirector.TCPTransport, streamredirector.TCPListeningTransport
 * @author Jorge Rafael Nogueras
 */

public interface ListeningTransport
{
	/*
	 * Notifies the listening transport that it should start listening for incoming messages.
	 */

	public void startListening();
	
	/*
	 * Notifies the listening transport that it should stop listening for incoming messages.
	 */

	public void stopListening();
}