package streamredirector;

import java.util.*;
/**
 *
 * @see ins.api.Application, ins.namespace.NameSpecifier, streamredirector.Controller
 * @author Jorge Rafael Nogueras
 */

public class LocationUpdateRegistrar
{
  /** this variable sets whether or not debug println's are printed */
  protected static final boolean DEBUG = false;//Controller.DEBUG;
  
  /** keeps track of the registered listeners for location update events */
  protected Hashtable locationUpdateListeners = new Hashtable();
  
  /**
   * Adds a location update listener to the list.
   *
   * @param listener the location update listener to be added
   */

  public void addLocationUpdateListener (LocationUpdateListener listener)
  {
    if (DEBUG)
      System.out.println ("LocationUpdateRegistrar::addLocationUpdateListener -> Added update listener: " + listener);

    locationUpdateListeners.put (listener, listener);
  }
  
  /**
   * Removes a location update listener from the list.
   *
   * @param listener the location update listener to be removed
   * @return the listener that has been removed
   */

  public LocationUpdateListener removeLocationUpdateListener (LocationUpdateListener listener)
  {
    return (LocationUpdateListener) locationUpdateListeners.remove (listener);
  }
  
  /**
   * Notifies all the location update listeners in the list that the location information
   * has changed.
   *
   * @param info the new location information
   */

  public void fireLocationUpdatedEvent (LocationInformation info)
  {
//System.out.println ("LocationUpdateRegistrar::fireLocationUpdatedEvent -> Sending new location info: " + info + ", listeners are: " + locationUpdateListeners);

    Enumeration list = locationUpdateListeners.elements();
    
    while (list.hasMoreElements())
    {
      LocationUpdateListener listener = (LocationUpdateListener) list.nextElement();
//System.out.println ("LocationUpdateRegistrar::fireLocationUpdatedEvent -> Sending new location info to listener: " + listener);
      listener.updateLocation (info);
    }
  }
}