package streamredirector;

import java.io.*;

/**
 * This is a convenience class that stores information pertaining to a method, namely, its
 * name and the types of its parameters, very much like the java.lang.reflect.Method class does.
 * However, said class is not serializable and we wanted to be able to send this object across
 * the network so it can tell the remote listening transport exactly which method will follow:
 * thus, this class has some of the same information as the java.lang.reflect.Method class, but
 * can be serialized and thus sent across the network.
 *
 * @see java.lang.reflect.Method
 * @author Jorge Rafael Nogueras
 */

public class MethodDescriptor implements Serializable
{
	/** the name of the method being described */
	protected String name;
	
	/** the types of the parameters of the method being described */
	protected Class [] parameters;

	/** Creates a MethodDescriptor with the name and he types of the parameters of the method
	 * being described.
	 *
	 * @param the_name the name of the method being described
	 * @param the_parameters the types of the parameters of the method being described
	 */

	public MethodDescriptor (String the_name, Class [] the_parameters)
	{
		name = the_name;
		parameters = the_parameters;
	}
	
	/**
   * Gets the name of the method being described.
   *
   * @return the name of the method being described
	 */
	 
	public String getName()
	{
		return name;
	}
	
	/**
   * Gets the types of the parameters of the method being described.
   *
   * @return the types of the parameters of the method being described
	 */

	public Class [] getParameters()
	{
		return parameters;
	}
	
  /**
   * Prints a useful description of this object.
   *
   * @return a String with a useful description of this object
   */

	public String toString()
	{
		StringBuffer buf = new StringBuffer ("[" + this.getClass().getName() + ": name = " + name + ", parameters = [");
		
		if (parameters.length > 0)
			buf.append (parameters [0].getName());

		for (int i = 1; i < parameters.length; i++)
			buf.append (", " + parameters [i].getName());
			
		buf.append ("]]");
		
		return buf.toString();
	}
}