package streamredirector;

import java.util.*;

/**
 *
 * @see ins.api.Application, ins.namespace.NameSpecifier, streamredirector.Controller
 * @author Jorge Rafael Nogueras
 */

public interface NetworkResourceManager extends LocationUpdateListener
{
  /** used to specify that a searching operation should refresh (wait) until at least one value is found */
  public static final boolean REFRESH = true;
  
  /** used to specify that a searching operation return immediately */
  public static final boolean NO_REFRESH = false;

  /*
   * This method gets called when the location information has changed. The current location
   * information should be updated.
   *
   * @param info the object that contains the location information (new and old)
   */

  public void updateLocation (LocationInformation info);

  /**
   * This method asks the network resource discovery system for any sink proxies that have announced their existence to
   * the network. It can ask that the list for sink proxies discovered on the network be refreshed, or it can use the
   * last list cached. Only active sinks (those whose listening transports are actually listening) will be returned.
   *
   * @param refresh if set to REFRESH, we will refresh the list of sink proxies on the network; if it set to
   *              NO_REFRESH, we will use the last-cached list of sink proxies
   * @return a vector of SinkProxyDescriptor which is the list of the names for the active sink proxies that
   *        were found, or an empty (size == 0) vector if none are found
   */
  

  public Vector getAllSinks (boolean refresh);

  /**
   * This method asks the network resource discovery system for a sink proxy that exists in the current location
   * of the controller. It will return "null" if none of the sink proxies found is at the same location
   * as the controller.
   *
   * @param refresh if set to REFRESH, we will refresh the list of sink proxies on the network; if it set to
   *              NO_REFRESH, we will use the last-cached list of sink proxies
   * @return a vector of SinkProxyDescriptor which are the names of the sink proxies that were found 
   *         in the same location as the controller, or an empty (size == 0) vector if none are found
   */

  public Vector getLocalSinks (boolean refresh);
  
  /**
   * Gets all the sinks local to the controller (i.e. those that are in the same location) and
   * arbitrarily returns the first one. This should be used when, given a choice of several local sinks
   * they are all equally good. If no local sinks are found, "null" will be returned.
   *
   * @param refresh if set to REFRESH, we will refresh the list of sink proxies on the network; if it set to
   *              NO_REFRESH, we will use the last-cached list of sink proxies
   * @return the SinkProxyDescriptor of an active sink that is local to the controller
   */

  public SinkProxyDescriptor getAnyLocalSink (boolean refresh);
}