package streamredirector;

/**
 * This sub-class of TCPSinkInfo knows how to create a TCPTransport object to
 * connect to the sink proxy and it creates a NullSinkConnection. This class should be
 * used when communicating with a sink proxy through TCP, but that does not need
 * an actual connection to the sink.
 *
 * @see streamredirector.TCPTransport, streamredirector.NullSinkConnection
 * @author Jorge Rafael Nogueras
 */

public class NullTCPSinkInfo extends TCPSinkInfo
{
  /**
   * Initializes the object with the information necessary to connect to the
   * specified sink proxy using TCP.
   *
   * @param the_hostName the hostname of the sink proxy
   * @param the_port prefix the port where the sink proxy is listening
   */

  public NullTCPSinkInfo (String the_hostName, int the_port)
  {
    super (the_hostName, the_port);
  }

  /**
   * This method will return a NullSinkConnection object: this means that no connection
   * to the sink must be established.
   *
   * @return a NullSinkConnection indicating that no sink connection is necessary
   */

  public SinkConnection getSinkConnection()
  {
    return new NullSinkConnection();
  }
  
  /**
   * Prints a useful description of this object.
   *
   * @return a String with a useful description of this object
   */

  public String toString()
  {
    return "[" + this.getClass().getName() + ": host = " + hostName + ":" + port +
           ", sink = <NullSinkConnection>" + "]";
  }
}