package streamredirector;

/**
 * This class represents the basic information that must be included in the description
 * of every sink proxy, regardless of the network resource discovery system being used.
 * It should be implemented by an object that knows the naming scheme of the specific
 * network resource discovery system being used so it returns the appropriate information.
 *
 * @author Jorge Rafael Nogueras
 */

public interface SinkProxyDescriptor
{
  /**
   * Gets the type of transport used by the sink proxy.
   *
   * @return the type of transport used by the sink proxy or "null" if the attribute does not exist
   */

  public String getTransportType();

  /**
   * Gets the location of the sink proxy.
   *
   * @return the location of the sink proxy or "null" if the attribute does not exist
   */

  public String getLocation();
}