package streamredirector;

/**
 * This class represents the basic information that must be included in the description
 * of every sink proxy whose transport uses TCP: this includes the information about
 * the sink proxy's host name and listening port, aside from the basic information
 * common to all sink proxies.
 * It should be implemented by an object that knows the naming scheme of the specific
 * network resource discovery system being used so it returns the appropriate information.
 *
 * @author Jorge Rafael Nogueras
 */

public interface TCPSinkProxyDescriptor extends SinkProxyDescriptor
{
  /**
   * Gets the hostname of the sink proxy.
   *
   * @return the hostname of the sink proxy or "null" if the attribute does not exist
   */

  public String getHostName();

  /**
   * Gets the port where the sink proxy is listening.
   *
   * @return the port where the sink proxy is listening  or "null" if the attribute does not exist
   */

  public int getPort();
}