package streamredirector;

/**
 * This class keeps all the state necessary to identify a connection between a client
 * (in our case, an HTTP-enabled media player) and the web server where the resource resides.
 * It gets created whenever a connection to this web proxy is established.
 *
 * @see streamredirector.WebProxyManager
 * @author Jorge Rafael Nogueras
 */

public class WebProxyConnectionState
{
  /** this state describes a file that is currently being transferred */
  public static final int IN_PROGRESS = 0;
  
  /** this state describes a file whose transfer has been stopped */
  public static final int STOPPED = 1;
  
  /** this state describes a file whose transfer has finished */
  public static final int FINISHED = 2;

  /** this state describes a file whose resource was not found in the web server */
  public static final int NOT_FOUND = 3;

  /** the cookie that uniquely identifies this connection */
  public Long cookie;
  
  /** the full path to the resource being transferred */
  public String fullPath;

  /** the offset of the last byte of the file the server has written out */
  public long offset;

  /** the thread that is processing the request and sending out the resource data */
  public WebProxyInstance webProxy;
  
  /** denotes the status of the current connection */
  public int status;
  
  /**
   * Initializes the HTTP connection state with the appropriate parameters
   *
   * @param the_cookie the cookie that uniquely identifies this connection
   * @param the_fullPath the full path to the resource being transferred
   * @param the_offset the offset of the last byte of the file the server has written out
   */

  public WebProxyConnectionState (Long the_cookie, String the_fullPath, long the_offset,
        WebProxyInstance the_webProxy, int the_status)
  {
    cookie = the_cookie;
    fullPath = the_fullPath;
    offset = the_offset;
    webProxy = the_webProxy;
    status = the_status;
  }
  
  /**
   * Prints a useful description of this object.
   *
   * @return a String with a useful description of this object
   */

  public String toString()
  {
    return "[" + this.getClass().getName() + ": cookie = " + cookie + ", resource path = " + fullPath +
            ", offset = " + offset + ", web proxy = " + webProxy + ", status: " + status + "]";
  }
}