/*
 * BookmarkEditPanel.java
 *
 * Bookmark Edit Panel used by Bookmark Panel to edit
 * bookmarks
 */

package streamredirector.gui;

import streamredirector.*;

import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;


/**
 *
 * @author  Kalpak Kothari
 */
public class BookmarkEditPanel extends JFrame implements ActionListener {

    // Variables declaration

    /** this variable sets whether or not debug println's are printed */
    protected static final boolean DEBUG = Controller.DEBUG;

    /** color constants **/
    private static Color bgColor = new Color(204,204,204);
    private static Color fgColor = new Color(0,0,0);
    private static Color errColor = new Color(255,0,0);

    /** variables for GUI and ServerDatabaseManager **/

    private BookmarkPanel bookmarkPanel = null;
    private URL curURL = null;
    private Vector fieldNames = ResourceEntry.getFieldNames();
    JLabel[] labels = new JLabel[fieldNames.size() + 1];
    JTextField[] textFields = new JTextField[fieldNames.size() + 1];
    
    // GUI variables
    private JPanel editPanel;
    private JPanel commandPanel;
    private JButton addButton;
    private JButton deleteButton;
    private JButton closeButton;
    private JTextField statusText;

    // End of variables declaration

    /** Creates new form BookmarkEditPanel */
    public BookmarkEditPanel() {
        initComponents();
    }

    public BookmarkEditPanel(BookmarkPanel bookmark_Panel) {
	bookmarkPanel = bookmark_Panel;
        initComponents();
    }

    public BookmarkEditPanel(BookmarkPanel bookmark_Panel, URL url) {
	bookmarkPanel = bookmark_Panel;
	curURL = url;
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        setTitle("ControllerGUI:: Edit Bookmark");
        getContentPane().setLayout(new GridBagLayout());

        editPanel = new JPanel();	
        commandPanel = new JPanel();
        addButton = new JButton("Add/Replace");
        deleteButton = new JButton("Delete");
        closeButton = new JButton("Close");
	statusText = new JTextField();
      
        GridBagConstraints gridBagConstraints5;

	// configure editPanel
	int height = (fieldNames.size() + 1) * 30;
	editPanel.setPreferredSize(new Dimension(220, height));
	editPanel.setMinimumSize(new Dimension(220, height));
	editPanel.setMaximumSize(new Dimension(420, height));
        editPanel.setBorder(new TitledBorder(null, "Edit Bookmark", TitledBorder.CENTER, TitledBorder.DEFAULT_POSITION));

	// configure all fields
	
	for(int i=1; i < fieldNames.size()+1; i++) {
	    labels[i] = new JLabel(fieldNames.get(i-1).toString());
	    textFields[i] = new JTextField();
	}
	labels[0] = new JLabel("URL");
	textFields[0] = new JTextField();

	// if editing, fill the textfields with info from curURL
	if(curURL != null) {
	    textFields[0].setText(curURL.toString());
	    ResourceEntry entry = ServerDatabaseManager.getEntry(curURL);
	    for(int i=1; i < fieldNames.size()+1; i++) {
		labels[i] = new JLabel(fieldNames.get(i-1).toString());
		textFields[i] = new JTextField(entry.getValue(i-1).toString());
	    }
	}
	
	GridBagLayout gridbag = new GridBagLayout();
	editPanel.setLayout(gridbag);
	addLabelTextRows(labels, textFields, gridbag, editPanel);

        // configure and add edit/apply/cancel buttons to commandPanel
        addButton.setFont(new Font ("Dialog", 0, 10));
	addButton.setMargin(new Insets(2,1,2,1));
	addButton.setToolTipText("Add/Replace Bookmark");
	addButton.setMnemonic(KeyEvent.VK_A);
	addButton.setActionCommand("addbutton");
	addButton.addActionListener(this);

        deleteButton.setFont(new Font ("Dialog", 0, 10));
	deleteButton.setMargin(new Insets(2,1,2,1));
        deleteButton.setToolTipText("Delete Bookmark");
	deleteButton.setMnemonic(KeyEvent.VK_D);
	deleteButton.setActionCommand("deletebutton");
	deleteButton.addActionListener(this);

        closeButton.setFont(new Font ("Dialog", 0, 10));
	closeButton.setMargin(new Insets(2,1,2,1));
        closeButton.setToolTipText("Close Window");
	closeButton.setMnemonic(KeyEvent.VK_C);
	closeButton.setActionCommand("closebutton");
	closeButton.addActionListener(this);

	commandPanel.add(addButton);
	commandPanel.add(deleteButton);
        commandPanel.add(closeButton);
        

	// configure statusText
	statusText = new JTextField();
	statusText.setBackground(bgColor);
	statusText.setForeground(fgColor);
	statusText.setEditable(false);

	// add treeScrollPane
        gridBagConstraints5 = new GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 0;
        gridBagConstraints5.anchor = GridBagConstraints.NORTH;
        getContentPane().add(editPanel, gridBagConstraints5);

	// add commandPanel
        gridBagConstraints5 = new GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 1;
        getContentPane().add(commandPanel, gridBagConstraints5);

	// add statusText
        gridBagConstraints5 = new GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 2;
	gridBagConstraints5.gridwidth = GridBagConstraints.REMAINDER;  //end row
	gridBagConstraints5.fill = GridBagConstraints.BOTH;

        getContentPane().add(statusText, gridBagConstraints5);
	
	pack();
    }


    private void addLabelTextRows(JLabel[] labels,
                                  JTextField[] textFields,
                                  GridBagLayout gridbag,
                                  Container container) {
        GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.EAST;
        int numLabels = labels.length;

        for (int i = 0; i < numLabels; i++) {
            c.gridwidth = GridBagConstraints.RELATIVE; //next-to-last
            c.fill = GridBagConstraints.NONE;      //reset to default
            c.weightx = 0.0;                       //reset to default
            gridbag.setConstraints(labels[i], c);
            container.add(labels[i]);
	    
            c.gridwidth = GridBagConstraints.REMAINDER;     //end row
            c.fill = GridBagConstraints.HORIZONTAL;
            c.weightx = 1.0;
            gridbag.setConstraints(textFields[i], c);
            container.add(textFields[i]);
        }
    }

    public void actionPerformed(ActionEvent e) {
	// GUI event handler
	if(e.getActionCommand().equals("addbutton")) {
	    // add/replace bookmark entry
	    addButtonActionPerformed();
        } 
	else if(e.getActionCommand().equals("deletebutton")) {
	    // delete bookmark entry
	    deleteButtonActionPerformed();
	} 
	else if(e.getActionCommand().equals("closebutton")) {
	    setVisible(false);
	} 
    }

    private void addButtonActionPerformed() {
	URL newURL;
	ResourceEntry newEntry;
	ResourceEntry oldEntry;
	String resString;

	try {
	    newURL = new URL(textFields[0].getText());
	    oldEntry = ServerDatabaseManager.getEntry(newURL);
	    
	    String newEntryString = new String(textFields[1].getText());
	    for(int i=2; i < fieldNames.size()+1; i++) {
		newEntryString += " | " + textFields[i].getText();
	    }
	    newEntry = new ResourceEntry(newEntryString);
	}
	catch(MalformedURLException e) {
	    printError("Error: Malformed URL");
	    e.printStackTrace();
	    return;
	}

	if(oldEntry != null) {
	    // replace entry
	    ServerDatabaseManager.replaceEntry(newURL, newEntry);
	    resString = "Bookmark replaced";
	}
	else {
	    // add entry
	    ServerDatabaseManager.addEntry(newURL, newEntry);
	    resString = "Bookmark added";
	}
	try {
	    ServerDatabaseManager.writeServerDatabase();
	    printSuccess(resString);
	    bookmarkPanel.refreshBookmarks();
	}
	catch(Exception ex) {
	    printError("Error: Failed to write bookmark file.");
	    ex.printStackTrace();
	    return;
	}
    }
    
    private void deleteButtonActionPerformed() {
	URL newURL;
	ResourceEntry oldEntry;
	String resString = "";

	try {
	    newURL = new URL(textFields[0].getText());
	    oldEntry = ServerDatabaseManager.getEntry(newURL);
	}
	catch(MalformedURLException e) {
	    printError("Error: Malformed URL");
	    e.printStackTrace();
	    return;
	}

	if(oldEntry != null) {
	    // delete entry
	    ServerDatabaseManager.removeEntry(newURL);
	    resString = "Bookmark removed";
	    try {
		ServerDatabaseManager.writeServerDatabase();
		printSuccess(resString);
		bookmarkPanel.refreshBookmarks();
	    }
	    catch(Exception ex) {
		printError("Error: Failed to write bookmark file.");
		ex.printStackTrace();
		return;
	    }
	}
	else {
	    printError("Error: URL not found in bookmarks");
	}
    }

    private void printError(String s) {
	statusText.setForeground(errColor);
	statusText.setText(s);
    }

    private void printSuccess(String s) {
	statusText.setForeground(fgColor);
	statusText.setText(s);
    }
}


