package streamredirector.player;

import streamredirector.player.media.protocol.http.*;

import javax.media.*;
import java.io.*;
import java.util.*;

/**
 * This is the actual audio player that uses the Java Media Framework (JMF)
 * to play.
 */
 
public class JavaPlayer implements ControllerListener//, Runnable
{
  /** this variable sets whether or not debug println's are printed */
  public static final boolean DEBUG = NativePlayer.DEBUG;

  protected static final String packageName = "streamredirector.player";
  protected javax.media.Player player;
  
  protected DataSource dataSource;
  
  Thread thread;
  long start = 0, stop = 0;

    /**
     * Adds a content prefix to the JMF content prefix list.  If
     * the content prefix already exists in the list, no action
     * is taken.  If commit is true, the list will be commited and
     * become persistent over future JMF sessions.  Otherwise, the
     * new content prefix list will only be valid during the
     * current session.
     *
     * @param          prefix
     *                 the content prefix to add
     * @param          commit
     *                 boolean indicating whether to make new
     *                 content list persistent
     */
//  static
//  {
//    Vector packagePrefix = PackageManager.getProtocolPrefixList();
//
//    if (!packagePrefix.contains (packageName))
//    {
//      packagePrefix.add (0, packageName);
//      PackageManager.setProtocolPrefixList (packagePrefix);
//      PackageManager.commitProtocolPrefixList();
//System.out.println ("JavaPlayer::<clinit> -> Protocol prefix list after installation is: " + packagePrefix);
//    }
//  }

  public JavaPlayer (String mediaLocator, long offset) throws Exception
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::<init> -> Media locator: " + mediaLocator);
      
    dataSource = new DataSource (new MediaLocator (mediaLocator), offset);
    
//    dataSource = Manager.createDataSource (new MediaLocator (mediaLocator));

//System.out.println ("Plugin list of content type: " + dataSource.getContentType() + " = " + PlugInManager.getPlugInList (new ContentDescriptor (dataSource.getContentType()), null, 1));
//    System.out.println ("Plugin list: " + PlugInManager.getPlugInList (new ContentDescriptor ("unknown"), null, 1));

    if (DEBUG)
      System.out.println ("JavaPlayer::<init> -> DataSource: " + dataSource);
      
    player = Manager.createPlayer (dataSource);
//    player = Manager.createPlayer (new MediaLocator (mediaLocator));
    
    if (DEBUG)
      System.out.println ("JavaPlayer::<init> -> Player: " + player);

    dataSource.setPlayer (player);

    // Add ourselves as a listener to the player's events
    player.addControllerListener (this);

    start = System.currentTimeMillis();

    // Start Player
//    player.start();
    player.realize();

    if (DEBUG)
      System.out.println ("JavaPlayer::<init> -> Player #" + hashCode() + ": start initiated");
      
//    thread = new Thread (this, "JavaPlayer #" + hashCode());
//    thread.start();
  }

//  public void run()
//  {
//    BufferedReader br = new BufferedReader (new InputStreamReader (System.in));
//
//    while (true) 
//    {
//      try
//      {
//        System.out.print ("Command [stop|start|restart] ==> ");
//        String str = br.readLine();
//        
//        if (str == null || str.equals (""))
//          continue;
//    
//        StringTokenizer st = new StringTokenizer (str);
//
//        if (!st.hasMoreElements())
//          continue;
//    
//        // commands are case insensitive, so we compare them always in lowercase
//        String cmd = st.nextToken().toLowerCase();
//    
//        if (cmd.equals ("stop"))
//        {
//System.out.println ("JavaPlayer::run -> Player: stop initiated");
//          stop = System.currentTimeMillis();
//          stop();
//        }
//        else if (cmd.equals ("start"))
//        {
//System.out.println ("JavaPlayer::run -> Player: start playing initiated");
//          start = System.currentTimeMillis();
//          start();
//        }
//        else if (cmd.equals ("restart"))
//        {
//System.out.println ("JavaPlayer::run -> Player: re-start playing initiated");
//          start = System.currentTimeMillis();
//          restart();
//        }
//      }
//      catch (Exception e)
//      {
//        e.printStackTrace();
//        continue;
//      }
//    }
//  }
//
  /**
   * Does nothing by default since the ExecutablePlayer cannot be stopped programatically:
   * subclasses may overload this method if it is possible to do so.
   *
   * @exception Exception if there are errors stopping the player
   */

  public void stop() throws Exception
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::stop -> Player #" + hashCode() + " stopping");
      
    if (player != null)
    {
      // <HERE> may be used to let buffer run out
//      dataSource.disconnect();
      player.stop();
      
// System.out.println ("JavaPlayer::stop -> Last offset is: " + dataSource.getLastByteOffset());
    }
  }

  public void start() throws Exception
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::start -> Player #" + hashCode() + " starting to play");
      
    if (player != null)
    {
      player.start();
    }
  }

  public void restart() throws Exception
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::restart -> Player re-starting");
      
    if (player != null)
    {
      player.setMediaTime (new Time (0));
      player.start();
    }      
  }

  public void deallocate() throws Exception
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::deallocate -> Player deallocating");
    
    if (player != null)
    {
      player.deallocate();
      player.close();
    }
    
    player = null;
  }
  
  public long getLastByteOffset() throws Exception
  {
    long lastByteOffset = dataSource.getLastByteOffset();
    
    if (DEBUG)
      System.out.println ("JavaPlayer::getLastByteOffset -> Player returning offset: " + lastByteOffset);

    return lastByteOffset;
  }

  /**
   * This controllerUpdate function must be defined in order to
   * implement a ControllerListener interface. This 
   * function will be called whenever there is a media event.
   *
   * @param          event
   *                 the media event
   */

  public synchronized void controllerUpdate (ControllerEvent event)
  {
    long realize = 0, prefetch = 0;
    
    if (event instanceof RealizeCompleteEvent)
    {
      if (DEBUG)
      {
        System.out.println ("JavaPlayer::controllerUpdate -> Player was realized!");
//        System.out.println ("JavaPlayer::controllerUpdate -> It took " + (System.currentTimeMillis() - start) + " to realize");
      }
      
      player.prefetch();
    }
    else if (event instanceof PrefetchCompleteEvent)
    {
      if (DEBUG)
      {
        System.out.println ("JavaPlayer::controllerUpdate -> Player was prefetched!");
//        System.out.println ("JavaPlayer::controllerUpdate -> It took " + (System.currentTimeMillis() - start) + " to prefetch");
      }
      
      prefetch = System.currentTimeMillis();
      
      player.start();
    }
    else if (event instanceof StartEvent)
    {
      if (DEBUG)
//      if (true)
      {
        System.out.println ("JavaPlayer::controllerUpdate -> Player was started!");
        System.out.println ("JavaPlayer::controllerUpdate -> It took " + (System.currentTimeMillis() - start) + " to start");
      }
      
      start = System.currentTimeMillis();
    }
    else if (event instanceof StopEvent)
    {
      if (event instanceof StopByRequestEvent)
      {
        if (DEBUG)
        {
          System.out.println ("JavaPlayer::controllerUpdate -> Player was stopped by request: media time is: " + ((StopEvent) event).getMediaTime().getSeconds());
//          System.out.println ("JavaPlayer::controllerUpdate -> It took " + (System.currentTimeMillis() - stop) + " to stop");
        }
      }
      else if (event instanceof DeallocateEvent)
      {
        if (DEBUG)
        {
          System.out.println ("JavaPlayer::controllerUpdate -> Player #"  + hashCode() + " was deallocated: media time is: " + ((StopEvent) event).getMediaTime().getSeconds());
        }
      }
      else if (event instanceof EndOfMediaEvent)
      {
        if (DEBUG)
          System.out.println ("JavaPlayer::controllerUpdate -> Player finished playing: media time is: " + ((StopEvent) event).getMediaTime().getSeconds());
        // End of the media -- rewind
//        player.setMediaTime (new Time (0));
      }
    }
    else if (event instanceof DurationUpdateEvent)
    {
      if (DEBUG)
        System.out.println ("JavaPlayer::controllerUpdate -> Player updated its duration to: " + ((DurationUpdateEvent) event).getDuration().getNanoseconds()/1e9);
    }
    else if (event instanceof ControllerClosedEvent)
    {
      if (DEBUG)
      {
        System.out.println ("JavaPlayer::controllerUpdate -> Player #"  + hashCode() + " was completely closed");
      }
    }
    else
    {
      if (DEBUG)
        System.out.println ("JavaPlayer::controllerUpdate -> Player received event: " + event);
    }
  }
  
  public void finalize()
  {
    if (DEBUG)
      System.out.println ("JavaPlayer::finalize -> Player #"  + hashCode() + " finalized!!!");
  }
}

/*
static Player createPlayerForSource(javax.media.protocol.DataSource datasource, String s, boolean aflag[])
        throws IOException, NoPlayerException
    {
System.out.println ("Looking for content type: " + s);
        Player player = null;
        if(aflag != null)
            aflag[0] = true;
        Enumeration enumeration = Manager.getHandlerClassList(s).elements();
        Object obj = null;
        while(enumeration.hasMoreElements()) 
        {
            String s1 = (String)enumeration.nextElement();
            try
            {
                Class class1 = Class.forName(s1);
                MediaHandler mediahandler = (MediaHandler)class1.newInstance();
                mediahandler.setSource(datasource);
                if(mediahandler instanceof Player)
                {
                    player = (Player)mediahandler;
                    break;
                }
                MediaProxy mediaproxy = (MediaProxy)mediahandler;
                javax.media.protocol.DataSource datasource1 = mediaproxy.getDataSource();
                String s4 = datasource1.getContentType();

System.out.println ("Examining datasource: " + datasource1 + " with content type: " + s4);
                try
                {
                    player = createPlayerForSource(datasource1, s4, null);
                    continue;
                }
                catch(NoPlayerException _ex)
                {
                    _ex.printStackTrace();
                    player = createPlayerForSource(datasource1, "unknown", null);
                }
                if(player != null)
                    break;
            }
            catch(ClassNotFoundException _ex)
            {
                _ex.printStackTrace();

                player = null;
                if(aflag != null)
                    aflag[0] = false;
            }
            catch(InstantiationException _ex)
            {
                _ex.printStackTrace();
                player = null;
                if(aflag != null)
                    aflag[0] = false;
            }
            catch(IllegalAccessException _ex)
            {
                _ex.printStackTrace();
                player = null;
                if(aflag != null)
                    aflag[0] = false;
            }
            catch(IncompatibleSourceException _ex)
            {
                _ex.printStackTrace();
                player = null;
            }
            catch(NoDataSourceException _ex)
            {
                _ex.printStackTrace();
                player = null;
            }
            catch(Exception exception)
            {
                exception.printStackTrace();
                player = null;
                String s2 = "Error instantiating class: " + s1 + " : " + exception;
                throw new NoPlayerException(s2);
            }
            catch(Error error)
            {
                error.printStackTrace();
                String s3 = "Error instantiating class: " + s1 + " : " + error;
//                Log.error(error);
                throw new NoPlayerException(s3);
            }
        }
        if(player == null)
        {
            throw new NoPlayerException("Cannot find a Player for: " + datasource);
        } else
        {
//            Log.comment("Player created: " + player);
//            Log.comment("  using DataSource: " + datasource + "\n");
            return player;
        }
    }
*/