package streamredirector.player.media.protocol.http;

import java.net.*;
import java.io.*;
import java.util.*;

import javax.media.*;
import javax.media.protocol.*;
import com.sun.media.protocol.*;

import streamredirector.player.*;

public class DataSource extends PullDataSource implements Streamable
{
  /** this variable sets whether or not debug println's are printed */
  protected static final boolean DEBUG = NativePlayer.DEBUG;
  
  /** the size (in bytes) of the buffer where data will be read into */
  protected static final int BUFFER_SIZE = 30 * 1024;

  /** the URL where the data source will connect */
  protected URL theURL;
  
  /** the object representing the connectio to the resource */
  protected URLConnection connection;
  
  /** the MIME type of the resource */
  protected String contentType = null;
  
  /** the length (in bytes) of the resource */
  protected long contentLength = -1L;
  
  /** the server where the resource resides */
  protected String server;
  
  /** the path to the resource on the server */
  protected String resource;

  /** where data from the resource are being read */
  protected InputStream is;
  
  /** whether or not the connection has already been established */
  protected boolean connected = false;
  
  /** the offset (in bytes) from where the resource should be retrieved */
  protected long offset;
  
  /** the player in charge of rendering the media */
  protected javax.media.Player player;
  
  /** the array of streams returned by the data source */
  protected HTTPSourceStream [] streams;
  
  /** the actual source stream where bytes from the resource are read */
  protected HTTPSourceStream sourceStream;

  public DataSource (MediaLocator locator, long offset) throws IOException
  {
    setLocator (locator);
    setOffset (offset);
    
    connect();
  }
  
  public boolean isPrefetchable()
  {
    return true;
  }
  
  public void setPlayer (javax.media.Player the_player)
  {
    player = the_player;
    player.addControllerListener (sourceStream);

  }

  public void setOffset (long the_offset)
  {
    offset = the_offset;
  }

  public long getLastByteOffset()
  {
    return sourceStream.getLastByteOffset();
  }
  
  public void setLocator (MediaLocator locator)
  {
//System.out.println ("DataSource::setLocator -> Testing setLocator with media locator: " + locator);
    try
    {
      theURL = locator.getURL();
      
      if (!theURL.getProtocol().equalsIgnoreCase ("http"))
      {
        throw new Error ("Invalid MediaLocator set on DataSource");
      }

      server = theURL.getHost();
      resource = theURL.getPath();
      
      if (server == null || server.length() == 0 ||
          resource == null || resource.length() == 0)
      {
        throw new Error ("Invalid MediaLocator set on DataSource");
      }

//System.out.println ("DataSource::setLocator -> Accepted DataSource with URL: " + theURL + " and resource: " + resource);
    
      super.setLocator (locator);
    }
    catch (MalformedURLException mue)
    {
      throw new Error ("Invalid MediaLocator set on DataSource");
    }
  }

  public String getContentType()
  {
//    String contentType = "audio.mpeg";
//System.out.println ("DataSource::getContentType -> Someone is checking the content type: " + contentType);
    initCheck();
    
//System.out.println ("DataSource::getContentType -> List of handlers is: " + Manager.getHandlerClassList (contentType));
    
    return contentType;
  }
  
  public void connect() throws IOException
  {
    if (DEBUG)
      System.out.println ("DataSource::connect -> Opening connection to resource...");

    initCheck();
    
    if (connected)
    {
      return;
    }

    // we open a connection to this URL (although the request is not yet written out since we still need to pass the request
    // headers from the original request)
    connection = theURL.openConnection();
    connection.setAllowUserInteraction (true);

//System.out.println ("DataSource::start -> Opened connection to resource!");

    connection.setRequestProperty ("Server", "Stream Redirector Java Player/1.0");
    connection.setRequestProperty ("Connection", "close");
//    connection.setRequestProperty ("Range", "bytes=" + offset + "-");

    try
    {
//System.out.println ("DataSource::connect -> Getting input stream...");
      // we finally establish the connection to the web server and write out the request and the request headers
      is = connection.getInputStream();
      
      contentType = connection.getContentType();
      
      // MIME types have a "/" separator (e.g. "audio/mpeg"), but for JMF's purposes
      // they must have a "." separator (e.g. "audio.mpeg"), so we replace all
      // occurences of "/" by "."
      if (contentType != null)
      {
        contentType = contentType.replace ('/', '.');
      }
      
      if (DEBUG)
        System.out.println ("DataSource::connect -> Got content type: " + contentType);

      contentLength = connection.getContentLength();
      
      if (DEBUG)
        System.out.println ("DataSource::connect -> Got content length: " + contentLength);

      sourceStream = new HTTPSourceStream (theURL, is, contentLength, contentType, offset);
      
      streams = new HTTPSourceStream [] {sourceStream};
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    connected = true;
    
    if (DEBUG)
      System.out.println ("DataSource::connect -> Connected to URL!");
  }

  public void disconnect()
  {
    initCheck();

    if (!connected)
    {
      return;
    }
    
    try
    {
      is.close();
      
      // <HERE> trying to either stop immediately with less offset or let buffer run out
 //     streams [0].close();

      streams = null;
      connected = false;
    }
    catch (IOException e) {}
  }

  public void start() throws IOException
  {
    initCheck();
    
    return;
  }

  public void stop() throws IOException
  {
    initCheck();

    return;
  }

  public PullSourceStream [] getStreams()
  {
    MediaLocator locator = getLocator();

    if (locator == null || is == null)
    {
      return null;
    }

//    if (streams == null)
//    {
//      HTTPSourceStream sourceStream = new HTTPSourceStream (theURL, is, contentLength, contentType);
//      player.addControllerListener (sourceStream);
//      
//      streams = new PullSourceStream [] {sourceStream};
//    }
    
//System.out.println ("DataSource::getStreams -> Returning stream: " + streams [0]);
    return streams;
  }

  public Object[] getControls()
  {
    return new Object [0];
  }

  public Object getControl (String controlType)
  {
    return null;
  }
  
  public Time getDuration()
  {
    return DURATION_UNKNOWN;
  }
}