package streamredirector.player.media.protocol.http;

import java.net.*;
import java.io.*;
import javax.media.*;
import javax.media.protocol.*;

import streamredirector.player.*;

public class HTTPSourceStream implements ControllerListener, PullSourceStream, Seekable
{
  /** this variable sets whether or not debug println's are printed */
  protected static final boolean DEBUG = NativePlayer.DEBUG;
  
  protected URL url;
  protected URLConnection connection;

  protected BufferedInputStream is;
  protected long contentLength;
  protected ContentDescriptor contentDescriptor;
  
  /** the offset from where the playing should begin */
  protected long offset;

  /** says whether or not the player has achieved the realized state, that is, it is ready to prefetch its data */
  protected boolean realized = false;
  
  protected boolean endOfStream;
  protected long bytesRead;

  protected long location;

  public HTTPSourceStream (URL the_URL, InputStream the_is, long the_contentLength, String the_contentType, long the_offset)
  {
    url = the_URL;
   
    try
    {
      is = new BufferedInputStream (the_is);
    }
    catch (Exception e) { e.printStackTrace(); }
    
    contentLength = the_contentLength;
    contentDescriptor = new ContentDescriptor (the_contentType);
    offset = the_offset;
    
    endOfStream = false;
    bytesRead = 0;
  }

  public long getLastByteOffset()
  {
    // before the player is realized, the variable "location" changes its value several times as the stream
    // is read back and forth to determine its header information; during this time (called "realization") we
    // would like to simply return as the last byte offset the offset that was passed in the constructor, since
    // no playing has really begun, and if the user wants to restart the player, he must do so from this offset
    if (!realized)
    {
      return offset;
    }
    else
    {
      // if we have been realized, the current position of the stream should be the offset returned
      return location;
    }
  }
  
  public synchronized void controllerUpdate (ControllerEvent event)
  {
    if (event instanceof RealizeCompleteEvent)
    {
      realized = true;
      
      if (DEBUG)
        System.out.println ("HTTPSourceStream::controllerUpdate -> Player was realized!");

      seek (offset);
    }
  }
  
  public long seek (long offset)
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::seek -> Called seeking to offset: " + offset);

    long l1 = location;
    location = offset;
    
//    try
//    {
      if (offset < l1)
      {
//System.out.println ("HTTPSourceStream::seek -> 1) Offset is: " + offset + " and l1 is: " + l1);
        endOfStream = false;
//        reopenStream (offset);
//        return offset;
//  
//        return skip (is, offset);
      }
      else
      {
//System.out.println ("HTTPSourceStream::seek -> 2) Offset is: " + offset + " and l1 is: " + l1);
//        return skip (is, offset - l1);
      }

      reopenStream (offset);
      return offset;
//    }
//    catch (IOException e)
//    {
//      e.printStackTrace();
//      return 0L;
//    }
  }
  
  private long skip (InputStream inputstream, long l) throws IOException
  {
    long l2;

    if (DEBUG)
      System.out.println ("HTTPSourceStream::skip -> Skipping " + l + " bytes");

    inputstream.skip (l);
//    for (long l1 = l; l1 > 0L; l1 -= l2)
//    {
//      l2 = inputstream.skip (l1);
//    }

    return l;
  }

  public long tell()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::tell -> Called tell: returning: " + location);

    return location;
  }

  void reopenStream (long offset)
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::reopenStream -> Called reopenStream with offset: " + offset);

    try
    {
      if (is != null)
      {
//        final InputStream the_is = is;
        
        new Thread()
        {
          public void run()
          {
            try
            {
//System.out.println ("HTTPSourceStream::run -> Closing previous input stream: " + is);
              is.close();
//System.out.println ("HTTPSourceStream::run -> Closed previous input stream: " + is + "!!!");
            }
            catch (Exception e) {}
          }
        }.start();
      }

//System.out.println ("HTTPSourceStream::reopenStream -> Had to reopen stream with offset: " + offset);

      connection = url.openConnection();
      connection.setAllowUserInteraction (true);
      connection.setRequestProperty ("Server", "Stream Redirector Java Player/1.0");
      connection.setRequestProperty ("Connection", "close");
      connection.setRequestProperty ("Range", "bytes=" + offset + "-");
//System.out.println ("HTTPSourceStream::reopenStream -> Reopened!");

      try
      {
        is = new BufferedInputStream (connection.getInputStream());
//System.out.println ("HTTPSourceStream::reopenStream -> Got new input stream: " + is);
      }
      catch (Exception exception)
      {
        System.err.println ("Unable to re-open a URL connection " + exception);
        throw new IOException ("Unable to re-open a URL connection " + exception.getMessage());
      }
    }
    catch (IOException _ex) { }
  }

  ////////////////////////////////////////////////////////////
  //
  //  javax.media.protocol.SourceStream methods
  //
  ////////////////////////////////////////////////////////////

  public ContentDescriptor getContentDescriptor()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::getContentDescriptor -> Called getContentDescriptor");

//    return contentDescriptor;
    return null;
  }

  public long getContentLength()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::getContentLength -> Called getContentLength: returning " + contentLength);
      
    return contentLength;
  }
  
  public boolean endOfStream()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::endOfStream -> Called endOfStream");
      
    return endOfStream;
  }

  ////////////////////////////////////////////////////////////
  //
  //  javax.media.protocol.PullSourceStream methods
  //
  ////////////////////////////////////////////////////////////

  public boolean willReadBlock()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::willReadBlock -> Called willReadBlock");
      
    try
    {
      return (is.available() == 0);
    }
    catch(IOException e)
    {
      return true;
    }
  }

  public int read (byte [] buffer, int offset, int length) throws IOException
  {
    int length1 = length;
    int offset1 = offset;
    
    do
    {
      int r = is.read (buffer, offset1, length1);
      
      if (r == -1)
      {
        endOfStream = true;
        int length2 = length - length1;

        if (DEBUG)
          System.out.println ("HTTPSourceStream::read -> EOF: Returning " + length2 + " bytes");
          
        return length2 <= 0 ? -1 : length2;
      }
      
      location += r;
      length1 -= r;
      offset1 += r;
    }
    while (length1 != 0);

    if (DEBUG)
      System.out.println ("HTTPSourceStream::read -> Returning " + length + " bytes");

    return length;

/*
    // read the required bytes into the buffer
    int result = is.read (buffer, offset, length);
    
    // if the result of the read() is -1, we have reached the end of the stream
    endOfStream = (result == -1);

    if (!endOfStream)
    {
      location += result;
    }

    return result;
*/
  }

/*
// more efficient read (returns a full buffer at a time except at EOF)
 public int read(byte abyte0[], int offset, int length)
        throws IOException
    {
        int length1 = length;
        int offset1 = offset;
        do
        {
            int bytesRead = stream.read(abyte0, offset1, length1);
            if(bytesRead == -1)
            {
                eofReached = true;
                int length2 = length - length1;
                return length2 <= 0 ? -1 : length2;
            }
            location += bytesRead;
            length1 -= bytesRead;
            offset1 += bytesRead;
        } while(length1 != 0);
        return length;
    }
*/
  
  public boolean isRandomAccess()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::isRandomAccess -> Called isRandomAccess");
      
    return true;
//    return false;
  }

  ////////////////////////////////////////////////////////////
  //
  //  javax.media.Controls methods
  //
  ////////////////////////////////////////////////////////////

  public Object[] getControls()
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::getControls -> Called getControls");
      
    return new Object [0];
  }

  public Object getControl (String controlType)
  {
    if (DEBUG)
      System.out.println ("HTTPSourceStream::getControl -> Called getControl");
      
    return null;
  }
}