/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package viewfinder;

import java.awt.event.*;
import cricketdaemon.clientlib.sim.SimControl;

/**
 * Controls for navigation in the GUI of ViewFinderApp.
 *
 * @author Allen Miu
 */
class Controls implements KeyListener 
{
    SimControl simCntl;
    StatusBar status;
    
    public Controls (SimControl s, StatusBar st) {
	super();
	simCntl = s;
	status  = st;
    }
    
    public void keyTyped(KeyEvent e) 
    {
	if(simCntl == null)
	    return;

	char k = Character.toLowerCase(e.getKeyChar());
	Util.dbgPrintln("key typed "+k);
	double deltaAngle = 0;
	switch(k) {
	case 'd':
	    deltaAngle = simCntl.incrDeviceRotate();
	    break;
	case 'f': 
	    deltaAngle = simCntl.decrDeviceRotate();
	    break;
	}
	status.setOrient(Coord.fullCircleInRadians-deltaAngle);
	simCntl.notice();
    }

    public void keyReleased(KeyEvent e)
    {
	// do nothing
    }

    public void keyPressed(KeyEvent e) 
    {
	if(simCntl == null)
	    return;

	//int code = e.getKeyCode();
	char k = Character.toLowerCase(e.getKeyChar());
	Util.dbgPrintln("key pressed "+k);

	switch(k) {
	case 'i':
	    status.setY(simCntl.moveDeviceForward());
	    break;
	case 'm':
	    status.setY(simCntl.moveDeviceBackward());
	    break;
	case 'j':
	    status.setX(simCntl.moveDeviceLeft());
	    break;
	case 'k':
	    status.setX(simCntl.moveDeviceRight());
	    break;	
	}
	simCntl.notice();
    }
}

