/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package viewfinder;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.*;
import cricketdaemon.clientlib.*;
import cricketdaemon.clientlib.sim.*;

/**
 * GUI for the ViewFinderApp.
 *
 * @author Allen Miu
 */
public class GUI extends JPanel implements FocusListener
{
    ArrayList beaconCoords, serviceCoords;
    String highlightAnchorBeacon;
    String highlightSpaceBeacon;
    Vector highlightViewables;
    ArrayList highlightBeaconsHeard;

    protected JFrame f;
    protected Controls cntl;

    Image im;
    Graphics imgc;
    JList list;

    private static String FONTNAME = "Arial";
    private static int FONTSTYLE = Font.BOLD;
    private static int FONTSIZE = 14;

    private FontMetrics fontMetrics;

    /* bounding box variables */
    private int minX, minY, maxX, maxY;

    private StatusBar status;
    private boolean demo;

    MouseListener selectServiceListener = new MouseAdapter() {
	    public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
		    int index = list.locationToIndex(e.getPoint());
		    String s = ((Coord) list.getModel().
				getElementAt(index)).toString();
		    //System.out.println("Service selected: "+s);
		    CustomFrame f = new CustomFrame("Hello from "+s,
						    220, 1);
		    f.pack();
		    f.setVisible(true);
		    f.toFront();
		}
	    }
	};

    private void setFontDefault(Graphics g)
    {
	Font f = new Font(FONTNAME, Font.PLAIN, 12);
	g.setFont(f);
	fontMetrics = g.getFontMetrics();
    }

    private void setFonts(Graphics g)
    {
	Font f = new Font(FONTNAME, FONTSTYLE, FONTSIZE);
	g.setFont(f);
	fontMetrics = g.getFontMetrics();
    }

    private void updateBoundingBox(int minx, int miny, int maxx, int maxy)
    {
	if(minx < minX) minX = minx; if(maxx < minX) minX = maxx;
	if(miny < minY) minY = miny; if(maxy < minY) minY = maxy;
	if(maxx > maxX) maxX = maxx; if(minx > maxX) maxX = minx;
	if(maxy > maxY) maxY = maxy; if(miny > maxY) maxY = miny;
    }

    private void drawString(String s, int x, int y) 
    {
	int width  = fontMetrics.stringWidth( s );
	int height = fontMetrics.getHeight();
	int xx    = x-width/2;
	imgc.drawString(s, xx, y);
	//updateBoundingBox(xx, y, xx+width, y+height);
    }

    public GUI(String title, int x, int y, Broker s) 
    {
	preferredSize = new Dimension(x,y);
	setBorder(createBorder());
	
	/*
	  boundX = 0;
	  boundY = 0;
	  boundWidth = preferredSize.width;
	  boundHeight = preferredSize.height;
	*/

	f = new CustomFrame(title);
	//f.addFocusListener(this);
	//addFocusListener(this);

	Container container = f.getContentPane();
	container.setLayout(new BorderLayout());
	container.add(this, "North");

	list = new JList();
	Coord dumb = new Coord(123, 123, "0123456789");
	list.setPrototypeCellValue(dumb.toString());
	list.addMouseListener(selectServiceListener);

	//list.addFocusListener(this);

	JScrollPane scroll = new JScrollPane(list);
	scroll.setBorder(createBorder());
	scroll.setPreferredSize(new Dimension(x, y/10));
	container.add(scroll, "Center");

	//scroll.addFocusListener(this);

	status = new StatusBar();
	status.setBorder(BorderFactory.createLoweredBevelBorder());
	container.add(status, "South");

	//status.addFocusListener(this);

	if(s instanceof SimControl) {
	    Controls cntl = new Controls((SimControl) s, status);
	    //f.addKeyListener(cntl);
	    list.addKeyListener(cntl);
	}

	f.setBackground(Color.white);
	f.validate();
	f.pack();
	f.setVisible(true);

	im = createImage(4*x,4*y);
	imgc = (Graphics) im.getGraphics();
	originX = ((BufferedImage) im).getWidth()/2;
	originY = ((BufferedImage) im).getHeight()/2;
	imgc.translate(originX, originY);
		       
	//	imgc.setXORMode(Color.red);
	//	imgc.setColor(Color.blue);

	if(s instanceof SimControl) {
	    setFontDefault(imgc);
	    demo = true;
	}
	else {
	    setFonts(imgc);
	    demo = false;
	}

	minX = minY = Integer.MAX_VALUE;
	maxX = maxY = Integer.MIN_VALUE;

	Util.dbgPrintln("view created and exposed");
    }

    /* public API for program */

    public void setService(ArrayList v) 
    {
	serviceCoords = v;
	//list.setListData(serviceCoords);
	//	repaint();
    }

    public void setBeacons(ArrayList v)
    {
	beaconCoords = v;
	//repaint();
    }

    public void highlightAnchorBeacon(String beaconName) 
    {
	highlightAnchorBeacon = beaconName;
	//repaint();
    }

    public void highlightSpaceBeacon(String beaconName)
    {
	highlightSpaceBeacon = beaconName;
	//repaint();
    }

    public void highlightBeaconsHeard(ArrayList beaconsHeard)
    {
	highlightBeaconsHeard = beaconsHeard;
	//repaint();
    }

    public void highlightViewableServices(Vector viewable)
    {
	highlightViewables = viewable;
	list.setListData(viewable);
    }

    double PiToDegreeConv = 180.0/Math.PI;
    /*
      private final int boundBox = 10;
      
      private int boundX;
      private int boundY;
      private int boundHeight;
      private int boundWidth;
    */

    // FIXME:  all coordinates passed to the GUI are expected in 
    // real-world units.  So we need to perform conversions to
    // fit everything to the screen.
    public void drawDevice(double x, double y, double or,
			   double vangle, double radius, boolean doViewCone) 
    {
	if(!demo) {
	    status.setX(x);
	    status.setY(y);
	    status.setOrient(or);
	}
	    
	imgc.setColor(Color.blue);
	imgc.clearRect(-originX, 
		       -originY, 
		       ((BufferedImage)im).getWidth(), 
		       ((BufferedImage)im).getHeight());
	drawString("ME", (int) x, (int) y);
	int originX = (int)(x-radius);
	int originY = (int)(y-radius);
	int boxSize = (int)(2*radius); 
	int start   = (int)((-or-vangle)*PiToDegreeConv);
	int degree  = (int)(2*vangle*PiToDegreeConv) - 1;
	
	/*
	  int boundX   = originX-boundBox/2;
	  int boundY   = originY-boundBox/2;
	  int boundSize = boxSize+boundBox;
	*/
	
	// imgc.clearRect(boundX, boundY, boundSize, boundSize); 
	if(doViewCone) {
	    _drawLineRadians(x, y, radius, or-vangle);
	    _drawLineRadians(x, y, radius, or+vangle);
	}
	imgc.drawArc(originX, originY, boxSize, boxSize, start, degree);
	//	repaint(boundX, boundY, boundSize, boundSize);
	
	
	imgc.drawRect((int)x, (int)y, pointSize, pointSize);
	updateBoundingBox((int)x, (int)y, (int)x+pointSize, (int)y+pointSize);
    }

    /* private methods for drawing other stuff */
    public final int pointSize = 3;
    private void _drawService() 
    {
	//list.clearSelection();
	if( serviceCoords != null && imgc != null) {
	    for(int i = 0; i < serviceCoords.size(); i++) {
		Coord p = (Coord) serviceCoords.get(i);
		if(highlightViewables != null &&
		   highlightViewables.contains(p)) {
		    imgc.setColor(Color.blue);
		    //list.setSelectedValue(p, false);
		}
		else {
		    imgc.setColor(Color.black);
		}
		int ix = (int)p.x;
		int iy = (int)p.y;
		imgc.drawRect(ix, iy, pointSize, pointSize);
		//updateBoundingBox(ix, iy, ix+pointSize, iy+pointSize);
		drawString(p.name, ix, iy);
	    }
	}
	imgc.setColor(Color.black);
    }
    
    private void _drawBeacons()
    {
	if( beaconCoords != null && imgc != null ) {
	    for(int i = 0; i < beaconCoords.size(); i++) {
		Coord p = (Coord) beaconCoords.get(i);
		int ix = (int)p.x;
		int iy = (int)p.y;		
		//imgc.clearRect(boundX, boundY, boundSize, boundSize); 
		//System.out.println(highlightSpaceBeacon);
		//System.out.println(p.name);
		
		// erase underline
		imgc.setColor(Color.white);
		int width = fontMetrics.stringWidth( p.name );
		int xx0   = ix -width/2;
		int xx1   = ix +width/2;
		imgc.drawLine(xx0, iy + 1, xx1, iy + 1);
		//updateBoundingBox(xx0, iy+1, xx1, iy+1);		
		
		if(p.name.equals(highlightAnchorBeacon)) {
		    imgc.setColor(Color.pink);
		}
		else if(p.name.equals(highlightSpaceBeacon)) {
		    imgc.setColor(Color.red);
		    // underline 
		    imgc.drawLine(ix - width/2, iy + 1, ix + width/2, iy + 1);
		}
		else if(highlightBeaconsHeard != null &&
			highlightBeaconsHeard.contains(p)) {
		    imgc.setColor(Color.green);
		}
		else {
		    imgc.setColor(Color.gray);		    
		}
		imgc.drawRect(ix, iy, pointSize, pointSize);
		//updateBoundingBox(ix, iy, ix+pointSize, iy+pointSize);
		drawString(p.name, ix, iy);
	    }
	}	
	imgc.setColor(Color.black);
    }

    private void _drawLineRadians(double x, double y, double r, double a)
    {
	int ix = (int) x;
	int iy = (int) y;
	int x2 = (int) (r*Math.cos(a));
	int y2 = (int) (r*Math.sin(a));
	int xx = ix+x2;
	int yy = iy+y2;
	imgc.drawLine(ix, iy, xx, yy);
	// don't update the full extent of the viewing cone for centering
	// the image
	updateBoundingBox(ix, iy, ix+x2/2, iy+y2/2);
    }

    /* Java AWT stuff */

    private int lastXX = 0;
    private int lastYY = 0;

    private int originX;
    private int originY;

    private long repaintTime = 0;

    public void repaint()
    {
	repaintTime = System.currentTimeMillis();
	super.repaint();
    }

    public void paintComponent(Graphics g) 
    {
        //super.paintComponent(g);
	long time0 = System.currentTimeMillis();

	if( im != null ) {
	    _drawService(); // draw these first to serve as "background"
	    //_drawBeacons();
	    int xx = 0;
	    int yy = 0;
	    if(maxX > Integer.MIN_VALUE) {
		xx = (preferredSize.width - (maxX + minX))/2;
		yy = (preferredSize.height - (maxY + minY))/2;
		if(Math.abs(xx - lastXX) < 20)
		    xx = lastXX;
		else 
		    lastXX = xx;
		if(Math.abs(yy - lastYY) < 20)
		    yy = lastYY;
		else
		    lastYY = yy;
	    }
	    g.drawImage(im,xx-originX,yy-originY,null);
	}
	minX = minY = Integer.MAX_VALUE;
	maxX = maxY = Integer.MIN_VALUE;
	long time1 = System.currentTimeMillis();
	System.out.println("foobar: "+ (time1-time0) +
			   " "+ (time0-repaintTime));
    }

    Dimension preferredSize;
    public Dimension getPreferredSize() 
    {
	return preferredSize;
    }

    private Border createBorder() {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }

    /* window focus stuff */
    public void focusGained(FocusEvent e) {
	System.out.println("Focus gained "+e.getComponent());
    }

    public void focusLost(FocusEvent e) {
	System.out.println("Focus lost "+e.getComponent());
    }

}
