/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
/* CVS INFO
 * $Header: /var/cvs/Cricket/src/cricketdaemon/Util.java,v 1.13 2004/04/21 02:05:07 aklmiu Exp $
 */
package cricketdaemon;

import java.io.*;

/**
 * Contains a bunch of miscellaneous methods useful for debugging and
 * debug logging.
 *
 * @author Allen Miu
 */
public class Util {

    public static final boolean APPEND_OLDLOGFILE  = true;
    public static final boolean DESTROY_OLDLOGFILE = false;

    private static PrintWriter _logFile = null;

    /*
      public static final int CLOCK_FREQ_HZ  = 15625;
      public static final double TEMPERATURE_C = 20.0;
      public static final double HUMIDITY_PERCENT = 50.0;
      public static final double DELAY_OFFSET_CM = 80.0;
      public static final double SPEED_OF_SOUND = 34449.0;
    */
    
    /* Cricket receiver offset */
    /*
      public static final int BEACON_OFFSET_BODHI = 36;
      public static final int BEACON_OFFSET_CM = 80;
    */


    /*
      public static double bToCentimeters(double bodhi)
      {
      return (double) bodhi*34449.0/CLOCK_FREQ_HZ; // - DELAY_OFFSET_CM;
      }
    */

    /*
    public static double centimetersToB(double cm)
    {
	return (double) cm*CLOCK_FREQ_HZ/SPEED_OF_SOUND;
    }
    */

    public static void assert(boolean b, String errMsg) {
	try {
	    if(!b)
		throw new Exception();
	}
	catch (Exception e) {
	    System.err.println("ASSERTION FAILED--> "+errMsg);
	    e.printStackTrace();
	    //need to close log file
	    System.exit(1);
	}
    }
    
    // logs message to some file
    public static void warn(String s)
    {
	String w = "WARNING: "+s;
	if(!CricketDaemon.debugMode)
	    System.err.println(w);
	log(w);
    }

    public static void log(String s) 
    {
	if(CricketDaemon.debugMode)
	    System.out.println(s);

	if(_logFile != null && s != null) {
	    _logFile.println(s);
	    _logFile.flush();
	}
    }

    public static void init(String file)
    {
	initLogFile(file);
    }
    
    private static void initLogFile(String fName) {
	try {
	    File f = new File(fName);
	    assert(!f.exists(), fName+" exists, please remove or use another filename");
	    _logFile = new PrintWriter(new FileWriter(fName, DESTROY_OLDLOGFILE));
	} 
	catch (IOException e) {
	    assert(false,"Error opening file '" + fName + "'");
	}
    }

    public static void exit() {
	log("Util::exit()");
	if(_logFile != null) {
	    _logFile.println(""); // pad a null line to separate different
	    _logFile.close();     // log sessions when the append mode is used
	}
    }

    public static int min(int a, int b)
    {
	if(b < a)
	    return b;
	return a;
    }

    public static String textOf(double d, int places) 
    {
	String s = String.valueOf(d);
	int endIndex = s.length();
	int decIndex = s.indexOf(".");
	if(decIndex >= 0)
	    endIndex = min(endIndex, decIndex + places + 1);
	
	return s.substring(0, endIndex);
    }

    /* java doesn't parse NaN properly, hence this function */
    public static double parseDouble(String value) 
    {
	//System.out.println("parseDouble "+value);
	if(value.equals("NaN")) 
	    return Double.NaN;
	else {
	    //	    System.out.println("parsing double");
	    double result = Double.parseDouble(value);
	    //double result = 3.0;
	    //	    System.out.println("parseDouble returning "+ result);
	    return result;
	}
    }

    // not very efficient
    public static void bufcpy(byte[] dst, byte[] src, int dstoffset, int length)
    {
	for(int i = 0; i < length; i++) {
	    dst[dstoffset+i] = src[i];
	}
    }
}
