/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.beaconstat;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import java.util.*;

/**
 * The data type that is passed to higher layer module that registers a
 * callback with the BeaconStat module.
 *
 * @author Allen Miu
 */
public class BeaconStatData implements ModuleThread.Data
{
    private static final int MAX_SEQNUM = 65536;
    private static int globalSeqNum = 0;
    public int seqNum;

    public long timeStamp; // time when stat is generated
    public ArrayList values;
    
    public BeaconStatData(ArrayList v, long curTime)
    {
	seqNum = globalSeqNum;
	globalSeqNum++;             // ensures unique sequence number
	globalSeqNum %= MAX_SEQNUM; // wrap around
	timeStamp = curTime;
	values = v;
	Util.assert(v.size() > 0, "BeaconStatData::BeaconStatData()");
    }

    public interface BeaconStatDataHandler {
	public void processBeaconStatData(BeaconStatData newData, 
					  BitSet fieldMask);
    }

    public void process(Object handler, BitSet fieldMask) 
    {
	Util.assert(handler instanceof BeaconStatDataHandler,
		    "BeaconStatData::process() incorrect handler type");
	BeaconStatDataHandler h = (BeaconStatDataHandler) handler;
	h.processBeaconStatData(this, fieldMask);
    }
    
    public String toString()
    {
	String result = seqNum+" "+timeStamp;

	if(values != null ) {
	    result += " {\n";
	    for(int i = 0; i < values.size(); i++)
		result += values.get(i)+"\n";
	    result += " }\n";
	}
	return result;
    }
}
