/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib;

import cricketdaemon.clientlib.data.CricketData;
import java.util.*;

/**
 * Cricket Applications should implement this interface to receive
 * updates from the <code>cricketdameon</code>.
 *
 * @author Allen Miu
 */
public interface Callback
{
    /**
     * The update from the cricketdaemon is contained in <code>data</code>, 
     * The updated fields are specified in <code>mask</code>.  
     * Beware when multiple
     * threads have registered callbacks with the Broker: the
     * same <code>data</code> gets passed to every thread.  As a result,
     * any modifications made to data may become visible to another
     * thread.  In such case, each thread is responsible for deep
     * cloning the relevant data structure.  Be advised that deep cloning
     * is not implemented for any data strucuture contained within
     * <code>CricketData</code>.
     */
    void callback(CricketData data, BitSet mask); 
}

