/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib.data;

import cricketdaemon.clientlib.Associable;

/**
 * Contains information about a Beacon.  The unique id used for
 * Association is the string space+"-"+id.  The unique id is also used
 * for comparison between different BeaconRecords.
 *
 * @author Allen Miu
 */
public class BeaconRecord implements Associable, Comparable, Cloneable
{
    public String uniqueName;
    public String space;
    public Integer id;
    public Long lastUpdate;
    public DistanceStat distStat;
    public Position pos;

    public Object getUniqueIdentifier() { return uniqueName; }
    public boolean isAssociated(Associable o)
    {
	Object id = o.getUniqueIdentifier();
	if(!(id instanceof String))
	    return false;

	return id.equals(getUniqueIdentifier());
    }

    // facilitate sorting by ascending name order
    public int compareTo(Object o) 
    {
	return uniqueName.compareTo(((BeaconRecord)o).uniqueName);
    }
    
    public boolean equals(Object o)
    {
	if(!(o instanceof BeaconRecord))
	    return false;
	
	boolean result = compareTo(o) == 0;
	return result;
    }

    public void setUniqueName() { uniqueName = space+"-"+id; }
    
    public String toString()
    {
	String result = uniqueName+" "+lastUpdate+" "+distStat+" "+pos;
	return result;
    }

    public double getDist()
    {
	return distStat.dist;
    }

    // shallow clone
    public Object clone()
    {
	BeaconRecord result = new BeaconRecord();
	result.uniqueName = uniqueName;
	result.space = space;
	result.id = id;
	result.lastUpdate = lastUpdate;
	result.distStat = distStat;
	result.pos = pos;
	return result;
    }
}
