/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a "device_pos"ition type.
 *
 * @author Allen Miu
 */
class DevicePosParseHandlerMap extends ParseHandlerMap
{
    public static DevicePosParseHandlerMap instance = 
	new DevicePosParseHandlerMap();

    protected DevicePosParseHandlerMap()
    {
	super();
	register("pos", "check_datatype", "parse_pos");
	register("array:dist_est", "check_datatype", 
		 "parse_adist_est");
    }

    public boolean check_datatype(Object o) 
    {
	return o instanceof CricketData;
    }

    public void parse_pos(Object dataStructure, String value)
	throws IOException 
    {
	CricketData c = (CricketData) dataStructure;
	Position pos = new Position(-1, -1, -1);
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(pos,
			     sr, 
			     PosParseHandlerMap.instance);
	c.setDevicePosition(pos);
    }

    public void parse_adist_est(Object dataStructure, String value)
	throws IOException 
    {
	ArrayList calcBeaconDist = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(calcBeaconDist,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	cricketData.setCalcBeaconDist(calcBeaconDist);
	//PacketParseHandlerMap.parse_adist_est(dataStructure, value);
    }
}
