/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a Distance Estimate.
 *
 * @author Allen Miu
 */
class DistanceEstimateParseHandlerMap extends ParseHandlerMap
{
    public static DistanceEstimateParseHandlerMap instance = 
	new DistanceEstimateParseHandlerMap();

    protected DistanceEstimateParseHandlerMap()
    {
	super();
	register("space", "check_beaconrec", "parse_space");
	register("id", "check_beaconrec", "parse_id");
	register("dist", "check_beaconrec", "parse_dist");
	register("dist_stat", "check_beaconrec", "parse_dist_stat");
	register("pos", "check_beaconrec", "parse_pos");
	register("last_update", "check_beaconrec", "parse_last_update");
    }

    public boolean check_beaconrec(Object o)
    {
	return o instanceof BeaconRecord;
    }

    public void parse_space(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.space = value;
	c.setUniqueName();
    }

    public void parse_id(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.id = Integer.valueOf(value);
	c.setUniqueName();
    }
	
    public void parse_dist(Object dataStructure, String value)
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.distStat = new DistanceStat();
	c.distStat.dist = Util.parseDouble(value);
    }
    
    public void parse_dist_stat(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.distStat = new DistanceStat();
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) {
	    Field.parseField(c.distStat, 
			     sr,
			     StatParseHandlerMap.instance);
	}
    }
	
    public void parse_pos(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.pos = new Position(-1, -1, -1);
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) {
	    Field.parseField(c.pos, 
			     sr,
			     PosParseHandlerMap.instance);
	}
    }

    public void parse_last_update(Object dataStructure, String value)
	throws IOException
    {
	BeaconRecord c = (BeaconRecord) dataStructure;
	c.lastUpdate = Long.valueOf(value);
    }
}
