/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a Packet.
 *
 * @author Allen Miu
 */
public class PacketParseHandlerMap extends ParseHandlerMap
{
    public static PacketParseHandlerMap instance = 
	new PacketParseHandlerMap();

    protected PacketParseHandlerMap()
    {
	super();
	register("array:dist_est", "check_datatype", "parse_adist_est");
	register("array:stat_est", "check_datatype", "parse_astat_est");
	register("device_pos", "check_datatype", "parse_device_pos");
	register("cur_space", "check_datatype", "parse_cur_space");
	register("orient", "check_datatype", "parse_orient");
    }

    public void parse_device_pos(Object dataStructure, String value)
	throws IOException
    {
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(dataStructure,
			     sr, 
			     DevicePosParseHandlerMap.instance);
    }

    public boolean check_datatype(Object o) 
    {
	return o instanceof CricketData;
    }

    public static void parse_adist_est(Object dataStructure, String value)
	throws IOException
    {
	ArrayList beaconsHeard = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;

	System.err.println("PacketParseHandlerMap::parse_adist_est() do"+
			   " not support data from SpaceModule anymore");
	return;

	/*
	cricketData.setBeaconsHeard(beaconsHeard);

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(beaconsHeard,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);
	
	// assumes server returns distance estimates in ascending order
	// if not, we can easily add a field (a handler) to specify the 
	// current space object
	cricketData.setCurrentSpaceObject((BeaconRecord) 
					  beaconsHeard.get(0));
	*/
    }

    public void parse_astat_est(Object dataStructure, String value)
	throws IOException
    {
	ArrayList beaconsHeard = new ArrayList();
	CricketData cricketData = (CricketData) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(beaconsHeard,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	//System.out.println("parsed beacons list "+beaconsHeard.size());
	cricketData.setBeaconsHeard(beaconsHeard);
	cricketData.setBeaconStat();
    }

    public void parse_cur_space(Object dataStructure, String value)
	throws IOException
    {
	ArrayList curspace = new ArrayList(); // for now, this is just a container to parse dist_est
	CricketData cricketData = (CricketData) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr))
	    Field.parseField(curspace,
			     sr, 
			     DistanceEstimateArrayParseHandlerMap.instance);

	cricketData.setCurrentSpaceObject((BeaconRecord) curspace.get(0));
    }

    public void parse_orient(Object dataStructure, String value)
	throws IOException
    {
	CricketData cricketData = (CricketData) dataStructure;
	cricketData.setOrient(Util.parseDouble(value));
    }

}
