/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within a stat type.
 *
 * @author Allen Miu
 */
class StatParseHandlerMap extends ParseHandlerMap
{
    public static StatParseHandlerMap instance = 
	new StatParseHandlerMap();

    protected StatParseHandlerMap()
    {
	super();
	register("dist", "check_stat", "parse_dist" );
	register("median", "check_stat", "parse_median" );
	register("mean", "check_stat", "parse_mean" );
	register("mode", "check_stat", "parse_mode" );
	register("max", "check_stat", "parse_max" );
	register("min", "check_stat", "parse_min" );
	register("stddev", "check_stat", "parse_stddev" );
	register("var", "check_stat", "parse_var" );
	register("array:sample", "check_stat", "parse_asample");
    }

    public boolean check_stat(Object o)
    {
	return o instanceof DistanceStat;
    }
	
    public void parse_dist(Object dataStructure, String value)
	throws IOException 
    {
	//System.out.println("parse_dist "+value);
	DistanceStat a = (DistanceStat) dataStructure;
	a.dist = Util.parseDouble(value);
	//	System.out.println("parse_dist finish");
    }

    public void parse_median(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.median = Util.parseDouble(value);
    }
	
    public void parse_mean(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.mean = Util.parseDouble(value);
    }
	
    public void parse_mode(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.mode = Util.parseDouble(value);
    }
    
    public void parse_max(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.max = Util.parseDouble(value);
    }

    public void parse_min(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.min = Util.parseDouble(value);
    }
	
    public void parse_stddev(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.stddev = Util.parseDouble(value);
    }

    public void parse_var(Object dataStructure, String value)
	throws IOException 
    {
	DistanceStat a = (DistanceStat) dataStructure;
	a.variance = Util.parseDouble(value);
    }

    public void parse_asample(Object dataStructure, String value)
	throws IOException
    {
	DistanceStat a = (DistanceStat) dataStructure;

	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) // new dest_est record
	    Field.parseField(a.samples,
			     sr, 
			     SampleArrayParseHandlerMap.instance);
    }

}

