package cricketdaemon.evaluate;

import cricketdaemon.reader.*;
import cricketdaemon.beaconstat.*;
import cricketdaemon.position.*;
import cricketdaemon.UnitConv;
import cricketdaemon.Util;
import java.util.*;

public class TestAlgor extends PlaybackReader 
{
    Algorithm beaconFilter, positionFilter;

    public TestAlgor(Algorithm bf, 
		     Algorithm pf, 
		     String inputFile, 
		     long breakP)
    {
	super(inputFile, false, breakP);
	beaconFilter = bf;
	positionFilter = pf;
    }

    public void start()
    {
	Util.assert(false, "TestAlgor::start() is not supported");
    }

    //private final double idealKinches = Util.SPEED_OF_SOUND/(Util.CLOCK_FREQ_HZ * 2.54);
    private final double idealKinches = UnitConv.cricketDistToIN(1);

    protected void transfer(Data value, BitSet fieldMask)
    {
	DataCapsule input = new DataCapsule(value, fieldMask);
	DataCapsule outputA = beaconFilter.exec(input);
	if(outputA != null) 
	    dump((BeaconStatData) outputA.data, fieldMask);
	DataCapsule outputB = positionFilter.exec(outputA);
	if(outputB != null) {
	    PositionData pd = (PositionData) outputB.data;
	    System.out.println(outputB);
	}
    }

    public static void main(String[] args)
    {
	long breakPoint = Long.MAX_VALUE;
	int  window, binSize;
	double threshold = Double.MAX_VALUE;
	
	if(args.length < 5)
	    Util.assert(false, "Usage: java TestAlgor logfile <window> <binSize> <mean|mode> <threshold> [breakpoint]");
	
	String inputFile = args[0];
	window  = Integer.parseInt(args[1]);
	binSize = Integer.parseInt(args[2]);

	int statSelect = 0; // use mode

	if(args[3].equals("mean"))
	    statSelect = 1; // use mean

	threshold = Double.parseDouble(args[4]);

	if(args.length > 5)
	    breakPoint = Long.parseLong(args[5]);

	BeaconFilter bf   = new BeaconFilter(window, binSize, statSelect);
	PositionFilter pf = new PositionFilter(threshold);

	TestAlgor ta = new TestAlgor(bf, pf, inputFile, breakPoint);
	ta.run();
    }

    private void dump(BeaconStatData data, BitSet fieldMask) {
	ArrayList v   = data.values;
	String result = "beaconstat: ";
	for(int i = 0; i < v.size(); i++) {
	    BeaconDistStat d = (BeaconDistStat) v.get(i);
	    result += d.space+" "+d.id+" "+d.lastUpdate+" "+d.distanceEst+" ";
	}
	System.out.println(result);
    }
}
