/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.logging;

import javax.swing.border.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import cricketdaemon.reader.sim.CustomFrame;
import cricketdaemon.Util;

/**
 * A GUI that dumps the beacon readings onto a scroll pane.  The scroll
 * pane scrolls up automatically as new data is read.  It turns out that
 * such a feature runs extremely slowly on the iPAQ.  We will phase out the
 * GUI soon.
 *
 * @author Allen Miu
 */
class GUI extends JTextArea implements Runnable, MouseListener {

    private CustomFrame frame;
    private JButton button;
    private JScrollBar scrollBar;
    private LoggingModule ptm;

    private class TextChangeListener 
	extends JTextComponent.AccessibleJTextComponent 
    {    
	private GUI selfRef;
	public TextChangeListener(GUI g) { selfRef = g; }

	public void insertUpdate(DocumentEvent e) 
	{
	    SwingUtilities.invokeLater(selfRef);
	}
    }

    public GUI(LoggingModule ptm, int x, int y)
    {
	this.ptm = ptm;
	Dimension size = new Dimension(x,y);

	CustomFrame frame = new CustomFrame("Logging Logging");
	frame.setResizable(true);
	Container container = frame.getContentPane();
	container.setLayout(new BorderLayout());

	setLineWrap(true);
	setWrapStyleWord(true);
	setEditable(false);
	setAutoscrolls(true);

	JScrollPane scroll = 
	    new JScrollPane(this, 
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	scroll.setBorder(createBorder());
	scroll.setPreferredSize(size);
	scroll.setAutoscrolls(true);
	container.add(scroll, "Center");
	Document textDoc = getDocument();
	textDoc.addDocumentListener(new TextChangeListener(this));

	scrollBar = scroll.getVerticalScrollBar();
	Util.assert(scrollBar != null, "GUI::GUI() scrollBar null");

	button = new JButton("Separation Mark");
	button.addMouseListener(this);

	button.setBorder(BorderFactory.createRaisedBevelBorder());
	container.add(button, "South");
	
	frame.setBackground(Color.white);
	frame.validate();
	frame.pack();
	frame.setVisible(true);

	Util.log("view created and exposed");
    }

    /* MouseListener stuff */
    public void mouseClicked(MouseEvent e) {
	long curTime = System.currentTimeMillis();
	String s = LoggingModule.HASH + " " + curTime;
	dumpln(s);
	ptm.dumpln(s);
    }
    
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }
    public void mousePressed(MouseEvent e) { return; }
    public void mouseReleased(MouseEvent e) { return; }


    /* public API for program */
    public void dump(String s)
    {
	append(s);
    }
    
    public void dumpln(String s)
    {
	dump(s+"\n");
    }

    private Border createBorder() 
    {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }

    public void run() 
    {
	scrollBar.setValue(scrollBar.getMaximum());
    }
}
