/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.logging;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.reader.*;
import cricketdaemon.reader.parse.ParseMap;
import cricketdaemon.reader.PlaybackReader.*;
import java.util.*;
import java.io.*;

/**
 * A module that simple dumps the results from the Reader module.
 *
 * @author Allen Miu
 */
public class LoggingModule extends ModuleThread 
    implements Beacon.BeaconHandler
{
    public static final String HASH = "#####";

    private PrintWriter file;
    GUI gui;

    public LoggingModule(ModuleThread mt, String fileName)
    {
	super("LoggingModule",0);
	BitSet fieldMask = new BitSet();
	fieldMask.set(ParseMap.SPACE);
	fieldMask.set(ParseMap.BEACONID);
	fieldMask.set(ParseMap.DISTANCE);
	mt.register(this, fieldMask);

	gui = new GUI(this, 220,50);
	if(fileName != null)
	    initFile(fileName);
    }

    private void initFile(String fName)
    {
	try {
	    File f = new File(fName);
	    Util.assert(!f.exists(), 
		 fName+" exists, please remove or use another filename");
	    file = 
		new PrintWriter(new FileWriter(fName, 
					       Util.DESTROY_OLDLOGFILE));
	}
	catch (IOException ie) {
	    //Util.assert(false, "Error creating file "+f);
	}
    }

    protected void moduleCleanUp()
    {
	if(file != null) {
	    file.flush();
	    file.close();
	}
    }

    // protected void process(); // using the inherited process method

    public void processBeacon(Beacon newData, BitSet fieldMask)
    {
	String output = System.currentTimeMillis()+" "+newData.toString2();
	//System.out.println(output);
	if(gui != null)
	    gui.dumpln(output);
	dumpln(output);
    }

    public void dumpln(String s)
    {
	System.out.println(s);
	dump(s+"\n");
    }

    public void dump(String s)
    {
	if(file != null) {
	    file.print(s);
	    file.flush();
	}
    }

    public static PlaybackBeacon parsePlayback(StringTokenizer s)
    {
	if(s.equals(HASH))
	    return null;

	long timeStamp = -1;
	try {
	    timeStamp = Long.parseLong(s.nextToken());
	}
	catch (NoSuchElementException e) {
	    Util.log("LoggingModule::parsePlayback() invalid line");
	    return null;
	}
	catch (NumberFormatException ie) {
	    Util.log("LoggingModule::parsePlayback() invalid line n");
	    return null;
	}

	PlaybackBeacon result = Beacon.parsePlayback(s, timeStamp);
	if(result != null)
	    result.timeStamp = timeStamp;
	return result;
    }
}
