package cricketdaemon.module;

import cricketdaemon.beacontable.*;
import cricketdaemon.beaconstat.*;
import cricketdaemon.space.*;
import cricketdaemon.space2.*;
import cricketdaemon.position.*;
import cricketdaemon.reader.*;
import cricketdaemon.reader.sim.*;
import cricketdaemon.reader.parse.ParseMap;
import java.util.*;

/**
 * A module that simple dumps the results from various modules.
 *
 * @author Allen Miu
 */
public class DebugModule extends ModuleThread 
    implements 
	Beacon.BeaconHandler,
	BeaconStatData.BeaconStatDataHandler,
	SpaceData.SpaceDataHandler, 
	PositionData.PositionDataHandler
{

    public DebugModule()
    {
	super("DebugModule",0);
    }

    public DebugModule(ModuleThread sp)
    {
	this();
	register(sp);
    }

    public void register(ModuleThread sp)
    {
	BitSet fieldMask = new BitSet();
	if(sp instanceof SpaceModule || 
	   sp instanceof Space2Module || 
	   sp instanceof BeaconTableModule) {
	    fieldMask.set(SpaceModule.MEAN);
	    fieldMask.set(SpaceModule.MODE);
	}
	else if(sp instanceof GPSDReader || sp instanceof SerialPortReader) {
	    fieldMask.set(ParseMap.SPACE);
	    fieldMask.set(ParseMap.BEACONID);
	    fieldMask.set(ParseMap.DISTANCE);
	    fieldMask.set(ParseMap.CARTESIAN_COORD);
	}
	else if(sp instanceof BeaconStatModule) {
	    fieldMask.set(BeaconStatModule.STAT);
	}
	else if(sp instanceof SimReader) {
	    fieldMask.set(ParseMap.SPACE);
	    fieldMask.set(ParseMap.BEACONID);
	    fieldMask.set(ParseMap.DISTANCE);
	}
	else
	    return; // silently ignore

	sp.register(this, fieldMask);
    }

    protected void process()
    {
	while(inputBuf.size() > 0)
	    dequeueDataAndDispatchHandler(this); 	    
	System.out.println("DebugModule::process() "+
			   System.currentTimeMillis());
    }

    protected void moduleCleanUp() { return; }

    public void processBeacon(Beacon data, BitSet fieldMask)
    {
	System.out.println("***DebugModule*** processBeacon\n"+data.toString2());
    }

    public void processSpaceData(SpaceData newData, BitSet fieldMask) 
    {
	System.out.println("***DebugModule*** processSpaceData\n"+newData);
    }

    public void processPositionData(PositionData newData, BitSet fieldMask)
    {
	System.out.println("***DebugModule*** processPositionData\n"+newData);
    }

    public void processBeaconStatData(BeaconStatData newData, BitSet mask)
    {
	System.out.println("***DebugModule*** processBeaconStatData\n"+newData);
    }
}
