/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.orient;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import java.util.*;

/**
 * The data type that is passed to higher layer module that registers a
 * callback with the Orient module.
 *
 * @author Allen Miu
 */
public class OrientData implements ModuleThread.Data
{
    private static final int MAX_SEQNUM = 65536;
    private static int globalSeqNum = 0;
    public int seqNum;

    public long timeStamp;
    public Float value;
    
    public OrientData(Float angle)
    {
	seqNum = globalSeqNum;
	globalSeqNum++;             // ensures unique sequence number
	globalSeqNum %= MAX_SEQNUM; // wrap around
	timeStamp = System.currentTimeMillis();
	value = angle;
    }

    public interface OrientDataHandler {
	public void processOrientData(OrientData newData, BitSet fieldMask);
    }

    public void process(Object handler, BitSet fieldMask) 
    {
	Util.assert(handler instanceof OrientDataHandler,
		    "OrientData::process() incorrect handler type");
	OrientDataHandler h = (OrientDataHandler) handler;
	h.processOrientData(this, fieldMask);
    }
    
    public String toString()
    {
	return value.toString();
    }
}

