/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.orient;


import cricketdaemon.module.*;
import cricketdaemon.position.*;
import cricketdaemon.space.DistanceEstimate;
import cricketdaemon.reader.CartesianCoord;
import cricketdaemon.reader.PhaseArrayRecord;
import cricketdaemon.reader.parse.ParseMap;
import cricketdaemon.reader.Beacon;
import cricketdaemon.UnitConv;
import cricketdaemon.Util;
import java.util.*;

/**
 * A module that computes the Orientation angle with respect to an
 * anchor beacon.
 *
 * FIXME: currently, it reports the absolute angle reported by SimReader
 *
 * Probably should use some averaged angle value and take hints from
 * BeaconStatModule.
 *
 * @author Allen Miu
 */
public class OrientModule extends ModuleThread 
    implements 
	Beacon.BeaconHandler,
	PositionData.PositionDataHandler
{
    /* OrientModule field types */
    public static final int ANGLE    = 1;
    public static final int ANCHOR   = 2;

    public OrientModule(ModuleThread mt, ModuleThread pt)
    {
	super("OrientModule", 0);
	BitSet desiredTypeMask = new BitSet();
	desiredTypeMask.set(ParseMap.ORIENT);     // for simulation demos
	desiredTypeMask.set(ParseMap.PHASEARRAY); // for Kevin
	mt.register(this, desiredTypeMask);

	if(pt != null) {
	    desiredTypeMask.clear();
	    desiredTypeMask.set(PositionModule.CARTESIAN_COORD);
	    pt.register(this, desiredTypeMask);
	}
    }

    protected void moduleCleanUp() { return; }

    public void processBeacon(Beacon newData, BitSet fieldMask)
    {
	Float orient;
	
	if(fieldMask.get(ParseMap.ORIENT)) {
	    orient = (Float) newData.getField(ParseMap.ORIENT);
	    OrientData newProcessedData = new OrientData(orient);
	    BitSet orientFieldMask = (BitSet) CLEARMASK.clone();

	    // output - backward compatible
	    orientFieldMask.set(ANGLE);
	    transfer(newProcessedData, orientFieldMask);
	}

	if(fieldMask.get(ParseMap.PHASEARRAY)) {
	    ArrayList pa = (ArrayList) newData.getField(ParseMap.PHASEARRAY);
	    if(pa != null) {
		orient = processPhaseArray(pa);
		if(orient != null) {
		    OrientData newProcessedData = new OrientData(orient);
		    BitSet orientFieldMask = (BitSet) CLEARMASK.clone();
		    orientFieldMask.set(ANGLE);
		    transfer(newProcessedData, orientFieldMask);
		}
	    }
	}
    }

    private Float processPhaseArray(ArrayList pa)
    {
	// kevin's algorithm goes here
	if(pa.size() > 0) {
	    PhaseArrayRecord r = (PhaseArrayRecord) pa.get(0);
	    //System.out.println("debug phase=" + r.orient + " " +r.sepDist);
	}
	return null;
    }
    
    public void processPositionData(PositionData newData, BitSet fieldMask)
    {
	if(fieldMask.get(PositionModule.CARTESIAN_COORD)) {
	    // array of beacon information containing 
	    // 1) the computed distance (and stats) between the listener 
	    //    and beacon
	    // 2) the beacon's space, id, and coordinates
	    // The length of the returned array represents the number of 
	    // beacons used/ in computing the listener's position
	    DistanceEstimate[] des = newData.getDistances();

	    /*
	    System.out.println("debug beacon="+des[0].space+"  coord="+
			       des[0].coord+" dist="+
			       UnitConv.cricketDistToCM(des[0].distance));
	    */

	    // Listener's position
	    CartesianCoord listenerPos = newData.getCoord();

	    // kevin's algorithm goes here
	    
	}
    }
}

// *TODO: change position/PositionData to include beacon coordinates
// *TODO: modify positionModule so that it filters beacons that do not have beacon coordinates
// *TODO: implement Kevin-specific input data types
// TODO: implement Kevin-specific output data types

