import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

class DrawingArea 
    extends JPanel
    implements MouseListener, MouseMotionListener
{

    public static SquareVertex device;

    private Node prevMousePos, selectPos, anchorPoint;
    private Image   img;
    private Graphics gc;

    public static final int nodeSize = 8; // assume this to be even
    public static final int nodeSizeDiv2 = nodeSize/2;

    /* transform stuff */
    public static Vertex minCoord, maxCoord, translate;
    public double scaleX, scaleY;

    Vertex currentSelectObject;
    SolverPanel solver;

    public DrawingArea(int width, SolverPanel sp) 
    {
	super();
	preferredSize = new Dimension(width, width);
	setBackground(Color.gray.darker());
	addMouseListener(this);
	addMouseMotionListener(this);

	translate = new Vertex(width/2.0, width/2.0);
	minCoord = new Vertex(-500, -500);
	maxCoord = new Vertex(500, 500);
	
	scaleX = width/(maxCoord.x-minCoord.x);
	scaleY = -width/(maxCoord.y-minCoord.y);

	device = new SquareVertex(0, 0);
	solver = sp;
    }

    Dimension preferredSize;
    public Dimension getPreferredSize() 
    {
	return preferredSize;
    }

    // Initialize data structure
    public void initialize() 
    {
	setEnabled(true);
	anchorPoint  = new Node(-10, -10);
	prevMousePos = new Node(-10, -10);
	selectPos    = new Node(-10, -10);
	currentSelectObject = null;
	img = createImage(getPreferredSize().width, 
			  getPreferredSize().height);
	gc  = img.getGraphics();
	Vertex.clearList();
	device.x = device.y = 0;
	
	Beacon.snap = true;
	
	repaint();
    }

    public void mousePressed(MouseEvent e) {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	Util.dbgPrintln("mpressed: "+modifier+" "+MouseEvent.BUTTON3_MASK+
			"at (" +x+","+y+").");

	anchorPoint.setLocation(x, y);
	prevMousePos.setLocation(x, y); 
	selectPos.setLocation(x, y);

	Vertex v = invTransformStatic(x, y);
	currentSelectObject = findObject(v.x, v.y);

	// checking MouseEvent.BUTTON1_MASK doesn't work for netscape,
	// so check for button3 and reverse the if-condition claues instead
	if(currentSelectObject != null) {
	    currentSelectObject.mousePressed(v.x, v.y, modifier);
	    if ((modifier & MouseEvent.BUTTON3_MASK) > 0 &&
		currentSelectObject != device) {
		currentSelectObject.remove();
		currentSelectObject = null;
	    }
	}

	repaint();
    }

    private boolean isVerticalDisplacement(int x, int y)
    {
	Util.assert(selectPos != null, 
		    "DrawingArea::isVerticalDisplacement");
	double slope = (double) (x - selectPos.x)/(y - selectPos.y);
	return Math.abs(slope) >= 1.0;
    }

    public void mouseDragged(MouseEvent e) 
    {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	/* to restrict displacement to either HORIZONTAL or VERTICAL */
	if ((modifier & MouseEvent.ALT_MASK) > 0) {
	    if(isVerticalDisplacement(x, y))
		y = selectPos.y;
	    else
		x = selectPos.x;
	}

	int xx = Math.max(Math.min(x, getPreferredSize().width-nodeSizeDiv2), nodeSizeDiv2);
	int yy = Math.max(Math.min(y, getPreferredSize().height-nodeSizeDiv2), nodeSizeDiv2);

	Util.assert(prevMousePos != null, "DrawingArea::mouseDragged()");
	Util.dbgPrintln("Dragging: (" + xx + ", " + yy + ")");
	prevMousePos.setLocation(xx, yy);

	//if(currentObjectUnderMouse != null && !insertMode) {
	Vertex v = invTransformStatic(xx, yy);
	Vertex anchor = invTransform(anchorPoint.x, anchorPoint.y);
	if(currentSelectObject != null) 
	    currentSelectObject.mouseDragged(v.x, v.y, anchor, modifier);
	    
	repaint();
    }
    
    public void mouseMoved(MouseEvent e) { return; }

    public void addBeacon()
    {
	Beacon.createInstance();
	repaint();
    }

    public void mouseReleased(MouseEvent e) 
    {
	int x        = e.getX();
	int y        = e.getY();
	int modifier = e.getModifiers();

	Util.dbgPrintln("mouseReleased "+x+" "+y);

	anchorPoint.setLocation(-10, -10);
	prevMousePos.setLocation(-10, -10);
	selectPos.setLocation(-10, -10);

	currentSelectObject = null;
	repaint();
    }

    public void mouseClicked(MouseEvent e) { return; }
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }
    
    public void update(Graphics g)
    {
	//double buffering to avoid flickering
	if(img != null) {
	    gc = img.getGraphics();
	    paint(gc);
	    g.drawImage(img, 0, 0, this);
	}
    }

    public void paint(Graphics g) 
    {
	super.paint(g);
	g.clearRect(0, 0, 
		    getPreferredSize().width, 
		    getPreferredSize().height);

	if(anchorPoint != null) {
	    // dragging
	    g.setColor(Color.yellow);
	    anchorPoint.draw(g, Color.yellow);
	    prevMousePos.draw(g, Color.yellow);
	    g.drawLine(anchorPoint.x, anchorPoint.y,
		       prevMousePos.x, prevMousePos.y);
	}

	Vertex.drawAll(g, this);
	solver.update(device);
	solver.draw(g, this);
	device.draw(g, this);
	drawGrid(g, 1, 100, 500, preferredSize.width/2);
    }

    public void  drawGrid(Graphics g, int size, float space, 
			  float realWidth, int screenWidth)
    {
	float scale = screenWidth/realWidth;
	int ulTrans = screenWidth-size;

	float i = 0;
	g.setColor(Color.black);
	do {
	    g.drawOval(Math.round(i*scale)+ulTrans, ulTrans, 2*size, 2*size);
	    g.drawOval(Math.round(-i*scale)+ulTrans, ulTrans, 2*size, 2*size);
	    g.drawOval(ulTrans, Math.round(i*scale)+ulTrans, 2*size, 2*size);
	    g.drawOval(ulTrans, Math.round(-i*scale)+ulTrans, 2*size, 2*size);
	    i += space;
	} while(i <= realWidth);	    
    }
    

    public Vertex transform(double x, double y)
    {
	return new Vertex(x*scaleX+translate.x, y*scaleY+translate.y);
    }

    public Vertex invTransform(double x, double y)
    {
	return new Vertex((x-translate.x)/scaleX,(y-translate.y)/scaleY);
    }

    private Vertex transformStatic = new Vertex(0,0);
    public Vertex invTransformStatic(double x, double y)
    {
	 transformStatic.x = (x-translate.x)/scaleX;
	 transformStatic.y = (y-translate.y)/scaleY;
	 return transformStatic;
    }

    public Vertex findObject(double x, double y)
    {
	if(device.inBound(x, y))
	    return device;       
	return Vertex.findObject(x, y);
    }
}



