import java.awt.*;
import java.util.*;
import javax.swing.*;

import Jama.*;

class SolverPanel extends JPanel
{
    Vertex curDevice;
    
    AttribValueLabel constantK, constantKSqr, 
	xvalue, yvalue, zvalue, zvalueSqrA, 
	zvalueSqrB, zvalueSqrC, zvalueSqrD,
	poserr, kerr, deter;

    //protected static double K_NOMINAL  = 34449.0/(15625 * 2.54);
    static double K_NOMINAL  = 1.0;

    public SolverPanel()
    {
	curDevice = null;
	GridLayout grid = new GridLayout(12,2);
	grid.setHgap(10);
	setLayout(grid);
	xvalue       = new AttribValueLabel(this, "x"); 
	yvalue       = new AttribValueLabel(this, "y"); 
	zvalue       = new AttribValueLabel(this, "z"); 
	zvalueSqrA   = new AttribValueLabel(this, "z^2 A");
	zvalueSqrB   = new AttribValueLabel(this, "z^2 B");
	zvalueSqrC   = new AttribValueLabel(this, "z^2 C");
	zvalueSqrD   = new AttribValueLabel(this, "z^2 D");
	constantK    = new AttribValueLabel(this, "Constant K");
	constantKSqr = new AttribValueLabel(this, "Constant K^2");
	poserr       = new AttribValueLabel(this, "Position Error");
	kerr         = new AttribValueLabel(this, "Constant Error"); 
	deter        = new AttribValueLabel(this, "Determinate"); 

	constantK.setValue(K_NOMINAL);
	validate();
    }

    public void update(Vertex device)
    {
	curDevice = device;
	update();
    }

    public void update()
    {
	ArrayList beacons = Vertex.beacons;
	if(beacons.size() >= 3) {
	    double[] x = new double[beacons.size()];
	    double[] y = new double[beacons.size()];
	    double[] z = new double[beacons.size()];
	    double[] dist = new double[beacons.size()];

	    for(int i = 0; i < beacons.size(); i++) {
		Beacon b = (Beacon) beacons.get(i);
		
		x[i]    = b.x;
		y[i]    = b.y;
		z[i]    = b.z;
		dist[i] = b.known.radius;
	    }

	    positionEstimate3Beacons(x, y, z, dist, beacons.size(), 0);
	}
    }

    public void draw(Graphics g, DrawingArea d)
    {
	ArrayList a = Vertex.beacons;
	
	Vertex p = d.transform(curDevice.x, curDevice.y);

	for(int i = 0; i < a.size(); i++) {
	    Beacon v = (Beacon) a.get(i);
	    Vertex q = d.transform(v.x, v.y);

	    Circle k = v.known;

	    g.setColor(Color.blue.darker().darker());
	    g.drawString("("+Util.textOf(v.x, 2)+", "+Util.textOf(v.y, 2)+", "+
			 Util.textOf(v.z, 2)+", "+ Util.textOf(k.radius, 2)+")",
			 Math.round((float)(q.x+5)),
			 Math.round((float)(q.y+5)));
	    g.drawLine(Math.round((float)p.x),
		       Math.round((float)p.y),
		       Math.round((float)q.x),
		       Math.round((float)q.y));
	    double dist = Math.sqrt(v.distSqr(curDevice));
	    g.drawString(Util.textOf(dist, 2), 
			 Math.round((float)((p.x+q.x)/2)),
			 Math.round((float)((p.y+q.y)/2)));
	}
    }

    /* select equation combos */
    private static int[] comboIndexA = 
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 3 };
    private static int[] comboIndexB = 
    { 1, 1, 1, 2, 2, 2, 3, 3, 4, 2, 2, 2, 3, 3, 4, 3, 3, 4, 4 };
    private static int[] comboIndexC = 
    { 2, 4, 5, 3, 4, 5, 4, 5, 5, 3, 4, 5, 4, 5, 5, 4, 5, 5, 5 };
    //        2  3  4     6  7  8  9  10 11 12 13 14 15 16 17 18 19

    /* equation combos */
    private static int[] indexI = { 0, 0, 0, 1, 1, 2 };
    private static int[] indexJ = { 1, 2, 3, 2, 3, 3 };

    public void positionEstimate3Beacons(double[] x, 
					 double[] y,
					 double[] z,
					 double[] dist,
					 int numBeacons,
					 int combo)
    {
	//double[] d_bar = new double[3];

	int j0 = indexJ[comboIndexA[combo]]; int i0 = indexI[comboIndexA[combo]];
	int j1 = indexJ[comboIndexB[combo]]; int i1 = indexI[comboIndexB[combo]];
	int j2 = indexJ[comboIndexC[combo]]; int i2 = indexI[comboIndexC[combo]];

	if(numBeacons < 3)
	    return;

	for(int i = 0; i < numBeacons; i++) {
	    System.out.println("input "+i+": "+x[i]+" "+y[i]+" "+" "+dist[i]);
	}

	double[][] amatrix = null;

	if(numBeacons == 3) {
	    amatrix = new double[2][];
	    amatrix[0] = new double[2];
	    amatrix[1] = new double[2];
	    amatrix[0][0] = 2*(x[j0]-x[i0]);
	    amatrix[0][1] = 2*(y[j0]-y[i0]);
	    amatrix[1][0] = 2*(x[j1]-x[i1]);
	    amatrix[1][1] = 2*(y[j1]-y[i1]);
	}
	else {
	    amatrix    = new double[3][];
	    amatrix[0] = new double[2];
	    amatrix[1] = new double[2];
	    amatrix[2] = new double[2];
	    
	    amatrix[0][0] = 2*(x[j0]-x[i0]);
	    amatrix[0][1] = 2*(y[j0]-y[i0]);
	    amatrix[1][0] = 2*(x[j1]-x[i1]);
	    amatrix[1][1] = 2*(y[j1]-y[i1]);
	    amatrix[2][0] = 2*(x[j2]-x[i2]);
	    amatrix[2][1] = 2*(y[j2]-y[i2]);
	}
	  
	double[][] bmatrix = null;
	
	if(numBeacons == 3) {
	    bmatrix = new double[2][];
	    bmatrix[0] = new double[1];
	    bmatrix[1] = new double[1];

	    bmatrix[0][0] = diffsqr(x[j0], x[i0]) + diffsqr(y[j0], y[i0]) - K_NOMINAL*K_NOMINAL*diffsqr(dist[j0],dist[i0]);
	    bmatrix[1][0] = diffsqr(x[j1], x[i1]) + diffsqr(y[j1], y[i1]) - K_NOMINAL*K_NOMINAL*diffsqr(dist[j1],dist[i1]);
	}
	else {
	    bmatrix = new double[3][];
	    bmatrix[0] = new double[1];
	    bmatrix[1] = new double[1];
	    bmatrix[2] = new double[1];

	    bmatrix[0][0] = diffsqr(x[j0], x[i0]) + diffsqr(y[j0], y[i0]) - K_NOMINAL*K_NOMINAL*diffsqr(dist[j0],dist[i0]);
	    bmatrix[1][0] = diffsqr(x[j1], x[i1]) + diffsqr(y[j1], y[i1]) - K_NOMINAL*K_NOMINAL*diffsqr(dist[j1],dist[i1]);
	    bmatrix[2][0] = diffsqr(x[j2], x[i2]) + diffsqr(y[j2], y[i2]) - K_NOMINAL*K_NOMINAL*diffsqr(dist[j2],dist[i2]);
	}

	Matrix A = new Matrix(amatrix);
	Matrix B = new Matrix(bmatrix);

	System.out.println("begin");
	A.print(amatrix.length , 2);
	A.inverse().print(amatrix.length , 10);
	System.out.println("end");
	//double determinate = A.det();

	// do a determinate check to see if a unique solution exists!!!
	//deter.setValue(determinate);
	
	//if(Math.abs(determinate) < 0.000000001)
	//return; // throw away sample.

	Matrix X = (new QRDecomposition(A)).solve(B);

	double[][] xmatrix = X.getArray();
	    
	double resultX = xmatrix[0][0];
	double resultY = xmatrix[1][0];
 	double resultZ = diffsqr(K_NOMINAL*dist[0], resultX - x[0]) - sqr(resultY - y[0]);
	double resultZB = diffsqr(K_NOMINAL*dist[1], resultX - x[1]) - sqr(resultY - y[1]);
	double resultZC = diffsqr(K_NOMINAL*dist[2], resultX - x[2]) - sqr(resultY - y[2]);

	
	// resultZ is resultZ squared now
	System.out.println(K_NOMINAL+" "+dist[0]+" "+resultX+" "+x[0]+" "+resultY+" "+y[0]+" "+resultZ);
	//resultZ = Math.abs(resultZ);

	xvalue.setValue(resultX);
	yvalue.setValue(resultY);
	zvalueSqrA.setValue(resultZ);
	zvalueSqrB.setValue(resultZB);
	zvalueSqrC.setValue(resultZC);
	if(numBeacons > 3) {
	    double resultZD = diffsqr(K_NOMINAL*dist[3], resultX - x[3]) - sqr(resultY - y[3]);
	    zvalueSqrD.setValue(resultZD);
	}

	if(resultZ >= 0) {
	    resultZ = Math.sqrt(resultZ);
	    zvalue.setValue(resultZ);
	}

	// ignore z component error
	double posErr = Math.sqrt(curDevice.distSqr(resultX, resultY, curDevice.z));
	System.out.println(posErr);
	poserr.setValue(posErr);

	// find all the d_bars
	//for(int i = 0; i < 3; i++)
	//d_bar[i] = K_NOMINAL*dist[i];
    }

    // more accurate way to calculate a^2 - b^2
    public static double diffsqr(double a, double b) 
    {
	return (a+b)*(a-b);
    }
    public static double sqr(double x) { return x*x; }

}

