import java.awt.*;
import java.util.*;
import javax.swing.*;

public class Vertex
{
    public static ArrayList beacons        = new ArrayList();
    public static ArrayList knownPositions = new ArrayList();

    public static final double TOLERANCE = 15;

    public double x;
    public double y;
    public double z;
    
    public Vertex(double x, double y) 
    {
	this.x = x; this.y = y;
    }

    public static Vertex createInstance()
    {
	return new Vertex(0, 0);
    }

    public static void clearList()
    {
	beacons.clear();
	knownPositions.clear();
    }

    public boolean inBound(double x, double y)
    {
	//System.out.println(this.x+" "+this.y);
	return
	    Math.abs(this.x - x) < TOLERANCE &&
	    Math.abs(this.y - y) < TOLERANCE;
    }

    public static void drawAll(Graphics g, DrawingArea drawArea)
    {
	for(int i = 0; i < beacons.size(); i++) {
	    g.setColor(Color.blue);
	    ((Vertex)beacons.get(i)).draw(g, drawArea);
	    g.setColor(Color.red);
	    ((Vertex)knownPositions.get(i)).draw(g, drawArea);

	    if(i < beacons.size() - 1) {
		drawLine(g, ((Beacon) beacons.get(i+1)).known, 
			 ((Beacon) beacons.get(0)).known, 
			 drawArea);
	    }
	}
    }

    public static void drawLine(Graphics g,
				Circle a, 
				Circle b, DrawingArea drawArea)
    {
	if(Math.abs(a.y - b.y) < 0.000001)
	    return;

	double slope = -(a.x - b.x)/(a.y - b.y);
	double yint  = (a.x*a.x - b.x*b.x + a.y*a.y - b.y*b.y -
			(a.radius*a.radius - b.radius*b.radius))/(2*(a.y-b.y));
	
	double y1 = slope*500 + yint;
	double y2 = slope*(-500) + yint;

	System.out.println(slope+" "+yint+" "+y1+" "+y2);

	Vertex p = drawArea.transform(500, y1);
	Vertex q = drawArea.transform(-500, y2);

	g.setColor(Color.green);
	g.drawLine(Math.round((float)p.x),
		   Math.round((float)p.y),
		   Math.round((float)q.x),
		   Math.round((float)q.y));
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	Vertex v = drawArea.transform(x, y);
	
	g.fillOval(Math.round((float)(v.x - DrawingArea.nodeSizeDiv2)), 
		   Math.round((float)(v.y - DrawingArea.nodeSizeDiv2)), 
		   DrawingArea.nodeSize, 
		   DrawingArea.nodeSize);
    }

    public String toString()
    {
	return "("+x+","+y+")";
    }

    public double distSqr(Vertex v)
    {
	return distSqr(x, y, z, v.x, v.y, v.z);
    }

    public double distSqr(double xx, double yy, double zz)
    {
	return distSqr(x, y, z, xx, yy, zz);
    }

    public double distSqr(double X, double Y, double Z, double XX, double YY, double ZZ)
    {
	return (X - XX)*(X - XX) + (Y - YY)*(Y - YY) + (Z-ZZ)*(Z-ZZ);
    }

    public boolean equals(Object o)
    {
	return this == o;
    }

    public static Vertex findObject(double x, double y)
    {
	for(int i = 0; i < knownPositions.size(); i++) {
	    Vertex obj = (Vertex) knownPositions.get(i);
	    if(obj.inBound(x, y))
		return obj;
	}
	for(int i = 0; i < beacons.size(); i++) {
	    Vertex obj = (Vertex) beacons.get(i);
	    if(obj.inBound(x, y))
		return obj;
	}
	return null;
    }

    public void mousePressed(double x, double y, int modifier)
    {
	return;
    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier)
    {
	this.x = x; this.y = y;
    }

    public void remove()
    {
	return;
    }
}
