/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader;

import cricketdaemon.clientlib.data.Position;
import cricketdaemon.Util;
import java.util.*;

/**
 * 3D coordinate object used as a primitive type of
 * the CricketClient beacon packet.
 *
 * @author Allen Miu
 */
public class CartesianCoord extends Position
{
    public CartesianCoord(double x, double y, double z)
    {
	super(x, y, z);
    }

    public static CartesianCoord parseCoord(String s)
    {
	StringTokenizer st = new StringTokenizer(s, " \t\n\r\f");
	st.nextToken(); // pop (
	return parseCoord(st);
    }

    public static CartesianCoord parseCoord(StringTokenizer st)
    {
	try {
	    double x = Double.parseDouble(st.nextToken());
	    double y = Double.parseDouble(st.nextToken());
	    double z = Double.parseDouble(st.nextToken());

	    return new CartesianCoord(x, y, z);
	}
	catch(NoSuchElementException e) {
	    Util.log("CartesianCoord::parseCoord() bad input");
	    return null;
	}
	catch(NumberFormatException ne) {
	    Util.log("CartesianCoord::parseCoord() bad input");
	    return null;
	}
    }
}
