/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader;

import cricketdaemon.*;
import cricketdaemon.module.*;
import cricketdaemon.reader.serial.*;
import cricketdaemon.reader.parse.ParseMap;
import java.util.*;
import java.io.*;

/**
 * Retrieves and parses beacon data from the FIRST VERSION
 * of the cricket listeners.
 *
 * @author Bodhi Priyantha, Allen Miu
 */
public class OldSerialPortReader extends SerialPortReader 
{
    // maximum size of location string (Should be set from properties.)
    private static final int MAX_STRING = 100; 

    // start and end of message markers
    private static final byte STX = 2;
    private static final byte ETX = 3;
    private static final byte US  = 31; //unit seperator

    static class OldSerialPortReaderFactory implements PortReaderFactory
    {
	public ModuleThread create(String configFile)
	{
	    return new OldSerialPortReader(configFile);
	}
    }

    static 
    {
	PortReaderCreator.
	    portReaderFactories.put(PortReaderCreator.PATH+
				    "OldSerialPortReader", 
				    new OldSerialPortReaderFactory());
    }

    public OldSerialPortReader(String configFile)
    {
	super(configFile);
    }

    // the space string from the beacon
    private byte[] buf      = new byte[MAX_STRING]; 
    private byte[] distance = new byte[2];  // buffer for the distance


    protected Beacon readData(BitSet mask) 
	throws IOException 
    {
	byte char1, char2, character;
	
	// sync
	char2 = (byte) this.in.read();
	do{
	    char1 = char2;
	    char2 = (byte) this.in.read();
	} while((char1 != ETX)||(char2 != STX));

	Beacon beacon = new Beacon(System.currentTimeMillis());

	// read the location and distance info
	int length = 0; // number of characters in the space string
	int us_count = 0;

	// warning: FIXME: will die if the string length >= MAX_STRING-2 !!!
	while(length < MAX_STRING) {
	    character = (byte) in.read();
	    if (character == US) {
		us_count++;
		if(us_count == 2) // reached the UID field
		    break;
	    }
	    else {
		buf[length] = character;
		length++;
	    }
	}
	byte[] space = new byte[length];
	System.arraycopy(buf, 0, space, 0, length);

	//byte id = in.read(); // reads the ID of the beacon
	distance[0] = (byte) in.read();  // reads higher order byte of distance
	distance[1] = (byte) in.read();  // reads lower order byte of distance

	// now transfer this reading to the buffer
	beacon.insertValue(space, ParseMap.SPACE);
	mask.set(ParseMap.SPACE);
	//	beacon.insertValue(id, ParseMap.BEACONID);
	//      mask.set(ParseMap.BEACONID);
	beacon.insertValue(distance, ParseMap.DISTANCE);
	mask.set(ParseMap.DISTANCE);

	return beacon;
    }

    public static void main(String[] args) 
    {
	OldSerialPortReader opr = 
	    new OldSerialPortReader(CricketDaemon.CONFIGFILE);
	Util.init(CricketDaemon.DEBUGLOGFILE);

	opr.run();
	/*
	opr.start();
	try {
	    opr.join();
	}
	catch (InterruptedException ie) {
	    System.err.println("interrupted!!");
	}
	finally {
	    Util.exit();
	}
	*/
	Util.exit();
    }
}
