/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.parse;

/**
 * Interface to support customizable routines for parsing field values.
 *
 * @author Allen Miu
 */
public interface FieldParser 
{
    /**
     * Implements the routine that parses the value of the
     * field type represented by the FieldParser.  The input
     * is a byte array representing the entire beacon, the offset
     * of the byte array at which the value of the field begins, 
     * and the length of the field.  Returns the Object 
     * representing the parsed data field.
     *
     * Byte ordering:  LSB ... MSB
     */
    // some wierd Java property: cannot override static methods!
    public Object parse(byte[] beacon, int offset, int length);

    /**
     * The inverse function of <code>parse()</code>. 
     *
     * @see FieldParser#parse
     */
    public byte[] unparse(Object o);

    /**
     * Converts the <code>toString()</code> representation of the
     * <code>Object</code> returned by <code>parse()</code> into the
     * original Object.  The relationship between the
     * <code>Object</code> and <code>stringToReturnType</code> is akin to,
     * for example, <code>Integer</code> and <code>Integer.valueOf()</code>.
     */
    public Object stringToReturnType(String o);
    
    /**
     * Same as <code>stringToReturnType()</code> except o takes the
     * format of the values returned by GPSD.  
     */
    public Object GPSDStringToReturnType(String o);
}
