/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;

/**
 * A FieldParser that parses the input bytes as an Integer.
 *
 * @author Allen Miu
 */
public class FieldParser_Integer implements FieldParser
{
    public static FieldParser instance = new FieldParser_Integer();

    private FieldParser_Integer()
    {
    }

    // Byte order: LSB ... MSB
    /*
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Integer: length is invalid "+length);

	int result = normalize(beacon[offset]);
	
	if(length > 1)
	    result += normalize(beacon[offset+1]) * 256;      // 2^8
	if(length > 2)
	    result += normalize(beacon[offset+2]) * 65536;    // 2^16
	if(length > 3)
	    result += normalize(beacon[offset+3]) * 16777216; // 2^24
	
	//System.out.println("FieldParser_Integer "+result);

	return new Integer(result);
    }
    */
    // MSB ... LSB
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length < 1 || length > 4)
	    Util.warn("FieldParser_Integer: length is invalid "+length);

	int result = normalize(beacon[offset]);
	
	if(length > 1)
	    result = normalize(beacon[offset+1]) + result*256;
	if(length > 2)
	    result = normalize(beacon[offset+2]) + result*256;
	if(length > 3)
	    result = normalize(beacon[offset+3]) + result*256;

	//System.out.println("FieldParser_Integer "+result);

	return new Integer(result);
    }

    // WARNING: assumes positive integer only !!!!
    public byte[] unparse(Object o)
    {
	int i = ((Integer) o).intValue();
	byte[] result = new byte[4];
	
	for(int c = 3; c >= 0; c--) {
	    result[c] = (byte) (i % 256);
	    i -= normalize(result[c]);
	    i /= 256;  // right shift
	    if( i <= 0 )
		break;
	}

	return result;
    }

    public Object stringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    result = Integer.valueOf(s);
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    public Object GPSDStringToReturnType(String s)
    {
	Object result = null;
	
	try {
	    result = Integer.valueOf(s, 16); // expects hex
	}
	catch(NumberFormatException e) {
	    // silently ignores;
	}
	
	return result;
    }

    private int normalize(byte b) {
	if( b < 0 )
	    return b + 256;
	else
	    return b;
    }

    public static void main(String[] args) {
	FieldParser p = FieldParser_Integer.instance;
	//byte[] r = p.unparse(new Integer(0x12345678));
	byte[] r = p.unparse(new Integer(152));
	for(int i = 0; i < r.length; i++)
	    System.out.println(i+" "+Integer.toHexString(r[i]));

	Integer parsed = (Integer) p.parse(r, 0, r.length);
	//System.out.println(Integer.toHexString(parsed.intValue()));
	System.out.println(Integer.toString(parsed.intValue()));
    }
}
