/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import cricketdaemon.reader.PhaseArrayRecord;
import java.util.*;

/**
 * A FieldParser that parses the input bytes as a PhaseArray.
 *
 * @author Allen Miu
 */
public class FieldParser_PhaseArray implements FieldParser 
{
    public static FieldParser instance = new FieldParser_PhaseArray();

    private FieldParser_PhaseArray()
    {
    }

    public static final int VALUE_SIZE = 4; 
    public static final int CHUNK_SIZE = 3*VALUE_SIZE; 

	
    public Object parse(byte[] beacon, int offset, int length)
    {
	if(length % CHUNK_SIZE != 0)
	    Util.warn("FieldParser_PhaseArray: length not a multiple of "
		      +CHUNK_SIZE+" "+length);

	FieldParser floatParser = FieldParser_Float.instance;

	ArrayList result = new ArrayList();

	while(length > 0) {
	    Float o = 
		(Float) floatParser.parse(beacon, offset, VALUE_SIZE);
	    Float s = 
		(Float) floatParser.parse(beacon, offset+VALUE_SIZE, VALUE_SIZE);
	    Float p = 
		(Float) floatParser.parse(beacon, offset+2*VALUE_SIZE, VALUE_SIZE);

	    result.add(new PhaseArrayRecord(o.floatValue(), s.floatValue(), p.floatValue()));
	    length -= CHUNK_SIZE;
	    offset += CHUNK_SIZE;
	}
	
	return result;
    }

    public byte[] unparse(Object o)
    {
	ArrayList a = (ArrayList) o;
	Iterator it = a.iterator();
	byte[] result = new byte[a.size() * CHUNK_SIZE];
	int offset = 0;

	while(it.hasNext()) {
	    PhaseArrayRecord par = (PhaseArrayRecord) it.next();

	    FieldParser floatParser = FieldParser_Float.instance;
	    byte[] temp = floatParser.unparse(new Float(par.orient));
	    Util.bufcpy(result, temp, offset, VALUE_SIZE);
	    
	    temp = floatParser.unparse(new Float(par.sepDist));
	    Util.bufcpy(result, temp, offset+VALUE_SIZE, VALUE_SIZE);
	    
	    temp = floatParser.unparse(new Float(par.phaseDiff));
	    Util.bufcpy(result, temp, offset+2*VALUE_SIZE, VALUE_SIZE);

	    offset += CHUNK_SIZE;
	}
	
	return result;
    }
    
    public Object stringToReturnType(String s)
    {
	ArrayList result = new ArrayList();
	StringTokenizer st = new StringTokenizer(s, " ,");
	float[] triple = new float[3];

	int i = 0;
	while(st.hasMoreElements()) {
	    triple[i] = Float.parseFloat(st.nextToken());
	    i++;
	    if(i == 3) {
		i = 0;
		result.add(new PhaseArrayRecord(triple[0], triple[1], triple[2]));
	    }
	}
	if(i != 0) {
	    Util.warn("FieldParser_PhaseArray.stringToReturnType(): "+
		      "invalid number of values from cricketd\n"+s);
	}

	return result;
    }

    public Object GPSDStringToReturnType(String s)
    {
	return stringToReturnType(s);
    }

    public static void main(String[] args)
    {
	FieldParser p = FieldParser_PhaseArray.instance;
	ArrayList a   = new ArrayList();
	a.add(new PhaseArrayRecord((float) 4.5, (float) 2.21234412, (float) 100004.6663));
	a.add(new PhaseArrayRecord((float) 5.4, (float) 2.12234412, (float) 100006.4663));
	byte[] r = p.unparse(a);

	for(int i = 0; i < r.length; i++) 
	    System.out.println(i+" "+Integer.toHexString(r[i]));
	
	System.out.println(p.parse(r, 0, 24));
    }


}
