/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.parse;

import cricketdaemon.Util;
import java.util.*;

/**
 * Maps the default field type value to the corresponding FieldParser.
 *
 * @author Allen Miu
 */
public class ParseMap
{
    /* Type values */
    //    public final static int NULL       = 0; // should never use this
    public final static int SPACE      = 1;
    public final static int BEACONID   = 2;
    public final static int DISTANCE   = 3;
    public final static int CARTESIAN_COORD = 4;
    public final static int PHASEARRAY = 5;
    public final static int VERSION    = 6;
    public final static int DURATION   = 7;
    public final static int ORIENT     = 8; // hack to propagate orientation info from SimReader
    public final static int LISTENTIME = 9; // timestamp from listener
 
    /**
     * Field types that do not change value across multiple readings from
     * the same beacon.
     */
    public final static int[] persistentFields = 
    { SPACE, BEACONID, CARTESIAN_COORD };


   /** 
     * The ith entry corresponds to the type that has a value i, 
     * as defined above. 
     */
    public static FieldParser[] map =
    { 
	null,                       // NULL
	FieldParser_ASCII.instance,    // SPACE
	FieldParser_Integer.instance,  // BEACONID     
	FieldParser_Integer.instance,  // DISTANCE
	FieldParser_CartesianCoord.instance, // COORD
	FieldParser_PhaseArray.instance, 
	FieldParser_Float.instance, // VERSION
	FieldParser_Integer.instance, // PULSE DURATION
	FieldParser_Float.instance, // ORIENT	
	FieldParser_Integer.instance,  
    };

    public static final int NUM_TYPES = map.length;

    public static Hashtable GPSDMap; // must be consistent with "map"
    static {
	GPSDMap = new Hashtable();
	GPSDMap.put("space", new Integer(SPACE)); 
	GPSDMap.put("id",    new Integer(BEACONID));
	GPSDMap.put("dist",  new Integer(DISTANCE));
	GPSDMap.put("coord", new Integer(CARTESIAN_COORD));
	GPSDMap.put("phase", new Integer(PHASEARRAY));
	GPSDMap.put("ver", new Integer(VERSION));
	GPSDMap.put("duration", new Integer(DURATION));
	GPSDMap.put("orient", new Integer(ORIENT));
	GPSDMap.put("time", new Integer(LISTENTIME));
    }

    public static int typeToFieldID(String type)
    {
	Object o = GPSDMap.get(type);
	if(o == null)
	    return -1;
	else
	    return ((Integer) o).intValue();
    }
}

