/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.serial;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import java.util.*;

/**
 * Class used to support dynamic class loading for the OldSerialPortReader
 * and SerialPortReader.
 *
 * @author Allen Miu
 */
public abstract class PortReaderCreator
{
    public static Hashtable portReaderFactories = new Hashtable();

    public static final String PATH = "cricketdaemon.reader.";

    // The return type is ModuleThread to prevent the loading
    // of the SerialPortReader class, unless this method is called.
    public static ModuleThread createPortReader(String name,
						String configFile)
    {
	PortReaderFactory p = 
	    (PortReaderFactory) portReaderFactories.get(name);
	if(p == null) {
	    try {
		Class.forName(name);
		p = (PortReaderFactory) portReaderFactories.get(name);
		Util.assert(p != null, 
			    "PortReaderCreator::createPortReader()");
	    }
	    catch (ClassNotFoundException e) {
		Util.assert(false, "PortReaderCreator::createPortReader() "+
			    name+" class not found");
	    }
	}
	return p.create(configFile);
    }
}
