/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
import java.util.*;
import java.io.*;
import javax.comm.*;
import java.io.*;

/**
 * A simple application to test the serial port.
 *
 * @author Allen Miu
 */
public class PortTester implements SerialPortEventListener
{
    protected InputStream in;

    public PortTester()
    {
	CommPortIdentifier portId;
	SerialPort sp = null;
	//	String port   = "/dev/ttySA0";
	String port   = "COM1";

	// plus 6 bytes padding for end of field and end of beacon markers
	try{
	    int baudrate  = 9600;
            System.out.println("Trying to open port "+port);
            portId = CommPortIdentifier.getPortIdentifier(port);
	    System.out.println("Opening port "+ portId.getName());
	    sp = addPort(portId);
	    configPort(sp, port, baudrate);
        } 
	catch (NumberFormatException e) {
	    System.out.println("Invalid baudrate specified in config file");
	}
	catch (NoSuchPortException e) {
            e.printStackTrace();
            System.out.println("Port " + port + " not found!");
	}
	// get I/O streams
	try{
            in = sp.getInputStream();
	}
	catch (IOException e) {
            e.printStackTrace();
            System.out.println("Could not get serial port I/O streams");
        }

	System.out.println("PortReader initialization done");
    }


    /**
     * Opens the port and assign to serial port.
    */
    private SerialPort addPort(CommPortIdentifier portId)
    {
	SerialPort sp = null;

        // is this a serial port ?
	if(portId.getPortType() != CommPortIdentifier.PORT_SERIAL)
	    System.out.println(portId.getName() + " is not a serial port!");

	if(portId.isCurrentlyOwned())
	    System.out.println("Detected " + portId.getName() + " in use by "
		    + portId.getCurrentOwner());

	// try to open the port
	try {
	    sp = (SerialPort) portId.open("cricket", 2000);
	    if(sp == null)
		System.out.println("Error opening port " + portId.getName());
	} catch (PortInUseException e) {
	    System.out.println(portId.getName()+" is in use by "+ e.currentOwner);
	}
	return sp;
    }

    /**
     * Configure the serail port accordingly
     */
    private void configPort(SerialPort sp, String port, int baudrate) 
    {

	/*
	 *  Setup an event listener for the port
	 */
	/*
	try {
	    sp.addEventListener(this);
	} catch (TooManyListenersException tmle) {
	    tmle.printStackTrace();
	}
	*/
        // set serial port parameters
	try{
            sp.setSerialPortParams(baudrate,
				   SerialPort.DATABITS_8,
				   SerialPort.STOPBITS_1,
				   SerialPort.PARITY_NONE);
            sp.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
	    /*
	    sp.notifyOnBreakInterrupt(true);
	    sp.notifyOnCarrierDetect(true);
	    sp.notifyOnCTS(true);
	    sp.notifyOnDataAvailable(true);
	    sp.notifyOnDSR(true);
	    sp.notifyOnFramingError(true);
	    sp.notifyOnOutputEmpty(true);
	    sp.notifyOnOverrunError(true);
	    sp.notifyOnParityError(true);
	    sp.notifyOnRingIndicator(true);
	    */
	    System.out.println("configPort "+baudrate+" "+port+" "+sp);
	} catch (UnsupportedCommOperationException e) {
            System.out.println("Couldn't set serial port parameters");
	}
    }

    public void readData() 
	throws IOException 
    {
	char char2;
	
	// sync
	//System.out.println("about to read a byte "+in.available());
	char2 = (char) in.read();
	//if(Character.isDigit(char2)) {
	    if(char2 == '5')
		System.out.print("\n");
	    System.out.print(char2);
	    //}
    }


    public static void main(String[] args) {
	PortTester t = new PortTester();
	try {
	    while(true)
		t.readData();
	}
	catch (IOException ie) {
	    ie.printStackTrace();
	    System.out.println("main exited with IOException");
	}
    }

	/**
	 * Implementation of the event listener
	 */
	public void serialEvent( SerialPortEvent e) {
		int i = e.getEventType();
		switch(i){
		case SerialPortEvent.DATA_AVAILABLE:
			System.out.println("Data Available");
			break;
		case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
			System.out.println("Output Buffer Empty");
			break;
		case SerialPortEvent.CTS:
			System.out.println("Clear to Send");
			break;
		case SerialPortEvent.DSR:
			System.out.println("Data Set Ready");
			break;
		case SerialPortEvent.RI:
			System.out.println("Ring Indicator");
			break;
		case SerialPortEvent.CD:
			System.out.println("Carrier Detect");
			break;
		case SerialPortEvent.OE:
			System.out.println("Overrun Error");
			break;
		case SerialPortEvent.PE:
			System.out.println("Parity Error");
			break;
		case SerialPortEvent.FE:
			System.out.println("Framing Error");
			break;
		case SerialPortEvent.BI:
			System.out.println("Break Interrupt");
			break;
		default: 
			System.out.println("Unknown event");
		}
	}

}
