/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.sim;

import cricketdaemon.clientlib.sim.SimControl;
import cricketdaemon.Util;
import viewfinder.Coord;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Keyboard controls for the simulation GUI.
 *
 * @author Allen Miu
 */
class Controls extends JPanel implements KeyListener {

    JLabel beacon_title = new JLabel("Beacon: ");
    JLabel device_title = new JLabel("Device: ");

    public JLabel x, y, z, orient, curBeaconX, curBeaconY;

    SimControl simCntl;

    public Controls (SimControl s) {
	super();
	simCntl = s;

	setLayout(new FlowLayout());

	//_addInfoLabel("SweepAngle:", "PI/8");
	//_addInfoLabel("SweepDist:", "150");
	//	_addInfoLabel("Beacon ", null);
	//	curBeaconX = _addInfoLabel(" x:", "0");
	//	curBeaconY = _addInfoLabel(" y:", "0");
	_addInfoLabel(" Coords", null);
	x = _addInfoLabel(" x:", "0");
	y = _addInfoLabel(" y:", "0");    
	z = _addInfoLabel(" z:", "0");    
	orient = _addInfoLabel(" angle:", "0.0");    
    }

    private JLabel _addInfoLabel(String title, String value) 
    {
	JLabel t = new JLabel(title, JLabel.LEFT);
	add(t);
	
	if( value != null ) {
	    JLabel v = new JLabel(value, JLabel.RIGHT);
	    add(v);
	    return v;
	}
	return null;
    }
    
    public void keyTyped(KeyEvent e) 
    {
	// do nothing
    }

    public void keyReleased(KeyEvent e)
    {
	// do nothing
    }

    public void keyPressed(KeyEvent e) 
    {
	if(simCntl == null)
	    return;

	int k = KeyEvent.VK_UNDEFINED;
	if(e.isActionKey())
	    k = e.getKeyCode();
	else
	    k = (int) Character.toLowerCase(e.getKeyChar());
	
	//System.out.println("key typed "+k);
	switch(k) {
	case (int) 'f':
	    orient.setText(Util.textOf(Coord.fullCircleInRadians-
				       simCntl.incrDeviceRotate(),3));
	    break;
	case (int) 'd':
	    orient.setText(Util.textOf(Coord.fullCircleInRadians-
				       simCntl.decrDeviceRotate(),3));
	    break;
	case KeyEvent.VK_UP:
	case (int)'i':
	    y.setText(Util.textOf(simCntl.moveDeviceBackward(),3));
	    break;
	case KeyEvent.VK_DOWN:
	case (int)'m':
	    y.setText(Util.textOf(simCntl.moveDeviceForward(),3));
	    break;
	case KeyEvent.VK_LEFT:
	case (int)'j':
	    x.setText(Util.textOf(simCntl.moveDeviceLeft(),3));
	    break;
	case KeyEvent.VK_RIGHT:
	case (int)'k':
	    x.setText(Util.textOf(simCntl.moveDeviceRight(),3));
	    break;
	case (int)'u':
	    z.setText(Util.textOf(simCntl.moveDeviceUp(),3));
	    break;
	case (int)'n':
	    z.setText(Util.textOf(simCntl.moveDeviceDown(),3));
	    break;	
	}
	
	simCntl.notice();
    }
}


