/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.reader.sim;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.clientlib.sim.SimControl;
import cricketdaemon.reader.*;
import cricketdaemon.reader.parse.*;
import cricketdaemon.position.*;
//import pathfinder.Waypoint;
import viewfinder.Coord;
import java.util.*;
import java.io.*;

/**
 * Simulates a PortReader by generating and feeding test beacon data.
 *
 * @author Allen Miu
 */
public class SimReader extends ModuleThread implements SimControl
{
    protected BitSet packetFieldMask;
    protected final static Random rand = new Random();
    //private final static double JITTER = 0.4;
    private final static double JITTER = 0;

    /* if Viewfinder is used, should be consistent ??!! */
    private final static int X_MAX = 600;
    private final static int Y_MAX = 600;
    private final static int Z_MAX = 150;

    protected ArrayList beacons;
    private ArrayList path;

    private int[] deviceCoord = { 111, 52, 74 };
    private double _absOrient = 0.0;
    private final int X = 0;
    private final int Y = 1;
    private final int Z = 2;

    //public static final int RANGE = 190;
    public static final int RANGE = 236;
    private int prevCycle;
    protected int cycle = 0;

    private GUI gui;

    private static final long BEACONPERIOD = 50;

    /*
      ByteArrayOutputStream distanceArray;
      DataOutputStream distanceData;
    */
    protected SimReader(String beaconConfig)
    {
	//super("SimReader", 175);
	//super("SimReader", 500);
	super("SimReader", BEACONPERIOD);
	beacons = new ArrayList();
	initBeacons(beacons, beaconConfig);
    }

    public SimReader(String beaconConfig, String pathConfig)
    {
	this(beaconConfig);
	prevCycle = beacons.size()-1;
	simDistances = new double[beacons.size()];

	gui = new GUI("SimReader", X_MAX, Y_MAX, this);
	gui.setBeacons(beacons);

	/*
	if(pathConfig != null) {
	    path = new ArrayList();
	    initPath(path, pathConfig);
	    Util.assert(path.size() > 0, "SimReader() path init error");
	    Waypoint start = (Waypoint) path.get(0);
	    deviceCoord[X] = (int) start.pos.x;
	    deviceCoord[Y] = (int) start.pos.y;
	    deviceCoord[Z] = (int) start.pos.z;
	    gui.setPath(path);
	}
	*/

	/*
	  distanceArray = new ByteArrayOutputStream();
	  distanceData  = new DataOutputStream(distanceArray);
	*/
	notice();
    }

    protected void moduleCleanUp() { return; }

    public final static String COMMENT = "#";

    /*
    public static void initPath(ArrayList path, String configFile)
    {
	InputStream i = ClassLoader.getSystemResourceAsStream(configFile);
	Util.assert(i != null, "SimReader::initBeacons() "+configFile+
		    " not found");
	//Properties p = new Properties();
	BufferedReader r = null;
	try {
	    r = new BufferedReader(new InputStreamReader(i));
	    //p.load(i);
	    //Enumeration entries = p.propertyNames();
	    String aline;
	    Util.assert(r != null, "SimReader::initPath() r is null");
	    //while(entries.hasMoreElements()) {
	    while((aline = r.readLine()) != null) {
		StringTokenizer entry = 
		    new StringTokenizer(aline);
		
		String key = entry.nextToken();
		
		if(key.startsWith(COMMENT))
		    continue;
		
		String x  = entry.nextToken();
		String y  = entry.nextToken();
		String z  = entry.nextToken();
		
		System.out.println("read: "+key+" "+x+" "+y+" "+z);
		
		Waypoint w = new Waypoint(Double.parseDouble(x),
					  Double.parseDouble(y),
					  Double.parseDouble(z),
					  key);
		path.add(w);
		//if(key.equals("START"))
		//start = w;
		//else if(key.equals("GOAL"))
		//goal = w;
		//else
		//path.add(w);
	    }
	}
	catch (IOException io) {
	    io.printStackTrace();
	    Util.assert(false, "SimReader::initPath() "+configFile);
	}

	//Util.assert(start != null, "SimReader::initPath() start null");
	//Util.assert(goal  != null, "SimReader::initPath() goal null");

	//path.add(goal);
	//path.add(0, start);
    }
    */

    public static void initBeacons(ArrayList beacons, String configFile) 
    {
	InputStream i = ClassLoader.getSystemResourceAsStream(configFile);
	Util.assert(i != null, "SimReader::initBeacons() "+configFile+
		    " not found");
	Properties p = new Properties();
	
	try {
	    p.load(i);
	}
	catch (IOException io) {
	    io.printStackTrace();
	    Util.assert(false, "SimReader::initBeacons() "+configFile);
	}
	Enumeration entries = p.propertyNames();
	while(entries.hasMoreElements()) {
	    String keyAndId = (String) entries.nextElement();
	    System.out.println("fetching key "+keyAndId);
	    StringTokenizer entry = 
		new StringTokenizer(p.getProperty(keyAndId));

	    int delim = keyAndId.indexOf("%");
	    String key = keyAndId.substring(0,delim);
	    String id = keyAndId.substring(delim+1);
	    String x  = entry.nextToken();
	    String y  = entry.nextToken();
	    String z  = entry.nextToken();

	    System.out.println(id+" "+x+" "+y+" "+key);

	    beacons.add(new Coord(Double.parseDouble(x),
				  Double.parseDouble(y),
				  key, Integer.parseInt(id)));
	}
    }

    // for collecting exact distances
    private double[] simDistances;
    //private boolean POSITION_TEST = true;
    private boolean POSITION_TEST = false;

    // all distance measurements are assumed to be in centimeters
    protected void process()
    {

	Coord prevBeaconCoord = (Coord) beacons.get(prevCycle);
	Coord currBeaconCoord = (Coord) beacons.get(cycle);

	int prevBeaconX = (int) prevBeaconCoord.x;
	int prevBeaconY = (int) prevBeaconCoord.y;
	int currBeaconX = (int) currBeaconCoord.x;
	int currBeaconY = (int) currBeaconCoord.y;

	gui.clearBeaconTransmit(prevBeaconX, prevBeaconY, RANGE);
	gui.drawBeaconTransmit(currBeaconX, currBeaconY, RANGE);
	gui.drawDevice(deviceCoord[X], deviceCoord[Y], _absOrient, 0, 10);

	CartesianCoord beaconXYZ = 
	    new CartesianCoord(currBeaconCoord.x, currBeaconCoord.y, 0);
	CartesianCoord deviceXYZ =
	    new CartesianCoord(deviceCoord[X], deviceCoord[Y], deviceCoord[Z]);

	double dist = beaconXYZ.dist(deviceXYZ);

	System.out.println("##### "+currBeaconCoord.beaconID+" dist: "+dist);

	// log simDistances before adding the offset for
	// the cricketdaemon
	simDistances[cycle] = dist; // centimeters
	send(dist, currBeaconCoord);

	/* next cycle */
	prevCycle = cycle;
	cycle++;
	cycle %= beacons.size();
	if(cycle == 0 && POSITION_TEST)
	    positionTest();
    }

    protected void send(double dist, Coord currBeaconCoord)
    {
	if(dist <= RANGE) {
	    double error = JITTER*dist;
	    
	    //if(error > 0 && cycle == 3)
	    //dist += (rand.nextDouble() - 0.5)*error;

	    dist += (rand.nextDouble())*error; // want to simulate one sided error


	    // putting dist before calculating the error may 
	    // not be as convenient... we want to express the errors
	    // in terms of the distance rather in Bodhi's

	    //dist = Util.centimetersToB(dist)*2.54;
	    //dist = Util.centimetersToB(dist*2.54);

	    // log simDistances before adding the offset for
	    // the cricketdaemon
	    //simDistances[cycle] = dist;
	    
	    // add Cricket receiver offset
	    //dist += Util.BEACON_OFFSET_BODHI;

	    /*
	      distanceArray.reset();
	      try {
	      distanceData.writeDouble(dist);
	      }
	      catch(IOException e) {
	      Util.assert(false, "SimReader::process()");
	      }
	      byte[] distanceDouble = distanceArray.toByteArray();
	    */
	    // MSB ... LSB
	    byte[] distanceByte   = new byte[2];
	    // conversion from byte to int may cause sign extension
	    distanceByte[1] = (byte) (Math.round(dist) % 256);
	    if(dist > 255)
		distanceByte[0] = (byte) (Math.round(dist) / 256);
	    else
		distanceByte[0] = 0;
	    
	    //System.out.println(" "+(int)distanceByte[1]);

	    // assume that we have parsed one full beacon here
	    /* create new beacon */
	    Beacon newBeacon = new Beacon(System.currentTimeMillis());
	    newBeacon.insertValue(currBeaconCoord.space.getBytes(), 
				  ParseMap.SPACE);
	    newBeacon.insertValue(distanceByte, ParseMap.DISTANCE);
	    //newBeacon.insertValue(distanceDouble, ParseMap.DISTANCE_DOUBLE);

	    //newBeacon.insertInv(beaconXYZ, ParseMap.CARTESIAN_COORD);

	    byte[] bID = new byte[1];
	    bID[0] = (byte) currBeaconCoord.beaconID;
	    newBeacon.insertValue(bID, ParseMap.BEACONID);

	    /* phase */
	    ArrayList pa   = new ArrayList();
	    PhaseArrayRecord par = new PhaseArrayRecord( distanceByte[0], distanceByte[1], 0 );
	    pa.add(par);
	    newBeacon.insertInv(pa, ParseMap.PHASEARRAY);

	    /* orientation */
	    newBeacon.insertInv(new Float((float)_absOrient), ParseMap.ORIENT);

	    /* set mask that represents the received fields */
	    packetFieldMask = (BitSet) CLEARMASK.clone();
	    packetFieldMask.set(ParseMap.SPACE);
	    packetFieldMask.set(ParseMap.BEACONID);
	    packetFieldMask.set(ParseMap.DISTANCE);
	    packetFieldMask.set(ParseMap.ORIENT);
	    packetFieldMask.set(ParseMap.PHASEARRAY);

	    //packetFieldMask.set(ParseMap.CARTESIAN_COORD);

	    /* send beacons */
	    //System.out.println("transferring "+newBeacon);
	    transfer(newBeacon, packetFieldMask);
	}
    }

    private static PositionSolver ps = new PositionSolver(PositionModule.K_NOMINAL, 0.5);

    private void positionTest() 
    {
	System.out.print("PositionTest: ");

	if(beacons.size() < 3)
	    return;

	/* the following setup can be done once statically */
	double[] x     = new double[beacons.size()];
	double[] y     = new double[beacons.size()];
	double[] z     = new double[beacons.size()];
	String[] space = new String[beacons.size()];
	int[]    id    = new int[beacons.size()];

	for(int i = 0; i < beacons.size(); i++) {
	    Coord b = (Coord) beacons.get(i);
	    x[i] = b.x;
	    y[i] = b.y;
	    z[i] = 0;
	    space[i] = b.space;
	    id[i]    = b.beaconID;
	}
	PositionData newEstimate = 
	    ps.positionEstimateVKnown(x, y, z, space, id, simDistances);
	System.out.println(newEstimate);
    }

    /* SimControl interface */

    public double moveDeviceUp() 
    {
	if(deviceCoord[Z] > 0)
	    deviceCoord[Z]--;
	return deviceCoord[Z];
    }

    public double moveDeviceDown() 
    {
	if(deviceCoord[Z] < Z_MAX)
	    deviceCoord[Z]++;
	return deviceCoord[Z];
    }

    public double moveDeviceForward() 
    {
	if(deviceCoord[Y] > 0)
	    deviceCoord[Y]--;
	return deviceCoord[Y];
    }

    public double moveDeviceBackward()
    {
	if(deviceCoord[Y] < Y_MAX)
	    deviceCoord[Y]++;
	return deviceCoord[Y];
    }
    public double moveDeviceLeft()
    {
	if(deviceCoord[X] > 0)
	    deviceCoord[X]--;
	return deviceCoord[X];
    }

    public double moveDeviceRight()
    {
	if(deviceCoord[Y] < X_MAX)
	    deviceCoord[X]++;
	return (double) deviceCoord[X];
    }

    public double incrDeviceRotate()
    {
	_absOrient += 0.1;
	_absOrient %= Coord.fullCircleInRadians;
	return _absOrient;
    }

    public double decrDeviceRotate()
    {
	_absOrient -= 0.1;
	if(_absOrient < 0)
	    _absOrient = Coord.fullCircleInRadians + _absOrient;
	return _absOrient;
    }

    public void notice()
    {
	if(gui != null && deviceCoord != null)
	    gui.paintDevice(deviceCoord[X], deviceCoord[Y],
			    _absOrient, 0, 10);
    }
}

