/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.server;

import cricketdaemon.*;
import java.net.*;
import java.util.*;
import java.io.*;

/**
 * States of a registered client that wish to obtain updates from the
 * CricketDaemon.
 *
 * @author Allen Miu
 */
class ClientRecord {
    BufferedReader reader = null;

    InetAddress addr;
    int port;
    long expiration;
    Socket s = null;

    int state;

    public final static int CONNECTING  = 0;
    public final static int ESTABLISHED = 1;

    public ClientRecord(Socket sock, long e)
    {
	this(sock.getInetAddress(), sock.getPort(), e);
	s = sock;
    }

    public ClientRecord(InetAddress a, int p, long e)
    {
	addr = a; port = p; expiration = e; state = CONNECTING;
    }
    
    public boolean equals(ClientRecord other)
    {
	return addr.equals(other.addr) && port == other.port;
    }

    static public int indexOf(ClientRecord query, ArrayList a)
    {
	for(int i = 0; i < a.size(); i++) {
	    ClientRecord item = (ClientRecord) a.get(i);
	    if(query.equals(item))
		return i;
	}
	return -1;
    }

    public DatagramPacket createPacket(byte[] payload)
	throws IOException
    {
	Util.assert(s == null, "ClientRecord::createPacket()");
	return new DatagramPacket(payload, payload.length, addr, port);
    }

    public Socket getSocket()
    {
	return s;
    }

    public BufferedReader getBufferedReader()
	throws IOException
    {
	if(reader == null)
	    reader = new BufferedReader(new InputStreamReader(getSocket().getInputStream()));
	return reader;
    }

    public String toString()
    {
	return addr.toString() + ":" + port;
    }
}

