/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.server;

import cricketdaemon.Util;
import java.net.*;
import java.io.*;

/**
 * A thread that listens to incoming registrations.
 *
 * @author Allen Miu
 */
public class ServerThread extends Thread
{
    public static final String REGISTER_TOKEN    = "register";
    public static final String BEACONTABLE_TOKEN = "beacontable";
    public static final String POSITION_TOKEN    = "position";

    ServerConnListener scl;
    ServerSocket s;
    
    ServerThread(ServerConnListener listener, int port) 
    {
	scl = listener;
	try {
	    s = new ServerSocket(port);
	    s.setSoTimeout(0); // enable blocking
	}
	catch (IOException e) {
	    Util.assert(false, "error opening port "+port);
	}
    }
    
    public void run()
    {
	//this should just register each client
	byte[] buf = new byte[64];
	while(true) {
	    try {
		Socket newSock = s.accept();
		newSock.setTcpNoDelay(true);
		System.out.println("accept TcpNoDelay="+newSock.getTcpNoDelay());
		scl.registerClient(newSock); 
	    }
	    catch (IOException io) {
		io.printStackTrace();
		Util.assert(false,"ServerThread::run() ioexception");
	    }
	}
	//s.close(); never reached 
    }
}
