/*
 * Copyright (c) 2000, 2001 Massachusetts Institute of Technology.
 *
 * This software is being provided by the copyright holders under the GNU
 * General Public License, either version 2 or, at your discretion, any later
 * version. (See the file `LICENSE' in the source distribution.)
 *
 * This software is provided "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. The entire risk as
 * to the quality and performance of the program is with you. See the GNU
 * General Public License for more details.
 *
 * The name and trademarks of copyright holders may NOT be used in 
 * advertising or publicity pertaining to the software without specific, 
 * written prior permission. Title to copyright in this software and any 
 * associated documentation will at all times remain with copyright 
 * holders. See the file CRICKETAUTHORS which should have accompanied 
 * this software for a list of all copyright holders. 
 * 
 * Particular files in this distribution may be derived from previously
 * copyrighted software. This copyright applies only to those changes made by
 * the copyright holders listed in the CRICKETAUTHORS file.
 */
package cricketdaemon.space;

import cricketdaemon.Util;
import cricketdaemon.reader.CartesianCoord;
import java.util.*;

/** 
 * A distance measurment from a a single beacon. Implements
 * Comparator to allow sorting by distance in an array.
 *
 * @author Allen Miu
 */
public class DistanceEstimate implements Comparator {
    public String space;
    public int beaconID;    // need to get rid of these primitives??
    public double distance;
    public CartesianCoord coord;

    public Integer beaconIDInteger;
    public Double distanceDouble;

    public DistanceEstimate(String s, int b, CartesianCoord c, double d) {
	space = s; beaconID = b; coord = c; distance = d;
	beaconIDInteger = new Integer(b);
	distanceDouble = new Double(d);
    }

    public int compare(Object o1, Object o2) {
	DistanceEstimate e1 = (DistanceEstimate) o1;
	DistanceEstimate e2 = (DistanceEstimate) o2;

	int diff = (int) (e1.distance - e2.distance);
	
	if(diff == 0) {
	    diff = e1.space.compareTo(e2.space);
	    if(diff == 0)
		return e1.beaconID - e2.beaconID;
	}
	
	return diff;
    }

    public boolean equals(Object o) {
	return compare(this, o) == 0;
    }

    public int hashCode() {
	return space.hashCode()+beaconIDInteger.hashCode()+
	    distanceDouble.hashCode();
    }
    
    public String toString() {
	return space+" "+beaconID+" "+distance;
    }
}

